#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "eigend.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef eigend::c2p_t c2p_type_; 


 
static std::map<c2p_type_ *, PyObject *> c2p_wrapped_;
 


struct c2p_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(c2p_type_ *, bool owned);
    PyObject *(*wrapcopy)(c2p_type_ *);
    c2p_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< c2p_type_ *, PyObject * > &);
    void (*erase)(c2p_type_ *);
    PyObject *(*find)(c2p_type_ *);
};

struct c2p_dispatch_ *c2p_dispatcher_;



typedef eigend::p2c_t p2c_type_; 


 
static std::map<p2c_type_ *, PyObject *> p2c_wrapped_;
 


struct p2c_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(p2c_type_ *, bool owned);
    PyObject *(*wrapcopy)(p2c_type_ *);
    p2c_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< p2c_type_ *, PyObject * > &);
    void (*erase)(p2c_type_ *);
    PyObject *(*find)(p2c_type_ *);
};

struct p2c_dispatch_ *p2c_dispatcher_;



typedef pia::scaffold_gui_t scaffold_gui_type_; 





struct scaffold_gui_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scaffold_gui_type_ *, bool owned);
    PyObject *(*wrapcopy)(scaffold_gui_type_ *);
    scaffold_gui_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scaffold_gui_type_ *, PyObject * > &);
    void (*erase)(scaffold_gui_type_ *);
    PyObject *(*find)(scaffold_gui_type_ *);
};

struct scaffold_gui_dispatch_ *scaffold_gui_dispatcher_;



typedef piw::term_t term_type_; 





struct term_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(term_type_ *, bool owned);
    PyObject *(*wrapcopy)(term_type_ *);
    term_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< term_type_ *, PyObject * > &);
    void (*erase)(term_type_ *);
    PyObject *(*find)(term_type_ *);
};

struct term_dispatch_ *term_dispatcher_;



typedef pic::f_string_t f_string_type_; 





struct f_string_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_string_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_string_type_ *);
    f_string_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_string_type_ *, PyObject * > &);
    void (*erase)(f_string_type_ *);
    PyObject *(*find)(f_string_type_ *);
};

struct f_string_dispatch_ *f_string_dispatcher_;





int fpcvt_c2p(PyObject *o, void *a)
{
    c2p_type_ *object;
    if(o == Py_None)
    {
        *((c2p_type_ **)a) = 0;
        return 1;
    }
    object = c2p_dispatcher_->unwrap(o);
    *((c2p_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_c2p_copy fpcvt_c2p

PyObject *tpcvt_c2p(void *a)
{
    c2p_type_ *o = *((c2p_type_ **)a);
    return c2p_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_c2p_copy(void *a)
{
    c2p_type_ *o = (c2p_type_ *)a;
    return c2p_dispatcher_->wrapcopy(o);
}



int fpcvt_p2c(PyObject *o, void *a)
{
    p2c_type_ *object;
    if(o == Py_None)
    {
        *((p2c_type_ **)a) = 0;
        return 1;
    }
    object = p2c_dispatcher_->unwrap(o);
    *((p2c_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_p2c_copy fpcvt_p2c

PyObject *tpcvt_p2c(void *a)
{
    p2c_type_ *o = *((p2c_type_ **)a);
    return p2c_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_p2c_copy(void *a)
{
    p2c_type_ *o = (p2c_type_ *)a;
    return p2c_dispatcher_->wrapcopy(o);
}



int fpcvt_scaffold_gui(PyObject *o, void *a)
{
    scaffold_gui_type_ *object;
    if(o == Py_None)
    {
        *((scaffold_gui_type_ **)a) = 0;
        return 1;
    }
    object = scaffold_gui_dispatcher_->unwrap(o);
    *((scaffold_gui_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scaffold_gui_copy fpcvt_scaffold_gui

PyObject *tpcvt_scaffold_gui(void *a)
{
    scaffold_gui_type_ *o = *((scaffold_gui_type_ **)a);
    return scaffold_gui_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scaffold_gui_copy(void *a)
{
    scaffold_gui_type_ *o = (scaffold_gui_type_ *)a;
    return scaffold_gui_dispatcher_->wrapcopy(o);
}



int fpcvt_term(PyObject *o, void *a)
{
    term_type_ *object;
    if(o == Py_None)
    {
        *((term_type_ **)a) = 0;
        return 1;
    }
    object = term_dispatcher_->unwrap(o);
    *((term_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_term_copy fpcvt_term

PyObject *tpcvt_term(void *a)
{
    term_type_ *o = *((term_type_ **)a);
    return term_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_term_copy(void *a)
{
    term_type_ *o = (term_type_ *)a;
    return term_dispatcher_->wrapcopy(o);
}



int fpcvt_f_string(PyObject *o, void *a)
{
    f_string_type_ *object;
    if(o == Py_None)
    {
        *((f_string_type_ **)a) = 0;
        return 1;
    }
    object = f_string_dispatcher_->unwrap(o);
    *((f_string_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_string_copy fpcvt_f_string

PyObject *tpcvt_f_string(void *a)
{
    f_string_type_ *o = *((f_string_type_ **)a);
    return f_string_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_string_copy(void *a)
{
    f_string_type_ *o = (f_string_type_ *)a;
    return f_string_dispatcher_->wrapcopy(o);
}



 

struct c2p_object_ {
    PyObject_HEAD
    c2p_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct c2p_wrapper_: public c2p_type_ 
{

    static void __deleter__(c2p_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
        cache_8_ = 0;
    
        cache_9_ = 0;
    
        cache_10_ = 0;
    
        cache_11_ = 0;
    
        cache_12_ = 0;
    
        cache_13_ = 0;
    
        cache_14_ = 0;
    
        cache_15_ = 0;
    
        cache_16_ = 0;
    
        cache_17_ = 0;
    
        cache_18_ = 0;
    
        cache_19_ = 0;
    
        cache_20_ = 0;
    
        cache_21_ = 0;
    
        cache_22_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
        if(cache_8_)
        {
            Py_DECREF(cache_8_);
            cache_8_ = 0;
        }
    
        if(cache_9_)
        {
            Py_DECREF(cache_9_);
            cache_9_ = 0;
        }
    
        if(cache_10_)
        {
            Py_DECREF(cache_10_);
            cache_10_ = 0;
        }
    
        if(cache_11_)
        {
            Py_DECREF(cache_11_);
            cache_11_ = 0;
        }
    
        if(cache_12_)
        {
            Py_DECREF(cache_12_);
            cache_12_ = 0;
        }
    
        if(cache_13_)
        {
            Py_DECREF(cache_13_);
            cache_13_ = 0;
        }
    
        if(cache_14_)
        {
            Py_DECREF(cache_14_);
            cache_14_ = 0;
        }
    
        if(cache_15_)
        {
            Py_DECREF(cache_15_);
            cache_15_ = 0;
        }
    
        if(cache_16_)
        {
            Py_DECREF(cache_16_);
            cache_16_ = 0;
        }
    
        if(cache_17_)
        {
            Py_DECREF(cache_17_);
            cache_17_ = 0;
        }
    
        if(cache_18_)
        {
            Py_DECREF(cache_18_);
            cache_18_ = 0;
        }
    
        if(cache_19_)
        {
            Py_DECREF(cache_19_);
            cache_19_ = 0;
        }
    
        if(cache_20_)
        {
            Py_DECREF(cache_20_);
            cache_20_ = 0;
        }
    
        if(cache_21_)
        {
            Py_DECREF(cache_21_);
            cache_21_ = 0;
        }
    
        if(cache_22_)
        {
            Py_DECREF(cache_22_);
            cache_22_ = 0;
        }
    
    }
*/



    c2p_wrapper_( PyObject *o, PyInterpreterState *i  ): c2p_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~c2p_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void set_args(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_args");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("set_args before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_args after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void initialise(  p2c_type_ * a0  ,scaffold_gui_type_ * a1  ,const std::string & a2  ,const std::string & a3  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"initialise");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&O&)" ,tpcvt_p2c,&a0,tpcvt_scaffold_gui,&a1,tpcvt_stdstr,&a2,tpcvt_stdstr,&a3);
        //printf("initialise before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("initialise after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    term_type_ get_setups(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_setups");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_setups before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_setups after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            term_type_ * rptr;

            if(!fpcvt_term_copy(result,&rptr))
            {
                throw pip_err_t("wrong type result converting term_copy");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        term_type_ rcopy(*rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_3_;
 
    term_type_ get_user_setups(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_user_setups");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_user_setups before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_user_setups after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            term_type_ * rptr;

            if(!fpcvt_term_copy(result,&rptr))
            {
                throw pip_err_t("wrong type result converting term_copy");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        term_type_ rcopy(*rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_4_;
 
    std::string get_setup_slot(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_setup_slot");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("get_setup_slot before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_setup_slot after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_5_;
 
    void load_setup(  const char * a0  ,bool a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"load_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_str,&a0,tpcvt_b,&a1);
        //printf("load_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("load_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void delete_setup(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"delete_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("delete_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("delete_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    std::string save_setup(  const char * a0  ,const char * a1  ,const char * a2  ,bool a3  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"save_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&O&)" ,tpcvt_str,&a0,tpcvt_str,&a1,tpcvt_str,&a2,tpcvt_b,&a3);
        //printf("save_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("save_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_8_;
 
    std::string edit_setup(  const char * a0  ,const char * a1  ,const char * a2  ,const char * a3  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_8_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"edit_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_8_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&O&)" ,tpcvt_str,&a0,tpcvt_str,&a1,tpcvt_str,&a2,tpcvt_str,&a3);
        //printf("edit_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("edit_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_9_;
 
    std::string get_description(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_9_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_description");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_9_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("get_description before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_description after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_10_;
 
    std::string get_logfile(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_10_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_logfile");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_10_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_logfile before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_logfile after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_11_;
 
    std::string get_email(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_11_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_email");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_11_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_email before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_email after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_12_;
 
    std::string get_username(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_12_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_username");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_12_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_username before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_username after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_13_;
 
    std::string get_subject(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_13_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_subject");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_13_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_subject before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_subject after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_14_;
 
    std::string get_default_setup(  bool a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_14_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_default_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_14_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_b,&a0);
        //printf("get_default_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_default_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_15_;
 
    void set_default_setup(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_15_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_default_setup");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_15_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("set_default_setup before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_default_setup after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_16_;
 
    std::string get_latest_release(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_16_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_latest_release");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_16_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("get_latest_release before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_latest_release after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_17_;
 
    std::string notes_to_words(  const std::string & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_17_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"notes_to_words");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_17_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_stdstr,&a0);
        //printf("notes_to_words before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("notes_to_words after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_18_;
 
    std::string words_to_notes(  const std::string & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_18_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"words_to_notes");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_18_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_stdstr,&a0);
        //printf("words_to_notes before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("words_to_notes after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_19_;
 
    void file_bug(  const std::string & a0  ,const std::string & a1  ,const std::string & a2  ,const std::string & a3  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_19_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"file_bug");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_19_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&O&)" ,tpcvt_stdstr,&a0,tpcvt_stdstr,&a1,tpcvt_stdstr,&a2,tpcvt_stdstr,&a3);
        //printf("file_bug before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("file_bug after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_20_;
 
    void upgrade_setups(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_20_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"upgrade_setups");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_20_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("upgrade_setups before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("upgrade_setups after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_21_;
 
    void prepare_quit(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_21_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"prepare_quit");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_21_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("prepare_quit before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("prepare_quit after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_22_;
 
    void quit(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_22_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"quit");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_22_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("quit before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("quit after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void set_args_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::set_args");
                
                
            
            
        }

        return
            
                set_args
            
            
                ( a0 );
    }

    static PyObject *set_args_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_args_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::set_args");
        }

        return 0;
    }



    void initialise_call_(bool owned ,p2c_type_ * a0 ,scaffold_gui_type_ * a1 ,const std::string & a2 ,const std::string & a3 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::initialise");
                
                
            
            
        }

        return
            
                initialise
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *initialise_method_(c2p_object_ *self, PyObject *args)
    {
         p2c_type_ * a0;  scaffold_gui_type_ * a1;  std::string a2;  std::string a3; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_p2c,&a0  ,fpcvt_scaffold_gui,&a1  ,fpcvt_stdstr,&a2  ,fpcvt_stdstr,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->initialise_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::initialise");
        }

        return 0;
    }



    term_type_ get_setups_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_setups");
                
                
            
            
        }

        return
            
                get_setups
            
            
                ();
    }

    static PyObject *get_setups_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    term_type_ r = o->get_setups_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_term_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_setups");
        }

        return 0;
    }



    term_type_ get_user_setups_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_user_setups");
                
                
            
            
        }

        return
            
                get_user_setups
            
            
                ();
    }

    static PyObject *get_user_setups_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    term_type_ r = o->get_user_setups_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_term_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_user_setups");
        }

        return 0;
    }



    std::string get_setup_slot_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_setup_slot");
                
                
            
            
        }

        return
            
                get_setup_slot
            
            
                ( a0 );
    }

    static PyObject *get_setup_slot_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_setup_slot_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_setup_slot");
        }

        return 0;
    }



    void load_setup_call_(bool owned ,const char * a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::load_setup");
                
                
            
            
        }

        return
            
                load_setup
            
            
                ( a0  ,a1 );
    }

    static PyObject *load_setup_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0;  bool a1; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_str,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_setup_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::load_setup");
        }

        return 0;
    }



    void delete_setup_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::delete_setup");
                
                
            
            
        }

        return
            
                delete_setup
            
            
                ( a0 );
    }

    static PyObject *delete_setup_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->delete_setup_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::delete_setup");
        }

        return 0;
    }



    std::string save_setup_call_(bool owned ,const char * a0 ,const char * a1 ,const char * a2 ,bool a3 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::save_setup");
                
                
            
            
        }

        return
            
                save_setup
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *save_setup_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0;  const char * a1;  const char * a2;  bool a3; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_str,&a1  ,fpcvt_str,&a2  ,fpcvt_b,&a3 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->save_setup_call_ (self->owned ,a0 ,a1 ,a2 ,a3  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::save_setup");
        }

        return 0;
    }



    std::string edit_setup_call_(bool owned ,const char * a0 ,const char * a1 ,const char * a2 ,const char * a3 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::edit_setup");
                
                
            
            
        }

        return
            
                edit_setup
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *edit_setup_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0;  const char * a1;  const char * a2;  const char * a3; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_str,&a1  ,fpcvt_str,&a2  ,fpcvt_str,&a3 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->edit_setup_call_ (self->owned ,a0 ,a1 ,a2 ,a3  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::edit_setup");
        }

        return 0;
    }



    std::string get_description_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_description");
                
                
            
            
        }

        return
            
                get_description
            
            
                ( a0 );
    }

    static PyObject *get_description_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_description_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_description");
        }

        return 0;
    }



    std::string get_logfile_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_logfile");
                
                
            
            
        }

        return
            
                get_logfile
            
            
                ();
    }

    static PyObject *get_logfile_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_logfile_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_logfile");
        }

        return 0;
    }



    std::string get_email_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_email");
                
                
            
            
        }

        return
            
                get_email
            
            
                ();
    }

    static PyObject *get_email_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_email_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_email");
        }

        return 0;
    }



    std::string get_username_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_username");
                
                
            
            
        }

        return
            
                get_username
            
            
                ();
    }

    static PyObject *get_username_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_username_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_username");
        }

        return 0;
    }



    std::string get_subject_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_subject");
                
                
            
            
        }

        return
            
                get_subject
            
            
                ();
    }

    static PyObject *get_subject_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_subject_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_subject");
        }

        return 0;
    }



    std::string get_default_setup_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_default_setup");
                
                
            
            
        }

        return
            
                get_default_setup
            
            
                ( a0 );
    }

    static PyObject *get_default_setup_method_(c2p_object_ *self, PyObject *args)
    {
         bool a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_default_setup_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_default_setup");
        }

        return 0;
    }



    void set_default_setup_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::set_default_setup");
                
                
            
            
        }

        return
            
                set_default_setup
            
            
                ( a0 );
    }

    static PyObject *set_default_setup_method_(c2p_object_ *self, PyObject *args)
    {
         const char * a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_default_setup_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::set_default_setup");
        }

        return 0;
    }



    std::string get_latest_release_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::get_latest_release");
                
                
            
            
        }

        return
            
                get_latest_release
            
            
                ();
    }

    static PyObject *get_latest_release_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_latest_release_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::get_latest_release");
        }

        return 0;
    }



    std::string notes_to_words_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::notes_to_words");
                
                
            
            
        }

        return
            
                notes_to_words
            
            
                ( a0 );
    }

    static PyObject *notes_to_words_method_(c2p_object_ *self, PyObject *args)
    {
         std::string a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->notes_to_words_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::notes_to_words");
        }

        return 0;
    }



    std::string words_to_notes_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::words_to_notes");
                
                
            
            
        }

        return
            
                words_to_notes
            
            
                ( a0 );
    }

    static PyObject *words_to_notes_method_(c2p_object_ *self, PyObject *args)
    {
         std::string a0; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->words_to_notes_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::words_to_notes");
        }

        return 0;
    }



    void file_bug_call_(bool owned ,const std::string & a0 ,const std::string & a1 ,const std::string & a2 ,const std::string & a3 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::file_bug");
                
                
            
            
        }

        return
            
                file_bug
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *file_bug_method_(c2p_object_ *self, PyObject *args)
    {
         std::string a0;  std::string a1;  std::string a2;  std::string a3; 

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_stdstr,&a1  ,fpcvt_stdstr,&a2  ,fpcvt_stdstr,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->file_bug_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::file_bug");
        }

        return 0;
    }



    void upgrade_setups_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::upgrade_setups");
                
                
            
            
        }

        return
            
                upgrade_setups
            
            
                ();
    }

    static PyObject *upgrade_setups_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->upgrade_setups_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::upgrade_setups");
        }

        return 0;
    }



    void prepare_quit_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::prepare_quit");
                
                
            
            
        }

        return
            
                prepare_quit
            
            
                ();
    }

    static PyObject *prepare_quit_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->prepare_quit_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::prepare_quit");
        }

        return 0;
    }



    void quit_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method c2p::quit");
                
                
            
            
        }

        return
            
                quit
            
            
                ();
    }

    static PyObject *quit_method_(c2p_object_ *self, PyObject *args)
    {
        

        c2p_wrapper_ *o = (c2p_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null c2p c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->quit_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::quit");
        }

        return 0;
    }





    static PyObject *_method_inner_token(c2p_object_ *_self_, PyObject *args)
    {
        c2p_type_ *self = _self_->object;
        
        return PyCObject_FromVoidPtr(self,0);
    
    }

    static PyObject *_method_token(c2p_object_ *self, PyObject *args)
    {
        try
        {
            PyObject *rv = _method_inner_token(self,args);
            return rv;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in c2p::token");
        }

        return 0;
    }



    static PyObject *_method_lock(c2p_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(c2p_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void c2p_delete_(c2p_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            c2p_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (c2p_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        c2p_wrapper_::__deleter__((c2p_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void c2p_dealloc_(c2p_object_* self)
{
    
    c2p_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * c2p_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    c2p_object_ *self;

    if(!(self = (c2p_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int c2p_init_(c2p_object_ *self, PyObject *args, PyObject *kwds)
{
    c2p_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new c2p_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing c2p");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            c2p_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"c2p invalid arguments");
    return -1;

}


static const char *c2p_set_args_docstring_ = "";

static const char *c2p_initialise_docstring_ = "";

static const char *c2p_get_setups_docstring_ = "";

static const char *c2p_get_user_setups_docstring_ = "";

static const char *c2p_get_setup_slot_docstring_ = "";

static const char *c2p_load_setup_docstring_ = "";

static const char *c2p_delete_setup_docstring_ = "";

static const char *c2p_save_setup_docstring_ = "";

static const char *c2p_edit_setup_docstring_ = "";

static const char *c2p_get_description_docstring_ = "";

static const char *c2p_get_logfile_docstring_ = "";

static const char *c2p_get_email_docstring_ = "";

static const char *c2p_get_username_docstring_ = "";

static const char *c2p_get_subject_docstring_ = "";

static const char *c2p_get_default_setup_docstring_ = "";

static const char *c2p_set_default_setup_docstring_ = "";

static const char *c2p_get_latest_release_docstring_ = "";

static const char *c2p_notes_to_words_docstring_ = "";

static const char *c2p_words_to_notes_docstring_ = "";

static const char *c2p_file_bug_docstring_ = "";

static const char *c2p_upgrade_setups_docstring_ = "";

static const char *c2p_prepare_quit_docstring_ = "";

static const char *c2p_quit_docstring_ = "";



static const char *c2p_token_docstring_ = "";


static PyMethodDef c2p_methods_[] = {
    
        { "set_args",(PyCFunction)c2p_wrapper_::set_args_method_,METH_VARARGS,(char *)c2p_set_args_docstring_ },
    
        { "initialise",(PyCFunction)c2p_wrapper_::initialise_method_,METH_VARARGS,(char *)c2p_initialise_docstring_ },
    
        { "get_setups",(PyCFunction)c2p_wrapper_::get_setups_method_,METH_VARARGS,(char *)c2p_get_setups_docstring_ },
    
        { "get_user_setups",(PyCFunction)c2p_wrapper_::get_user_setups_method_,METH_VARARGS,(char *)c2p_get_user_setups_docstring_ },
    
        { "get_setup_slot",(PyCFunction)c2p_wrapper_::get_setup_slot_method_,METH_VARARGS,(char *)c2p_get_setup_slot_docstring_ },
    
        { "load_setup",(PyCFunction)c2p_wrapper_::load_setup_method_,METH_VARARGS,(char *)c2p_load_setup_docstring_ },
    
        { "delete_setup",(PyCFunction)c2p_wrapper_::delete_setup_method_,METH_VARARGS,(char *)c2p_delete_setup_docstring_ },
    
        { "save_setup",(PyCFunction)c2p_wrapper_::save_setup_method_,METH_VARARGS,(char *)c2p_save_setup_docstring_ },
    
        { "edit_setup",(PyCFunction)c2p_wrapper_::edit_setup_method_,METH_VARARGS,(char *)c2p_edit_setup_docstring_ },
    
        { "get_description",(PyCFunction)c2p_wrapper_::get_description_method_,METH_VARARGS,(char *)c2p_get_description_docstring_ },
    
        { "get_logfile",(PyCFunction)c2p_wrapper_::get_logfile_method_,METH_VARARGS,(char *)c2p_get_logfile_docstring_ },
    
        { "get_email",(PyCFunction)c2p_wrapper_::get_email_method_,METH_VARARGS,(char *)c2p_get_email_docstring_ },
    
        { "get_username",(PyCFunction)c2p_wrapper_::get_username_method_,METH_VARARGS,(char *)c2p_get_username_docstring_ },
    
        { "get_subject",(PyCFunction)c2p_wrapper_::get_subject_method_,METH_VARARGS,(char *)c2p_get_subject_docstring_ },
    
        { "get_default_setup",(PyCFunction)c2p_wrapper_::get_default_setup_method_,METH_VARARGS,(char *)c2p_get_default_setup_docstring_ },
    
        { "set_default_setup",(PyCFunction)c2p_wrapper_::set_default_setup_method_,METH_VARARGS,(char *)c2p_set_default_setup_docstring_ },
    
        { "get_latest_release",(PyCFunction)c2p_wrapper_::get_latest_release_method_,METH_VARARGS,(char *)c2p_get_latest_release_docstring_ },
    
        { "notes_to_words",(PyCFunction)c2p_wrapper_::notes_to_words_method_,METH_VARARGS,(char *)c2p_notes_to_words_docstring_ },
    
        { "words_to_notes",(PyCFunction)c2p_wrapper_::words_to_notes_method_,METH_VARARGS,(char *)c2p_words_to_notes_docstring_ },
    
        { "file_bug",(PyCFunction)c2p_wrapper_::file_bug_method_,METH_VARARGS,(char *)c2p_file_bug_docstring_ },
    
        { "upgrade_setups",(PyCFunction)c2p_wrapper_::upgrade_setups_method_,METH_VARARGS,(char *)c2p_upgrade_setups_docstring_ },
    
        { "prepare_quit",(PyCFunction)c2p_wrapper_::prepare_quit_method_,METH_VARARGS,(char *)c2p_prepare_quit_docstring_ },
    
        { "quit",(PyCFunction)c2p_wrapper_::quit_method_,METH_VARARGS,(char *)c2p_quit_docstring_ },
    
    
        { "token",(PyCFunction)c2p_wrapper_::_method_token,METH_VARARGS,(char *)c2p_token_docstring_ },
    
        { "py_lock",(PyCFunction)c2p_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)c2p_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *c2p_docstring_ = "";

PyTypeObject c2p_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "eigend_native.c2p",/*tp_name*/
    sizeof(c2p_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)c2p_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)c2p_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    c2p_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)c2p_init_,  /* tp_init */
    0,                         /* tp_alloc */
    c2p_new_,             /* tp_new */
};

PyObject *c2p_wrapcopy_(c2p_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *c2p_wrap_(c2p_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = c2p_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    c2p_object_ *self = 
        PyObject_NEW(c2p_object_, &c2p_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        c2p_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

c2p_type_ *c2p_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &c2p_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected c2p object");
        return 0;
    }

    c2p_type_ *object = static_cast<c2p_type_ *>(((c2p_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null c2p object in unwrap");
        return 0;
    }

    return object;
}

void c2p_insert_(const std::pair< c2p_type_ *, PyObject *> &p)
{




    c2p_wrapped_.insert(p);


}

void c2p_erase_(c2p_type_ *p)
{




    c2p_wrapped_.erase(p);


}

PyObject * c2p_find_(c2p_type_ *p)
{




    PyObject *ret = 0;
    std::map< c2p_type_ *, PyObject *>::iterator where = c2p_wrapped_.find(p);
    if(where != c2p_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct c2p_dispatch_ c2p_realdispatcher_ =
{
    &c2p_typeobject_,
    c2p_wrap_,
    c2p_wrapcopy_,
    c2p_unwrap_,
    c2p_insert_,
    c2p_erase_,
    c2p_find_
};

  

struct p2c_object_ {
    PyObject_HEAD
    p2c_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct p2c_wrapper_: public p2c_type_ 
{

    static void __deleter__(p2c_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
    }
*/



    p2c_wrapper_( PyObject *o, PyInterpreterState *i  ): p2c_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~p2c_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void load_started(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"load_started");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("load_started before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("load_started after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void load_status(  const char * a0  ,unsigned int a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"load_status");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_str,&a0,tpcvt_ui,&a1);
        //printf("load_status before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("load_status after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void load_ended(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"load_ended");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("load_ended before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("load_ended after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    f_string_type_ make_logger(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"make_logger");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("make_logger before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("make_logger after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            f_string_type_ * rptr;

            if(!fpcvt_f_string_copy(result,&rptr))
            {
                throw pip_err_t("wrong type result converting f_string_copy");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        f_string_type_ rcopy(*rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }


    //PyObject *cache_4_;
 
    void setups_changed(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"setups_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("setups_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("setups_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_5_;
 
    void alert_dialog(  const char * a0  ,const char * a1  ,const char * a2  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"alert_dialog");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&)" ,tpcvt_str,&a0,tpcvt_str,&a1,tpcvt_str,&a2);
        //printf("alert_dialog before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("alert_dialog after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void info_dialog(  const char * a0  ,const char * a1  ,const char * a2  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"info_dialog");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&)" ,tpcvt_str,&a0,tpcvt_str,&a1,tpcvt_str,&a2);
        //printf("info_dialog before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("info_dialog after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    void set_latest_release(  const char * a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"set_latest_release");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_str,&a0);
        //printf("set_latest_release before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("set_latest_release after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void load_started_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::load_started");
                
                
            
            
        }

        return
            
                load_started
            
            
                ( a0 );
    }

    static PyObject *load_started_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_started_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::load_started");
        }

        return 0;
    }



    void load_status_call_(bool owned ,const char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::load_status");
                
                
            
            
        }

        return
            
                load_status
            
            
                ( a0  ,a1 );
    }

    static PyObject *load_status_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0;  unsigned int a1; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_str,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_status_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::load_status");
        }

        return 0;
    }



    void load_ended_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::load_ended");
                
                
            
            
        }

        return
            
                load_ended
            
            
                ();
    }

    static PyObject *load_ended_method_(p2c_object_ *self, PyObject *args)
    {
        

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_ended_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::load_ended");
        }

        return 0;
    }



    f_string_type_ make_logger_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::make_logger");
                
                
            
            
        }

        return
            
                make_logger
            
            
                ( a0 );
    }

    static PyObject *make_logger_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    f_string_type_ r = o->make_logger_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f_string_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::make_logger");
        }

        return 0;
    }



    void setups_changed_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::setups_changed");
                
                
            
            
        }

        return
            
                setups_changed
            
            
                ( a0 );
    }

    static PyObject *setups_changed_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->setups_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::setups_changed");
        }

        return 0;
    }



    void alert_dialog_call_(bool owned ,const char * a0 ,const char * a1 ,const char * a2 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::alert_dialog");
                
                
            
            
        }

        return
            
                alert_dialog
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *alert_dialog_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0;  const char * a1;  const char * a2; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_str,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->alert_dialog_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::alert_dialog");
        }

        return 0;
    }



    void info_dialog_call_(bool owned ,const char * a0 ,const char * a1 ,const char * a2 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::info_dialog");
                
                
            
            
        }

        return
            
                info_dialog
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *info_dialog_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0;  const char * a1;  const char * a2; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_str,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->info_dialog_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::info_dialog");
        }

        return 0;
    }



    void set_latest_release_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method p2c::set_latest_release");
                
                
            
            
        }

        return
            
                set_latest_release
            
            
                ( a0 );
    }

    static PyObject *set_latest_release_method_(p2c_object_ *self, PyObject *args)
    {
         const char * a0; 

        p2c_wrapper_ *o = (p2c_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null p2c c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_latest_release_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in p2c::set_latest_release");
        }

        return 0;
    }





    static PyObject *_method_lock(p2c_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(p2c_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void p2c_delete_(p2c_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            p2c_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (p2c_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        p2c_wrapper_::__deleter__((p2c_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void p2c_dealloc_(p2c_object_* self)
{
    
    p2c_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * p2c_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    p2c_object_ *self;

    if(!(self = (p2c_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int p2c_init_(p2c_object_ *self, PyObject *args, PyObject *kwds)
{
    p2c_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new p2c_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing p2c");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            p2c_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"p2c invalid arguments");
    return -1;

}


static const char *p2c_load_started_docstring_ = "";

static const char *p2c_load_status_docstring_ = "";

static const char *p2c_load_ended_docstring_ = "";

static const char *p2c_make_logger_docstring_ = "";

static const char *p2c_setups_changed_docstring_ = "";

static const char *p2c_alert_dialog_docstring_ = "";

static const char *p2c_info_dialog_docstring_ = "";

static const char *p2c_set_latest_release_docstring_ = "";




static PyMethodDef p2c_methods_[] = {
    
        { "load_started",(PyCFunction)p2c_wrapper_::load_started_method_,METH_VARARGS,(char *)p2c_load_started_docstring_ },
    
        { "load_status",(PyCFunction)p2c_wrapper_::load_status_method_,METH_VARARGS,(char *)p2c_load_status_docstring_ },
    
        { "load_ended",(PyCFunction)p2c_wrapper_::load_ended_method_,METH_VARARGS,(char *)p2c_load_ended_docstring_ },
    
        { "make_logger",(PyCFunction)p2c_wrapper_::make_logger_method_,METH_VARARGS,(char *)p2c_make_logger_docstring_ },
    
        { "setups_changed",(PyCFunction)p2c_wrapper_::setups_changed_method_,METH_VARARGS,(char *)p2c_setups_changed_docstring_ },
    
        { "alert_dialog",(PyCFunction)p2c_wrapper_::alert_dialog_method_,METH_VARARGS,(char *)p2c_alert_dialog_docstring_ },
    
        { "info_dialog",(PyCFunction)p2c_wrapper_::info_dialog_method_,METH_VARARGS,(char *)p2c_info_dialog_docstring_ },
    
        { "set_latest_release",(PyCFunction)p2c_wrapper_::set_latest_release_method_,METH_VARARGS,(char *)p2c_set_latest_release_docstring_ },
    
    
        { "py_lock",(PyCFunction)p2c_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)p2c_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *p2c_docstring_ = "";

PyTypeObject p2c_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "eigend_native.p2c",/*tp_name*/
    sizeof(p2c_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)p2c_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)p2c_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    p2c_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)p2c_init_,  /* tp_init */
    0,                         /* tp_alloc */
    p2c_new_,             /* tp_new */
};

PyObject *p2c_wrapcopy_(p2c_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *p2c_wrap_(p2c_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = p2c_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    p2c_object_ *self = 
        PyObject_NEW(p2c_object_, &p2c_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        p2c_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

p2c_type_ *p2c_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &p2c_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected p2c object");
        return 0;
    }

    p2c_type_ *object = static_cast<p2c_type_ *>(((p2c_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null p2c object in unwrap");
        return 0;
    }

    return object;
}

void p2c_insert_(const std::pair< p2c_type_ *, PyObject *> &p)
{




    p2c_wrapped_.insert(p);


}

void p2c_erase_(p2c_type_ *p)
{




    p2c_wrapped_.erase(p);


}

PyObject * p2c_find_(p2c_type_ *p)
{




    PyObject *ret = 0;
    std::map< p2c_type_ *, PyObject *>::iterator where = p2c_wrapped_.find(p);
    if(where != p2c_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct p2c_dispatch_ p2c_realdispatcher_ =
{
    &p2c_typeobject_,
    p2c_wrap_,
    p2c_wrapcopy_,
    p2c_unwrap_,
    p2c_insert_,
    p2c_erase_,
    p2c_find_
};

       





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initeigend_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            c2p_dispatcher_ = &c2p_realdispatcher_;
        
    
        
            p2c_dispatcher_ = &p2c_realdispatcher_;
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piagent_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_scaffold_gui_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_scaffold_gui_dispatch in module piagent_native");
                return;
            }

            scaffold_gui_dispatcher_ = (scaffold_gui_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_term_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_term_dispatch in module piw");
                return;
            }

            term_dispatcher_ = (term_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    
        if(!(m=PyImport_ImportModule("picross")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_f_string_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_f_string_dispatch in module picross");
                return;
            }

            f_string_dispatcher_ = (f_string_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&c2p_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&p2c_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("eigend_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(c2p_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_c2p_dispatch",s);
            Py_INCREF(&c2p_typeobject_);
            PyModule_AddObject(m,"c2p",(PyObject *)&c2p_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(p2c_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_p2c_dispatch",s);
            Py_INCREF(&p2c_typeobject_);
            PyModule_AddObject(m,"p2c",(PyObject *)&p2c_typeobject_);
        
    
        
    
        
    
        
    

    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

