#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <lib_alpha2/alpha2_active.h>
#include <lib_alpha2/alpha2_passive.h>


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef alpha2::passive_t passive_type_; 


 
static std::map<passive_type_ *, PyObject *> passive_wrapped_;
 


struct passive_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(passive_type_ *, bool owned);
    PyObject *(*wrapcopy)(passive_type_ *);
    passive_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< passive_type_ *, PyObject * > &);
    void (*erase)(passive_type_ *);
    PyObject *(*find)(passive_type_ *);
};

struct passive_dispatch_ *passive_dispatcher_;



typedef alpha2::active_t active_type_; 


 
static std::map<active_type_ *, PyObject *> active_wrapped_;
 


struct active_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(active_type_ *, bool owned);
    PyObject *(*wrapcopy)(active_type_ *);
    active_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< active_type_ *, PyObject * > &);
    void (*erase)(active_type_ *);
    PyObject *(*find)(active_type_ *);
};

struct active_dispatch_ *active_dispatcher_;



typedef alpha2::active_t::delegate_t delegate_type_; 


 
static std::map<delegate_type_ *, PyObject *> delegate_wrapped_;
 


struct delegate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(delegate_type_ *, bool owned);
    PyObject *(*wrapcopy)(delegate_type_ *);
    delegate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< delegate_type_ *, PyObject * > &);
    void (*erase)(delegate_type_ *);
    PyObject *(*find)(delegate_type_ *);
};

struct delegate_dispatch_ *delegate_dispatcher_;



typedef alpha2::printer_t printer_type_; 


  


struct printer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(printer_type_ *, bool owned);
    PyObject *(*wrapcopy)(printer_type_ *);
    printer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< printer_type_ *, PyObject * > &);
    void (*erase)(printer_type_ *);
    PyObject *(*find)(printer_type_ *);
};

struct printer_dispatch_ *printer_dispatcher_;



typedef pic::usbdevice_t usbdevice_type_; 





struct usbdevice_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(usbdevice_type_ *, bool owned);
    PyObject *(*wrapcopy)(usbdevice_type_ *);
    usbdevice_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< usbdevice_type_ *, PyObject * > &);
    void (*erase)(usbdevice_type_ *);
    PyObject *(*find)(usbdevice_type_ *);
};

struct usbdevice_dispatch_ *usbdevice_dispatcher_;





int fpcvt_passive(PyObject *o, void *a)
{
    passive_type_ *object;
    if(o == Py_None)
    {
        *((passive_type_ **)a) = 0;
        return 1;
    }
    object = passive_dispatcher_->unwrap(o);
    *((passive_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_passive_copy fpcvt_passive

PyObject *tpcvt_passive(void *a)
{
    passive_type_ *o = *((passive_type_ **)a);
    return passive_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_passive_copy(void *a)
{
    passive_type_ *o = (passive_type_ *)a;
    return passive_dispatcher_->wrapcopy(o);
}



int fpcvt_active(PyObject *o, void *a)
{
    active_type_ *object;
    if(o == Py_None)
    {
        *((active_type_ **)a) = 0;
        return 1;
    }
    object = active_dispatcher_->unwrap(o);
    *((active_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_active_copy fpcvt_active

PyObject *tpcvt_active(void *a)
{
    active_type_ *o = *((active_type_ **)a);
    return active_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_active_copy(void *a)
{
    active_type_ *o = (active_type_ *)a;
    return active_dispatcher_->wrapcopy(o);
}



int fpcvt_delegate(PyObject *o, void *a)
{
    delegate_type_ *object;
    if(o == Py_None)
    {
        *((delegate_type_ **)a) = 0;
        return 1;
    }
    object = delegate_dispatcher_->unwrap(o);
    *((delegate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_delegate_copy fpcvt_delegate

PyObject *tpcvt_delegate(void *a)
{
    delegate_type_ *o = *((delegate_type_ **)a);
    return delegate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_delegate_copy(void *a)
{
    delegate_type_ *o = (delegate_type_ *)a;
    return delegate_dispatcher_->wrapcopy(o);
}



int fpcvt_printer(PyObject *o, void *a)
{
    printer_type_ *object;
    if(o == Py_None)
    {
        *((printer_type_ **)a) = 0;
        return 1;
    }
    object = printer_dispatcher_->unwrap(o);
    *((printer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_printer_copy fpcvt_printer

PyObject *tpcvt_printer(void *a)
{
    printer_type_ *o = *((printer_type_ **)a);
    return printer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_printer_copy(void *a)
{
    printer_type_ *o = (printer_type_ *)a;
    return printer_dispatcher_->wrapcopy(o);
}



int fpcvt_usbdevice(PyObject *o, void *a)
{
    usbdevice_type_ *object;
    if(o == Py_None)
    {
        *((usbdevice_type_ **)a) = 0;
        return 1;
    }
    object = usbdevice_dispatcher_->unwrap(o);
    *((usbdevice_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_usbdevice_copy fpcvt_usbdevice

PyObject *tpcvt_usbdevice(void *a)
{
    usbdevice_type_ *o = *((usbdevice_type_ **)a);
    return usbdevice_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_usbdevice_copy(void *a)
{
    usbdevice_type_ *o = (usbdevice_type_ *)a;
    return usbdevice_dispatcher_->wrapcopy(o);
}



 

struct passive_object_ {
    PyObject_HEAD
    passive_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct passive_wrapper_: public passive_type_ 
{

    static void __deleter__(passive_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    passive_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,unsigned int a1  ): passive_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~passive_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned short get_rawkey_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_rawkey( a0  ,a1 );
                
            
            
        }

        return
            
                get_rawkey
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_rawkey_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_rawkey_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_rawkey");
        }

        return 0;
    }



    void start_calibration_row_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start_calibration_row( a0  ,a1 );
                
            
            
        }

        return
            
                start_calibration_row
            
            
                ( a0  ,a1 );
    }

    static PyObject *start_calibration_row_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_calibration_row_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start_calibration_row");
        }

        return 0;
    }



    void set_calibration_range_call_(bool owned ,unsigned short a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_calibration_range( a0  ,a1 );
                
            
            
        }

        return
            
                set_calibration_range
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_calibration_range_method_(passive_object_ *self, PyObject *args)
    {
         unsigned short a0;  unsigned short a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_us,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_calibration_range_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_calibration_range");
        }

        return 0;
    }



    void set_calibration_point_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_calibration_point( a0  ,a1 );
                
            
            
        }

        return
            
                set_calibration_point
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_calibration_point_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_calibration_point_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_calibration_point");
        }

        return 0;
    }



    void write_calibration_row_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::write_calibration_row();
                
            
            
        }

        return
            
                write_calibration_row
            
            
                ();
    }

    static PyObject *write_calibration_row_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->write_calibration_row_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::write_calibration_row");
        }

        return 0;
    }



    void commit_calibration_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::commit_calibration();
                
            
            
        }

        return
            
                commit_calibration
            
            
                ();
    }

    static PyObject *commit_calibration_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->commit_calibration_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::commit_calibration");
        }

        return 0;
    }



    const char * get_name_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_name();
                
            
            
        }

        return
            
                get_name
            
            
                ();
    }

    static PyObject *get_name_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    const char * r = o->get_name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_str((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_name");
        }

        return 0;
    }



    bool is_kbd_died_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::is_kbd_died();
                
            
            
        }

        return
            
                is_kbd_died
            
            
                ();
    }

    static PyObject *is_kbd_died_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_kbd_died_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::is_kbd_died");
        }

        return 0;
    }



    bool wait_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::wait();
                
            
            
        }

        return
            
                wait
            
            
                ();
    }

    static PyObject *wait_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->wait_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::wait");
        }

        return 0;
    }



    void sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::sync();
                
            
            
        }

        return
            
                sync
            
            
                ();
    }

    static PyObject *sync_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::sync");
        }

        return 0;
    }



    void set_ledcolour_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_ledcolour( a0  ,a1 );
                
            
            
        }

        return
            
                set_ledcolour
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_ledcolour_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_ledcolour_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_ledcolour");
        }

        return 0;
    }



    unsigned long count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::count();
                
            
            
        }

        return
            
                count
            
            
                ();
    }

    static PyObject *count_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::count");
        }

        return 0;
    }



    void clear_calibration_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::clear_calibration();
                
            
            
        }

        return
            
                clear_calibration
            
            
                ();
    }

    static PyObject *clear_calibration_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_calibration_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::clear_calibration");
        }

        return 0;
    }



    unsigned int get_temperature_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_temperature();
                
            
            
        }

        return
            
                get_temperature
            
            
                ();
    }

    static PyObject *get_temperature_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_temperature_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_temperature");
        }

        return 0;
    }



    void start_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start();
                
            
            
        }

        return
            
                start
            
            
                ();
    }

    static PyObject *start_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::stop");
        }

        return 0;
    }



    void start_polling_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start_polling();
                
            
            
        }

        return
            
                start_polling
            
            
                ();
    }

    static PyObject *start_polling_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_polling_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start_polling");
        }

        return 0;
    }



    void stop_polling_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::stop_polling();
                
            
            
        }

        return
            
                stop_polling
            
            
                ();
    }

    static PyObject *stop_polling_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_polling_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::stop_polling");
        }

        return 0;
    }



    void start_collecting_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start_collecting();
                
            
            
        }

        return
            
                start_collecting
            
            
                ();
    }

    static PyObject *start_collecting_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_collecting_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start_collecting");
        }

        return 0;
    }



    void stop_collecting_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::stop_collecting();
                
            
            
        }

        return
            
                stop_collecting
            
            
                ();
    }

    static PyObject *stop_collecting_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_collecting_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::stop_collecting");
        }

        return 0;
    }



    unsigned short get_collected_key_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned int a2 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_collected_key( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                get_collected_key
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *get_collected_key_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned int a2; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_collected_key_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_collected_key");
        }

        return 0;
    }



    unsigned int collect_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::collect_count();
                
            
            
        }

        return
            
                collect_count
            
            
                ();
    }

    static PyObject *collect_count_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->collect_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::collect_count");
        }

        return 0;
    }



    void set_active_colour_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_active_colour( a0 );
                
            
            
        }

        return
            
                set_active_colour
            
            
                ( a0 );
    }

    static PyObject *set_active_colour_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_active_colour_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_active_colour");
        }

        return 0;
    }



    void control_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::control( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                control
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *control_method_(passive_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->control_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::control");
        }

        return 0;
    }



    void control_out_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 ,const std::string & a4 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::control_out( a0  ,a1  ,a2  ,a3  ,a4 );
                
            
            
        }

        return
            
                control_out
            
            
                ( a0  ,a1  ,a2  ,a3  ,a4 );
    }

    static PyObject *control_out_method_(passive_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3;  std::string a4; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3  ,fpcvt_stdstr,&a4 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->control_out_call_ (self->owned ,a0,a1,a2,a3,a4 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::control_out");
        }

        return 0;
    }



    std::string control_in_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 ,unsigned int a4 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::control_in( a0  ,a1  ,a2  ,a3  ,a4 );
                
            
            
        }

        return
            
                control_in
            
            
                ( a0  ,a1  ,a2  ,a3  ,a4 );
    }

    static PyObject *control_in_method_(passive_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3;  unsigned int a4; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3  ,fpcvt_ui,&a4 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->control_in_call_ (self->owned ,a0 ,a1 ,a2 ,a3 ,a4  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::control_in");
        }

        return 0;
    }





    static PyObject *_method_lock(passive_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(passive_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void passive_delete_(passive_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            passive_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (passive_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        passive_wrapper_::__deleter__((passive_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void passive_dealloc_(passive_object_* self)
{
    
    passive_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * passive_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    passive_object_ *self;

    if(!(self = (passive_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int passive_init_(passive_object_ *self, PyObject *args, PyObject *kwds)
{
    passive_delete_(self);

    

    {
         const char * a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new passive_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing passive");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            passive_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"passive invalid arguments");
    return -1;

}


static const char *passive_get_rawkey_docstring_ = ""
"        keyboard.get_rawkey(key,corner) -> sensor_value"
"        "
"        Get the current raw sensor reading for corner of key"
"        "
"        "
""
;

static const char *passive_start_calibration_row_docstring_ = ""
"        keyboard.start_calibration_row(key,sensor)"
""
"        Set the key and sensor of the row currently being calibrated"
"        "
""
;

static const char *passive_set_calibration_range_docstring_ = ""
"        keyboard.set_normalisation(min,max)"
"        "
"        Set the range for corner of key."
"        "
""
;

static const char *passive_set_calibration_point_docstring_ = ""
"        keyboard.set_normalisation(point,value)"
"        "
"        Set a point in the calibration table."
"        "
""
;

static const char *passive_write_calibration_row_docstring_ = ""
"        keyboard.write_calibration_row()"
"        "
"        Send one row of the calibration table to the instrument"
"        "
""
;

static const char *passive_commit_calibration_docstring_ = ""
"        commit the instrument's calibration data to flash"
"        "
""
;

static const char *passive_get_name_docstring_ = ""
"        keyboard.get_name() -> name"
"        "
"        Return the dsp serial number of the keyboard"
"        "
""
;

static const char *passive_is_kbd_died_docstring_ = "";

static const char *passive_wait_docstring_ = ""
"        keyboard.wait() -> success"
"        "
"        Wait for a complete raw scan cycle."
"        If return is false, you missed your frame (taking into account"
"        the decimation factor)"
"        "
""
;

static const char *passive_sync_docstring_ = ""
"        keyboard.sync() -> ."
""
"        Reset the sync state of the keyboard raw data with the receiver."
"        "
""
;

static const char *passive_set_ledcolour_docstring_ = ""
"        keyboard.set_ledcolour(key,colour) -> ."
"        "
"        Set the LED on a given key to the indicated colour.  Colour is"
"        made up of two bit rgb values with r occupying bits 4 and 5, g"
"        occupying bits 2 and 3, and b occupying bits 0 and 1"
"        "
""
;

static const char *passive_count_docstring_ = "";

static const char *passive_clear_calibration_docstring_ = "";

static const char *passive_get_temperature_docstring_ = "";

static const char *passive_start_docstring_ = "";

static const char *passive_stop_docstring_ = "";

static const char *passive_start_polling_docstring_ = "";

static const char *passive_stop_polling_docstring_ = "";

static const char *passive_start_collecting_docstring_ = "";

static const char *passive_stop_collecting_docstring_ = "";

static const char *passive_get_collected_key_docstring_ = "";

static const char *passive_collect_count_docstring_ = "";

static const char *passive_set_active_colour_docstring_ = "";

static const char *passive_control_docstring_ = "";

static const char *passive_control_out_docstring_ = "";

static const char *passive_control_in_docstring_ = "";




static PyMethodDef passive_methods_[] = {
    
        { "get_rawkey",(PyCFunction)passive_wrapper_::get_rawkey_method_,METH_VARARGS,(char *)passive_get_rawkey_docstring_ },
    
        { "start_calibration_row",(PyCFunction)passive_wrapper_::start_calibration_row_method_,METH_VARARGS,(char *)passive_start_calibration_row_docstring_ },
    
        { "set_calibration_range",(PyCFunction)passive_wrapper_::set_calibration_range_method_,METH_VARARGS,(char *)passive_set_calibration_range_docstring_ },
    
        { "set_calibration_point",(PyCFunction)passive_wrapper_::set_calibration_point_method_,METH_VARARGS,(char *)passive_set_calibration_point_docstring_ },
    
        { "write_calibration_row",(PyCFunction)passive_wrapper_::write_calibration_row_method_,METH_VARARGS,(char *)passive_write_calibration_row_docstring_ },
    
        { "commit_calibration",(PyCFunction)passive_wrapper_::commit_calibration_method_,METH_VARARGS,(char *)passive_commit_calibration_docstring_ },
    
        { "get_name",(PyCFunction)passive_wrapper_::get_name_method_,METH_VARARGS,(char *)passive_get_name_docstring_ },
    
        { "is_kbd_died",(PyCFunction)passive_wrapper_::is_kbd_died_method_,METH_VARARGS,(char *)passive_is_kbd_died_docstring_ },
    
        { "wait",(PyCFunction)passive_wrapper_::wait_method_,METH_VARARGS,(char *)passive_wait_docstring_ },
    
        { "sync",(PyCFunction)passive_wrapper_::sync_method_,METH_VARARGS,(char *)passive_sync_docstring_ },
    
        { "set_ledcolour",(PyCFunction)passive_wrapper_::set_ledcolour_method_,METH_VARARGS,(char *)passive_set_ledcolour_docstring_ },
    
        { "count",(PyCFunction)passive_wrapper_::count_method_,METH_VARARGS,(char *)passive_count_docstring_ },
    
        { "clear_calibration",(PyCFunction)passive_wrapper_::clear_calibration_method_,METH_VARARGS,(char *)passive_clear_calibration_docstring_ },
    
        { "get_temperature",(PyCFunction)passive_wrapper_::get_temperature_method_,METH_VARARGS,(char *)passive_get_temperature_docstring_ },
    
        { "start",(PyCFunction)passive_wrapper_::start_method_,METH_VARARGS,(char *)passive_start_docstring_ },
    
        { "stop",(PyCFunction)passive_wrapper_::stop_method_,METH_VARARGS,(char *)passive_stop_docstring_ },
    
        { "start_polling",(PyCFunction)passive_wrapper_::start_polling_method_,METH_VARARGS,(char *)passive_start_polling_docstring_ },
    
        { "stop_polling",(PyCFunction)passive_wrapper_::stop_polling_method_,METH_VARARGS,(char *)passive_stop_polling_docstring_ },
    
        { "start_collecting",(PyCFunction)passive_wrapper_::start_collecting_method_,METH_VARARGS,(char *)passive_start_collecting_docstring_ },
    
        { "stop_collecting",(PyCFunction)passive_wrapper_::stop_collecting_method_,METH_VARARGS,(char *)passive_stop_collecting_docstring_ },
    
        { "get_collected_key",(PyCFunction)passive_wrapper_::get_collected_key_method_,METH_VARARGS,(char *)passive_get_collected_key_docstring_ },
    
        { "collect_count",(PyCFunction)passive_wrapper_::collect_count_method_,METH_VARARGS,(char *)passive_collect_count_docstring_ },
    
        { "set_active_colour",(PyCFunction)passive_wrapper_::set_active_colour_method_,METH_VARARGS,(char *)passive_set_active_colour_docstring_ },
    
        { "control",(PyCFunction)passive_wrapper_::control_method_,METH_VARARGS,(char *)passive_control_docstring_ },
    
        { "control_out",(PyCFunction)passive_wrapper_::control_out_method_,METH_VARARGS,(char *)passive_control_out_docstring_ },
    
        { "control_in",(PyCFunction)passive_wrapper_::control_in_method_,METH_VARARGS,(char *)passive_control_in_docstring_ },
    
    
        { "py_lock",(PyCFunction)passive_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)passive_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *passive_docstring_ = ""
"    Passive USB keyboard controller"
"    "
"    a passive connects to the usb instrument with the specified name."
"    The name is an OS specific name, e.g. /proc/bus/usb/xxx/yyy on linux"
"    It provides no callbacks; key data is passively collected and stored"
"    for retrieval"
"    "
"    Initialisers"
"    ============"
"    "
"    passive(name,decimate)"
"       - usbname: usbname of keyboard"
"       - decimate: decimation factor"
"    "
""
;

PyTypeObject passive_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "alpha2_native.passive",/*tp_name*/
    sizeof(passive_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)passive_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)passive_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    passive_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)passive_init_,  /* tp_init */
    0,                         /* tp_alloc */
    passive_new_,             /* tp_new */
};

PyObject *passive_wrapcopy_(passive_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *passive_wrap_(passive_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = passive_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    passive_object_ *self = 
        PyObject_NEW(passive_object_, &passive_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        passive_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

passive_type_ *passive_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &passive_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected passive object");
        return 0;
    }

    passive_type_ *object = static_cast<passive_type_ *>(((passive_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null passive object in unwrap");
        return 0;
    }

    return object;
}

void passive_insert_(const std::pair< passive_type_ *, PyObject *> &p)
{




    passive_wrapped_.insert(p);


}

void passive_erase_(passive_type_ *p)
{




    passive_wrapped_.erase(p);


}

PyObject * passive_find_(passive_type_ *p)
{




    PyObject *ret = 0;
    std::map< passive_type_ *, PyObject *>::iterator where = passive_wrapped_.find(p);
    if(where != passive_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct passive_dispatch_ passive_realdispatcher_ =
{
    &passive_typeobject_,
    passive_wrap_,
    passive_wrapcopy_,
    passive_unwrap_,
    passive_insert_,
    passive_erase_,
    passive_find_
};

  

struct active_object_ {
    PyObject_HEAD
    active_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct active_wrapper_: public active_type_ 
{

    static void __deleter__(active_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    active_wrapper_( PyObject *o, PyInterpreterState *i  ,usbdevice_type_ * a0  ,delegate_type_ * a1  ): active_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~active_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void start_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::start();
                
            
            
        }

        return
            
                start
            
            
                ();
    }

    static PyObject *start_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::start");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::stop");
        }

        return 0;
    }



    void msg_test_write_lib_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned int a2 ,unsigned int a3 )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_test_write_lib( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                msg_test_write_lib
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *msg_test_write_lib_method_(active_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned int a2;  unsigned int a3; 

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2  ,fpcvt_ui,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_test_write_lib_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_test_write_lib");
        }

        return 0;
    }



    void msg_test_finish_lib_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_test_finish_lib();
                
            
            
        }

        return
            
                msg_test_finish_lib
            
            
                ();
    }

    static PyObject *msg_test_finish_lib_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_test_finish_lib_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_test_finish_lib");
        }

        return 0;
    }



    void msg_test_write_seq_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned int a2 )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_test_write_seq( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                msg_test_write_seq
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *msg_test_write_seq_method_(active_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned int a2; 

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_test_write_seq_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_test_write_seq");
        }

        return 0;
    }



    void msg_test_finish_seq_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_test_finish_seq();
                
            
            
        }

        return
            
                msg_test_finish_seq
            
            
                ();
    }

    static PyObject *msg_test_finish_seq_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_test_finish_seq_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_test_finish_seq");
        }

        return 0;
    }



    void msg_test_start_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_test_start();
                
            
            
        }

        return
            
                msg_test_start
            
            
                ();
    }

    static PyObject *msg_test_start_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_test_start_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_test_start");
        }

        return 0;
    }



    void msg_flush_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_flush();
                
            
            
        }

        return
            
                msg_flush
            
            
                ();
    }

    static PyObject *msg_flush_method_(active_object_ *self, PyObject *args)
    {
        

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_flush_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_flush");
        }

        return 0;
    }



    void msg_set_led_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_set_led( a0  ,a1 );
                
            
            
        }

        return
            
                msg_set_led
            
            
                ( a0  ,a1 );
    }

    static PyObject *msg_set_led_method_(active_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_set_led_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_set_led");
        }

        return 0;
    }



    void set_tau_mode_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return active_type_::set_tau_mode( a0 );
                
            
            
        }

        return
            
                set_tau_mode
            
            
                ( a0 );
    }

    static PyObject *set_tau_mode_method_(active_object_ *self, PyObject *args)
    {
         bool a0; 

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_tau_mode_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::set_tau_mode");
        }

        return 0;
    }



    void msg_send_midi_call_(bool owned ,const unsigned char * a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return active_type_::msg_send_midi( a0  ,a1 );
                
            
            
        }

        return
            
                msg_send_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *msg_send_midi_method_(active_object_ *self, PyObject *args)
    {
         const unsigned char * a0;  unsigned int a1; 

        active_wrapper_ *o = (active_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null active c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ustr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->msg_send_midi_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in active::msg_send_midi");
        }

        return 0;
    }





    static PyObject *_method_lock(active_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(active_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void active_delete_(active_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            active_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (active_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        active_wrapper_::__deleter__((active_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void active_dealloc_(active_object_* self)
{
    
    active_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * active_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    active_object_ *self;

    if(!(self = (active_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int active_init_(active_object_ *self, PyObject *args, PyObject *kwds)
{
    active_delete_(self);

    

    {
         usbdevice_type_ * a0;  delegate_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_usbdevice,&a0,fpcvt_delegate,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new active_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing active");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            active_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"active invalid arguments");
    return -1;

}


static const char *active_start_docstring_ = "";

static const char *active_stop_docstring_ = "";

static const char *active_msg_test_write_lib_docstring_ = "";

static const char *active_msg_test_finish_lib_docstring_ = "";

static const char *active_msg_test_write_seq_docstring_ = "";

static const char *active_msg_test_finish_seq_docstring_ = "";

static const char *active_msg_test_start_docstring_ = "";

static const char *active_msg_flush_docstring_ = "";

static const char *active_msg_set_led_docstring_ = "";

static const char *active_set_tau_mode_docstring_ = "";

static const char *active_msg_send_midi_docstring_ = "";




static PyMethodDef active_methods_[] = {
    
        { "start",(PyCFunction)active_wrapper_::start_method_,METH_VARARGS,(char *)active_start_docstring_ },
    
        { "stop",(PyCFunction)active_wrapper_::stop_method_,METH_VARARGS,(char *)active_stop_docstring_ },
    
        { "msg_test_write_lib",(PyCFunction)active_wrapper_::msg_test_write_lib_method_,METH_VARARGS,(char *)active_msg_test_write_lib_docstring_ },
    
        { "msg_test_finish_lib",(PyCFunction)active_wrapper_::msg_test_finish_lib_method_,METH_VARARGS,(char *)active_msg_test_finish_lib_docstring_ },
    
        { "msg_test_write_seq",(PyCFunction)active_wrapper_::msg_test_write_seq_method_,METH_VARARGS,(char *)active_msg_test_write_seq_docstring_ },
    
        { "msg_test_finish_seq",(PyCFunction)active_wrapper_::msg_test_finish_seq_method_,METH_VARARGS,(char *)active_msg_test_finish_seq_docstring_ },
    
        { "msg_test_start",(PyCFunction)active_wrapper_::msg_test_start_method_,METH_VARARGS,(char *)active_msg_test_start_docstring_ },
    
        { "msg_flush",(PyCFunction)active_wrapper_::msg_flush_method_,METH_VARARGS,(char *)active_msg_flush_docstring_ },
    
        { "msg_set_led",(PyCFunction)active_wrapper_::msg_set_led_method_,METH_VARARGS,(char *)active_msg_set_led_docstring_ },
    
        { "set_tau_mode",(PyCFunction)active_wrapper_::set_tau_mode_method_,METH_VARARGS,(char *)active_set_tau_mode_docstring_ },
    
        { "msg_send_midi",(PyCFunction)active_wrapper_::msg_send_midi_method_,METH_VARARGS,(char *)active_msg_send_midi_docstring_ },
    
    
        { "py_lock",(PyCFunction)active_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)active_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *active_docstring_ = "";

PyTypeObject active_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "alpha2_native.active",/*tp_name*/
    sizeof(active_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)active_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)active_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    active_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)active_init_,  /* tp_init */
    0,                         /* tp_alloc */
    active_new_,             /* tp_new */
};

PyObject *active_wrapcopy_(active_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *active_wrap_(active_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = active_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    active_object_ *self = 
        PyObject_NEW(active_object_, &active_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        active_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

active_type_ *active_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &active_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected active object");
        return 0;
    }

    active_type_ *object = static_cast<active_type_ *>(((active_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null active object in unwrap");
        return 0;
    }

    return object;
}

void active_insert_(const std::pair< active_type_ *, PyObject *> &p)
{




    active_wrapped_.insert(p);


}

void active_erase_(active_type_ *p)
{




    active_wrapped_.erase(p);


}

PyObject * active_find_(active_type_ *p)
{




    PyObject *ret = 0;
    std::map< active_type_ *, PyObject *>::iterator where = active_wrapped_.find(p);
    if(where != active_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct active_dispatch_ active_realdispatcher_ =
{
    &active_typeobject_,
    active_wrap_,
    active_wrapcopy_,
    active_unwrap_,
    active_insert_,
    active_erase_,
    active_find_
};

  

struct delegate_object_ {
    PyObject_HEAD
    delegate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct delegate_wrapper_: public delegate_type_ 
{

    static void __deleter__(delegate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    void __safe_dtors()
    {
        
    }

    virtual ~delegate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(delegate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void delegate_delete_(delegate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            delegate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (delegate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        delegate_wrapper_::__deleter__((delegate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void delegate_dealloc_(delegate_object_* self)
{
    
    delegate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * delegate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    delegate_object_ *self;

    if(!(self = (delegate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int delegate_init_(delegate_object_ *self, PyObject *args, PyObject *kwds)
{
    delegate_delete_(self);

    

    PyErr_SetString(PyExc_RuntimeError,"delegate invalid arguments");
    return -1;

}





static PyMethodDef delegate_methods_[] = {
    
    
        { "py_lock",(PyCFunction)delegate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)delegate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *delegate_docstring_ = "";

PyTypeObject delegate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "alpha2_native.delegate",/*tp_name*/
    sizeof(delegate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)delegate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)delegate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    delegate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)delegate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    delegate_new_,             /* tp_new */
};

PyObject *delegate_wrapcopy_(delegate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *delegate_wrap_(delegate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = delegate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    delegate_object_ *self = 
        PyObject_NEW(delegate_object_, &delegate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

delegate_type_ *delegate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &delegate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected delegate object");
        return 0;
    }

    delegate_type_ *object = static_cast<delegate_type_ *>(((delegate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null delegate object in unwrap");
        return 0;
    }

    return object;
}

void delegate_insert_(const std::pair< delegate_type_ *, PyObject *> &p)
{




    delegate_wrapped_.insert(p);


}

void delegate_erase_(delegate_type_ *p)
{




    delegate_wrapped_.erase(p);


}

PyObject * delegate_find_(delegate_type_ *p)
{




    PyObject *ret = 0;
    std::map< delegate_type_ *, PyObject *>::iterator where = delegate_wrapped_.find(p);
    if(where != delegate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct delegate_dispatch_ delegate_realdispatcher_ =
{
    &delegate_typeobject_,
    delegate_wrap_,
    delegate_wrapcopy_,
    delegate_unwrap_,
    delegate_insert_,
    delegate_erase_,
    delegate_find_
};

  

struct printer_object_ {
    PyObject_HEAD
    delegate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct printer_wrapper_: public printer_type_ 
{

    static void __deleter__(printer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    printer_wrapper_( PyObject *o, PyInterpreterState *i  ): printer_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~printer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(printer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(printer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void printer_delete_(printer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            delegate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (printer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        printer_wrapper_::__deleter__((printer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void printer_dealloc_(printer_object_* self)
{
    
    printer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * printer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    printer_object_ *self;

    if(!(self = (printer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int printer_init_(printer_object_ *self, PyObject *args, PyObject *kwds)
{
    printer_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new printer_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing printer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"printer invalid arguments");
    return -1;

}





static PyMethodDef printer_methods_[] = {
    
    
        { "py_lock",(PyCFunction)printer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)printer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *printer_docstring_ = "";

PyTypeObject printer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "alpha2_native.printer",/*tp_name*/
    sizeof(printer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)printer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)printer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    printer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)printer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    printer_new_,             /* tp_new */
};

PyObject *printer_wrapcopy_(printer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *printer_wrap_(printer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = delegate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    printer_object_ *self = 
        PyObject_NEW(printer_object_, &printer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        delegate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

printer_type_ *printer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &printer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected printer object");
        return 0;
    }

    printer_type_ *object = static_cast<printer_type_ *>(((printer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null printer object in unwrap");
        return 0;
    }

    return object;
}

void printer_insert_(const std::pair< printer_type_ *, PyObject *> &p)
{


    delegate_dispatcher_->insert(p);




}

void printer_erase_(printer_type_ *p)
{


    delegate_dispatcher_->erase(p);




}

PyObject * printer_find_(printer_type_ *p)
{


    return delegate_dispatcher_->find(p);




}

static struct printer_dispatch_ printer_realdispatcher_ =
{
    &printer_typeobject_,
    printer_wrap_,
    printer_wrapcopy_,
    printer_unwrap_,
    printer_insert_,
    printer_erase_,
    printer_find_
};

   





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initalpha2_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            passive_dispatcher_ = &passive_realdispatcher_;
        
    
        
            active_dispatcher_ = &active_realdispatcher_;
        
    
        
            delegate_dispatcher_ = &delegate_realdispatcher_;
        
    
        
            printer_dispatcher_ = &printer_realdispatcher_;
        
    
        
    

    
        if(!(m=PyImport_ImportModule("picross_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_usbdevice_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_usbdevice_dispatch in module picross_native");
                return;
            }

            usbdevice_dispatcher_ = (usbdevice_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                printer_typeobject_.tp_base = delegate_dispatcher_->type;
            
        
    
        
    

    
        
            if(PyType_Ready(&passive_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&active_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&delegate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&printer_typeobject_)<0)
                return;
        
    
        
    

    if(!(m=Py_InitModule("alpha2_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(passive_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_passive_dispatch",s);
            Py_INCREF(&passive_typeobject_);
            PyModule_AddObject(m,"passive",(PyObject *)&passive_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(active_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_active_dispatch",s);
            Py_INCREF(&active_typeobject_);
            PyModule_AddObject(m,"active",(PyObject *)&active_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(delegate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_delegate_dispatch",s);
            Py_INCREF(&delegate_typeobject_);
            PyModule_AddObject(m,"delegate",(PyObject *)&delegate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(printer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_printer_dispatch",s);
            Py_INCREF(&printer_typeobject_);
            PyModule_AddObject(m,"printer",(PyObject *)&printer_typeobject_);
        
    
        
    

    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

