#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <lib_micro/micro_passive.h>


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef micro::passive_t passive_type_; 


 
static std::map<passive_type_ *, PyObject *> passive_wrapped_;
 


struct passive_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(passive_type_ *, bool owned);
    PyObject *(*wrapcopy)(passive_type_ *);
    passive_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< passive_type_ *, PyObject * > &);
    void (*erase)(passive_type_ *);
    PyObject *(*find)(passive_type_ *);
};

struct passive_dispatch_ *passive_dispatcher_;





int fpcvt_passive(PyObject *o, void *a)
{
    passive_type_ *object;
    if(o == Py_None)
    {
        *((passive_type_ **)a) = 0;
        return 1;
    }
    object = passive_dispatcher_->unwrap(o);
    *((passive_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_passive_copy fpcvt_passive

PyObject *tpcvt_passive(void *a)
{
    passive_type_ *o = *((passive_type_ **)a);
    return passive_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_passive_copy(void *a)
{
    passive_type_ *o = (passive_type_ *)a;
    return passive_dispatcher_->wrapcopy(o);
}



 

struct passive_object_ {
    PyObject_HEAD
    passive_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct passive_wrapper_: public passive_type_ 
{

    static void __deleter__(passive_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    passive_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,unsigned int a1  ): passive_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~passive_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned short get_rawkey_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_rawkey( a0  ,a1 );
                
            
            
        }

        return
            
                get_rawkey
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_rawkey_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_rawkey_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_rawkey");
        }

        return 0;
    }



    unsigned short get_strip_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_strip();
                
            
            
        }

        return
            
                get_strip
            
            
                ();
    }

    static PyObject *get_strip_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_strip_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_strip");
        }

        return 0;
    }



    bool wait_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::wait();
                
            
            
        }

        return
            
                wait
            
            
                ();
    }

    static PyObject *wait_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->wait_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::wait");
        }

        return 0;
    }



    void sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::sync();
                
            
            
        }

        return
            
                sync
            
            
                ();
    }

    static PyObject *sync_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::sync");
        }

        return 0;
    }



    void set_ledcolour_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_ledcolour( a0  ,a1 );
                
            
            
        }

        return
            
                set_ledcolour
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_ledcolour_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_ledcolour_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_ledcolour");
        }

        return 0;
    }



    void set_ledrgb_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned int a2 ,unsigned int a3 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_ledrgb( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                set_ledrgb
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *set_ledrgb_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned int a2;  unsigned int a3; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ui,&a2  ,fpcvt_ui,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_ledrgb_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_ledrgb");
        }

        return 0;
    }



    std::string debug_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::debug();
                
            
            
        }

        return
            
                debug
            
            
                ();
    }

    static PyObject *debug_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->debug_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::debug");
        }

        return 0;
    }



    void start_calibration_row_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start_calibration_row( a0  ,a1 );
                
            
            
        }

        return
            
                start_calibration_row
            
            
                ( a0  ,a1 );
    }

    static PyObject *start_calibration_row_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_calibration_row_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start_calibration_row");
        }

        return 0;
    }



    void set_calibration_range_call_(bool owned ,unsigned short a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_calibration_range( a0  ,a1 );
                
            
            
        }

        return
            
                set_calibration_range
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_calibration_range_method_(passive_object_ *self, PyObject *args)
    {
         unsigned short a0;  unsigned short a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_us,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_calibration_range_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_calibration_range");
        }

        return 0;
    }



    void set_calibration_point_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::set_calibration_point( a0  ,a1 );
                
            
            
        }

        return
            
                set_calibration_point
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_calibration_point_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_calibration_point_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::set_calibration_point");
        }

        return 0;
    }



    void write_calibration_row_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::write_calibration_row();
                
            
            
        }

        return
            
                write_calibration_row
            
            
                ();
    }

    static PyObject *write_calibration_row_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->write_calibration_row_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::write_calibration_row");
        }

        return 0;
    }



    void commit_calibration_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::commit_calibration();
                
            
            
        }

        return
            
                commit_calibration
            
            
                ();
    }

    static PyObject *commit_calibration_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->commit_calibration_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::commit_calibration");
        }

        return 0;
    }



    void clear_calibration_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::clear_calibration();
                
            
            
        }

        return
            
                clear_calibration
            
            
                ();
    }

    static PyObject *clear_calibration_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_calibration_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::clear_calibration");
        }

        return 0;
    }



    void read_calibration_row_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::read_calibration_row();
                
            
            
        }

        return
            
                read_calibration_row
            
            
                ();
    }

    static PyObject *read_calibration_row_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->read_calibration_row_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::read_calibration_row");
        }

        return 0;
    }



    unsigned short get_calibration_min_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_calibration_min();
                
            
            
        }

        return
            
                get_calibration_min
            
            
                ();
    }

    static PyObject *get_calibration_min_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_calibration_min_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_calibration_min");
        }

        return 0;
    }



    unsigned short get_calibration_max_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_calibration_max();
                
            
            
        }

        return
            
                get_calibration_max
            
            
                ();
    }

    static PyObject *get_calibration_max_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_calibration_max_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_calibration_max");
        }

        return 0;
    }



    unsigned short get_calibration_point_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_calibration_point( a0 );
                
            
            
        }

        return
            
                get_calibration_point
            
            
                ( a0 );
    }

    static PyObject *get_calibration_point_method_(passive_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned short r = o->get_calibration_point_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_calibration_point");
        }

        return 0;
    }



    unsigned int get_temperature_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::get_temperature();
                
            
            
        }

        return
            
                get_temperature
            
            
                ();
    }

    static PyObject *get_temperature_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->get_temperature_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::get_temperature");
        }

        return 0;
    }



    void start_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::start();
                
            
            
        }

        return
            
                start
            
            
                ();
    }

    static PyObject *start_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::start");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return passive_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(passive_object_ *self, PyObject *args)
    {
        

        passive_wrapper_ *o = (passive_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null passive c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in passive::stop");
        }

        return 0;
    }





    static PyObject *_method_lock(passive_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(passive_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void passive_delete_(passive_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            passive_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (passive_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        passive_wrapper_::__deleter__((passive_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void passive_dealloc_(passive_object_* self)
{
    
    passive_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * passive_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    passive_object_ *self;

    if(!(self = (passive_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int passive_init_(passive_object_ *self, PyObject *args, PyObject *kwds)
{
    passive_delete_(self);

    

    {
         const char * a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new passive_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing passive");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            passive_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"passive invalid arguments");
    return -1;

}


static const char *passive_get_rawkey_docstring_ = "";

static const char *passive_get_strip_docstring_ = "";

static const char *passive_wait_docstring_ = "";

static const char *passive_sync_docstring_ = "";

static const char *passive_set_ledcolour_docstring_ = "";

static const char *passive_set_ledrgb_docstring_ = "";

static const char *passive_debug_docstring_ = "";

static const char *passive_start_calibration_row_docstring_ = "";

static const char *passive_set_calibration_range_docstring_ = "";

static const char *passive_set_calibration_point_docstring_ = "";

static const char *passive_write_calibration_row_docstring_ = "";

static const char *passive_commit_calibration_docstring_ = "";

static const char *passive_clear_calibration_docstring_ = "";

static const char *passive_read_calibration_row_docstring_ = "";

static const char *passive_get_calibration_min_docstring_ = "";

static const char *passive_get_calibration_max_docstring_ = "";

static const char *passive_get_calibration_point_docstring_ = "";

static const char *passive_get_temperature_docstring_ = "";

static const char *passive_start_docstring_ = "";

static const char *passive_stop_docstring_ = "";




static PyMethodDef passive_methods_[] = {
    
        { "get_rawkey",(PyCFunction)passive_wrapper_::get_rawkey_method_,METH_VARARGS,(char *)passive_get_rawkey_docstring_ },
    
        { "get_strip",(PyCFunction)passive_wrapper_::get_strip_method_,METH_VARARGS,(char *)passive_get_strip_docstring_ },
    
        { "wait",(PyCFunction)passive_wrapper_::wait_method_,METH_VARARGS,(char *)passive_wait_docstring_ },
    
        { "sync",(PyCFunction)passive_wrapper_::sync_method_,METH_VARARGS,(char *)passive_sync_docstring_ },
    
        { "set_ledcolour",(PyCFunction)passive_wrapper_::set_ledcolour_method_,METH_VARARGS,(char *)passive_set_ledcolour_docstring_ },
    
        { "set_ledrgb",(PyCFunction)passive_wrapper_::set_ledrgb_method_,METH_VARARGS,(char *)passive_set_ledrgb_docstring_ },
    
        { "debug",(PyCFunction)passive_wrapper_::debug_method_,METH_VARARGS,(char *)passive_debug_docstring_ },
    
        { "start_calibration_row",(PyCFunction)passive_wrapper_::start_calibration_row_method_,METH_VARARGS,(char *)passive_start_calibration_row_docstring_ },
    
        { "set_calibration_range",(PyCFunction)passive_wrapper_::set_calibration_range_method_,METH_VARARGS,(char *)passive_set_calibration_range_docstring_ },
    
        { "set_calibration_point",(PyCFunction)passive_wrapper_::set_calibration_point_method_,METH_VARARGS,(char *)passive_set_calibration_point_docstring_ },
    
        { "write_calibration_row",(PyCFunction)passive_wrapper_::write_calibration_row_method_,METH_VARARGS,(char *)passive_write_calibration_row_docstring_ },
    
        { "commit_calibration",(PyCFunction)passive_wrapper_::commit_calibration_method_,METH_VARARGS,(char *)passive_commit_calibration_docstring_ },
    
        { "clear_calibration",(PyCFunction)passive_wrapper_::clear_calibration_method_,METH_VARARGS,(char *)passive_clear_calibration_docstring_ },
    
        { "read_calibration_row",(PyCFunction)passive_wrapper_::read_calibration_row_method_,METH_VARARGS,(char *)passive_read_calibration_row_docstring_ },
    
        { "get_calibration_min",(PyCFunction)passive_wrapper_::get_calibration_min_method_,METH_VARARGS,(char *)passive_get_calibration_min_docstring_ },
    
        { "get_calibration_max",(PyCFunction)passive_wrapper_::get_calibration_max_method_,METH_VARARGS,(char *)passive_get_calibration_max_docstring_ },
    
        { "get_calibration_point",(PyCFunction)passive_wrapper_::get_calibration_point_method_,METH_VARARGS,(char *)passive_get_calibration_point_docstring_ },
    
        { "get_temperature",(PyCFunction)passive_wrapper_::get_temperature_method_,METH_VARARGS,(char *)passive_get_temperature_docstring_ },
    
        { "start",(PyCFunction)passive_wrapper_::start_method_,METH_VARARGS,(char *)passive_start_docstring_ },
    
        { "stop",(PyCFunction)passive_wrapper_::stop_method_,METH_VARARGS,(char *)passive_stop_docstring_ },
    
    
        { "py_lock",(PyCFunction)passive_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)passive_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *passive_docstring_ = "";

PyTypeObject passive_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "micro_native.passive",/*tp_name*/
    sizeof(passive_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)passive_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)passive_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    passive_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)passive_init_,  /* tp_init */
    0,                         /* tp_alloc */
    passive_new_,             /* tp_new */
};

PyObject *passive_wrapcopy_(passive_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *passive_wrap_(passive_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = passive_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    passive_object_ *self = 
        PyObject_NEW(passive_object_, &passive_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        passive_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

passive_type_ *passive_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &passive_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected passive object");
        return 0;
    }

    passive_type_ *object = static_cast<passive_type_ *>(((passive_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null passive object in unwrap");
        return 0;
    }

    return object;
}

void passive_insert_(const std::pair< passive_type_ *, PyObject *> &p)
{




    passive_wrapped_.insert(p);


}

void passive_erase_(passive_type_ *p)
{




    passive_wrapped_.erase(p);


}

PyObject * passive_find_(passive_type_ *p)
{




    PyObject *ret = 0;
    std::map< passive_type_ *, PyObject *>::iterator where = passive_wrapped_.find(p);
    if(where != passive_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct passive_dispatch_ passive_realdispatcher_ =
{
    &passive_typeobject_,
    passive_wrap_,
    passive_wrapcopy_,
    passive_unwrap_,
    passive_insert_,
    passive_erase_,
    passive_find_
};

 





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initmicro_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            passive_dispatcher_ = &passive_realdispatcher_;
        
    

    

    
        
            
        
    

    
        
            if(PyType_Ready(&passive_typeobject_)<0)
                return;
        
    

    if(!(m=Py_InitModule("micro_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(passive_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_passive_dispatch",s);
            Py_INCREF(&passive_typeobject_);
            PyModule_AddObject(m,"passive",(PyObject *)&passive_typeobject_);
        
    

    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

