#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "sng_database.h"
#include "sng_mirror.h"
#include "sng_file.h"
#include "sng_mapping.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pi::state::mapref_t Mapping_type_; 


 
static std::map<Mapping_type_ *, PyObject *> Mapping_wrapped_;
 


struct Mapping_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Mapping_type_ *, bool owned);
    PyObject *(*wrapcopy)(Mapping_type_ *);
    Mapping_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Mapping_type_ *, PyObject * > &);
    void (*erase)(Mapping_type_ *);
    PyObject *(*find)(Mapping_type_ *);
};

struct Mapping_dispatch_ *Mapping_dispatcher_;



typedef pi::state::manager_t Manager_type_; 


  


struct Manager_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Manager_type_ *, bool owned);
    PyObject *(*wrapcopy)(Manager_type_ *);
    Manager_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Manager_type_ *, PyObject * > &);
    void (*erase)(Manager_type_ *);
    PyObject *(*find)(Manager_type_ *);
};

struct Manager_dispatch_ *Manager_dispatcher_;



typedef pi::state::fileref_t File_type_; 


 
static std::map<File_type_ *, PyObject *> File_wrapped_;
 


struct File_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(File_type_ *, bool owned);
    PyObject *(*wrapcopy)(File_type_ *);
    File_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< File_type_ *, PyObject * > &);
    void (*erase)(File_type_ *);
    PyObject *(*find)(File_type_ *);
};

struct File_dispatch_ *File_dispatcher_;



typedef pi::state::noderef_t Node_type_; 


 
static std::map<Node_type_ *, PyObject *> Node_wrapped_;
 


struct Node_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Node_type_ *, bool owned);
    PyObject *(*wrapcopy)(Node_type_ *);
    Node_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Node_type_ *, PyObject * > &);
    void (*erase)(Node_type_ *);
    PyObject *(*find)(Node_type_ *);
};

struct Node_dispatch_ *Node_dispatcher_;



typedef pi::state::agentref_t Agent_type_; 


 
static std::map<Agent_type_ *, PyObject *> Agent_wrapped_;
 


struct Agent_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Agent_type_ *, bool owned);
    PyObject *(*wrapcopy)(Agent_type_ *);
    Agent_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Agent_type_ *, PyObject * > &);
    void (*erase)(Agent_type_ *);
    PyObject *(*find)(Agent_type_ *);
};

struct Agent_dispatch_ *Agent_dispatcher_;



typedef pi::state::snapref_t Snapshot_type_; 


 
static std::map<Snapshot_type_ *, PyObject *> Snapshot_wrapped_;
 


struct Snapshot_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Snapshot_type_ *, bool owned);
    PyObject *(*wrapcopy)(Snapshot_type_ *);
    Snapshot_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Snapshot_type_ *, PyObject * > &);
    void (*erase)(Snapshot_type_ *);
    PyObject *(*find)(Snapshot_type_ *);
};

struct Snapshot_dispatch_ *Snapshot_dispatcher_;



typedef pi::state::dbref_t Database_type_; 


 
static std::map<Database_type_ *, PyObject *> Database_wrapped_;
 


struct Database_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(Database_type_ *, bool owned);
    PyObject *(*wrapcopy)(Database_type_ *);
    Database_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< Database_type_ *, PyObject * > &);
    void (*erase)(Database_type_ *);
    PyObject *(*find)(Database_type_ *);
};

struct Database_dispatch_ *Database_dispatcher_;



typedef piw::client_t client_type_; 





struct client_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(client_type_ *, bool owned);
    PyObject *(*wrapcopy)(client_type_ *);
    client_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< client_type_ *, PyObject * > &);
    void (*erase)(client_type_ *);
    PyObject *(*find)(client_type_ *);
};

struct client_dispatch_ *client_dispatcher_;



typedef piw::term_t term_type_; 





struct term_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(term_type_ *, bool owned);
    PyObject *(*wrapcopy)(term_type_ *);
    term_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< term_type_ *, PyObject * > &);
    void (*erase)(term_type_ *);
    PyObject *(*find)(term_type_ *);
};

struct term_dispatch_ *term_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;





int fpcvt_Mapping(PyObject *o, void *a)
{
    Mapping_type_ *object;
    if(o == Py_None)
    {
        *((Mapping_type_ **)a) = 0;
        return 1;
    }
    object = Mapping_dispatcher_->unwrap(o);
    *((Mapping_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Mapping_copy fpcvt_Mapping

PyObject *tpcvt_Mapping(void *a)
{
    Mapping_type_ *o = *((Mapping_type_ **)a);
    return Mapping_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Mapping_copy(void *a)
{
    Mapping_type_ *o = (Mapping_type_ *)a;
    return Mapping_dispatcher_->wrapcopy(o);
}



int fpcvt_Manager(PyObject *o, void *a)
{
    Manager_type_ *object;
    if(o == Py_None)
    {
        *((Manager_type_ **)a) = 0;
        return 1;
    }
    object = Manager_dispatcher_->unwrap(o);
    *((Manager_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Manager_copy fpcvt_Manager

PyObject *tpcvt_Manager(void *a)
{
    Manager_type_ *o = *((Manager_type_ **)a);
    return Manager_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Manager_copy(void *a)
{
    Manager_type_ *o = (Manager_type_ *)a;
    return Manager_dispatcher_->wrapcopy(o);
}



int fpcvt_File(PyObject *o, void *a)
{
    File_type_ *object;
    if(o == Py_None)
    {
        *((File_type_ **)a) = 0;
        return 1;
    }
    object = File_dispatcher_->unwrap(o);
    *((File_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_File_copy fpcvt_File

PyObject *tpcvt_File(void *a)
{
    File_type_ *o = *((File_type_ **)a);
    return File_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_File_copy(void *a)
{
    File_type_ *o = (File_type_ *)a;
    return File_dispatcher_->wrapcopy(o);
}



int fpcvt_Node(PyObject *o, void *a)
{
    Node_type_ *object;
    if(o == Py_None)
    {
        *((Node_type_ **)a) = 0;
        return 1;
    }
    object = Node_dispatcher_->unwrap(o);
    *((Node_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Node_copy fpcvt_Node

PyObject *tpcvt_Node(void *a)
{
    Node_type_ *o = *((Node_type_ **)a);
    return Node_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Node_copy(void *a)
{
    Node_type_ *o = (Node_type_ *)a;
    return Node_dispatcher_->wrapcopy(o);
}



int fpcvt_Agent(PyObject *o, void *a)
{
    Agent_type_ *object;
    if(o == Py_None)
    {
        *((Agent_type_ **)a) = 0;
        return 1;
    }
    object = Agent_dispatcher_->unwrap(o);
    *((Agent_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Agent_copy fpcvt_Agent

PyObject *tpcvt_Agent(void *a)
{
    Agent_type_ *o = *((Agent_type_ **)a);
    return Agent_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Agent_copy(void *a)
{
    Agent_type_ *o = (Agent_type_ *)a;
    return Agent_dispatcher_->wrapcopy(o);
}



int fpcvt_Snapshot(PyObject *o, void *a)
{
    Snapshot_type_ *object;
    if(o == Py_None)
    {
        *((Snapshot_type_ **)a) = 0;
        return 1;
    }
    object = Snapshot_dispatcher_->unwrap(o);
    *((Snapshot_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Snapshot_copy fpcvt_Snapshot

PyObject *tpcvt_Snapshot(void *a)
{
    Snapshot_type_ *o = *((Snapshot_type_ **)a);
    return Snapshot_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Snapshot_copy(void *a)
{
    Snapshot_type_ *o = (Snapshot_type_ *)a;
    return Snapshot_dispatcher_->wrapcopy(o);
}



int fpcvt_Database(PyObject *o, void *a)
{
    Database_type_ *object;
    if(o == Py_None)
    {
        *((Database_type_ **)a) = 0;
        return 1;
    }
    object = Database_dispatcher_->unwrap(o);
    *((Database_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_Database_copy fpcvt_Database

PyObject *tpcvt_Database(void *a)
{
    Database_type_ *o = *((Database_type_ **)a);
    return Database_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_Database_copy(void *a)
{
    Database_type_ *o = (Database_type_ *)a;
    return Database_dispatcher_->wrapcopy(o);
}



int fpcvt_client(PyObject *o, void *a)
{
    client_type_ *object;
    if(o == Py_None)
    {
        *((client_type_ **)a) = 0;
        return 1;
    }
    object = client_dispatcher_->unwrap(o);
    *((client_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_client_copy fpcvt_client

PyObject *tpcvt_client(void *a)
{
    client_type_ *o = *((client_type_ **)a);
    return client_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_client_copy(void *a)
{
    client_type_ *o = (client_type_ *)a;
    return client_dispatcher_->wrapcopy(o);
}



int fpcvt_term(PyObject *o, void *a)
{
    term_type_ *object;
    if(o == Py_None)
    {
        *((term_type_ **)a) = 0;
        return 1;
    }
    object = term_dispatcher_->unwrap(o);
    *((term_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_term_copy fpcvt_term

PyObject *tpcvt_term(void *a)
{
    term_type_ *o = *((term_type_ **)a);
    return term_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_term_copy(void *a)
{
    term_type_ *o = (term_type_ *)a;
    return term_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



 

struct Mapping_object_ {
    PyObject_HEAD
    Mapping_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Mapping_wrapper_: public Mapping_type_ 
{

    static void __deleter__(Mapping_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    Mapping_wrapper_( PyObject *o, PyInterpreterState *i  ): Mapping_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    Mapping_wrapper_( PyObject *o, PyInterpreterState *i  ,const Mapping_type_ & a0  ): Mapping_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Mapping_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void add_call_(bool owned ,const char * a0 ,const char * a1 )
    {
        if(owned)
        {
            
            
                return (operator->())->add( a0  ,a1 );
            
        }

        return
            
            
                (operator->())->add
            
                ( a0  ,a1 );
    }

    static PyObject *add_method_(Mapping_object_ *self, PyObject *args)
    {
         const char * a0;  const char * a1; 

        Mapping_wrapper_ *o = (Mapping_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Mapping c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_str,&a0  ,fpcvt_str,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->add_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Mapping::add");
        }

        return 0;
    }



    data_type_ substitute_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->substitute( a0 );
            
        }

        return
            
            
                (operator->())->substitute
            
                ( a0 );
    }

    static PyObject *substitute_method_(Mapping_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        Mapping_wrapper_ *o = (Mapping_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Mapping c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    data_type_ r = o->substitute_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Mapping::substitute");
        }

        return 0;
    }



    std::string render_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->render();
            
        }

        return
            
            
                (operator->())->render
            
                ();
    }

    static PyObject *render_method_(Mapping_object_ *self, PyObject *args)
    {
        

        Mapping_wrapper_ *o = (Mapping_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Mapping c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->render_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Mapping::render");
        }

        return 0;
    }





    static PyObject *_method_lock(Mapping_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Mapping_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Mapping_delete_(Mapping_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            Mapping_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Mapping_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Mapping_wrapper_::__deleter__((Mapping_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Mapping_dealloc_(Mapping_object_* self)
{
    
    Mapping_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Mapping_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Mapping_object_ *self;

    if(!(self = (Mapping_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Mapping_init_(Mapping_object_ *self, PyObject *args, PyObject *kwds)
{
    Mapping_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Mapping_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Mapping");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Mapping_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         Mapping_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Mapping_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Mapping_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Mapping");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Mapping_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Mapping invalid arguments");
    return -1;

}


static const char *Mapping_add_docstring_ = "";

static const char *Mapping_substitute_docstring_ = "";

static const char *Mapping_render_docstring_ = "";




static PyMethodDef Mapping_methods_[] = {
    
        { "add",(PyCFunction)Mapping_wrapper_::add_method_,METH_VARARGS,(char *)Mapping_add_docstring_ },
    
        { "substitute",(PyCFunction)Mapping_wrapper_::substitute_method_,METH_VARARGS,(char *)Mapping_substitute_docstring_ },
    
        { "render",(PyCFunction)Mapping_wrapper_::render_method_,METH_VARARGS,(char *)Mapping_render_docstring_ },
    
    
        { "py_lock",(PyCFunction)Mapping_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Mapping_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Mapping_docstring_ = "";

PyTypeObject Mapping_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Mapping",/*tp_name*/
    sizeof(Mapping_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Mapping_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)Mapping_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Mapping_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Mapping_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Mapping_new_,             /* tp_new */
};

PyObject *Mapping_wrapcopy_(Mapping_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            Mapping_object_ *obj = 
                PyObject_NEW(Mapping_object_, Mapping_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new Mapping_type_(*o);
                }

                try
                {
                    Mapping_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying Mapping");
                return 0;
            }
        

    
}

PyObject *Mapping_wrap_(Mapping_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = Mapping_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Mapping_object_ *self = 
        PyObject_NEW(Mapping_object_, &Mapping_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        Mapping_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Mapping_type_ *Mapping_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Mapping_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Mapping object");
        return 0;
    }

    Mapping_type_ *object = static_cast<Mapping_type_ *>(((Mapping_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Mapping object in unwrap");
        return 0;
    }

    return object;
}

void Mapping_insert_(const std::pair< Mapping_type_ *, PyObject *> &p)
{




    Mapping_wrapped_.insert(p);


}

void Mapping_erase_(Mapping_type_ *p)
{




    Mapping_wrapped_.erase(p);


}

PyObject * Mapping_find_(Mapping_type_ *p)
{




    PyObject *ret = 0;
    std::map< Mapping_type_ *, PyObject *>::iterator where = Mapping_wrapped_.find(p);
    if(where != Mapping_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct Mapping_dispatch_ Mapping_realdispatcher_ =
{
    &Mapping_typeobject_,
    Mapping_wrap_,
    Mapping_wrapcopy_,
    Mapping_unwrap_,
    Mapping_insert_,
    Mapping_erase_,
    Mapping_find_
};

  

struct Manager_object_ {
    PyObject_HEAD
    client_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Manager_wrapper_: public Manager_type_ 
{

    static void __deleter__(Manager_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
        cache_5_ = 0;
    
        cache_6_ = 0;
    
        cache_7_ = 0;
    
        cache_8_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
        if(cache_5_)
        {
            Py_DECREF(cache_5_);
            cache_5_ = 0;
        }
    
        if(cache_6_)
        {
            Py_DECREF(cache_6_);
            cache_6_ = 0;
        }
    
        if(cache_7_)
        {
            Py_DECREF(cache_7_);
            cache_7_ = 0;
        }
    
        if(cache_8_)
        {
            Py_DECREF(cache_8_);
            cache_8_ = 0;
        }
    
    }
*/



    Manager_wrapper_( PyObject *o, PyInterpreterState *i  ,const Node_type_ & a0  ): Manager_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Manager_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void client_sync(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_sync");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_sync before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_sync after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void client_tree(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_tree");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_tree before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_tree after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void manager_checkpoint(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"manager_checkpoint");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("manager_checkpoint before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("manager_checkpoint after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void client_opened(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_opened");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_opened before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_opened after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    void close_client(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"close_client");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("close_client before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("close_client after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_5_;
 
    void client_data(  const data_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_5_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_data");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_5_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_data_copy,&a0);
        //printf("client_data before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_data after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_6_;
 
    void client_clock(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_6_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_clock");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_6_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_clock before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_clock after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_7_;
 
    void client_closed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_7_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_closed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_7_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_closed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_closed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_8_;
 
    void client_child(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_8_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"client_child");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_8_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("client_child before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("client_child after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(Manager_object_ *self, visitproc visit, void *arg)
    {
        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(Manager_object_ *self)
    {
        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void client_sync_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_sync();
                
            
            
        }

        return
            
                client_sync
            
            
                ();
    }

    static PyObject *client_sync_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_sync_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_sync");
        }

        return 0;
    }



    void client_tree_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_tree();
                
            
            
        }

        return
            
                client_tree
            
            
                ();
    }

    static PyObject *client_tree_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_tree_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_tree");
        }

        return 0;
    }



    void client_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_clock();
                
            
            
        }

        return
            
                client_clock
            
            
                ();
    }

    static PyObject *client_clock_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_clock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_clock");
        }

        return 0;
    }



    void manager_checkpoint_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::manager_checkpoint();
                
            
            
        }

        return
            
                manager_checkpoint
            
            
                ();
    }

    static PyObject *manager_checkpoint_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->manager_checkpoint_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::manager_checkpoint");
        }

        return 0;
    }



    void client_opened_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_opened();
                
            
            
        }

        return
            
                client_opened
            
            
                ();
    }

    static PyObject *client_opened_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_opened_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_opened");
        }

        return 0;
    }



    void close_client_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::close_client();
                
            
            
        }

        return
            
                close_client
            
            
                ();
    }

    static PyObject *close_client_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_client_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::close_client");
        }

        return 0;
    }



    void client_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_data( a0 );
                
            
            
        }

        return
            
                client_data
            
            
                ( a0 );
    }

    static PyObject *client_data_method_(Manager_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_data_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_data");
        }

        return 0;
    }



    void save_template_call_(bool owned ,const Node_type_ & a0 ,const Mapping_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::save_template( a0  ,a1 );
                
            
            
        }

        return
            
                save_template
            
            
                ( a0  ,a1 );
    }

    static PyObject *save_template_method_(Manager_object_ *self, PyObject *args)
    {
         Node_type_ * a0;  Mapping_type_ * a1; 

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_Node_copy,&a0  ,fpcvt_Mapping_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->save_template_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::save_template");
        }

        return 0;
    }



    term_type_ get_diff_call_(bool owned ,const Node_type_ & a0 ,const Mapping_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::get_diff( a0  ,a1 );
                
            
            
        }

        return
            
                get_diff
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_diff_method_(Manager_object_ *self, PyObject *args)
    {
         Node_type_ * a0;  Mapping_type_ * a1; 

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_Node_copy,&a0  ,fpcvt_Mapping_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    term_type_ r = o->get_diff_call_ (self->owned ,*a0 ,*a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_term_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::get_diff");
        }

        return 0;
    }



    void client_closed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_closed();
                
            
            
        }

        return
            
                client_closed
            
            
                ();
    }

    static PyObject *client_closed_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_closed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_closed");
        }

        return 0;
    }



    void client_child_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Manager_type_::client_child();
                
            
            
        }

        return
            
                client_child
            
            
                ();
    }

    static PyObject *client_child_method_(Manager_object_ *self, PyObject *args)
    {
        

        Manager_wrapper_ *o = (Manager_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Manager c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->client_child_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Manager::client_child");
        }

        return 0;
    }





    static PyObject *_method_lock(Manager_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Manager_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Manager_delete_(Manager_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            client_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Manager_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Manager_wrapper_::__deleter__((Manager_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Manager_dealloc_(Manager_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    Manager_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Manager_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Manager_object_ *self;

    if(!(self = (Manager_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Manager_init_(Manager_object_ *self, PyObject *args, PyObject *kwds)
{
    Manager_delete_(self);

    

    {
         Node_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Node_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Manager_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Manager");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Manager invalid arguments");
    return -1;

}


static const char *Manager_client_sync_docstring_ = "";

static const char *Manager_client_tree_docstring_ = "";

static const char *Manager_client_clock_docstring_ = "";

static const char *Manager_manager_checkpoint_docstring_ = "";

static const char *Manager_client_opened_docstring_ = "";

static const char *Manager_close_client_docstring_ = "";

static const char *Manager_client_data_docstring_ = "";

static const char *Manager_save_template_docstring_ = "";

static const char *Manager_get_diff_docstring_ = "";

static const char *Manager_client_closed_docstring_ = "";

static const char *Manager_client_child_docstring_ = "";




static PyMethodDef Manager_methods_[] = {
    
        { "client_sync",(PyCFunction)Manager_wrapper_::client_sync_method_,METH_VARARGS,(char *)Manager_client_sync_docstring_ },
    
        { "client_tree",(PyCFunction)Manager_wrapper_::client_tree_method_,METH_VARARGS,(char *)Manager_client_tree_docstring_ },
    
        { "client_clock",(PyCFunction)Manager_wrapper_::client_clock_method_,METH_VARARGS,(char *)Manager_client_clock_docstring_ },
    
        { "manager_checkpoint",(PyCFunction)Manager_wrapper_::manager_checkpoint_method_,METH_VARARGS,(char *)Manager_manager_checkpoint_docstring_ },
    
        { "client_opened",(PyCFunction)Manager_wrapper_::client_opened_method_,METH_VARARGS,(char *)Manager_client_opened_docstring_ },
    
        { "close_client",(PyCFunction)Manager_wrapper_::close_client_method_,METH_VARARGS,(char *)Manager_close_client_docstring_ },
    
        { "client_data",(PyCFunction)Manager_wrapper_::client_data_method_,METH_VARARGS,(char *)Manager_client_data_docstring_ },
    
        { "save_template",(PyCFunction)Manager_wrapper_::save_template_method_,METH_VARARGS,(char *)Manager_save_template_docstring_ },
    
        { "get_diff",(PyCFunction)Manager_wrapper_::get_diff_method_,METH_VARARGS,(char *)Manager_get_diff_docstring_ },
    
        { "client_closed",(PyCFunction)Manager_wrapper_::client_closed_method_,METH_VARARGS,(char *)Manager_client_closed_docstring_ },
    
        { "client_child",(PyCFunction)Manager_wrapper_::client_child_method_,METH_VARARGS,(char *)Manager_client_child_docstring_ },
    
    
        { "py_lock",(PyCFunction)Manager_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Manager_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Manager_docstring_ = "";

PyTypeObject Manager_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Manager",/*tp_name*/
    sizeof(Manager_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Manager_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)Manager_docstring_,       /* tp_doc */
    (traverseproc)Manager_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)Manager_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Manager_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Manager_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Manager_new_,             /* tp_new */
};

PyObject *Manager_wrapcopy_(Manager_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *Manager_wrap_(Manager_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = client_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Manager_object_ *self = 
        
        PyObject_GC_New(Manager_object_, &Manager_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        client_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Manager_type_ *Manager_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Manager_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Manager object");
        return 0;
    }

    Manager_type_ *object = static_cast<Manager_type_ *>(((Manager_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Manager object in unwrap");
        return 0;
    }

    return object;
}

void Manager_insert_(const std::pair< Manager_type_ *, PyObject *> &p)
{


    client_dispatcher_->insert(p);




}

void Manager_erase_(Manager_type_ *p)
{


    client_dispatcher_->erase(p);




}

PyObject * Manager_find_(Manager_type_ *p)
{


    return client_dispatcher_->find(p);




}

static struct Manager_dispatch_ Manager_realdispatcher_ =
{
    &Manager_typeobject_,
    Manager_wrap_,
    Manager_wrapcopy_,
    Manager_unwrap_,
    Manager_insert_,
    Manager_erase_,
    Manager_find_
};

  

struct File_object_ {
    PyObject_HEAD
    File_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct File_wrapper_: public File_type_ 
{

    static void __deleter__(File_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    File_wrapper_( PyObject *o, PyInterpreterState *i  ,const File_type_ & a0  ): File_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~File_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned long checkpoint_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->checkpoint();
            
        }

        return
            
            
                (operator->())->checkpoint
            
                ();
    }

    static PyObject *checkpoint_method_(File_object_ *self, PyObject *args)
    {
        

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->checkpoint_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::checkpoint");
        }

        return 0;
    }



    std::string read_payload_string_call_(bool owned ,unsigned long a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->read_payload_string( a0 );
            
        }

        return
            
            
                (operator->())->read_payload_string
            
                ( a0 );
    }

    static PyObject *read_payload_string_method_(File_object_ *self, PyObject *args)
    {
         unsigned long a0; 

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ul,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->read_payload_string_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::read_payload_string");
        }

        return 0;
    }



    unsigned long write_payload_string_call_(bool owned ,const std::string & a0 ,bool a1 )
    {
        if(owned)
        {
            
            
                return (operator->())->write_payload_string( a0  ,a1 );
            
        }

        return
            
            
                (operator->())->write_payload_string
            
                ( a0  ,a1 );
    }

    static PyObject *write_payload_string_method_(File_object_ *self, PyObject *args)
    {
         std::string a0;  bool a1; 

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->write_payload_string_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::write_payload_string");
        }

        return 0;
    }



    void flush_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->flush();
            
        }

        return
            
            
                (operator->())->flush
            
                ();
    }

    static PyObject *flush_method_(File_object_ *self, PyObject *args)
    {
        

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->flush_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::flush");
        }

        return 0;
    }



    void close_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->close();
            
        }

        return
            
            
                (operator->())->close
            
                ();
    }

    static PyObject *close_method_(File_object_ *self, PyObject *args)
    {
        

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->close_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::close");
        }

        return 0;
    }



    bool isopen_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->isopen();
            
        }

        return
            
            
                (operator->())->isopen
            
                ();
    }

    static PyObject *isopen_method_(File_object_ *self, PyObject *args)
    {
        

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    bool r = o->isopen_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::isopen");
        }

        return 0;
    }



    std::string name_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->name();
            
        }

        return
            
            
                (operator->())->name
            
                ();
    }

    static PyObject *name_method_(File_object_ *self, PyObject *args)
    {
        

        File_wrapper_ *o = (File_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null File c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in File::name");
        }

        return 0;
    }





    static PyObject *_method_lock(File_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(File_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void File_delete_(File_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            File_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (File_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        File_wrapper_::__deleter__((File_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void File_dealloc_(File_object_* self)
{
    
    File_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * File_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    File_object_ *self;

    if(!(self = (File_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int File_init_(File_object_ *self, PyObject *args, PyObject *kwds)
{
    File_delete_(self);

    

    {
         File_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_File_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new File_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing File");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            File_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"File invalid arguments");
    return -1;

}


static const char *File_checkpoint_docstring_ = "";

static const char *File_read_payload_string_docstring_ = "";

static const char *File_write_payload_string_docstring_ = "";

static const char *File_flush_docstring_ = "";

static const char *File_close_docstring_ = "";

static const char *File_isopen_docstring_ = "";

static const char *File_name_docstring_ = "";




static PyMethodDef File_methods_[] = {
    
        { "checkpoint",(PyCFunction)File_wrapper_::checkpoint_method_,METH_VARARGS,(char *)File_checkpoint_docstring_ },
    
        { "read_payload_string",(PyCFunction)File_wrapper_::read_payload_string_method_,METH_VARARGS,(char *)File_read_payload_string_docstring_ },
    
        { "write_payload_string",(PyCFunction)File_wrapper_::write_payload_string_method_,METH_VARARGS,(char *)File_write_payload_string_docstring_ },
    
        { "flush",(PyCFunction)File_wrapper_::flush_method_,METH_VARARGS,(char *)File_flush_docstring_ },
    
        { "close",(PyCFunction)File_wrapper_::close_method_,METH_VARARGS,(char *)File_close_docstring_ },
    
        { "isopen",(PyCFunction)File_wrapper_::isopen_method_,METH_VARARGS,(char *)File_isopen_docstring_ },
    
        { "name",(PyCFunction)File_wrapper_::name_method_,METH_VARARGS,(char *)File_name_docstring_ },
    
    
        { "py_lock",(PyCFunction)File_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)File_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *File_docstring_ = "";

PyTypeObject File_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.File",/*tp_name*/
    sizeof(File_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)File_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)File_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    File_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)File_init_,  /* tp_init */
    0,                         /* tp_alloc */
    File_new_,             /* tp_new */
};

PyObject *File_wrapcopy_(File_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            File_object_ *obj = 
                PyObject_NEW(File_object_, File_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new File_type_(*o);
                }

                try
                {
                    File_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying File");
                return 0;
            }
        

    
}

PyObject *File_wrap_(File_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = File_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    File_object_ *self = 
        PyObject_NEW(File_object_, &File_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        File_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

File_type_ *File_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &File_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected File object");
        return 0;
    }

    File_type_ *object = static_cast<File_type_ *>(((File_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null File object in unwrap");
        return 0;
    }

    return object;
}

void File_insert_(const std::pair< File_type_ *, PyObject *> &p)
{




    File_wrapped_.insert(p);


}

void File_erase_(File_type_ *p)
{




    File_wrapped_.erase(p);


}

PyObject * File_find_(File_type_ *p)
{




    PyObject *ret = 0;
    std::map< File_type_ *, PyObject *>::iterator where = File_wrapped_.find(p);
    if(where != File_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct File_dispatch_ File_realdispatcher_ =
{
    &File_typeobject_,
    File_wrap_,
    File_wrapcopy_,
    File_unwrap_,
    File_insert_,
    File_erase_,
    File_find_
};

  

struct Node_object_ {
    PyObject_HEAD
    Node_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Node_wrapper_: public Node_type_ 
{

    static void __deleter__(Node_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    Node_wrapper_( PyObject *o, PyInterpreterState *i  ): Node_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    Node_wrapper_( PyObject *o, PyInterpreterState *i  ,const Node_type_ & a0  ): Node_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Node_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned char enum_children_call_(bool owned ,unsigned char a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->enum_children( a0 );
            
        }

        return
            
            
                (operator->())->enum_children
            
                ( a0 );
    }

    static PyObject *enum_children_method_(Node_object_ *self, PyObject *args)
    {
         unsigned char a0; 

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_uc,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned char r = o->enum_children_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_uc((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::enum_children");
        }

        return 0;
    }



    data_type_ get_data_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_data();
            
        }

        return
            
            
                (operator->())->get_data
            
                ();
    }

    static PyObject *get_data_method_(Node_object_ *self, PyObject *args)
    {
        

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    data_type_ r = o->get_data_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::get_data");
        }

        return 0;
    }



    Node_type_ get_child_call_(bool owned ,unsigned char a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->get_child( a0 );
            
        }

        return
            
            
                (operator->())->get_child
            
                ( a0 );
    }

    static PyObject *get_child_method_(Node_object_ *self, PyObject *args)
    {
         unsigned char a0; 

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_uc,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Node_type_ r = o->get_child_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Node_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::get_child");
        }

        return 0;
    }



    bool set_data_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->set_data( a0 );
            
        }

        return
            
            
                (operator->())->set_data
            
                ( a0 );
    }

    static PyObject *set_data_method_(Node_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_data_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::set_data");
        }

        return 0;
    }



    void erase_child_call_(bool owned ,unsigned char a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->erase_child( a0 );
            
        }

        return
            
            
                (operator->())->erase_child
            
                ( a0 );
    }

    static PyObject *erase_child_method_(Node_object_ *self, PyObject *args)
    {
         unsigned char a0; 

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_uc,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->erase_child_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::erase_child");
        }

        return 0;
    }



    void erase_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->erase();
            
        }

        return
            
            
                (operator->())->erase
            
                ();
    }

    static PyObject *erase_method_(Node_object_ *self, PyObject *args)
    {
        

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->erase_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::erase");
        }

        return 0;
    }



    std::string list_children_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->list_children();
            
        }

        return
            
            
                (operator->())->list_children
            
                ();
    }

    static PyObject *list_children_method_(Node_object_ *self, PyObject *args)
    {
        

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->list_children_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::list_children");
        }

        return 0;
    }



    Node_type_ snapshot_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->snapshot();
            
        }

        return
            
            
                (operator->())->snapshot
            
                ();
    }

    static PyObject *snapshot_method_(Node_object_ *self, PyObject *args)
    {
        

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Node_type_ r = o->snapshot_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Node_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::snapshot");
        }

        return 0;
    }



    void copy_call_(bool owned ,const Node_type_ & a0 ,const Mapping_type_ & a1 )
    {
        if(owned)
        {
            
            
                return (operator->())->copy( a0  ,a1 );
            
        }

        return
            
            
                (operator->())->copy
            
                ( a0  ,a1 );
    }

    static PyObject *copy_method_(Node_object_ *self, PyObject *args)
    {
         Node_type_ * a0;  Mapping_type_ * a1; 

        Node_wrapper_ *o = (Node_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Node c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_Node_copy,&a0  ,fpcvt_Mapping_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->copy_call_ (self->owned ,*a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Node::copy");
        }

        return 0;
    }





    static PyObject *_method_lock(Node_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Node_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Node_delete_(Node_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            Node_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Node_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Node_wrapper_::__deleter__((Node_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Node_dealloc_(Node_object_* self)
{
    
    Node_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Node_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Node_object_ *self;

    if(!(self = (Node_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Node_init_(Node_object_ *self, PyObject *args, PyObject *kwds)
{
    Node_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Node_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Node");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Node_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         Node_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Node_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Node_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Node");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Node_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Node invalid arguments");
    return -1;

}


static const char *Node_enum_children_docstring_ = "";

static const char *Node_get_data_docstring_ = "";

static const char *Node_get_child_docstring_ = "";

static const char *Node_set_data_docstring_ = "";

static const char *Node_erase_child_docstring_ = "";

static const char *Node_erase_docstring_ = "";

static const char *Node_list_children_docstring_ = "";

static const char *Node_snapshot_docstring_ = "";

static const char *Node_copy_docstring_ = "";




static PyMethodDef Node_methods_[] = {
    
        { "enum_children",(PyCFunction)Node_wrapper_::enum_children_method_,METH_VARARGS,(char *)Node_enum_children_docstring_ },
    
        { "get_data",(PyCFunction)Node_wrapper_::get_data_method_,METH_VARARGS,(char *)Node_get_data_docstring_ },
    
        { "get_child",(PyCFunction)Node_wrapper_::get_child_method_,METH_VARARGS,(char *)Node_get_child_docstring_ },
    
        { "set_data",(PyCFunction)Node_wrapper_::set_data_method_,METH_VARARGS,(char *)Node_set_data_docstring_ },
    
        { "erase_child",(PyCFunction)Node_wrapper_::erase_child_method_,METH_VARARGS,(char *)Node_erase_child_docstring_ },
    
        { "erase",(PyCFunction)Node_wrapper_::erase_method_,METH_VARARGS,(char *)Node_erase_docstring_ },
    
        { "list_children",(PyCFunction)Node_wrapper_::list_children_method_,METH_VARARGS,(char *)Node_list_children_docstring_ },
    
        { "snapshot",(PyCFunction)Node_wrapper_::snapshot_method_,METH_VARARGS,(char *)Node_snapshot_docstring_ },
    
        { "copy",(PyCFunction)Node_wrapper_::copy_method_,METH_VARARGS,(char *)Node_copy_docstring_ },
    
    
        { "py_lock",(PyCFunction)Node_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Node_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Node_docstring_ = "";

PyTypeObject Node_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Node",/*tp_name*/
    sizeof(Node_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Node_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)Node_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Node_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Node_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Node_new_,             /* tp_new */
};

PyObject *Node_wrapcopy_(Node_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            Node_object_ *obj = 
                PyObject_NEW(Node_object_, Node_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new Node_type_(*o);
                }

                try
                {
                    Node_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying Node");
                return 0;
            }
        

    
}

PyObject *Node_wrap_(Node_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = Node_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Node_object_ *self = 
        PyObject_NEW(Node_object_, &Node_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        Node_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Node_type_ *Node_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Node_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Node object");
        return 0;
    }

    Node_type_ *object = static_cast<Node_type_ *>(((Node_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Node object in unwrap");
        return 0;
    }

    return object;
}

void Node_insert_(const std::pair< Node_type_ *, PyObject *> &p)
{




    Node_wrapped_.insert(p);


}

void Node_erase_(Node_type_ *p)
{




    Node_wrapped_.erase(p);


}

PyObject * Node_find_(Node_type_ *p)
{




    PyObject *ret = 0;
    std::map< Node_type_ *, PyObject *>::iterator where = Node_wrapped_.find(p);
    if(where != Node_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct Node_dispatch_ Node_realdispatcher_ =
{
    &Node_typeobject_,
    Node_wrap_,
    Node_wrapcopy_,
    Node_unwrap_,
    Node_insert_,
    Node_erase_,
    Node_find_
};

  

struct Agent_object_ {
    PyObject_HEAD
    Agent_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Agent_wrapper_: public Agent_type_ 
{

    static void __deleter__(Agent_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    Agent_wrapper_( PyObject *o, PyInterpreterState *i  ): Agent_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    Agent_wrapper_( PyObject *o, PyInterpreterState *i  ,const Agent_type_ & a0  ): Agent_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Agent_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    std::string get_address_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_address();
            
        }

        return
            
            
                (operator->())->get_address
            
                ();
    }

    static PyObject *get_address_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->get_address_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_address");
        }

        return 0;
    }



    unsigned int get_type_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_type();
            
        }

        return
            
            
                (operator->())->get_type
            
                ();
    }

    static PyObject *get_type_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->get_type_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_type");
        }

        return 0;
    }



    Node_type_ get_root_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_root();
            
        }

        return
            
            
                (operator->())->get_root
            
                ();
    }

    static PyObject *get_root_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Node_type_ r = o->get_root_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Node_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_root");
        }

        return 0;
    }



    unsigned long set_checkpoint_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->set_checkpoint();
            
        }

        return
            
            
                (operator->())->set_checkpoint
            
                ();
    }

    static PyObject *set_checkpoint_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->set_checkpoint_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::set_checkpoint");
        }

        return 0;
    }



    unsigned long get_checkpoint_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_checkpoint();
            
        }

        return
            
            
                (operator->())->get_checkpoint
            
                ();
    }

    static PyObject *get_checkpoint_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->get_checkpoint_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_checkpoint");
        }

        return 0;
    }



    bool isdirty_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->isdirty();
            
        }

        return
            
            
                (operator->())->isdirty
            
                ();
    }

    static PyObject *isdirty_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    bool r = o->isdirty_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::isdirty");
        }

        return 0;
    }



    Agent_type_ checkpoint_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->checkpoint();
            
        }

        return
            
            
                (operator->())->checkpoint
            
                ();
    }

    static PyObject *checkpoint_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Agent_type_ r = o->checkpoint_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Agent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::checkpoint");
        }

        return 0;
    }



    void set_type_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->set_type( a0 );
            
        }

        return
            
            
                (operator->())->set_type
            
                ( a0 );
    }

    static PyObject *set_type_method_(Agent_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_type_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::set_type");
        }

        return 0;
    }



    bool isvalid_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return Agent_type_::isvalid();
                
            
            
        }

        return
            
                isvalid
            
            
                ();
    }

    static PyObject *isvalid_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    bool r = o->isvalid_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::isvalid");
        }

        return 0;
    }



    std::string get_name_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_name();
            
        }

        return
            
            
                (operator->())->get_name
            
                ();
    }

    static PyObject *get_name_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->get_name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_name");
        }

        return 0;
    }



    std::string get_plugin_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_plugin();
            
        }

        return
            
            
                (operator->())->get_plugin
            
                ();
    }

    static PyObject *get_plugin_method_(Agent_object_ *self, PyObject *args)
    {
        

        Agent_wrapper_ *o = (Agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->get_plugin_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Agent::get_plugin");
        }

        return 0;
    }





    static PyObject *_method_lock(Agent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Agent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Agent_delete_(Agent_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            Agent_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Agent_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Agent_wrapper_::__deleter__((Agent_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Agent_dealloc_(Agent_object_* self)
{
    
    Agent_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Agent_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Agent_object_ *self;

    if(!(self = (Agent_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Agent_init_(Agent_object_ *self, PyObject *args, PyObject *kwds)
{
    Agent_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Agent_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Agent");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         Agent_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Agent_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Agent_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Agent");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Agent invalid arguments");
    return -1;

}


static const char *Agent_get_address_docstring_ = "";

static const char *Agent_get_type_docstring_ = "";

static const char *Agent_get_root_docstring_ = "";

static const char *Agent_set_checkpoint_docstring_ = "";

static const char *Agent_get_checkpoint_docstring_ = "";

static const char *Agent_isdirty_docstring_ = "";

static const char *Agent_checkpoint_docstring_ = "";

static const char *Agent_set_type_docstring_ = "";

static const char *Agent_isvalid_docstring_ = "";

static const char *Agent_get_name_docstring_ = "";

static const char *Agent_get_plugin_docstring_ = "";




static PyMethodDef Agent_methods_[] = {
    
        { "get_address",(PyCFunction)Agent_wrapper_::get_address_method_,METH_VARARGS,(char *)Agent_get_address_docstring_ },
    
        { "get_type",(PyCFunction)Agent_wrapper_::get_type_method_,METH_VARARGS,(char *)Agent_get_type_docstring_ },
    
        { "get_root",(PyCFunction)Agent_wrapper_::get_root_method_,METH_VARARGS,(char *)Agent_get_root_docstring_ },
    
        { "set_checkpoint",(PyCFunction)Agent_wrapper_::set_checkpoint_method_,METH_VARARGS,(char *)Agent_set_checkpoint_docstring_ },
    
        { "get_checkpoint",(PyCFunction)Agent_wrapper_::get_checkpoint_method_,METH_VARARGS,(char *)Agent_get_checkpoint_docstring_ },
    
        { "isdirty",(PyCFunction)Agent_wrapper_::isdirty_method_,METH_VARARGS,(char *)Agent_isdirty_docstring_ },
    
        { "checkpoint",(PyCFunction)Agent_wrapper_::checkpoint_method_,METH_VARARGS,(char *)Agent_checkpoint_docstring_ },
    
        { "set_type",(PyCFunction)Agent_wrapper_::set_type_method_,METH_VARARGS,(char *)Agent_set_type_docstring_ },
    
        { "isvalid",(PyCFunction)Agent_wrapper_::isvalid_method_,METH_VARARGS,(char *)Agent_isvalid_docstring_ },
    
        { "get_name",(PyCFunction)Agent_wrapper_::get_name_method_,METH_VARARGS,(char *)Agent_get_name_docstring_ },
    
        { "get_plugin",(PyCFunction)Agent_wrapper_::get_plugin_method_,METH_VARARGS,(char *)Agent_get_plugin_docstring_ },
    
    
        { "py_lock",(PyCFunction)Agent_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Agent_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Agent_docstring_ = "";

PyTypeObject Agent_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Agent",/*tp_name*/
    sizeof(Agent_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Agent_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)Agent_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Agent_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Agent_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Agent_new_,             /* tp_new */
};

PyObject *Agent_wrapcopy_(Agent_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            Agent_object_ *obj = 
                PyObject_NEW(Agent_object_, Agent_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new Agent_type_(*o);
                }

                try
                {
                    Agent_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying Agent");
                return 0;
            }
        

    
}

PyObject *Agent_wrap_(Agent_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = Agent_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Agent_object_ *self = 
        PyObject_NEW(Agent_object_, &Agent_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        Agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Agent_type_ *Agent_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Agent_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Agent object");
        return 0;
    }

    Agent_type_ *object = static_cast<Agent_type_ *>(((Agent_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Agent object in unwrap");
        return 0;
    }

    return object;
}

void Agent_insert_(const std::pair< Agent_type_ *, PyObject *> &p)
{




    Agent_wrapped_.insert(p);


}

void Agent_erase_(Agent_type_ *p)
{




    Agent_wrapped_.erase(p);


}

PyObject * Agent_find_(Agent_type_ *p)
{




    PyObject *ret = 0;
    std::map< Agent_type_ *, PyObject *>::iterator where = Agent_wrapped_.find(p);
    if(where != Agent_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct Agent_dispatch_ Agent_realdispatcher_ =
{
    &Agent_typeobject_,
    Agent_wrap_,
    Agent_wrapcopy_,
    Agent_unwrap_,
    Agent_insert_,
    Agent_erase_,
    Agent_find_
};

  

struct Snapshot_object_ {
    PyObject_HEAD
    Snapshot_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Snapshot_wrapper_: public Snapshot_type_ 
{

    static void __deleter__(Snapshot_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    Snapshot_wrapper_( PyObject *o, PyInterpreterState *i  ): Snapshot_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    Snapshot_wrapper_( PyObject *o, PyInterpreterState *i  ,const Snapshot_type_ & a0  ): Snapshot_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Snapshot_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    Snapshot_type_ clone_call_(bool owned ,unsigned long long a0 ,const char * a1 )
    {
        if(owned)
        {
            
            
                return (operator->())->clone( a0  ,a1 );
            
        }

        return
            
            
                (operator->())->clone
            
                ( a0  ,a1 );
    }

    static PyObject *clone_method_(Snapshot_object_ *self, PyObject *args)
    {
         unsigned long long a0;  const char * a1; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ull,&a0  ,fpcvt_str,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    Snapshot_type_ r = o->clone_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Snapshot_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::clone");
        }

        return 0;
    }



    unsigned int agent_count_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->agent_count();
            
        }

        return
            
            
                (operator->())->agent_count
            
                ();
    }

    static PyObject *agent_count_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->agent_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::agent_count");
        }

        return 0;
    }



    Agent_type_ get_agent_index_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->get_agent_index( a0 );
            
        }

        return
            
            
                (operator->())->get_agent_index
            
                ( a0 );
    }

    static PyObject *get_agent_index_method_(Snapshot_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Agent_type_ r = o->get_agent_index_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Agent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::get_agent_index");
        }

        return 0;
    }



    Agent_type_ get_agent_address_call_(bool owned ,unsigned int a0 ,const std::string & a1 ,bool a2 )
    {
        if(owned)
        {
            
            
                return (operator->())->get_agent_address( a0  ,a1  ,a2 );
            
        }

        return
            
            
                (operator->())->get_agent_address
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *get_agent_address_method_(Snapshot_object_ *self, PyObject *args)
    {
         unsigned int a0;  std::string a1;  bool a2; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_stdstr,&a1  ,fpcvt_b,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Agent_type_ r = o->get_agent_address_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Agent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::get_agent_address");
        }

        return 0;
    }



    Agent_type_ set_agent_call_(bool owned ,const Agent_type_ & a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->set_agent( a0 );
            
        }

        return
            
            
                (operator->())->set_agent
            
                ( a0 );
    }

    static PyObject *set_agent_method_(Snapshot_object_ *self, PyObject *args)
    {
         Agent_type_ * a0; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_Agent_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Agent_type_ r = o->set_agent_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Agent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::set_agent");
        }

        return 0;
    }



    bool erase_agent_call_(bool owned ,const Agent_type_ & a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->erase_agent( a0 );
            
        }

        return
            
            
                (operator->())->erase_agent
            
                ( a0 );
    }

    static PyObject *erase_agent_method_(Snapshot_object_ *self, PyObject *args)
    {
         Agent_type_ * a0; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_Agent_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->erase_agent_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::erase_agent");
        }

        return 0;
    }



    void erase_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->erase();
            
        }

        return
            
            
                (operator->())->erase
            
                ();
    }

    static PyObject *erase_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->erase_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::erase");
        }

        return 0;
    }



    unsigned long version_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->version();
            
        }

        return
            
            
                (operator->())->version
            
                ();
    }

    static PyObject *version_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->version_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::version");
        }

        return 0;
    }



    unsigned long previous_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->previous();
            
        }

        return
            
            
                (operator->())->previous
            
                ();
    }

    static PyObject *previous_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->previous_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::previous");
        }

        return 0;
    }



    unsigned long save_call_(bool owned ,unsigned long long a0 ,const char * a1 )
    {
        if(owned)
        {
            
            
                return (operator->())->save( a0  ,a1 );
            
        }

        return
            
            
                (operator->())->save
            
                ( a0  ,a1 );
    }

    static PyObject *save_method_(Snapshot_object_ *self, PyObject *args)
    {
         unsigned long long a0;  const char * a1; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ull,&a0  ,fpcvt_str,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->save_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::save");
        }

        return 0;
    }



    unsigned long long timestamp_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->timestamp();
            
        }

        return
            
            
                (operator->())->timestamp
            
                ();
    }

    static PyObject *timestamp_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long long r = o->timestamp_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ull((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::timestamp");
        }

        return 0;
    }



    std::string tag_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->tag();
            
        }

        return
            
            
                (operator->())->tag
            
                ();
    }

    static PyObject *tag_method_(Snapshot_object_ *self, PyObject *args)
    {
        

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->tag_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::tag");
        }

        return 0;
    }



    void copy_call_(bool owned ,const Snapshot_type_ & a0 ,const Mapping_type_ & a1 ,bool a2 )
    {
        if(owned)
        {
            
            
                return (operator->())->copy( a0  ,a1  ,a2 );
            
        }

        return
            
            
                (operator->())->copy
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *copy_method_(Snapshot_object_ *self, PyObject *args)
    {
         Snapshot_type_ * a0;  Mapping_type_ * a1;  bool a2; 

        Snapshot_wrapper_ *o = (Snapshot_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Snapshot c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_Snapshot_copy,&a0  ,fpcvt_Mapping_copy,&a1  ,fpcvt_b,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->copy_call_ (self->owned ,*a0,*a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Snapshot::copy");
        }

        return 0;
    }





    static PyObject *_method_lock(Snapshot_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Snapshot_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Snapshot_delete_(Snapshot_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            Snapshot_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Snapshot_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Snapshot_wrapper_::__deleter__((Snapshot_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Snapshot_dealloc_(Snapshot_object_* self)
{
    
    Snapshot_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Snapshot_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Snapshot_object_ *self;

    if(!(self = (Snapshot_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Snapshot_init_(Snapshot_object_ *self, PyObject *args, PyObject *kwds)
{
    Snapshot_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Snapshot_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Snapshot");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Snapshot_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         Snapshot_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Snapshot_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Snapshot_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Snapshot");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Snapshot_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Snapshot invalid arguments");
    return -1;

}


static const char *Snapshot_clone_docstring_ = "";

static const char *Snapshot_agent_count_docstring_ = "";

static const char *Snapshot_get_agent_index_docstring_ = "";

static const char *Snapshot_get_agent_address_docstring_ = "";

static const char *Snapshot_set_agent_docstring_ = "";

static const char *Snapshot_erase_agent_docstring_ = "";

static const char *Snapshot_erase_docstring_ = "";

static const char *Snapshot_version_docstring_ = "";

static const char *Snapshot_previous_docstring_ = "";

static const char *Snapshot_save_docstring_ = "";

static const char *Snapshot_timestamp_docstring_ = "";

static const char *Snapshot_tag_docstring_ = "";

static const char *Snapshot_copy_docstring_ = "";




static PyMethodDef Snapshot_methods_[] = {
    
        { "clone",(PyCFunction)Snapshot_wrapper_::clone_method_,METH_VARARGS,(char *)Snapshot_clone_docstring_ },
    
        { "agent_count",(PyCFunction)Snapshot_wrapper_::agent_count_method_,METH_VARARGS,(char *)Snapshot_agent_count_docstring_ },
    
        { "get_agent_index",(PyCFunction)Snapshot_wrapper_::get_agent_index_method_,METH_VARARGS,(char *)Snapshot_get_agent_index_docstring_ },
    
        { "get_agent_address",(PyCFunction)Snapshot_wrapper_::get_agent_address_method_,METH_VARARGS,(char *)Snapshot_get_agent_address_docstring_ },
    
        { "set_agent",(PyCFunction)Snapshot_wrapper_::set_agent_method_,METH_VARARGS,(char *)Snapshot_set_agent_docstring_ },
    
        { "erase_agent",(PyCFunction)Snapshot_wrapper_::erase_agent_method_,METH_VARARGS,(char *)Snapshot_erase_agent_docstring_ },
    
        { "erase",(PyCFunction)Snapshot_wrapper_::erase_method_,METH_VARARGS,(char *)Snapshot_erase_docstring_ },
    
        { "version",(PyCFunction)Snapshot_wrapper_::version_method_,METH_VARARGS,(char *)Snapshot_version_docstring_ },
    
        { "previous",(PyCFunction)Snapshot_wrapper_::previous_method_,METH_VARARGS,(char *)Snapshot_previous_docstring_ },
    
        { "save",(PyCFunction)Snapshot_wrapper_::save_method_,METH_VARARGS,(char *)Snapshot_save_docstring_ },
    
        { "timestamp",(PyCFunction)Snapshot_wrapper_::timestamp_method_,METH_VARARGS,(char *)Snapshot_timestamp_docstring_ },
    
        { "tag",(PyCFunction)Snapshot_wrapper_::tag_method_,METH_VARARGS,(char *)Snapshot_tag_docstring_ },
    
        { "copy",(PyCFunction)Snapshot_wrapper_::copy_method_,METH_VARARGS,(char *)Snapshot_copy_docstring_ },
    
    
        { "py_lock",(PyCFunction)Snapshot_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Snapshot_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Snapshot_docstring_ = "";

PyTypeObject Snapshot_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Snapshot",/*tp_name*/
    sizeof(Snapshot_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Snapshot_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)Snapshot_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Snapshot_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Snapshot_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Snapshot_new_,             /* tp_new */
};

PyObject *Snapshot_wrapcopy_(Snapshot_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            Snapshot_object_ *obj = 
                PyObject_NEW(Snapshot_object_, Snapshot_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new Snapshot_type_(*o);
                }

                try
                {
                    Snapshot_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying Snapshot");
                return 0;
            }
        

    
}

PyObject *Snapshot_wrap_(Snapshot_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = Snapshot_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Snapshot_object_ *self = 
        PyObject_NEW(Snapshot_object_, &Snapshot_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        Snapshot_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Snapshot_type_ *Snapshot_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Snapshot_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Snapshot object");
        return 0;
    }

    Snapshot_type_ *object = static_cast<Snapshot_type_ *>(((Snapshot_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Snapshot object in unwrap");
        return 0;
    }

    return object;
}

void Snapshot_insert_(const std::pair< Snapshot_type_ *, PyObject *> &p)
{




    Snapshot_wrapped_.insert(p);


}

void Snapshot_erase_(Snapshot_type_ *p)
{




    Snapshot_wrapped_.erase(p);


}

PyObject * Snapshot_find_(Snapshot_type_ *p)
{




    PyObject *ret = 0;
    std::map< Snapshot_type_ *, PyObject *>::iterator where = Snapshot_wrapped_.find(p);
    if(where != Snapshot_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct Snapshot_dispatch_ Snapshot_realdispatcher_ =
{
    &Snapshot_typeobject_,
    Snapshot_wrap_,
    Snapshot_wrapcopy_,
    Snapshot_unwrap_,
    Snapshot_insert_,
    Snapshot_erase_,
    Snapshot_find_
};

  

struct Database_object_ {
    PyObject_HEAD
    Database_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct Database_wrapper_: public Database_type_ 
{

    static void __deleter__(Database_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    Database_wrapper_( PyObject *o, PyInterpreterState *i  ,const Database_type_ & a0  ): Database_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~Database_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    Snapshot_type_ get_trunk_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_trunk();
            
        }

        return
            
            
                (operator->())->get_trunk
            
                ();
    }

    static PyObject *get_trunk_method_(Database_object_ *self, PyObject *args)
    {
        

        Database_wrapper_ *o = (Database_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Database c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Snapshot_type_ r = o->get_trunk_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Snapshot_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Database::get_trunk");
        }

        return 0;
    }



    Snapshot_type_ get_version_call_(bool owned ,unsigned long a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->get_version( a0 );
            
        }

        return
            
            
                (operator->())->get_version
            
                ( a0 );
    }

    static PyObject *get_version_method_(Database_object_ *self, PyObject *args)
    {
         unsigned long a0; 

        Database_wrapper_ *o = (Database_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Database c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ul,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    Snapshot_type_ r = o->get_version_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Snapshot_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Database::get_version");
        }

        return 0;
    }



    File_type_ get_file_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->get_file();
            
        }

        return
            
            
                (operator->())->get_file
            
                ();
    }

    static PyObject *get_file_method_(Database_object_ *self, PyObject *args)
    {
        

        Database_wrapper_ *o = (Database_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Database c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    File_type_ r = o->get_file_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_File_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Database::get_file");
        }

        return 0;
    }



    void close_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->close();
            
        }

        return
            
            
                (operator->())->close
            
                ();
    }

    static PyObject *close_method_(Database_object_ *self, PyObject *args)
    {
        

        Database_wrapper_ *o = (Database_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Database c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Database::close");
        }

        return 0;
    }



    void flush_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->flush();
            
        }

        return
            
            
                (operator->())->flush
            
                ();
    }

    static PyObject *flush_method_(Database_object_ *self, PyObject *args)
    {
        

        Database_wrapper_ *o = (Database_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null Database c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->flush_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in Database::flush");
        }

        return 0;
    }





    static PyObject *_method_lock(Database_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(Database_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void Database_delete_(Database_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            Database_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (Database_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        Database_wrapper_::__deleter__((Database_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void Database_dealloc_(Database_object_* self)
{
    
    Database_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * Database_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    Database_object_ *self;

    if(!(self = (Database_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int Database_init_(Database_object_ *self, PyObject *args, PyObject *kwds)
{
    Database_delete_(self);

    

    {
         Database_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_Database_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new Database_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing Database");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            Database_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"Database invalid arguments");
    return -1;

}


static const char *Database_get_trunk_docstring_ = "";

static const char *Database_get_version_docstring_ = "";

static const char *Database_get_file_docstring_ = "";

static const char *Database_close_docstring_ = "";

static const char *Database_flush_docstring_ = "";




static PyMethodDef Database_methods_[] = {
    
        { "get_trunk",(PyCFunction)Database_wrapper_::get_trunk_method_,METH_VARARGS,(char *)Database_get_trunk_docstring_ },
    
        { "get_version",(PyCFunction)Database_wrapper_::get_version_method_,METH_VARARGS,(char *)Database_get_version_docstring_ },
    
        { "get_file",(PyCFunction)Database_wrapper_::get_file_method_,METH_VARARGS,(char *)Database_get_file_docstring_ },
    
        { "close",(PyCFunction)Database_wrapper_::close_method_,METH_VARARGS,(char *)Database_close_docstring_ },
    
        { "flush",(PyCFunction)Database_wrapper_::flush_method_,METH_VARARGS,(char *)Database_flush_docstring_ },
    
    
        { "py_lock",(PyCFunction)Database_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)Database_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *Database_docstring_ = "";

PyTypeObject Database_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pi_state_native.Database",/*tp_name*/
    sizeof(Database_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)Database_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)Database_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    Database_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)Database_init_,  /* tp_init */
    0,                         /* tp_alloc */
    Database_new_,             /* tp_new */
};

PyObject *Database_wrapcopy_(Database_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            Database_object_ *obj = 
                PyObject_NEW(Database_object_, Database_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new Database_type_(*o);
                }

                try
                {
                    Database_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying Database");
                return 0;
            }
        

    
}

PyObject *Database_wrap_(Database_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = Database_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    Database_object_ *self = 
        PyObject_NEW(Database_object_, &Database_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        Database_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

Database_type_ *Database_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &Database_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected Database object");
        return 0;
    }

    Database_type_ *object = static_cast<Database_type_ *>(((Database_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null Database object in unwrap");
        return 0;
    }

    return object;
}

void Database_insert_(const std::pair< Database_type_ *, PyObject *> &p)
{




    Database_wrapped_.insert(p);


}

void Database_erase_(Database_type_ *p)
{




    Database_wrapped_.erase(p);


}

PyObject * Database_find_(Database_type_ *p)
{




    PyObject *ret = 0;
    std::map< Database_type_ *, PyObject *>::iterator where = Database_wrapped_.find(p);
    if(where != Database_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct Database_dispatch_ Database_realdispatcher_ =
{
    &Database_typeobject_,
    Database_wrap_,
    Database_wrapcopy_,
    Database_unwrap_,
    Database_insert_,
    Database_erase_,
    Database_find_
};

       


static const char *module_method_open_database_docstring_ = "";

static PyObject *module_method_open_database(PyObject *self, PyObject *args)
{
     const char * a0;  bool a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_b,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    Database_type_ r = pi::state::open_database  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Database_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in open_database");
    }

    return 0;
}


static const char *module_method_open_file_docstring_ = "";

static PyObject *module_method_open_file(PyObject *self, PyObject *args)
{
     const char * a0;  bool a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_b,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    File_type_ r = pi::state::open_file  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_File_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in open_file");
    }

    return 0;
}


static const char *module_method_create_mapping_docstring_ = "";

static PyObject *module_method_create_mapping(PyObject *self, PyObject *args)
{
     bool a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_b,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    Mapping_type_ r = pi::state::create_mapping  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_Mapping_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_mapping");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "open_database",module_method_open_database,METH_VARARGS, (char *)module_method_open_database_docstring_ }, 
{ "open_file",module_method_open_file,METH_VARARGS, (char *)module_method_open_file_docstring_ }, 
{ "create_mapping",module_method_create_mapping,METH_VARARGS, (char *)module_method_create_mapping_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpi_state_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            Mapping_dispatcher_ = &Mapping_realdispatcher_;
        
    
        
            Manager_dispatcher_ = &Manager_realdispatcher_;
        
    
        
            File_dispatcher_ = &File_realdispatcher_;
        
    
        
            Node_dispatcher_ = &Node_realdispatcher_;
        
    
        
            Agent_dispatcher_ = &Agent_realdispatcher_;
        
    
        
            Snapshot_dispatcher_ = &Snapshot_realdispatcher_;
        
    
        
            Database_dispatcher_ = &Database_realdispatcher_;
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw_native");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_client_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_client_dispatch in module piw_native");
                return;
            }

            client_dispatcher_ = (client_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_term_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_term_dispatch in module piw_native");
                return;
            }

            term_dispatcher_ = (term_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
                Manager_typeobject_.tp_base = client_dispatcher_->type;
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&Mapping_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&Manager_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&File_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&Node_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&Agent_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&Snapshot_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&Database_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("pi_state_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(Mapping_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Mapping_dispatch",s);
            Py_INCREF(&Mapping_typeobject_);
            PyModule_AddObject(m,"Mapping",(PyObject *)&Mapping_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(Manager_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Manager_dispatch",s);
            Py_INCREF(&Manager_typeobject_);
            PyModule_AddObject(m,"Manager",(PyObject *)&Manager_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(File_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_File_dispatch",s);
            Py_INCREF(&File_typeobject_);
            PyModule_AddObject(m,"File",(PyObject *)&File_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(Node_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Node_dispatch",s);
            Py_INCREF(&Node_typeobject_);
            PyModule_AddObject(m,"Node",(PyObject *)&Node_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(Agent_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Agent_dispatch",s);
            Py_INCREF(&Agent_typeobject_);
            PyModule_AddObject(m,"Agent",(PyObject *)&Agent_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(Snapshot_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Snapshot_dispatch",s);
            Py_INCREF(&Snapshot_typeobject_);
            PyModule_AddObject(m,"Snapshot",(PyObject *)&Snapshot_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(Database_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_Database_dispatch",s);
            Py_INCREF(&Database_typeobject_);
            PyModule_AddObject(m,"Database",(PyObject *)&Database_typeobject_);
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

