#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <piagent/pia_scaffold.h>


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = """"
"Python interface to libpia c++ classes."
"======================================="
""
""
;


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pia::scaffold_mt_t scaffold_mt_type_; 


 
static std::map<scaffold_mt_type_ *, PyObject *> scaffold_mt_wrapped_;
 


struct scaffold_mt_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scaffold_mt_type_ *, bool owned);
    PyObject *(*wrapcopy)(scaffold_mt_type_ *);
    scaffold_mt_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scaffold_mt_type_ *, PyObject * > &);
    void (*erase)(scaffold_mt_type_ *);
    PyObject *(*find)(scaffold_mt_type_ *);
};

struct scaffold_mt_dispatch_ *scaffold_mt_dispatcher_;



typedef pia::scaffold_gui_t scaffold_gui_type_; 


 
static std::map<scaffold_gui_type_ *, PyObject *> scaffold_gui_wrapped_;
 


struct scaffold_gui_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(scaffold_gui_type_ *, bool owned);
    PyObject *(*wrapcopy)(scaffold_gui_type_ *);
    scaffold_gui_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< scaffold_gui_type_ *, PyObject * > &);
    void (*erase)(scaffold_gui_type_ *);
    PyObject *(*find)(scaffold_gui_type_ *);
};

struct scaffold_gui_dispatch_ *scaffold_gui_dispatcher_;



typedef pia::context_t context_type_; 


 
static std::map<context_type_ *, PyObject *> context_wrapped_;
 


struct context_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(context_type_ *, bool owned);
    PyObject *(*wrapcopy)(context_type_ *);
    context_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< context_type_ *, PyObject * > &);
    void (*erase)(context_type_ *);
    PyObject *(*find)(context_type_ *);
};

struct context_dispatch_ *context_dispatcher_;



typedef pic::notify_t notify_type_; 





struct notify_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(notify_type_ *, bool owned);
    PyObject *(*wrapcopy)(notify_type_ *);
    notify_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< notify_type_ *, PyObject * > &);
    void (*erase)(notify_type_ *);
    PyObject *(*find)(notify_type_ *);
};

struct notify_dispatch_ *notify_dispatcher_;



typedef pic::status_t status_type_; 





struct status_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(status_type_ *, bool owned);
    PyObject *(*wrapcopy)(status_type_ *);
    status_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< status_type_ *, PyObject * > &);
    void (*erase)(status_type_ *);
    PyObject *(*find)(status_type_ *);
};

struct status_dispatch_ *status_dispatcher_;



typedef pic::f_string_t f_string_type_; 





struct f_string_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_string_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_string_type_ *);
    f_string_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_string_type_ *, PyObject * > &);
    void (*erase)(f_string_type_ *);
    PyObject *(*find)(f_string_type_ *);
};

struct f_string_dispatch_ *f_string_dispatcher_;





int fpcvt_scaffold_mt(PyObject *o, void *a)
{
    scaffold_mt_type_ *object;
    if(o == Py_None)
    {
        *((scaffold_mt_type_ **)a) = 0;
        return 1;
    }
    object = scaffold_mt_dispatcher_->unwrap(o);
    *((scaffold_mt_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scaffold_mt_copy fpcvt_scaffold_mt

PyObject *tpcvt_scaffold_mt(void *a)
{
    scaffold_mt_type_ *o = *((scaffold_mt_type_ **)a);
    return scaffold_mt_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scaffold_mt_copy(void *a)
{
    scaffold_mt_type_ *o = (scaffold_mt_type_ *)a;
    return scaffold_mt_dispatcher_->wrapcopy(o);
}



int fpcvt_scaffold_gui(PyObject *o, void *a)
{
    scaffold_gui_type_ *object;
    if(o == Py_None)
    {
        *((scaffold_gui_type_ **)a) = 0;
        return 1;
    }
    object = scaffold_gui_dispatcher_->unwrap(o);
    *((scaffold_gui_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_scaffold_gui_copy fpcvt_scaffold_gui

PyObject *tpcvt_scaffold_gui(void *a)
{
    scaffold_gui_type_ *o = *((scaffold_gui_type_ **)a);
    return scaffold_gui_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_scaffold_gui_copy(void *a)
{
    scaffold_gui_type_ *o = (scaffold_gui_type_ *)a;
    return scaffold_gui_dispatcher_->wrapcopy(o);
}



int fpcvt_context(PyObject *o, void *a)
{
    context_type_ *object;
    if(o == Py_None)
    {
        *((context_type_ **)a) = 0;
        return 1;
    }
    object = context_dispatcher_->unwrap(o);
    *((context_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_context_copy fpcvt_context

PyObject *tpcvt_context(void *a)
{
    context_type_ *o = *((context_type_ **)a);
    return context_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_context_copy(void *a)
{
    context_type_ *o = (context_type_ *)a;
    return context_dispatcher_->wrapcopy(o);
}



int fpcvt_notify(PyObject *o, void *a)
{
    notify_type_ *object;
    if(o == Py_None)
    {
        *((notify_type_ **)a) = 0;
        return 1;
    }
    object = notify_dispatcher_->unwrap(o);
    *((notify_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_notify_copy fpcvt_notify

PyObject *tpcvt_notify(void *a)
{
    notify_type_ *o = *((notify_type_ **)a);
    return notify_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_notify_copy(void *a)
{
    notify_type_ *o = (notify_type_ *)a;
    return notify_dispatcher_->wrapcopy(o);
}



int fpcvt_status(PyObject *o, void *a)
{
    status_type_ *object;
    if(o == Py_None)
    {
        *((status_type_ **)a) = 0;
        return 1;
    }
    object = status_dispatcher_->unwrap(o);
    *((status_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_status_copy fpcvt_status

PyObject *tpcvt_status(void *a)
{
    status_type_ *o = *((status_type_ **)a);
    return status_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_status_copy(void *a)
{
    status_type_ *o = (status_type_ *)a;
    return status_dispatcher_->wrapcopy(o);
}



int fpcvt_f_string(PyObject *o, void *a)
{
    f_string_type_ *object;
    if(o == Py_None)
    {
        *((f_string_type_ **)a) = 0;
        return 1;
    }
    object = f_string_dispatcher_->unwrap(o);
    *((f_string_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_string_copy fpcvt_f_string

PyObject *tpcvt_f_string(void *a)
{
    f_string_type_ *o = *((f_string_type_ **)a);
    return f_string_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_string_copy(void *a)
{
    f_string_type_ *o = (f_string_type_ *)a;
    return f_string_dispatcher_->wrapcopy(o);
}



 

struct scaffold_mt_object_ {
    PyObject_HEAD
    scaffold_mt_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scaffold_mt_wrapper_: public scaffold_mt_type_ 
{

    static void __deleter__(scaffold_mt_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scaffold_mt_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,unsigned int a1  ,const f_string_type_ & a2  ,const f_string_type_ & a3  ,bool a4  ,bool a5  ): scaffold_mt_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scaffold_mt_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    bool global_lock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_mt_type_::global_lock();
                
            
            
        }

        return
            
                global_lock
            
            
                ();
    }

    static PyObject *global_lock_method_(scaffold_mt_object_ *self, PyObject *args)
    {
        

        scaffold_mt_wrapper_ *o = (scaffold_mt_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_mt c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->global_lock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_mt::global_lock");
        }

        return 0;
    }



    void global_unlock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_mt_type_::global_unlock();
                
            
            
        }

        return
            
                global_unlock
            
            
                ();
    }

    static PyObject *global_unlock_method_(scaffold_mt_object_ *self, PyObject *args)
    {
        

        scaffold_mt_wrapper_ *o = (scaffold_mt_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_mt c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->global_unlock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_mt::global_unlock");
        }

        return 0;
    }



    void wait_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_mt_type_::wait();
                
            
            
        }

        return
            
                wait
            
            
                ();
    }

    static PyObject *wait_method_(scaffold_mt_object_ *self, PyObject *args)
    {
        

        scaffold_mt_wrapper_ *o = (scaffold_mt_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_mt c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->wait_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_mt::wait");
        }

        return 0;
    }



    context_type_ context_call_(bool owned ,const status_type_ & a0 ,const f_string_type_ & a1 ,const char * a2 )
    {
        if(owned)
        {
            
                
                
                    return scaffold_mt_type_::context( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                context
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *context_method_(scaffold_mt_object_ *self, PyObject *args)
    {
         status_type_ * a0;  f_string_type_ * a1;  const char * a2; 

        scaffold_mt_wrapper_ *o = (scaffold_mt_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_mt c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_status_copy,&a0  ,fpcvt_f_string_copy,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    context_type_ r = o->context_call_ (self->owned ,*a0 ,*a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_context_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_mt::context");
        }

        return 0;
    }





    static PyObject *_method_lock(scaffold_mt_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scaffold_mt_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scaffold_mt_delete_(scaffold_mt_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scaffold_mt_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scaffold_mt_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scaffold_mt_wrapper_::__deleter__((scaffold_mt_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scaffold_mt_dealloc_(scaffold_mt_object_* self)
{
    
    scaffold_mt_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scaffold_mt_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scaffold_mt_object_ *self;

    if(!(self = (scaffold_mt_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scaffold_mt_init_(scaffold_mt_object_ *self, PyObject *args, PyObject *kwds)
{
    scaffold_mt_delete_(self);

    

    {
         const char * a0;  unsigned int a1;  f_string_type_ * a2;  f_string_type_ * a3;  bool a4;  bool a5; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1,fpcvt_f_string_copy,&a2,fpcvt_f_string_copy,&a3,fpcvt_b,&a4,fpcvt_b,&a5))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scaffold_mt_wrapper_((PyObject *)self,interp ,a0,a1,*a2,*a3,a4,a5);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scaffold_mt");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scaffold_mt_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scaffold_mt invalid arguments");
    return -1;

}


static const char *scaffold_mt_global_lock_docstring_ = "";

static const char *scaffold_mt_global_unlock_docstring_ = "";

static const char *scaffold_mt_wait_docstring_ = ""
"        self.wait() -> ."
"        run the scaffolds main loop"
"        "
""
;

static const char *scaffold_mt_context_docstring_ = ""
"        self.context(notify,tag) -> context"
"        "
"        Create a new context.  notify will be called when"
"        context has gone.  tag is used to tag log messages by agent"
"        "
""
;




static PyMethodDef scaffold_mt_methods_[] = {
    
        { "global_lock",(PyCFunction)scaffold_mt_wrapper_::global_lock_method_,METH_VARARGS,(char *)scaffold_mt_global_lock_docstring_ },
    
        { "global_unlock",(PyCFunction)scaffold_mt_wrapper_::global_unlock_method_,METH_VARARGS,(char *)scaffold_mt_global_unlock_docstring_ },
    
        { "wait",(PyCFunction)scaffold_mt_wrapper_::wait_method_,METH_VARARGS,(char *)scaffold_mt_wait_docstring_ },
    
        { "context",(PyCFunction)scaffold_mt_wrapper_::context_method_,METH_VARARGS,(char *)scaffold_mt_context_docstring_ },
    
    
        { "py_lock",(PyCFunction)scaffold_mt_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scaffold_mt_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scaffold_mt_docstring_ = ""
"    A server and client environment."
"    "
"    Starts up the various parts needed to run clients and servers in a"
"    process."
""
"    This one is completely multi threaded"
"    "
""
;

PyTypeObject scaffold_mt_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piagent_native.scaffold_mt",/*tp_name*/
    sizeof(scaffold_mt_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scaffold_mt_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scaffold_mt_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scaffold_mt_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scaffold_mt_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scaffold_mt_new_,             /* tp_new */
};

PyObject *scaffold_mt_wrapcopy_(scaffold_mt_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scaffold_mt_wrap_(scaffold_mt_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scaffold_mt_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scaffold_mt_object_ *self = 
        PyObject_NEW(scaffold_mt_object_, &scaffold_mt_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scaffold_mt_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scaffold_mt_type_ *scaffold_mt_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scaffold_mt_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scaffold_mt object");
        return 0;
    }

    scaffold_mt_type_ *object = static_cast<scaffold_mt_type_ *>(((scaffold_mt_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scaffold_mt object in unwrap");
        return 0;
    }

    return object;
}

void scaffold_mt_insert_(const std::pair< scaffold_mt_type_ *, PyObject *> &p)
{




    scaffold_mt_wrapped_.insert(p);


}

void scaffold_mt_erase_(scaffold_mt_type_ *p)
{




    scaffold_mt_wrapped_.erase(p);


}

PyObject * scaffold_mt_find_(scaffold_mt_type_ *p)
{




    PyObject *ret = 0;
    std::map< scaffold_mt_type_ *, PyObject *>::iterator where = scaffold_mt_wrapped_.find(p);
    if(where != scaffold_mt_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scaffold_mt_dispatch_ scaffold_mt_realdispatcher_ =
{
    &scaffold_mt_typeobject_,
    scaffold_mt_wrap_,
    scaffold_mt_wrapcopy_,
    scaffold_mt_unwrap_,
    scaffold_mt_insert_,
    scaffold_mt_erase_,
    scaffold_mt_find_
};

  

struct scaffold_gui_object_ {
    PyObject_HEAD
    scaffold_gui_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct scaffold_gui_wrapper_: public scaffold_gui_type_ 
{

    static void __deleter__(scaffold_gui_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    scaffold_gui_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,const notify_type_ & a1  ,const notify_type_ & a2  ,const f_string_type_ & a3  ,const f_string_type_ & a4  ,bool a5  ,bool a6  ): scaffold_gui_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ,a6 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~scaffold_gui_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void process_ctx_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::process_ctx();
                
            
            
        }

        return
            
                process_ctx
            
            
                ();
    }

    static PyObject *process_ctx_method_(scaffold_gui_object_ *self, PyObject *args)
    {
        

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->process_ctx_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::process_ctx");
        }

        return 0;
    }



    unsigned int cpu_usage_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::cpu_usage();
                
            
            
        }

        return
            
                cpu_usage
            
            
                ();
    }

    static PyObject *cpu_usage_method_(scaffold_gui_object_ *self, PyObject *args)
    {
        

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->cpu_usage_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::cpu_usage");
        }

        return 0;
    }



    bool global_lock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::global_lock();
                
            
            
        }

        return
            
                global_lock
            
            
                ();
    }

    static PyObject *global_lock_method_(scaffold_gui_object_ *self, PyObject *args)
    {
        

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->global_lock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::global_lock");
        }

        return 0;
    }



    void global_unlock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::global_unlock();
                
            
            
        }

        return
            
                global_unlock
            
            
                ();
    }

    static PyObject *global_unlock_method_(scaffold_gui_object_ *self, PyObject *args)
    {
        

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->global_unlock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::global_unlock");
        }

        return 0;
    }



    context_type_ bgcontext_call_(bool owned ,const status_type_ & a0 ,const f_string_type_ & a1 ,const char * a2 )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::bgcontext( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                bgcontext
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *bgcontext_method_(scaffold_gui_object_ *self, PyObject *args)
    {
         status_type_ * a0;  f_string_type_ * a1;  const char * a2; 

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_status_copy,&a0  ,fpcvt_f_string_copy,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    context_type_ r = o->bgcontext_call_ (self->owned ,*a0 ,*a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_context_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::bgcontext");
        }

        return 0;
    }



    context_type_ context_call_(bool owned ,const status_type_ & a0 ,const f_string_type_ & a1 ,const char * a2 )
    {
        if(owned)
        {
            
                
                
                    return scaffold_gui_type_::context( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                context
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *context_method_(scaffold_gui_object_ *self, PyObject *args)
    {
         status_type_ * a0;  f_string_type_ * a1;  const char * a2; 

        scaffold_gui_wrapper_ *o = (scaffold_gui_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null scaffold_gui c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_status_copy,&a0  ,fpcvt_f_string_copy,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    context_type_ r = o->context_call_ (self->owned ,*a0 ,*a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_context_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in scaffold_gui::context");
        }

        return 0;
    }





    static PyObject *_method_lock(scaffold_gui_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(scaffold_gui_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void scaffold_gui_delete_(scaffold_gui_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            scaffold_gui_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (scaffold_gui_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        scaffold_gui_wrapper_::__deleter__((scaffold_gui_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void scaffold_gui_dealloc_(scaffold_gui_object_* self)
{
    
    scaffold_gui_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * scaffold_gui_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    scaffold_gui_object_ *self;

    if(!(self = (scaffold_gui_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int scaffold_gui_init_(scaffold_gui_object_ *self, PyObject *args, PyObject *kwds)
{
    scaffold_gui_delete_(self);

    

    {
         const char * a0;  notify_type_ * a1;  notify_type_ * a2;  f_string_type_ * a3;  f_string_type_ * a4;  bool a5;  bool a6; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&O&" ,fpcvt_str,&a0,fpcvt_notify_copy,&a1,fpcvt_notify_copy,&a2,fpcvt_f_string_copy,&a3,fpcvt_f_string_copy,&a4,fpcvt_b,&a5,fpcvt_b,&a6))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new scaffold_gui_wrapper_((PyObject *)self,interp ,a0,*a1,*a2,*a3,*a4,a5,a6);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing scaffold_gui");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            scaffold_gui_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"scaffold_gui invalid arguments");
    return -1;

}


static const char *scaffold_gui_process_ctx_docstring_ = "";

static const char *scaffold_gui_cpu_usage_docstring_ = "";

static const char *scaffold_gui_global_lock_docstring_ = "";

static const char *scaffold_gui_global_unlock_docstring_ = "";

static const char *scaffold_gui_bgcontext_docstring_ = ""
"        self.context(notify,tag) -> context"
"        "
"        Create a new background context.  notify will be called when"
"        context has gone.  tag is used to tag log messages by agent"
"        "
""
;

static const char *scaffold_gui_context_docstring_ = ""
"        self.context(notify,tag) -> context"
"        "
"        Create a new foreground context.  notify will be called when"
"        context has gone.  tag is used to tag log messages by agent"
"        "
""
;




static PyMethodDef scaffold_gui_methods_[] = {
    
        { "process_ctx",(PyCFunction)scaffold_gui_wrapper_::process_ctx_method_,METH_VARARGS,(char *)scaffold_gui_process_ctx_docstring_ },
    
        { "cpu_usage",(PyCFunction)scaffold_gui_wrapper_::cpu_usage_method_,METH_VARARGS,(char *)scaffold_gui_cpu_usage_docstring_ },
    
        { "global_lock",(PyCFunction)scaffold_gui_wrapper_::global_lock_method_,METH_VARARGS,(char *)scaffold_gui_global_lock_docstring_ },
    
        { "global_unlock",(PyCFunction)scaffold_gui_wrapper_::global_unlock_method_,METH_VARARGS,(char *)scaffold_gui_global_unlock_docstring_ },
    
        { "bgcontext",(PyCFunction)scaffold_gui_wrapper_::bgcontext_method_,METH_VARARGS,(char *)scaffold_gui_bgcontext_docstring_ },
    
        { "context",(PyCFunction)scaffold_gui_wrapper_::context_method_,METH_VARARGS,(char *)scaffold_gui_context_docstring_ },
    
    
        { "py_lock",(PyCFunction)scaffold_gui_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)scaffold_gui_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *scaffold_gui_docstring_ = ""
"    A server and client environment."
"    "
"    Starts up the various parts needed to run clients and servers in a"
"    process."
""
"    This one maintains internal threads for main and fast loops and background contexts,"
"    but expects the user to drive the foreground context thread when convenient."
"    Background contexts are driven by a separate thread."
"    "
""
;

PyTypeObject scaffold_gui_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piagent_native.scaffold_gui",/*tp_name*/
    sizeof(scaffold_gui_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)scaffold_gui_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)scaffold_gui_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    scaffold_gui_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)scaffold_gui_init_,  /* tp_init */
    0,                         /* tp_alloc */
    scaffold_gui_new_,             /* tp_new */
};

PyObject *scaffold_gui_wrapcopy_(scaffold_gui_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *scaffold_gui_wrap_(scaffold_gui_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = scaffold_gui_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    scaffold_gui_object_ *self = 
        PyObject_NEW(scaffold_gui_object_, &scaffold_gui_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        scaffold_gui_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

scaffold_gui_type_ *scaffold_gui_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &scaffold_gui_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected scaffold_gui object");
        return 0;
    }

    scaffold_gui_type_ *object = static_cast<scaffold_gui_type_ *>(((scaffold_gui_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null scaffold_gui object in unwrap");
        return 0;
    }

    return object;
}

void scaffold_gui_insert_(const std::pair< scaffold_gui_type_ *, PyObject *> &p)
{




    scaffold_gui_wrapped_.insert(p);


}

void scaffold_gui_erase_(scaffold_gui_type_ *p)
{




    scaffold_gui_wrapped_.erase(p);


}

PyObject * scaffold_gui_find_(scaffold_gui_type_ *p)
{




    PyObject *ret = 0;
    std::map< scaffold_gui_type_ *, PyObject *>::iterator where = scaffold_gui_wrapped_.find(p);
    if(where != scaffold_gui_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct scaffold_gui_dispatch_ scaffold_gui_realdispatcher_ =
{
    &scaffold_gui_typeobject_,
    scaffold_gui_wrap_,
    scaffold_gui_wrapcopy_,
    scaffold_gui_unwrap_,
    scaffold_gui_insert_,
    scaffold_gui_erase_,
    scaffold_gui_find_
};

  

struct context_object_ {
    PyObject_HEAD
    context_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct context_wrapper_: public context_type_ 
{

    static void __deleter__(context_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    context_wrapper_( PyObject *o, PyInterpreterState *i  ): context_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    context_wrapper_( PyObject *o, PyInterpreterState *i  ,const context_type_ & a0  ): context_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~context_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void kill_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return context_type_::kill();
                
            
            
        }

        return
            
                kill
            
            
                ();
    }

    static PyObject *kill_method_(context_object_ *self, PyObject *args)
    {
        

        context_wrapper_ *o = (context_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null context c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->kill_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::kill");
        }

        return 0;
    }



    void release_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return context_type_::release();
                
            
            
        }

        return
            
                release
            
            
                ();
    }

    static PyObject *release_method_(context_object_ *self, PyObject *args)
    {
        

        context_wrapper_ *o = (context_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null context c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->release_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::release");
        }

        return 0;
    }



    void lock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return context_type_::lock();
                
            
            
        }

        return
            
                lock
            
            
                ();
    }

    static PyObject *lock_method_(context_object_ *self, PyObject *args)
    {
        

        context_wrapper_ *o = (context_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null context c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->lock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::lock");
        }

        return 0;
    }



    void unlock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return context_type_::unlock();
                
            
            
        }

        return
            
                unlock
            
            
                ();
    }

    static PyObject *unlock_method_(context_object_ *self, PyObject *args)
    {
        

        context_wrapper_ *o = (context_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null context c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unlock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::unlock");
        }

        return 0;
    }



    void trigger_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return context_type_::trigger();
                
            
            
        }

        return
            
                trigger
            
            
                ();
    }

    static PyObject *trigger_method_(context_object_ *self, PyObject *args)
    {
        

        context_wrapper_ *o = (context_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null context c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->trigger_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::trigger");
        }

        return 0;
    }





    static PyObject *_method_inner_getenv(context_object_ *_self_, PyObject *args)
    {
        context_type_ *self = _self_->object;
        
        return PyCObject_FromVoidPtr(self->entity(),0);
    
    }

    static PyObject *_method_getenv(context_object_ *self, PyObject *args)
    {
        try
        {
            PyObject *rv = _method_inner_getenv(self,args);
            return rv;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in context::getenv");
        }

        return 0;
    }



    static PyObject *_method_lock(context_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(context_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void context_delete_(context_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            context_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (context_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        context_wrapper_::__deleter__((context_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void context_dealloc_(context_object_* self)
{
    
    context_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * context_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    context_object_ *self;

    if(!(self = (context_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int context_init_(context_object_ *self, PyObject *args, PyObject *kwds)
{
    context_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new context_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing context");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            context_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         context_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_context_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new context_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing context");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            context_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"context invalid arguments");
    return -1;

}


static const char *context_kill_docstring_ = "";

static const char *context_release_docstring_ = "";

static const char *context_lock_docstring_ = "";

static const char *context_unlock_docstring_ = "";

static const char *context_trigger_docstring_ = "";



static const char *context_getenv_docstring_ = "";


static PyMethodDef context_methods_[] = {
    
        { "kill",(PyCFunction)context_wrapper_::kill_method_,METH_VARARGS,(char *)context_kill_docstring_ },
    
        { "release",(PyCFunction)context_wrapper_::release_method_,METH_VARARGS,(char *)context_release_docstring_ },
    
        { "lock",(PyCFunction)context_wrapper_::lock_method_,METH_VARARGS,(char *)context_lock_docstring_ },
    
        { "unlock",(PyCFunction)context_wrapper_::unlock_method_,METH_VARARGS,(char *)context_unlock_docstring_ },
    
        { "trigger",(PyCFunction)context_wrapper_::trigger_method_,METH_VARARGS,(char *)context_trigger_docstring_ },
    
    
        { "getenv",(PyCFunction)context_wrapper_::_method_getenv,METH_VARARGS,(char *)context_getenv_docstring_ },
    
        { "py_lock",(PyCFunction)context_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)context_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *context_docstring_ = ""
"    An Agent context"
"    "
"    Provides a context in which to run Agents."
"    "
""
;

PyTypeObject context_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "piagent_native.context",/*tp_name*/
    sizeof(context_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)context_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)context_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    context_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)context_init_,  /* tp_init */
    0,                         /* tp_alloc */
    context_new_,             /* tp_new */
};

PyObject *context_wrapcopy_(context_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            context_object_ *obj = 
                PyObject_NEW(context_object_, context_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new context_type_(*o);
                }

                try
                {
                    context_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying context");
                return 0;
            }
        

    
}

PyObject *context_wrap_(context_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = context_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    context_object_ *self = 
        PyObject_NEW(context_object_, &context_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        context_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

context_type_ *context_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &context_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected context object");
        return 0;
    }

    context_type_ *object = static_cast<context_type_ *>(((context_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null context object in unwrap");
        return 0;
    }

    return object;
}

void context_insert_(const std::pair< context_type_ *, PyObject *> &p)
{




    context_wrapped_.insert(p);


}

void context_erase_(context_type_ *p)
{




    context_wrapped_.erase(p);


}

PyObject * context_find_(context_type_ *p)
{




    PyObject *ret = 0;
    std::map< context_type_ *, PyObject *>::iterator where = context_wrapped_.find(p);
    if(where != context_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct context_dispatch_ context_realdispatcher_ =
{
    &context_typeobject_,
    context_wrap_,
    context_wrapcopy_,
    context_unwrap_,
    context_insert_,
    context_erase_,
    context_find_
};

       





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpiagent_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            scaffold_mt_dispatcher_ = &scaffold_mt_realdispatcher_;
        
    
        
            scaffold_gui_dispatcher_ = &scaffold_gui_realdispatcher_;
        
    
        
            context_dispatcher_ = &context_realdispatcher_;
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("picross_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_notify_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_notify_dispatch in module picross_native");
                return;
            }

            notify_dispatcher_ = (notify_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_f_string_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_f_string_dispatch in module picross_native");
                return;
            }

            f_string_dispatcher_ = (f_string_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_status_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_status_dispatch in module picross_native");
                return;
            }

            status_dispatcher_ = (status_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&scaffold_mt_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&scaffold_gui_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&context_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("piagent_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(scaffold_mt_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scaffold_mt_dispatch",s);
            Py_INCREF(&scaffold_mt_typeobject_);
            PyModule_AddObject(m,"scaffold_mt",(PyObject *)&scaffold_mt_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(scaffold_gui_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_scaffold_gui_dispatch",s);
            Py_INCREF(&scaffold_gui_typeobject_);
            PyModule_AddObject(m,"scaffold_gui",(PyObject *)&scaffold_gui_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(context_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_context_dispatch",s);
            Py_INCREF(&context_typeobject_);
            PyModule_AddObject(m,"context",(PyObject *)&context_typeobject_);
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

