#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "pis_python.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pisession::interpreter_t interpreter_type_; 


 
static std::map<interpreter_type_ *, PyObject *> interpreter_wrapped_;
 


struct interpreter_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(interpreter_type_ *, bool owned);
    PyObject *(*wrapcopy)(interpreter_type_ *);
    interpreter_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< interpreter_type_ *, PyObject * > &);
    void (*erase)(interpreter_type_ *);
    PyObject *(*find)(interpreter_type_ *);
};

struct interpreter_dispatch_ *interpreter_dispatcher_;



typedef pisession::agent_t agent_type_; 


 
static std::map<agent_type_ *, PyObject *> agent_wrapped_;
 


struct agent_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(agent_type_ *, bool owned);
    PyObject *(*wrapcopy)(agent_type_ *);
    agent_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< agent_type_ *, PyObject * > &);
    void (*erase)(agent_type_ *);
    PyObject *(*find)(agent_type_ *);
};

struct agent_dispatch_ *agent_dispatcher_;



typedef pia::context_t context_type_; 





struct context_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(context_type_ *, bool owned);
    PyObject *(*wrapcopy)(context_type_ *);
    context_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< context_type_ *, PyObject * > &);
    void (*erase)(context_type_ *);
    PyObject *(*find)(context_type_ *);
};

struct context_dispatch_ *context_dispatcher_;





int fpcvt_interpreter(PyObject *o, void *a)
{
    interpreter_type_ *object;
    if(o == Py_None)
    {
        *((interpreter_type_ **)a) = 0;
        return 1;
    }
    object = interpreter_dispatcher_->unwrap(o);
    *((interpreter_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_interpreter_copy fpcvt_interpreter

PyObject *tpcvt_interpreter(void *a)
{
    interpreter_type_ *o = *((interpreter_type_ **)a);
    return interpreter_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_interpreter_copy(void *a)
{
    interpreter_type_ *o = (interpreter_type_ *)a;
    return interpreter_dispatcher_->wrapcopy(o);
}



int fpcvt_agent(PyObject *o, void *a)
{
    agent_type_ *object;
    if(o == Py_None)
    {
        *((agent_type_ **)a) = 0;
        return 1;
    }
    object = agent_dispatcher_->unwrap(o);
    *((agent_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_agent_copy fpcvt_agent

PyObject *tpcvt_agent(void *a)
{
    agent_type_ *o = *((agent_type_ **)a);
    return agent_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_agent_copy(void *a)
{
    agent_type_ *o = (agent_type_ *)a;
    return agent_dispatcher_->wrapcopy(o);
}



int fpcvt_context(PyObject *o, void *a)
{
    context_type_ *object;
    if(o == Py_None)
    {
        *((context_type_ **)a) = 0;
        return 1;
    }
    object = context_dispatcher_->unwrap(o);
    *((context_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_context_copy fpcvt_context

PyObject *tpcvt_context(void *a)
{
    context_type_ *o = *((context_type_ **)a);
    return context_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_context_copy(void *a)
{
    context_type_ *o = (context_type_ *)a;
    return context_dispatcher_->wrapcopy(o);
}



 

struct interpreter_object_ {
    PyObject_HEAD
    interpreter_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct interpreter_wrapper_: public interpreter_type_ 
{

    static void __deleter__(interpreter_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    interpreter_wrapper_( PyObject *o, PyInterpreterState *i  ): interpreter_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    interpreter_wrapper_( PyObject *o, PyInterpreterState *i  ,const interpreter_type_ & a0  ): interpreter_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~interpreter_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void release_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return interpreter_type_::release();
                
            
            
        }

        return
            
                release
            
            
                ();
    }

    static PyObject *release_method_(interpreter_object_ *self, PyObject *args)
    {
        

        interpreter_wrapper_ *o = (interpreter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null interpreter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->release_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in interpreter::release");
        }

        return 0;
    }



    agent_type_ create_agent_call_(bool owned ,const context_type_ & a0 ,const char * a1 ,const char * a2 )
    {
        if(owned)
        {
            
            
                return (operator->())->create_agent( a0  ,a1  ,a2 );
            
        }

        return
            
            
                (operator->())->create_agent
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *create_agent_method_(interpreter_object_ *self, PyObject *args)
    {
         context_type_ * a0;  const char * a1;  const char * a2; 

        interpreter_wrapper_ *o = (interpreter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null interpreter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_context_copy,&a0  ,fpcvt_str,&a1  ,fpcvt_str,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    agent_type_ r = o->create_agent_call_ (self->owned ,*a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_agent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in interpreter::create_agent");
        }

        return 0;
    }





    static PyObject *_method_lock(interpreter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(interpreter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void interpreter_delete_(interpreter_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            interpreter_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (interpreter_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        interpreter_wrapper_::__deleter__((interpreter_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void interpreter_dealloc_(interpreter_object_* self)
{
    
    interpreter_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * interpreter_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    interpreter_object_ *self;

    if(!(self = (interpreter_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int interpreter_init_(interpreter_object_ *self, PyObject *args, PyObject *kwds)
{
    interpreter_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new interpreter_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing interpreter");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            interpreter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         interpreter_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_interpreter_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new interpreter_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing interpreter");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            interpreter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"interpreter invalid arguments");
    return -1;

}


static const char *interpreter_release_docstring_ = "";

static const char *interpreter_create_agent_docstring_ = "";




static PyMethodDef interpreter_methods_[] = {
    
        { "release",(PyCFunction)interpreter_wrapper_::release_method_,METH_VARARGS,(char *)interpreter_release_docstring_ },
    
        { "create_agent",(PyCFunction)interpreter_wrapper_::create_agent_method_,METH_VARARGS,(char *)interpreter_create_agent_docstring_ },
    
    
        { "py_lock",(PyCFunction)interpreter_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)interpreter_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *interpreter_docstring_ = "";

PyTypeObject interpreter_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pisession_native.interpreter",/*tp_name*/
    sizeof(interpreter_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)interpreter_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)interpreter_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    interpreter_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)interpreter_init_,  /* tp_init */
    0,                         /* tp_alloc */
    interpreter_new_,             /* tp_new */
};

PyObject *interpreter_wrapcopy_(interpreter_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            interpreter_object_ *obj = 
                PyObject_NEW(interpreter_object_, interpreter_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new interpreter_type_(*o);
                }

                try
                {
                    interpreter_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying interpreter");
                return 0;
            }
        

    
}

PyObject *interpreter_wrap_(interpreter_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = interpreter_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    interpreter_object_ *self = 
        PyObject_NEW(interpreter_object_, &interpreter_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        interpreter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

interpreter_type_ *interpreter_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &interpreter_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected interpreter object");
        return 0;
    }

    interpreter_type_ *object = static_cast<interpreter_type_ *>(((interpreter_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null interpreter object in unwrap");
        return 0;
    }

    return object;
}

void interpreter_insert_(const std::pair< interpreter_type_ *, PyObject *> &p)
{




    interpreter_wrapped_.insert(p);


}

void interpreter_erase_(interpreter_type_ *p)
{




    interpreter_wrapped_.erase(p);


}

PyObject * interpreter_find_(interpreter_type_ *p)
{




    PyObject *ret = 0;
    std::map< interpreter_type_ *, PyObject *>::iterator where = interpreter_wrapped_.find(p);
    if(where != interpreter_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct interpreter_dispatch_ interpreter_realdispatcher_ =
{
    &interpreter_typeobject_,
    interpreter_wrap_,
    interpreter_wrapcopy_,
    interpreter_unwrap_,
    interpreter_insert_,
    interpreter_erase_,
    interpreter_find_
};

  

struct agent_object_ {
    PyObject_HEAD
    agent_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct agent_wrapper_: public agent_type_ 
{

    static void __deleter__(agent_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    agent_wrapper_( PyObject *o, PyInterpreterState *i  ): agent_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    agent_wrapper_( PyObject *o, PyInterpreterState *i  ,const agent_type_ & a0  ): agent_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~agent_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void release_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return agent_type_::release();
                
            
            
        }

        return
            
                release
            
            
                ();
    }

    static PyObject *release_method_(agent_object_ *self, PyObject *args)
    {
        

        agent_wrapper_ *o = (agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->release_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in agent::release");
        }

        return 0;
    }



    context_type_ env_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->env();
            
        }

        return
            
            
                (operator->())->env
            
                ();
    }

    static PyObject *env_method_(agent_object_ *self, PyObject *args)
    {
        

        agent_wrapper_ *o = (agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    context_type_ r = o->env_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_context_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in agent::env");
        }

        return 0;
    }



    std::string unload_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->unload();
            
        }

        return
            
            
                (operator->())->unload
            
                ();
    }

    static PyObject *unload_method_(agent_object_ *self, PyObject *args)
    {
        

        agent_wrapper_ *o = (agent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null agent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->unload_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in agent::unload");
        }

        return 0;
    }





    static PyObject *_method_lock(agent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(agent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void agent_delete_(agent_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            agent_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (agent_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        agent_wrapper_::__deleter__((agent_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void agent_dealloc_(agent_object_* self)
{
    
    agent_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * agent_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    agent_object_ *self;

    if(!(self = (agent_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int agent_init_(agent_object_ *self, PyObject *args, PyObject *kwds)
{
    agent_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new agent_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing agent");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         agent_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_agent_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new agent_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing agent");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"agent invalid arguments");
    return -1;

}


static const char *agent_release_docstring_ = "";

static const char *agent_env_docstring_ = "";

static const char *agent_unload_docstring_ = "";




static PyMethodDef agent_methods_[] = {
    
        { "release",(PyCFunction)agent_wrapper_::release_method_,METH_VARARGS,(char *)agent_release_docstring_ },
    
        { "env",(PyCFunction)agent_wrapper_::env_method_,METH_VARARGS,(char *)agent_env_docstring_ },
    
        { "unload",(PyCFunction)agent_wrapper_::unload_method_,METH_VARARGS,(char *)agent_unload_docstring_ },
    
    
        { "py_lock",(PyCFunction)agent_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)agent_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *agent_docstring_ = "";

PyTypeObject agent_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pisession_native.agent",/*tp_name*/
    sizeof(agent_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)agent_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)agent_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    agent_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)agent_init_,  /* tp_init */
    0,                         /* tp_alloc */
    agent_new_,             /* tp_new */
};

PyObject *agent_wrapcopy_(agent_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            agent_object_ *obj = 
                PyObject_NEW(agent_object_, agent_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new agent_type_(*o);
                }

                try
                {
                    agent_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying agent");
                return 0;
            }
        

    
}

PyObject *agent_wrap_(agent_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = agent_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    agent_object_ *self = 
        PyObject_NEW(agent_object_, &agent_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        agent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

agent_type_ *agent_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &agent_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected agent object");
        return 0;
    }

    agent_type_ *object = static_cast<agent_type_ *>(((agent_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null agent object in unwrap");
        return 0;
    }

    return object;
}

void agent_insert_(const std::pair< agent_type_ *, PyObject *> &p)
{




    agent_wrapped_.insert(p);


}

void agent_erase_(agent_type_ *p)
{




    agent_wrapped_.erase(p);


}

PyObject * agent_find_(agent_type_ *p)
{




    PyObject *ret = 0;
    std::map< agent_type_ *, PyObject *>::iterator where = agent_wrapped_.find(p);
    if(where != agent_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct agent_dispatch_ agent_realdispatcher_ =
{
    &agent_typeobject_,
    agent_wrap_,
    agent_wrapcopy_,
    agent_unwrap_,
    agent_insert_,
    agent_erase_,
    agent_find_
};

   


static const char *module_method_create_interpreter_docstring_ = "";

static PyObject *module_method_create_interpreter(PyObject *self, PyObject *args)
{
     const char * a0;  const char * a1;  const char * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_str,&a0,fpcvt_str,&a1,fpcvt_str,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    interpreter_type_ r = pisession::create_interpreter  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_interpreter_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_interpreter");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "create_interpreter",module_method_create_interpreter,METH_VARARGS, (char *)module_method_create_interpreter_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpisession_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            interpreter_dispatcher_ = &interpreter_realdispatcher_;
        
    
        
            agent_dispatcher_ = &agent_realdispatcher_;
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piagent_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_context_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_context_dispatch in module piagent_native");
                return;
            }

            context_dispatcher_ = (context_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
    

    
        
            if(PyType_Ready(&interpreter_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&agent_typeobject_)<0)
                return;
        
    
        
    

    if(!(m=Py_InitModule("pisession_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(interpreter_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_interpreter_dispatch",s);
            Py_INCREF(&interpreter_typeobject_);
            PyModule_AddObject(m,"interpreter",(PyObject *)&interpreter_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(agent_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_agent_dispatch",s);
            Py_INCREF(&agent_typeobject_);
            PyModule_AddObject(m,"agent",(PyObject *)&agent_typeobject_);
        
    
        
    

    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

