#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "arranger_view.h"
#include "arranger_model.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef arranger::view_t view_type_; 


 
static std::map<view_type_ *, PyObject *> view_wrapped_;
 


struct view_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(view_type_ *, bool owned);
    PyObject *(*wrapcopy)(view_type_ *);
    view_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< view_type_ *, PyObject * > &);
    void (*erase)(view_type_ *);
    PyObject *(*find)(view_type_ *);
};

struct view_dispatch_ *view_dispatcher_;



typedef arranger::model_t model_type_; 


 
static std::map<model_type_ *, PyObject *> model_wrapped_;
 


struct model_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(model_type_ *, bool owned);
    PyObject *(*wrapcopy)(model_type_ *);
    model_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< model_type_ *, PyObject * > &);
    void (*erase)(model_type_ *);
    PyObject *(*find)(model_type_ *);
};

struct model_dispatch_ *model_dispatcher_;



typedef bct_clocksink_t clockref_type_; 





struct clockref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockref_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockref_type_ *);
    clockref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockref_type_ *, PyObject * > &);
    void (*erase)(clockref_type_ *);
    PyObject *(*find)(clockref_type_ *);
};

struct clockref_dispatch_ *clockref_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::data_nb_t data_nb_type_; 





struct data_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_nb_type_ *);
    data_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_nb_type_ *, PyObject * > &);
    void (*erase)(data_nb_type_ *);
    PyObject *(*find)(data_nb_type_ *);
};

struct data_nb_dispatch_ *data_nb_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_view(PyObject *o, void *a)
{
    view_type_ *object;
    if(o == Py_None)
    {
        *((view_type_ **)a) = 0;
        return 1;
    }
    object = view_dispatcher_->unwrap(o);
    *((view_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_view_copy fpcvt_view

PyObject *tpcvt_view(void *a)
{
    view_type_ *o = *((view_type_ **)a);
    return view_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_view_copy(void *a)
{
    view_type_ *o = (view_type_ *)a;
    return view_dispatcher_->wrapcopy(o);
}



int fpcvt_model(PyObject *o, void *a)
{
    model_type_ *object;
    if(o == Py_None)
    {
        *((model_type_ **)a) = 0;
        return 1;
    }
    object = model_dispatcher_->unwrap(o);
    *((model_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_model_copy fpcvt_model

PyObject *tpcvt_model(void *a)
{
    model_type_ *o = *((model_type_ **)a);
    return model_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_model_copy(void *a)
{
    model_type_ *o = (model_type_ *)a;
    return model_dispatcher_->wrapcopy(o);
}



int fpcvt_clockref(PyObject *o, void *a)
{
    clockref_type_ *object;
    if(o == Py_None)
    {
        *((clockref_type_ **)a) = 0;
        return 1;
    }
    object = clockref_dispatcher_->unwrap(o);
    *((clockref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockref_copy fpcvt_clockref

PyObject *tpcvt_clockref(void *a)
{
    clockref_type_ *o = *((clockref_type_ **)a);
    return clockref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockref_copy(void *a)
{
    clockref_type_ *o = (clockref_type_ *)a;
    return clockref_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_data_nb(PyObject *o, void *a)
{
    data_nb_type_ *object;
    if(o == Py_None)
    {
        *((data_nb_type_ **)a) = 0;
        return 1;
    }
    object = data_nb_dispatcher_->unwrap(o);
    *((data_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_nb_copy fpcvt_data_nb

PyObject *tpcvt_data_nb(void *a)
{
    data_nb_type_ *o = *((data_nb_type_ **)a);
    return data_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_nb_copy(void *a)
{
    data_nb_type_ *o = (data_nb_type_ *)a;
    return data_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct view_object_ {
    PyObject_HEAD
    view_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct view_wrapper_: public view_type_ 
{

    static void __deleter__(view_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    view_wrapper_( PyObject *o, PyInterpreterState *i  ,model_type_ * a0  ,const cookie_type_ & a1  ): view_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~view_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return view_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(view_object_ *self, PyObject *args)
    {
        

        view_wrapper_ *o = (view_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null view c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in view::cookie");
        }

        return 0;
    }



    change_nb_type_ control_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return view_type_::control();
                
            
            
        }

        return
            
                control
            
            
                ();
    }

    static PyObject *control_method_(view_object_ *self, PyObject *args)
    {
        

        view_wrapper_ *o = (view_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null view c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->control_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in view::control");
        }

        return 0;
    }



    void clear_events_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return view_type_::clear_events();
                
            
            
        }

        return
            
                clear_events
            
            
                ();
    }

    static PyObject *clear_events_method_(view_object_ *self, PyObject *args)
    {
        

        view_wrapper_ *o = (view_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null view c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_events_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in view::clear_events");
        }

        return 0;
    }



    change_nb_type_ set_doubletap_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return view_type_::set_doubletap();
                
            
            
        }

        return
            
                set_doubletap
            
            
                ();
    }

    static PyObject *set_doubletap_method_(view_object_ *self, PyObject *args)
    {
        

        view_wrapper_ *o = (view_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null view c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_doubletap_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in view::set_doubletap");
        }

        return 0;
    }



    void doubletap_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return view_type_::doubletap_set( a0 );
                
            
            
        }

        return
            
                doubletap_set
            
            
                ( a0 );
    }

    static PyObject *doubletap_set_method_(view_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        view_wrapper_ *o = (view_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null view c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->doubletap_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in view::doubletap_set");
        }

        return 0;
    }





    static PyObject *_method_lock(view_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(view_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void view_delete_(view_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            view_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (view_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        view_wrapper_::__deleter__((view_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void view_dealloc_(view_object_* self)
{
    
    view_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * view_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    view_object_ *self;

    if(!(self = (view_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int view_init_(view_object_ *self, PyObject *args, PyObject *kwds)
{
    view_delete_(self);

    

    {
         model_type_ * a0;  cookie_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_model,&a0,fpcvt_cookie_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new view_wrapper_((PyObject *)self,interp ,a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing view");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            view_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"view invalid arguments");
    return -1;

}


static const char *view_cookie_docstring_ = "";

static const char *view_control_docstring_ = "";

static const char *view_clear_events_docstring_ = "";

static const char *view_set_doubletap_docstring_ = "";

static const char *view_doubletap_set_docstring_ = "";




static PyMethodDef view_methods_[] = {
    
        { "cookie",(PyCFunction)view_wrapper_::cookie_method_,METH_VARARGS,(char *)view_cookie_docstring_ },
    
        { "control",(PyCFunction)view_wrapper_::control_method_,METH_VARARGS,(char *)view_control_docstring_ },
    
        { "clear_events",(PyCFunction)view_wrapper_::clear_events_method_,METH_VARARGS,(char *)view_clear_events_docstring_ },
    
        { "set_doubletap",(PyCFunction)view_wrapper_::set_doubletap_method_,METH_VARARGS,(char *)view_set_doubletap_docstring_ },
    
        { "doubletap_set",(PyCFunction)view_wrapper_::doubletap_set_method_,METH_VARARGS,(char *)view_doubletap_set_docstring_ },
    
    
        { "py_lock",(PyCFunction)view_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)view_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *view_docstring_ = "";

PyTypeObject view_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "arranger_native.view",/*tp_name*/
    sizeof(view_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)view_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)view_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    view_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)view_init_,  /* tp_init */
    0,                         /* tp_alloc */
    view_new_,             /* tp_new */
};

PyObject *view_wrapcopy_(view_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *view_wrap_(view_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = view_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    view_object_ *self = 
        PyObject_NEW(view_object_, &view_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        view_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

view_type_ *view_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &view_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected view object");
        return 0;
    }

    view_type_ *object = static_cast<view_type_ *>(((view_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null view object in unwrap");
        return 0;
    }

    return object;
}

void view_insert_(const std::pair< view_type_ *, PyObject *> &p)
{




    view_wrapped_.insert(p);


}

void view_erase_(view_type_ *p)
{




    view_wrapped_.erase(p);


}

PyObject * view_find_(view_type_ *p)
{




    PyObject *ret = 0;
    std::map< view_type_ *, PyObject *>::iterator where = view_wrapped_.find(p);
    if(where != view_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct view_dispatch_ view_realdispatcher_ =
{
    &view_typeobject_,
    view_wrap_,
    view_wrapcopy_,
    view_unwrap_,
    view_insert_,
    view_erase_,
    view_find_
};

  

struct model_object_ {
    PyObject_HEAD
    model_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct model_wrapper_: public model_type_ 
{

    static void __deleter__(model_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    model_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ): model_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~model_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(model_object_ *self, visitproc visit, void *arg)
    {
        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(model_object_ *self)
    {
        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::cookie");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::get_clock");
        }

        return 0;
    }



    void set_target_call_(bool owned ,unsigned int a0 ,const change_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_target( a0  ,a1 );
                
            
            
        }

        return
            
                set_target
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_target_method_(model_object_ *self, PyObject *args)
    {
         unsigned int a0;  change_type_ * a1; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_change_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_target_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_target");
        }

        return 0;
    }



    void clear_target_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::clear_target( a0 );
                
            
            
        }

        return
            
                clear_target
            
            
                ( a0 );
    }

    static PyObject *clear_target_method_(model_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_target_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::clear_target");
        }

        return 0;
    }



    change_nb_type_ set_event_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_event();
                
            
            
        }

        return
            
                set_event
            
            
                ();
    }

    static PyObject *set_event_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_event_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_event");
        }

        return 0;
    }



    void event_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::event_set( a0 );
                
            
            
        }

        return
            
                event_set
            
            
                ( a0 );
    }

    static PyObject *event_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->event_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::event_set");
        }

        return 0;
    }



    change_nb_type_ set_loopstart_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_loopstart();
                
            
            
        }

        return
            
                set_loopstart
            
            
                ();
    }

    static PyObject *set_loopstart_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_loopstart_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_loopstart");
        }

        return 0;
    }



    void loopstart_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::loopstart_set( a0 );
                
            
            
        }

        return
            
                loopstart_set
            
            
                ( a0 );
    }

    static PyObject *loopstart_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->loopstart_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::loopstart_set");
        }

        return 0;
    }



    change_nb_type_ set_loopend_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_loopend();
                
            
            
        }

        return
            
                set_loopend
            
            
                ();
    }

    static PyObject *set_loopend_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_loopend_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_loopend");
        }

        return 0;
    }



    void loopend_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::loopend_set( a0 );
                
            
            
        }

        return
            
                loopend_set
            
            
                ( a0 );
    }

    static PyObject *loopend_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->loopend_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::loopend_set");
        }

        return 0;
    }



    change_nb_type_ set_position_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_position();
                
            
            
        }

        return
            
                set_position
            
            
                ();
    }

    static PyObject *set_position_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_position_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_position");
        }

        return 0;
    }



    void position_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::position_set( a0 );
                
            
            
        }

        return
            
                position_set
            
            
                ( a0 );
    }

    static PyObject *position_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->position_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::position_set");
        }

        return 0;
    }



    change_nb_type_ set_stepnumerator_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_stepnumerator();
                
            
            
        }

        return
            
                set_stepnumerator
            
            
                ();
    }

    static PyObject *set_stepnumerator_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_stepnumerator_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_stepnumerator");
        }

        return 0;
    }



    void stepnumerator_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::stepnumerator_set( a0 );
                
            
            
        }

        return
            
                stepnumerator_set
            
            
                ( a0 );
    }

    static PyObject *stepnumerator_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stepnumerator_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::stepnumerator_set");
        }

        return 0;
    }



    change_nb_type_ set_stepdenominator_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_stepdenominator();
                
            
            
        }

        return
            
                set_stepdenominator
            
            
                ();
    }

    static PyObject *set_stepdenominator_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_stepdenominator_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_stepdenominator");
        }

        return 0;
    }



    void stepdenominator_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::stepdenominator_set( a0 );
                
            
            
        }

        return
            
                stepdenominator_set
            
            
                ( a0 );
    }

    static PyObject *stepdenominator_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stepdenominator_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::stepdenominator_set");
        }

        return 0;
    }



    change_nb_type_ set_playstop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return model_type_::set_playstop();
                
            
            
        }

        return
            
                set_playstop
            
            
                ();
    }

    static PyObject *set_playstop_method_(model_object_ *self, PyObject *args)
    {
        

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->set_playstop_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::set_playstop");
        }

        return 0;
    }



    void playstop_set_call_(bool owned ,const change_nb_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return model_type_::playstop_set( a0 );
                
            
            
        }

        return
            
                playstop_set
            
            
                ( a0 );
    }

    static PyObject *playstop_set_method_(model_object_ *self, PyObject *args)
    {
         change_nb_type_ * a0; 

        model_wrapper_ *o = (model_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null model c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_change_nb_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->playstop_set_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in model::playstop_set");
        }

        return 0;
    }





    static PyObject *_method_lock(model_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(model_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void model_delete_(model_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            model_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (model_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        model_wrapper_::__deleter__((model_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void model_dealloc_(model_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    model_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * model_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    model_object_ *self;

    if(!(self = (model_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int model_init_(model_object_ *self, PyObject *args, PyObject *kwds)
{
    model_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_clockdomain_ctl,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new model_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing model");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            model_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"model invalid arguments");
    return -1;

}


static const char *model_cookie_docstring_ = "";

static const char *model_get_clock_docstring_ = "";

static const char *model_set_target_docstring_ = "";

static const char *model_clear_target_docstring_ = "";

static const char *model_set_event_docstring_ = "";

static const char *model_event_set_docstring_ = "";

static const char *model_set_loopstart_docstring_ = "";

static const char *model_loopstart_set_docstring_ = "";

static const char *model_set_loopend_docstring_ = "";

static const char *model_loopend_set_docstring_ = "";

static const char *model_set_position_docstring_ = "";

static const char *model_position_set_docstring_ = "";

static const char *model_set_stepnumerator_docstring_ = "";

static const char *model_stepnumerator_set_docstring_ = "";

static const char *model_set_stepdenominator_docstring_ = "";

static const char *model_stepdenominator_set_docstring_ = "";

static const char *model_set_playstop_docstring_ = "";

static const char *model_playstop_set_docstring_ = "";




static PyMethodDef model_methods_[] = {
    
        { "cookie",(PyCFunction)model_wrapper_::cookie_method_,METH_VARARGS,(char *)model_cookie_docstring_ },
    
        { "get_clock",(PyCFunction)model_wrapper_::get_clock_method_,METH_VARARGS,(char *)model_get_clock_docstring_ },
    
        { "set_target",(PyCFunction)model_wrapper_::set_target_method_,METH_VARARGS,(char *)model_set_target_docstring_ },
    
        { "clear_target",(PyCFunction)model_wrapper_::clear_target_method_,METH_VARARGS,(char *)model_clear_target_docstring_ },
    
        { "set_event",(PyCFunction)model_wrapper_::set_event_method_,METH_VARARGS,(char *)model_set_event_docstring_ },
    
        { "event_set",(PyCFunction)model_wrapper_::event_set_method_,METH_VARARGS,(char *)model_event_set_docstring_ },
    
        { "set_loopstart",(PyCFunction)model_wrapper_::set_loopstart_method_,METH_VARARGS,(char *)model_set_loopstart_docstring_ },
    
        { "loopstart_set",(PyCFunction)model_wrapper_::loopstart_set_method_,METH_VARARGS,(char *)model_loopstart_set_docstring_ },
    
        { "set_loopend",(PyCFunction)model_wrapper_::set_loopend_method_,METH_VARARGS,(char *)model_set_loopend_docstring_ },
    
        { "loopend_set",(PyCFunction)model_wrapper_::loopend_set_method_,METH_VARARGS,(char *)model_loopend_set_docstring_ },
    
        { "set_position",(PyCFunction)model_wrapper_::set_position_method_,METH_VARARGS,(char *)model_set_position_docstring_ },
    
        { "position_set",(PyCFunction)model_wrapper_::position_set_method_,METH_VARARGS,(char *)model_position_set_docstring_ },
    
        { "set_stepnumerator",(PyCFunction)model_wrapper_::set_stepnumerator_method_,METH_VARARGS,(char *)model_set_stepnumerator_docstring_ },
    
        { "stepnumerator_set",(PyCFunction)model_wrapper_::stepnumerator_set_method_,METH_VARARGS,(char *)model_stepnumerator_set_docstring_ },
    
        { "set_stepdenominator",(PyCFunction)model_wrapper_::set_stepdenominator_method_,METH_VARARGS,(char *)model_set_stepdenominator_docstring_ },
    
        { "stepdenominator_set",(PyCFunction)model_wrapper_::stepdenominator_set_method_,METH_VARARGS,(char *)model_stepdenominator_set_docstring_ },
    
        { "set_playstop",(PyCFunction)model_wrapper_::set_playstop_method_,METH_VARARGS,(char *)model_set_playstop_docstring_ },
    
        { "playstop_set",(PyCFunction)model_wrapper_::playstop_set_method_,METH_VARARGS,(char *)model_playstop_set_docstring_ },
    
    
        { "py_lock",(PyCFunction)model_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)model_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *model_docstring_ = "";

PyTypeObject model_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "arranger_native.model",/*tp_name*/
    sizeof(model_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)model_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)model_docstring_,       /* tp_doc */
    (traverseproc)model_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)model_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    model_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)model_init_,  /* tp_init */
    0,                         /* tp_alloc */
    model_new_,             /* tp_new */
};

PyObject *model_wrapcopy_(model_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *model_wrap_(model_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = model_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    model_object_ *self = 
        
        PyObject_GC_New(model_object_, &model_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        model_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

model_type_ *model_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &model_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected model object");
        return 0;
    }

    model_type_ *object = static_cast<model_type_ *>(((model_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null model object in unwrap");
        return 0;
    }

    return object;
}

void model_insert_(const std::pair< model_type_ *, PyObject *> &p)
{




    model_wrapped_.insert(p);


}

void model_erase_(model_type_ *p)
{




    model_wrapped_.erase(p);


}

PyObject * model_find_(model_type_ *p)
{




    PyObject *ret = 0;
    std::map< model_type_ *, PyObject *>::iterator where = model_wrapped_.find(p);
    if(where != model_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct model_dispatch_ model_realdispatcher_ =
{
    &model_typeobject_,
    model_wrap_,
    model_wrapcopy_,
    model_unwrap_,
    model_insert_,
    model_erase_,
    model_find_
};

             





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initarranger_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            view_dispatcher_ = &view_realdispatcher_;
        
    
        
            model_dispatcher_ = &model_realdispatcher_;
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_nb_dispatch in module piw");
                return;
            }

            data_nb_dispatcher_ = (data_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockref_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockref_dispatch in module piw");
                return;
            }

            clockref_dispatcher_ = (clockref_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&view_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&model_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("arranger_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(view_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_view_dispatch",s);
            Py_INCREF(&view_typeobject_);
            PyModule_AddObject(m,"view",(PyObject *)&view_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(model_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_model_dispatch",s);
            Py_INCREF(&model_typeobject_);
            PyModule_AddObject(m,"model",(PyObject *)&model_typeobject_);
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

