#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "plg_convolver.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef plg_convolver::convolver_t convolver_type_; 


 
static std::map<convolver_type_ *, PyObject *> convolver_wrapped_;
 


struct convolver_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(convolver_type_ *, bool owned);
    PyObject *(*wrapcopy)(convolver_type_ *);
    convolver_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< convolver_type_ *, PyObject * > &);
    void (*erase)(convolver_type_ *);
    PyObject *(*find)(convolver_type_ *);
};

struct convolver_dispatch_ *convolver_dispatcher_;



typedef plg_convolver::samplearray2ref_t samplearray2_type_; 


 
static std::map<samplearray2_type_ *, PyObject *> samplearray2_wrapped_;
 


struct samplearray2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(samplearray2_type_ *, bool owned);
    PyObject *(*wrapcopy)(samplearray2_type_ *);
    samplearray2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< samplearray2_type_ *, PyObject * > &);
    void (*erase)(samplearray2_type_ *);
    PyObject *(*find)(samplearray2_type_ *);
};

struct samplearray2_dispatch_ *samplearray2_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_convolver(PyObject *o, void *a)
{
    convolver_type_ *object;
    if(o == Py_None)
    {
        *((convolver_type_ **)a) = 0;
        return 1;
    }
    object = convolver_dispatcher_->unwrap(o);
    *((convolver_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_convolver_copy fpcvt_convolver

PyObject *tpcvt_convolver(void *a)
{
    convolver_type_ *o = *((convolver_type_ **)a);
    return convolver_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_convolver_copy(void *a)
{
    convolver_type_ *o = (convolver_type_ *)a;
    return convolver_dispatcher_->wrapcopy(o);
}



int fpcvt_samplearray2(PyObject *o, void *a)
{
    samplearray2_type_ *object;
    if(o == Py_None)
    {
        *((samplearray2_type_ **)a) = 0;
        return 1;
    }
    object = samplearray2_dispatcher_->unwrap(o);
    *((samplearray2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_samplearray2_copy fpcvt_samplearray2

PyObject *tpcvt_samplearray2(void *a)
{
    samplearray2_type_ *o = *((samplearray2_type_ **)a);
    return samplearray2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_samplearray2_copy(void *a)
{
    samplearray2_type_ *o = (samplearray2_type_ *)a;
    return samplearray2_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct convolver_object_ {
    PyObject_HEAD
    convolver_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct convolver_wrapper_: public convolver_type_ 
{

    static void __deleter__(convolver_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    convolver_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): convolver_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~convolver_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return convolver_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(convolver_object_ *self, PyObject *args)
    {
        

        convolver_wrapper_ *o = (convolver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null convolver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in convolver::cookie");
        }

        return 0;
    }



    void set_impulse_response_call_(bool owned ,samplearray2_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return convolver_type_::set_impulse_response( a0 );
                
            
            
        }

        return
            
                set_impulse_response
            
            
                ( a0 );
    }

    static PyObject *set_impulse_response_method_(convolver_object_ *self, PyObject *args)
    {
         samplearray2_type_ * a0; 

        convolver_wrapper_ *o = (convolver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null convolver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_samplearray2,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_impulse_response_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in convolver::set_impulse_response");
        }

        return 0;
    }



    void set_mono_processing_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return convolver_type_::set_mono_processing( a0 );
                
            
            
        }

        return
            
                set_mono_processing
            
            
                ( a0 );
    }

    static PyObject *set_mono_processing_method_(convolver_object_ *self, PyObject *args)
    {
         bool a0; 

        convolver_wrapper_ *o = (convolver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null convolver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_mono_processing_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in convolver::set_mono_processing");
        }

        return 0;
    }



    void set_enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return convolver_type_::set_enable( a0 );
                
            
            
        }

        return
            
                set_enable
            
            
                ( a0 );
    }

    static PyObject *set_enable_method_(convolver_object_ *self, PyObject *args)
    {
         bool a0; 

        convolver_wrapper_ *o = (convolver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null convolver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in convolver::set_enable");
        }

        return 0;
    }



    void set_enable_time_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return convolver_type_::set_enable_time( a0 );
                
            
            
        }

        return
            
                set_enable_time
            
            
                ( a0 );
    }

    static PyObject *set_enable_time_method_(convolver_object_ *self, PyObject *args)
    {
         float a0; 

        convolver_wrapper_ *o = (convolver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null convolver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_enable_time_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in convolver::set_enable_time");
        }

        return 0;
    }





    static PyObject *_method_lock(convolver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(convolver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void convolver_delete_(convolver_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            convolver_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (convolver_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        convolver_wrapper_::__deleter__((convolver_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void convolver_dealloc_(convolver_object_* self)
{
    
    convolver_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * convolver_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    convolver_object_ *self;

    if(!(self = (convolver_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int convolver_init_(convolver_object_ *self, PyObject *args, PyObject *kwds)
{
    convolver_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new convolver_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing convolver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            convolver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"convolver invalid arguments");
    return -1;

}


static const char *convolver_cookie_docstring_ = "";

static const char *convolver_set_impulse_response_docstring_ = "";

static const char *convolver_set_mono_processing_docstring_ = "";

static const char *convolver_set_enable_docstring_ = "";

static const char *convolver_set_enable_time_docstring_ = "";




static PyMethodDef convolver_methods_[] = {
    
        { "cookie",(PyCFunction)convolver_wrapper_::cookie_method_,METH_VARARGS,(char *)convolver_cookie_docstring_ },
    
        { "set_impulse_response",(PyCFunction)convolver_wrapper_::set_impulse_response_method_,METH_VARARGS,(char *)convolver_set_impulse_response_docstring_ },
    
        { "set_mono_processing",(PyCFunction)convolver_wrapper_::set_mono_processing_method_,METH_VARARGS,(char *)convolver_set_mono_processing_docstring_ },
    
        { "set_enable",(PyCFunction)convolver_wrapper_::set_enable_method_,METH_VARARGS,(char *)convolver_set_enable_docstring_ },
    
        { "set_enable_time",(PyCFunction)convolver_wrapper_::set_enable_time_method_,METH_VARARGS,(char *)convolver_set_enable_time_docstring_ },
    
    
        { "py_lock",(PyCFunction)convolver_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)convolver_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *convolver_docstring_ = "";

PyTypeObject convolver_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "convolver_native.convolver",/*tp_name*/
    sizeof(convolver_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)convolver_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)convolver_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    convolver_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)convolver_init_,  /* tp_init */
    0,                         /* tp_alloc */
    convolver_new_,             /* tp_new */
};

PyObject *convolver_wrapcopy_(convolver_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *convolver_wrap_(convolver_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = convolver_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    convolver_object_ *self = 
        PyObject_NEW(convolver_object_, &convolver_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        convolver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

convolver_type_ *convolver_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &convolver_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected convolver object");
        return 0;
    }

    convolver_type_ *object = static_cast<convolver_type_ *>(((convolver_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null convolver object in unwrap");
        return 0;
    }

    return object;
}

void convolver_insert_(const std::pair< convolver_type_ *, PyObject *> &p)
{




    convolver_wrapped_.insert(p);


}

void convolver_erase_(convolver_type_ *p)
{




    convolver_wrapped_.erase(p);


}

PyObject * convolver_find_(convolver_type_ *p)
{




    PyObject *ret = 0;
    std::map< convolver_type_ *, PyObject *>::iterator where = convolver_wrapped_.find(p);
    if(where != convolver_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct convolver_dispatch_ convolver_realdispatcher_ =
{
    &convolver_typeobject_,
    convolver_wrap_,
    convolver_wrapcopy_,
    convolver_unwrap_,
    convolver_insert_,
    convolver_erase_,
    convolver_find_
};

  

struct samplearray2_object_ {
    PyObject_HEAD
    samplearray2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct samplearray2_wrapper_: public samplearray2_type_ 
{

    static void __deleter__(samplearray2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    samplearray2_wrapper_( PyObject *o, PyInterpreterState *i  ,const samplearray2_type_ & a0  ): samplearray2_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~samplearray2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(samplearray2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(samplearray2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void samplearray2_delete_(samplearray2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            samplearray2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (samplearray2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        samplearray2_wrapper_::__deleter__((samplearray2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void samplearray2_dealloc_(samplearray2_object_* self)
{
    
    samplearray2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * samplearray2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    samplearray2_object_ *self;

    if(!(self = (samplearray2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int samplearray2_init_(samplearray2_object_ *self, PyObject *args, PyObject *kwds)
{
    samplearray2_delete_(self);

    

    {
         samplearray2_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_samplearray2_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new samplearray2_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing samplearray2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            samplearray2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"samplearray2 invalid arguments");
    return -1;

}





static PyMethodDef samplearray2_methods_[] = {
    
    
        { "py_lock",(PyCFunction)samplearray2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)samplearray2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *samplearray2_docstring_ = "";

PyTypeObject samplearray2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "convolver_native.samplearray2",/*tp_name*/
    sizeof(samplearray2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)samplearray2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)samplearray2_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    samplearray2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)samplearray2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    samplearray2_new_,             /* tp_new */
};

PyObject *samplearray2_wrapcopy_(samplearray2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            samplearray2_object_ *obj = 
                PyObject_NEW(samplearray2_object_, samplearray2_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new samplearray2_type_(*o);
                }

                try
                {
                    samplearray2_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying samplearray2");
                return 0;
            }
        

    
}

PyObject *samplearray2_wrap_(samplearray2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = samplearray2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    samplearray2_object_ *self = 
        PyObject_NEW(samplearray2_object_, &samplearray2_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        samplearray2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

samplearray2_type_ *samplearray2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &samplearray2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected samplearray2 object");
        return 0;
    }

    samplearray2_type_ *object = static_cast<samplearray2_type_ *>(((samplearray2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null samplearray2 object in unwrap");
        return 0;
    }

    return object;
}

void samplearray2_insert_(const std::pair< samplearray2_type_ *, PyObject *> &p)
{




    samplearray2_wrapped_.insert(p);


}

void samplearray2_erase_(samplearray2_type_ *p)
{




    samplearray2_wrapped_.erase(p);


}

PyObject * samplearray2_find_(samplearray2_type_ *p)
{




    PyObject *ret = 0;
    std::map< samplearray2_type_ *, PyObject *>::iterator where = samplearray2_wrapped_.find(p);
    if(where != samplearray2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct samplearray2_dispatch_ samplearray2_realdispatcher_ =
{
    &samplearray2_typeobject_,
    samplearray2_wrap_,
    samplearray2_wrapcopy_,
    samplearray2_unwrap_,
    samplearray2_insert_,
    samplearray2_erase_,
    samplearray2_find_
};

     


static const char *module_method_canonicalise_samples_docstring_ = "";

static PyObject *module_method_canonicalise_samples(PyObject *self, PyObject *args)
{
     std::string a0;  float a1;  unsigned int a2;  unsigned int a3; 

    if(!PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_stdstr,&a0,fpcvt_f,&a1,fpcvt_ui,&a2,fpcvt_ui,&a3))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    samplearray2_type_ r = plg_convolver::canonicalise_samples  (  a0  ,a1  ,a2  ,a3 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_samplearray2_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in canonicalise_samples");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "canonicalise_samples",module_method_canonicalise_samples,METH_VARARGS, (char *)module_method_canonicalise_samples_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initconvolver_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            convolver_dispatcher_ = &convolver_realdispatcher_;
        
    
        
            samplearray2_dispatcher_ = &samplearray2_realdispatcher_;
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
    
        
    

    
        
            if(PyType_Ready(&convolver_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&samplearray2_typeobject_)<0)
                return;
        
    
        
    
        
    

    if(!(m=Py_InitModule("convolver_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(convolver_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_convolver_dispatch",s);
            Py_INCREF(&convolver_typeobject_);
            PyModule_AddObject(m,"convolver",(PyObject *)&convolver_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(samplearray2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_samplearray2_dispatch",s);
            Py_INCREF(&samplearray2_typeobject_);
            PyModule_AddObject(m,"samplearray2",(PyObject *)&samplearray2_typeobject_);
        
    
        
    
        
    

    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

