#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "host.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef host::plugin_description_t plugin_description_type_; 


 
static std::map<plugin_description_type_ *, PyObject *> plugin_description_wrapped_;
 


struct plugin_description_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(plugin_description_type_ *, bool owned);
    PyObject *(*wrapcopy)(plugin_description_type_ *);
    plugin_description_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< plugin_description_type_ *, PyObject * > &);
    void (*erase)(plugin_description_type_ *);
    PyObject *(*find)(plugin_description_type_ *);
};

struct plugin_description_dispatch_ *plugin_description_dispatcher_;



typedef host::plugin_list_t plugin_list_type_; 


 
static std::map<plugin_list_type_ *, PyObject *> plugin_list_wrapped_;
 


struct plugin_list_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(plugin_list_type_ *, bool owned);
    PyObject *(*wrapcopy)(plugin_list_type_ *);
    plugin_list_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< plugin_list_type_ *, PyObject * > &);
    void (*erase)(plugin_list_type_ *);
    PyObject *(*find)(plugin_list_type_ *);
};

struct plugin_list_dispatch_ *plugin_list_dispatcher_;



typedef host::plugin_observer_t plugin_observer_type_; 


 
static std::map<plugin_observer_type_ *, PyObject *> plugin_observer_wrapped_;
 


struct plugin_observer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(plugin_observer_type_ *, bool owned);
    PyObject *(*wrapcopy)(plugin_observer_type_ *);
    plugin_observer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< plugin_observer_type_ *, PyObject * > &);
    void (*erase)(plugin_observer_type_ *);
    PyObject *(*find)(plugin_observer_type_ *);
};

struct plugin_observer_dispatch_ *plugin_observer_dispatcher_;



typedef host::plugin_instance_t plugin_instance_type_; 


 
static std::map<plugin_instance_type_ *, PyObject *> plugin_instance_wrapped_;
 


struct plugin_instance_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(plugin_instance_type_ *, bool owned);
    PyObject *(*wrapcopy)(plugin_instance_type_ *);
    plugin_instance_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< plugin_instance_type_ *, PyObject * > &);
    void (*erase)(plugin_instance_type_ *);
    PyObject *(*find)(plugin_instance_type_ *);
};

struct plugin_instance_dispatch_ *plugin_instance_dispatcher_;



typedef pic::notify_t notify_type_; 





struct notify_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(notify_type_ *, bool owned);
    PyObject *(*wrapcopy)(notify_type_ *);
    notify_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< notify_type_ *, PyObject * > &);
    void (*erase)(notify_type_ *);
    PyObject *(*find)(notify_type_ *);
};

struct notify_dispatch_ *notify_dispatcher_;



typedef pic::status_t status_type_; 





struct status_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(status_type_ *, bool owned);
    PyObject *(*wrapcopy)(status_type_ *);
    status_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< status_type_ *, PyObject * > &);
    void (*erase)(status_type_ *);
    PyObject *(*find)(status_type_ *);
};

struct status_dispatch_ *status_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::mapping_info_t mapping_info_type_; 





struct mapping_info_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(mapping_info_type_ *, bool owned);
    PyObject *(*wrapcopy)(mapping_info_type_ *);
    mapping_info_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< mapping_info_type_ *, PyObject * > &);
    void (*erase)(mapping_info_type_ *);
    PyObject *(*find)(mapping_info_type_ *);
};

struct mapping_info_dispatch_ *mapping_info_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;



typedef piw::midi_channel_delegate_t midi_channel_delegate_type_; 





struct midi_channel_delegate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_channel_delegate_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_channel_delegate_type_ *);
    midi_channel_delegate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_channel_delegate_type_ *, PyObject * > &);
    void (*erase)(midi_channel_delegate_type_ *);
    PyObject *(*find)(midi_channel_delegate_type_ *);
};

struct midi_channel_delegate_dispatch_ *midi_channel_delegate_dispatcher_;





int fpcvt_plugin_description(PyObject *o, void *a)
{
    plugin_description_type_ *object;
    if(o == Py_None)
    {
        *((plugin_description_type_ **)a) = 0;
        return 1;
    }
    object = plugin_description_dispatcher_->unwrap(o);
    *((plugin_description_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_plugin_description_copy fpcvt_plugin_description

PyObject *tpcvt_plugin_description(void *a)
{
    plugin_description_type_ *o = *((plugin_description_type_ **)a);
    return plugin_description_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_plugin_description_copy(void *a)
{
    plugin_description_type_ *o = (plugin_description_type_ *)a;
    return plugin_description_dispatcher_->wrapcopy(o);
}



int fpcvt_plugin_list(PyObject *o, void *a)
{
    plugin_list_type_ *object;
    if(o == Py_None)
    {
        *((plugin_list_type_ **)a) = 0;
        return 1;
    }
    object = plugin_list_dispatcher_->unwrap(o);
    *((plugin_list_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_plugin_list_copy fpcvt_plugin_list

PyObject *tpcvt_plugin_list(void *a)
{
    plugin_list_type_ *o = *((plugin_list_type_ **)a);
    return plugin_list_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_plugin_list_copy(void *a)
{
    plugin_list_type_ *o = (plugin_list_type_ *)a;
    return plugin_list_dispatcher_->wrapcopy(o);
}



int fpcvt_plugin_observer(PyObject *o, void *a)
{
    plugin_observer_type_ *object;
    if(o == Py_None)
    {
        *((plugin_observer_type_ **)a) = 0;
        return 1;
    }
    object = plugin_observer_dispatcher_->unwrap(o);
    *((plugin_observer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_plugin_observer_copy fpcvt_plugin_observer

PyObject *tpcvt_plugin_observer(void *a)
{
    plugin_observer_type_ *o = *((plugin_observer_type_ **)a);
    return plugin_observer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_plugin_observer_copy(void *a)
{
    plugin_observer_type_ *o = (plugin_observer_type_ *)a;
    return plugin_observer_dispatcher_->wrapcopy(o);
}



int fpcvt_plugin_instance(PyObject *o, void *a)
{
    plugin_instance_type_ *object;
    if(o == Py_None)
    {
        *((plugin_instance_type_ **)a) = 0;
        return 1;
    }
    object = plugin_instance_dispatcher_->unwrap(o);
    *((plugin_instance_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_plugin_instance_copy fpcvt_plugin_instance

PyObject *tpcvt_plugin_instance(void *a)
{
    plugin_instance_type_ *o = *((plugin_instance_type_ **)a);
    return plugin_instance_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_plugin_instance_copy(void *a)
{
    plugin_instance_type_ *o = (plugin_instance_type_ *)a;
    return plugin_instance_dispatcher_->wrapcopy(o);
}



int fpcvt_notify(PyObject *o, void *a)
{
    notify_type_ *object;
    if(o == Py_None)
    {
        *((notify_type_ **)a) = 0;
        return 1;
    }
    object = notify_dispatcher_->unwrap(o);
    *((notify_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_notify_copy fpcvt_notify

PyObject *tpcvt_notify(void *a)
{
    notify_type_ *o = *((notify_type_ **)a);
    return notify_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_notify_copy(void *a)
{
    notify_type_ *o = (notify_type_ *)a;
    return notify_dispatcher_->wrapcopy(o);
}



int fpcvt_status(PyObject *o, void *a)
{
    status_type_ *object;
    if(o == Py_None)
    {
        *((status_type_ **)a) = 0;
        return 1;
    }
    object = status_dispatcher_->unwrap(o);
    *((status_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_status_copy fpcvt_status

PyObject *tpcvt_status(void *a)
{
    status_type_ *o = *((status_type_ **)a);
    return status_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_status_copy(void *a)
{
    status_type_ *o = (status_type_ *)a;
    return status_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_mapping_info(PyObject *o, void *a)
{
    mapping_info_type_ *object;
    if(o == Py_None)
    {
        *((mapping_info_type_ **)a) = 0;
        return 1;
    }
    object = mapping_info_dispatcher_->unwrap(o);
    *((mapping_info_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_mapping_info_copy fpcvt_mapping_info

PyObject *tpcvt_mapping_info(void *a)
{
    mapping_info_type_ *o = *((mapping_info_type_ **)a);
    return mapping_info_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_mapping_info_copy(void *a)
{
    mapping_info_type_ *o = (mapping_info_type_ *)a;
    return mapping_info_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_channel_delegate(PyObject *o, void *a)
{
    midi_channel_delegate_type_ *object;
    if(o == Py_None)
    {
        *((midi_channel_delegate_type_ **)a) = 0;
        return 1;
    }
    object = midi_channel_delegate_dispatcher_->unwrap(o);
    *((midi_channel_delegate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_channel_delegate_copy fpcvt_midi_channel_delegate

PyObject *tpcvt_midi_channel_delegate(void *a)
{
    midi_channel_delegate_type_ *o = *((midi_channel_delegate_type_ **)a);
    return midi_channel_delegate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_channel_delegate_copy(void *a)
{
    midi_channel_delegate_type_ *o = (midi_channel_delegate_type_ *)a;
    return midi_channel_delegate_dispatcher_->wrapcopy(o);
}



 

struct plugin_description_object_ {
    PyObject_HEAD
    plugin_description_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct plugin_description_wrapper_: public plugin_description_type_ 
{

    static void __deleter__(plugin_description_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    plugin_description_wrapper_( PyObject *o, PyInterpreterState *i  ,const plugin_description_type_ & a0  ): plugin_description_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    plugin_description_wrapper_( PyObject *o, PyInterpreterState *i  ,const std::string & a0  ,const std::string & a1  ): plugin_description_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    plugin_description_wrapper_( PyObject *o, PyInterpreterState *i  ): plugin_description_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~plugin_description_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }




    // special method __str__ returning std::string doc 

    std::string special_str_call_()
    {
        return
             id 
            
                ();
    }

    static PyObject *special_str_method_(plugin_description_object_ *self)
    {
        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        try
        {
            PyObject *ro;
            {   unlock_p2c unlock ;
                {   
                    std::string r = o->special_str_call_();
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::__str__");
        }

        return 0;
    }








    // special method __repr__ returning std::string doc 

    std::string special_repr_call_()
    {
        return
             id 
            
                ();
    }

    static PyObject *special_repr_method_(plugin_description_object_ *self)
    {
        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        try
        {
            PyObject *ro;
            {   unlock_p2c unlock  ;
                {
                    std::string r = o->special_repr_call_();
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            } 

            return ro;
        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::__repr__");
        }

        return 0;
    }









    std::string name_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::name();
                
            
            
        }

        return
            
                name
            
            
                ();
    }

    static PyObject *name_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::name");
        }

        return 0;
    }



    std::string format_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::format();
                
            
            
        }

        return
            
                format
            
            
                ();
    }

    static PyObject *format_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->format_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::format");
        }

        return 0;
    }



    std::string manufacturer_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::manufacturer();
                
            
            
        }

        return
            
                manufacturer
            
            
                ();
    }

    static PyObject *manufacturer_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->manufacturer_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::manufacturer");
        }

        return 0;
    }



    std::string id_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::id();
                
            
            
        }

        return
            
                id
            
            
                ();
    }

    static PyObject *id_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->id_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::id");
        }

        return 0;
    }



    std::string description_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::description();
                
            
            
        }

        return
            
                description
            
            
                ();
    }

    static PyObject *description_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->description_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::description");
        }

        return 0;
    }



    std::string to_xml_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::to_xml();
                
            
            
        }

        return
            
                to_xml
            
            
                ();
    }

    static PyObject *to_xml_method_(plugin_description_object_ *self, PyObject *args)
    {
        

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->to_xml_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::to_xml");
        }

        return 0;
    }



    bool from_xml_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_description_type_::from_xml( a0 );
                
            
            
        }

        return
            
                from_xml
            
            
                ( a0 );
    }

    static PyObject *from_xml_method_(plugin_description_object_ *self, PyObject *args)
    {
         std::string a0; 

        plugin_description_wrapper_ *o = (plugin_description_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_description c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->from_xml_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_description::from_xml");
        }

        return 0;
    }





    static PyObject *_method_lock(plugin_description_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(plugin_description_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void plugin_description_delete_(plugin_description_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            plugin_description_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (plugin_description_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        plugin_description_wrapper_::__deleter__((plugin_description_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void plugin_description_dealloc_(plugin_description_object_* self)
{
    
    plugin_description_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * plugin_description_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    plugin_description_object_ *self;

    if(!(self = (plugin_description_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int plugin_description_init_(plugin_description_object_ *self, PyObject *args, PyObject *kwds)
{
    plugin_description_delete_(self);

    

    {
         plugin_description_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_plugin_description_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_description_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_description");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_description_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         std::string a0;  std::string a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_stdstr,&a0,fpcvt_stdstr,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_description_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_description");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_description_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_description_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_description");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_description_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"plugin_description invalid arguments");
    return -1;

}


static const char *plugin_description_name_docstring_ = "";

static const char *plugin_description_format_docstring_ = "";

static const char *plugin_description_manufacturer_docstring_ = "";

static const char *plugin_description_id_docstring_ = "";

static const char *plugin_description_description_docstring_ = "";

static const char *plugin_description_to_xml_docstring_ = "";

static const char *plugin_description_from_xml_docstring_ = "";




static PyMethodDef plugin_description_methods_[] = {
    
        { "name",(PyCFunction)plugin_description_wrapper_::name_method_,METH_VARARGS,(char *)plugin_description_name_docstring_ },
    
        { "format",(PyCFunction)plugin_description_wrapper_::format_method_,METH_VARARGS,(char *)plugin_description_format_docstring_ },
    
        { "manufacturer",(PyCFunction)plugin_description_wrapper_::manufacturer_method_,METH_VARARGS,(char *)plugin_description_manufacturer_docstring_ },
    
        { "id",(PyCFunction)plugin_description_wrapper_::id_method_,METH_VARARGS,(char *)plugin_description_id_docstring_ },
    
        { "description",(PyCFunction)plugin_description_wrapper_::description_method_,METH_VARARGS,(char *)plugin_description_description_docstring_ },
    
        { "to_xml",(PyCFunction)plugin_description_wrapper_::to_xml_method_,METH_VARARGS,(char *)plugin_description_to_xml_docstring_ },
    
        { "from_xml",(PyCFunction)plugin_description_wrapper_::from_xml_method_,METH_VARARGS,(char *)plugin_description_from_xml_docstring_ },
    
    
        { "py_lock",(PyCFunction)plugin_description_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)plugin_description_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *plugin_description_docstring_ = "";

PyTypeObject plugin_description_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "host_native.plugin_description",/*tp_name*/
    sizeof(plugin_description_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)plugin_description_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
     (reprfunc)plugin_description_wrapper_::special_repr_method_  , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
     (reprfunc)plugin_description_wrapper_::special_str_method_  , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)plugin_description_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    plugin_description_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)plugin_description_init_,  /* tp_init */
    0,                         /* tp_alloc */
    plugin_description_new_,             /* tp_new */
};

PyObject *plugin_description_wrapcopy_(plugin_description_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            plugin_description_object_ *obj = 
                PyObject_NEW(plugin_description_object_, plugin_description_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new plugin_description_type_(*o);
                }

                try
                {
                    plugin_description_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying plugin_description");
                return 0;
            }
        

    
}

PyObject *plugin_description_wrap_(plugin_description_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = plugin_description_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    plugin_description_object_ *self = 
        PyObject_NEW(plugin_description_object_, &plugin_description_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        plugin_description_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

plugin_description_type_ *plugin_description_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &plugin_description_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected plugin_description object");
        return 0;
    }

    plugin_description_type_ *object = static_cast<plugin_description_type_ *>(((plugin_description_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null plugin_description object in unwrap");
        return 0;
    }

    return object;
}

void plugin_description_insert_(const std::pair< plugin_description_type_ *, PyObject *> &p)
{




    plugin_description_wrapped_.insert(p);


}

void plugin_description_erase_(plugin_description_type_ *p)
{




    plugin_description_wrapped_.erase(p);


}

PyObject * plugin_description_find_(plugin_description_type_ *p)
{




    PyObject *ret = 0;
    std::map< plugin_description_type_ *, PyObject *>::iterator where = plugin_description_wrapped_.find(p);
    if(where != plugin_description_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct plugin_description_dispatch_ plugin_description_realdispatcher_ =
{
    &plugin_description_typeobject_,
    plugin_description_wrap_,
    plugin_description_wrapcopy_,
    plugin_description_unwrap_,
    plugin_description_insert_,
    plugin_description_erase_,
    plugin_description_find_
};

  

struct plugin_list_object_ {
    PyObject_HEAD
    plugin_list_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct plugin_list_wrapper_: public plugin_list_type_ 
{

    static void __deleter__(plugin_list_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    plugin_list_wrapper_( PyObject *o, PyInterpreterState *i  ,const std::string & a0  ,const notify_type_ & a1  ): plugin_list_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~plugin_list_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(plugin_list_object_ *self, visitproc visit, void *arg)
    {
        plugin_list_wrapper_ *o = (plugin_list_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(plugin_list_object_ *self)
    {
        plugin_list_wrapper_ *o = (plugin_list_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    unsigned int num_plugins_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_list_type_::num_plugins();
                
            
            
        }

        return
            
                num_plugins
            
            
                ();
    }

    static PyObject *num_plugins_method_(plugin_list_object_ *self, PyObject *args)
    {
        

        plugin_list_wrapper_ *o = (plugin_list_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_list c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->num_plugins_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_list::num_plugins");
        }

        return 0;
    }



    plugin_description_type_ get_plugin_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_list_type_::get_plugin( a0 );
                
            
            
        }

        return
            
                get_plugin
            
            
                ( a0 );
    }

    static PyObject *get_plugin_method_(plugin_list_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_list_wrapper_ *o = (plugin_list_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_list c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    plugin_description_type_ r = o->get_plugin_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_plugin_description_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_list::get_plugin");
        }

        return 0;
    }



    int find_plugin_call_(bool owned ,const plugin_description_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_list_type_::find_plugin( a0 );
                
            
            
        }

        return
            
                find_plugin
            
            
                ( a0 );
    }

    static PyObject *find_plugin_method_(plugin_list_object_ *self, PyObject *args)
    {
         plugin_description_type_ * a0; 

        plugin_list_wrapper_ *o = (plugin_list_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_list c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_plugin_description_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->find_plugin_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_list::find_plugin");
        }

        return 0;
    }





    static PyObject *_method_lock(plugin_list_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(plugin_list_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void plugin_list_delete_(plugin_list_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            plugin_list_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (plugin_list_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        plugin_list_wrapper_::__deleter__((plugin_list_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void plugin_list_dealloc_(plugin_list_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    plugin_list_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * plugin_list_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    plugin_list_object_ *self;

    if(!(self = (plugin_list_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int plugin_list_init_(plugin_list_object_ *self, PyObject *args, PyObject *kwds)
{
    plugin_list_delete_(self);

    

    {
         std::string a0;  notify_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_stdstr,&a0,fpcvt_notify_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_list_wrapper_((PyObject *)self,interp ,a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_list");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_list_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"plugin_list invalid arguments");
    return -1;

}


static const char *plugin_list_num_plugins_docstring_ = "";

static const char *plugin_list_get_plugin_docstring_ = "";

static const char *plugin_list_find_plugin_docstring_ = "";




static PyMethodDef plugin_list_methods_[] = {
    
        { "num_plugins",(PyCFunction)plugin_list_wrapper_::num_plugins_method_,METH_VARARGS,(char *)plugin_list_num_plugins_docstring_ },
    
        { "get_plugin",(PyCFunction)plugin_list_wrapper_::get_plugin_method_,METH_VARARGS,(char *)plugin_list_get_plugin_docstring_ },
    
        { "find_plugin",(PyCFunction)plugin_list_wrapper_::find_plugin_method_,METH_VARARGS,(char *)plugin_list_find_plugin_docstring_ },
    
    
        { "py_lock",(PyCFunction)plugin_list_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)plugin_list_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *plugin_list_docstring_ = "";

PyTypeObject plugin_list_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "host_native.plugin_list",/*tp_name*/
    sizeof(plugin_list_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)plugin_list_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)plugin_list_docstring_,       /* tp_doc */
    (traverseproc)plugin_list_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)plugin_list_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    plugin_list_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)plugin_list_init_,  /* tp_init */
    0,                         /* tp_alloc */
    plugin_list_new_,             /* tp_new */
};

PyObject *plugin_list_wrapcopy_(plugin_list_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *plugin_list_wrap_(plugin_list_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = plugin_list_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    plugin_list_object_ *self = 
        
        PyObject_GC_New(plugin_list_object_, &plugin_list_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        plugin_list_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

plugin_list_type_ *plugin_list_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &plugin_list_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected plugin_list object");
        return 0;
    }

    plugin_list_type_ *object = static_cast<plugin_list_type_ *>(((plugin_list_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null plugin_list object in unwrap");
        return 0;
    }

    return object;
}

void plugin_list_insert_(const std::pair< plugin_list_type_ *, PyObject *> &p)
{




    plugin_list_wrapped_.insert(p);


}

void plugin_list_erase_(plugin_list_type_ *p)
{




    plugin_list_wrapped_.erase(p);


}

PyObject * plugin_list_find_(plugin_list_type_ *p)
{




    PyObject *ret = 0;
    std::map< plugin_list_type_ *, PyObject *>::iterator where = plugin_list_wrapped_.find(p);
    if(where != plugin_list_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct plugin_list_dispatch_ plugin_list_realdispatcher_ =
{
    &plugin_list_typeobject_,
    plugin_list_wrap_,
    plugin_list_wrapcopy_,
    plugin_list_unwrap_,
    plugin_list_insert_,
    plugin_list_erase_,
    plugin_list_find_
};

  

struct plugin_observer_object_ {
    PyObject_HEAD
    plugin_observer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct plugin_observer_wrapper_: public plugin_observer_type_ 
{

    static void __deleter__(plugin_observer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
        cache_3_ = 0;
    
        cache_4_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
        if(cache_3_)
        {
            Py_DECREF(cache_3_);
            cache_3_ = 0;
        }
    
        if(cache_4_)
        {
            Py_DECREF(cache_4_);
            cache_4_ = 0;
        }
    
    }
*/



    plugin_observer_wrapper_( PyObject *o, PyInterpreterState *i  ): plugin_observer_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~plugin_observer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void description_changed(  const std::string & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"description_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_stdstr,&a0);
        //printf("description_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("description_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void showing_changed(  bool a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"showing_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_b,&a0);
        //printf("showing_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("showing_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void bypassed_changed(  bool a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"bypassed_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_b,&a0);
        //printf("bypassed_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("bypassed_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_3_;
 
    void mapping_changed(  const std::string & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_3_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"mapping_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_3_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_stdstr,&a0);
        //printf("mapping_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("mapping_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_4_;
 
    std::string get_parameter_name(  unsigned int a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_4_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"get_parameter_name");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_4_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_ui,&a0);
        //printf("get_parameter_name before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("get_parameter_name after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        

        
        {
            std::string rptr;

            if(!fpcvt_stdstr(result,&rptr))
            {
                throw pip_err_t("wrong type result converting stdstr");
            }

            try
            {
                {   unlock_p2c unlock;
                    {
                        std::string rcopy(rptr);
                        {   relock_p2c relock(unlock);
                            Py_DECREF(result);
                            return rcopy;
                        }
                    }
                }
            }
            catch(...)
            {
                Py_DECREF(result);
                throw;
            }
        }

        
    }

















    void description_changed_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method plugin_observer::description_changed");
                
                
            
            
        }

        return
            
                description_changed
            
            
                ( a0 );
    }

    static PyObject *description_changed_method_(plugin_observer_object_ *self, PyObject *args)
    {
         std::string a0; 

        plugin_observer_wrapper_ *o = (plugin_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->description_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_observer::description_changed");
        }

        return 0;
    }



    void showing_changed_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method plugin_observer::showing_changed");
                
                
            
            
        }

        return
            
                showing_changed
            
            
                ( a0 );
    }

    static PyObject *showing_changed_method_(plugin_observer_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_observer_wrapper_ *o = (plugin_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->showing_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_observer::showing_changed");
        }

        return 0;
    }



    void bypassed_changed_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method plugin_observer::bypassed_changed");
                
                
            
            
        }

        return
            
                bypassed_changed
            
            
                ( a0 );
    }

    static PyObject *bypassed_changed_method_(plugin_observer_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_observer_wrapper_ *o = (plugin_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->bypassed_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_observer::bypassed_changed");
        }

        return 0;
    }



    void mapping_changed_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method plugin_observer::mapping_changed");
                
                
            
            
        }

        return
            
                mapping_changed
            
            
                ( a0 );
    }

    static PyObject *mapping_changed_method_(plugin_observer_object_ *self, PyObject *args)
    {
         std::string a0; 

        plugin_observer_wrapper_ *o = (plugin_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->mapping_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_observer::mapping_changed");
        }

        return 0;
    }



    std::string get_parameter_name_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method plugin_observer::get_parameter_name");
                
                
            
            
        }

        return
            
                get_parameter_name
            
            
                ( a0 );
    }

    static PyObject *get_parameter_name_method_(plugin_observer_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_observer_wrapper_ *o = (plugin_observer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_observer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_parameter_name_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_observer::get_parameter_name");
        }

        return 0;
    }





    static PyObject *_method_lock(plugin_observer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(plugin_observer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void plugin_observer_delete_(plugin_observer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            plugin_observer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (plugin_observer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        plugin_observer_wrapper_::__deleter__((plugin_observer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void plugin_observer_dealloc_(plugin_observer_object_* self)
{
    
    plugin_observer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * plugin_observer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    plugin_observer_object_ *self;

    if(!(self = (plugin_observer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int plugin_observer_init_(plugin_observer_object_ *self, PyObject *args, PyObject *kwds)
{
    plugin_observer_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_observer_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_observer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_observer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"plugin_observer invalid arguments");
    return -1;

}


static const char *plugin_observer_description_changed_docstring_ = "";

static const char *plugin_observer_showing_changed_docstring_ = "";

static const char *plugin_observer_bypassed_changed_docstring_ = "";

static const char *plugin_observer_mapping_changed_docstring_ = "";

static const char *plugin_observer_get_parameter_name_docstring_ = "";




static PyMethodDef plugin_observer_methods_[] = {
    
        { "description_changed",(PyCFunction)plugin_observer_wrapper_::description_changed_method_,METH_VARARGS,(char *)plugin_observer_description_changed_docstring_ },
    
        { "showing_changed",(PyCFunction)plugin_observer_wrapper_::showing_changed_method_,METH_VARARGS,(char *)plugin_observer_showing_changed_docstring_ },
    
        { "bypassed_changed",(PyCFunction)plugin_observer_wrapper_::bypassed_changed_method_,METH_VARARGS,(char *)plugin_observer_bypassed_changed_docstring_ },
    
        { "mapping_changed",(PyCFunction)plugin_observer_wrapper_::mapping_changed_method_,METH_VARARGS,(char *)plugin_observer_mapping_changed_docstring_ },
    
        { "get_parameter_name",(PyCFunction)plugin_observer_wrapper_::get_parameter_name_method_,METH_VARARGS,(char *)plugin_observer_get_parameter_name_docstring_ },
    
    
        { "py_lock",(PyCFunction)plugin_observer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)plugin_observer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *plugin_observer_docstring_ = "";

PyTypeObject plugin_observer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "host_native.plugin_observer",/*tp_name*/
    sizeof(plugin_observer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)plugin_observer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)plugin_observer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    plugin_observer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)plugin_observer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    plugin_observer_new_,             /* tp_new */
};

PyObject *plugin_observer_wrapcopy_(plugin_observer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *plugin_observer_wrap_(plugin_observer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = plugin_observer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    plugin_observer_object_ *self = 
        PyObject_NEW(plugin_observer_object_, &plugin_observer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        plugin_observer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

plugin_observer_type_ *plugin_observer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &plugin_observer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected plugin_observer object");
        return 0;
    }

    plugin_observer_type_ *object = static_cast<plugin_observer_type_ *>(((plugin_observer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null plugin_observer object in unwrap");
        return 0;
    }

    return object;
}

void plugin_observer_insert_(const std::pair< plugin_observer_type_ *, PyObject *> &p)
{




    plugin_observer_wrapped_.insert(p);


}

void plugin_observer_erase_(plugin_observer_type_ *p)
{




    plugin_observer_wrapped_.erase(p);


}

PyObject * plugin_observer_find_(plugin_observer_type_ *p)
{




    PyObject *ret = 0;
    std::map< plugin_observer_type_ *, PyObject *>::iterator where = plugin_observer_wrapped_.find(p);
    if(where != plugin_observer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct plugin_observer_dispatch_ plugin_observer_realdispatcher_ =
{
    &plugin_observer_typeobject_,
    plugin_observer_wrap_,
    plugin_observer_wrapcopy_,
    plugin_observer_unwrap_,
    plugin_observer_insert_,
    plugin_observer_erase_,
    plugin_observer_find_
};

  

struct plugin_instance_object_ {
    PyObject_HEAD
    plugin_instance_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct plugin_instance_wrapper_: public plugin_instance_type_ 
{

    static void __deleter__(plugin_instance_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    plugin_instance_wrapper_( PyObject *o, PyInterpreterState *i  ,plugin_observer_type_ * a0  ,midi_channel_delegate_type_ * a1  ,clockdomain_ctl_type_ * a2  ,const cookie_type_ & a3  ,const cookie_type_ & a4  ,const status_type_ & a5  ): plugin_instance_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~plugin_instance_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(plugin_instance_object_ *self, visitproc visit, void *arg)
    {
        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(plugin_instance_object_ *self)
    {
        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    clockdomain_ctl_type_ * clock_domain_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::clock_domain();
                
            
            
        }

        return
            
                clock_domain
            
            
                ();
    }

    static PyObject *clock_domain_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockdomain_ctl_type_ * r = o->clock_domain_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockdomain_ctl((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::clock_domain");
        }

        return 0;
    }



    cookie_type_ metronome_input_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::metronome_input();
                
            
            
        }

        return
            
                metronome_input
            
            
                ();
    }

    static PyObject *metronome_input_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->metronome_input_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::metronome_input");
        }

        return 0;
    }



    cookie_type_ midi_from_belcanto_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::midi_from_belcanto();
                
            
            
        }

        return
            
                midi_from_belcanto
            
            
                ();
    }

    static PyObject *midi_from_belcanto_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->midi_from_belcanto_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::midi_from_belcanto");
        }

        return 0;
    }



    cookie_type_ midi_aggregator_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::midi_aggregator();
                
            
            
        }

        return
            
                midi_aggregator
            
            
                ();
    }

    static PyObject *midi_aggregator_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->midi_aggregator_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::midi_aggregator");
        }

        return 0;
    }



    cookie_type_ audio_input_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::audio_input();
                
            
            
        }

        return
            
                audio_input
            
            
                ();
    }

    static PyObject *audio_input_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->audio_input_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::audio_input");
        }

        return 0;
    }



    void set_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_midi_channel( a0 );
                
            
            
        }

        return
            
                set_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_midi_channel_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_midi_channel");
        }

        return 0;
    }



    void set_min_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_min_midi_channel( a0 );
                
            
            
        }

        return
            
                set_min_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_min_midi_channel_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_min_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_min_midi_channel");
        }

        return 0;
    }



    void set_max_midi_channel_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_max_midi_channel( a0 );
                
            
            
        }

        return
            
                set_max_midi_channel
            
            
                ( a0 );
    }

    static PyObject *set_max_midi_channel_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_max_midi_channel_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_max_midi_channel");
        }

        return 0;
    }



    void set_program_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_program_change( a0 );
                
            
            
        }

        return
            
                set_program_change
            
            
                ( a0 );
    }

    static PyObject *set_program_change_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_program_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_program_change");
        }

        return 0;
    }



    change_nb_type_ change_program_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::change_program();
                
            
            
        }

        return
            
                change_program
            
            
                ();
    }

    static PyObject *change_program_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_program_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::change_program");
        }

        return 0;
    }



    void set_bank_change_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_bank_change( a0 );
                
            
            
        }

        return
            
                set_bank_change
            
            
                ( a0 );
    }

    static PyObject *set_bank_change_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bank_change_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_bank_change");
        }

        return 0;
    }



    change_nb_type_ change_bank_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::change_bank();
                
            
            
        }

        return
            
                change_bank
            
            
                ();
    }

    static PyObject *change_bank_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_bank_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::change_bank");
        }

        return 0;
    }



    void set_cc_call_(bool owned ,unsigned int a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_cc( a0  ,a1 );
                
            
            
        }

        return
            
                set_cc
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_cc_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_cc_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_cc");
        }

        return 0;
    }



    change_nb_type_ change_cc_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::change_cc();
                
            
            
        }

        return
            
                change_cc
            
            
                ();
    }

    static PyObject *change_cc_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->change_cc_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::change_cc");
        }

        return 0;
    }



    cookie_type_ parameter_input_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::parameter_input( a0 );
                
            
            
        }

        return
            
                parameter_input
            
            
                ( a0 );
    }

    static PyObject *parameter_input_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->parameter_input_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::parameter_input");
        }

        return 0;
    }



    void set_mapping_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_mapping( a0 );
                
            
            
        }

        return
            
                set_mapping
            
            
                ( a0 );
    }

    static PyObject *set_mapping_method_(plugin_instance_object_ *self, PyObject *args)
    {
         std::string a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_mapping_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_mapping");
        }

        return 0;
    }



    std::string get_mapping_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_mapping();
                
            
            
        }

        return
            
                get_mapping
            
            
                ();
    }

    static PyObject *get_mapping_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_mapping_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_mapping");
        }

        return 0;
    }



    void parameter_name_changed_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::parameter_name_changed( a0 );
                
            
            
        }

        return
            
                parameter_name_changed
            
            
                ( a0 );
    }

    static PyObject *parameter_name_changed_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->parameter_name_changed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::parameter_name_changed");
        }

        return 0;
    }



    void map_param_call_(bool owned ,unsigned int a0 ,mapping_info_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::map_param( a0  ,a1 );
                
            
            
        }

        return
            
                map_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *map_param_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  mapping_info_type_ * a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_mapping_info_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->map_param_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::map_param");
        }

        return 0;
    }



    void map_midi_call_(bool owned ,unsigned int a0 ,mapping_info_type_ a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::map_midi( a0  ,a1 );
                
            
            
        }

        return
            
                map_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *map_midi_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  mapping_info_type_ * a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_mapping_info_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->map_midi_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::map_midi");
        }

        return 0;
    }



    void unmap_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::unmap_param( a0  ,a1 );
                
            
            
        }

        return
            
                unmap_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *unmap_param_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unmap_param_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::unmap_param");
        }

        return 0;
    }



    void unmap_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::unmap_midi( a0  ,a1 );
                
            
            
        }

        return
            
                unmap_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *unmap_midi_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unmap_midi_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::unmap_midi");
        }

        return 0;
    }



    bool is_mapped_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::is_mapped_param( a0  ,a1 );
                
            
            
        }

        return
            
                is_mapped_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *is_mapped_param_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_mapped_param_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::is_mapped_param");
        }

        return 0;
    }



    bool is_mapped_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::is_mapped_midi( a0  ,a1 );
                
            
            
        }

        return
            
                is_mapped_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *is_mapped_midi_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_mapped_midi_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::is_mapped_midi");
        }

        return 0;
    }



    mapping_info_type_ get_info_param_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_info_param( a0  ,a1 );
                
            
            
        }

        return
            
                get_info_param
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_info_param_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->get_info_param_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_info_param");
        }

        return 0;
    }



    mapping_info_type_ get_info_midi_call_(bool owned ,unsigned int a0 ,unsigned short a1 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_info_midi( a0  ,a1 );
                
            
            
        }

        return
            
                get_info_midi
            
            
                ( a0  ,a1 );
    }

    static PyObject *get_info_midi_method_(plugin_instance_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned short a1; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_us,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    mapping_info_type_ r = o->get_info_midi_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_mapping_info_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_info_midi");
        }

        return 0;
    }



    void set_minimum_decimation_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_minimum_decimation( a0 );
                
            
            
        }

        return
            
                set_minimum_decimation
            
            
                ( a0 );
    }

    static PyObject *set_minimum_decimation_method_(plugin_instance_object_ *self, PyObject *args)
    {
         float a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_minimum_decimation_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_minimum_decimation");
        }

        return 0;
    }



    void set_midi_notes_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_midi_notes( a0 );
                
            
            
        }

        return
            
                set_midi_notes
            
            
                ( a0 );
    }

    static PyObject *set_midi_notes_method_(plugin_instance_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_notes_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_midi_notes");
        }

        return 0;
    }



    void set_midi_pitchbend_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_midi_pitchbend( a0 );
                
            
            
        }

        return
            
                set_midi_pitchbend
            
            
                ( a0 );
    }

    static PyObject *set_midi_pitchbend_method_(plugin_instance_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_midi_pitchbend_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_midi_pitchbend");
        }

        return 0;
    }



    bool open_call_(bool owned ,const plugin_description_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::open( a0 );
                
            
            
        }

        return
            
                open
            
            
                ( a0 );
    }

    static PyObject *open_method_(plugin_instance_object_ *self, PyObject *args)
    {
         plugin_description_type_ * a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_plugin_description_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned ,*a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::open");
        }

        return 0;
    }



    void close_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::close();
                
            
            
        }

        return
            
                close
            
            
                ();
    }

    static PyObject *close_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::close");
        }

        return 0;
    }



    unsigned int input_channel_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::input_channel_count();
                
            
            
        }

        return
            
                input_channel_count
            
            
                ();
    }

    static PyObject *input_channel_count_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->input_channel_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::input_channel_count");
        }

        return 0;
    }



    unsigned int output_channel_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::output_channel_count();
                
            
            
        }

        return
            
                output_channel_count
            
            
                ();
    }

    static PyObject *output_channel_count_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->output_channel_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::output_channel_count");
        }

        return 0;
    }



    void set_showing_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_showing( a0 );
                
            
            
        }

        return
            
                set_showing
            
            
                ( a0 );
    }

    static PyObject *set_showing_method_(plugin_instance_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_showing_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_showing");
        }

        return 0;
    }



    bool is_showing_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::is_showing();
                
            
            
        }

        return
            
                is_showing
            
            
                ();
    }

    static PyObject *is_showing_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_showing_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::is_showing");
        }

        return 0;
    }



    void set_title_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_title( a0 );
                
            
            
        }

        return
            
                set_title
            
            
                ( a0 );
    }

    static PyObject *set_title_method_(plugin_instance_object_ *self, PyObject *args)
    {
         std::string a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_title_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_title");
        }

        return 0;
    }



    void set_bypassed_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_bypassed( a0 );
                
            
            
        }

        return
            
                set_bypassed
            
            
                ( a0 );
    }

    static PyObject *set_bypassed_method_(plugin_instance_object_ *self, PyObject *args)
    {
         bool a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bypassed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_bypassed");
        }

        return 0;
    }



    bool is_bypassed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::is_bypassed();
                
            
            
        }

        return
            
                is_bypassed
            
            
                ();
    }

    static PyObject *is_bypassed_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->is_bypassed_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::is_bypassed");
        }

        return 0;
    }



    plugin_description_type_ get_description_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_description();
                
            
            
        }

        return
            
                get_description
            
            
                ();
    }

    static PyObject *get_description_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    plugin_description_type_ r = o->get_description_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_plugin_description_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_description");
        }

        return 0;
    }



    data_type_ get_state_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_state();
                
            
            
        }

        return
            
                get_state
            
            
                ();
    }

    static PyObject *get_state_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_state_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_state");
        }

        return 0;
    }



    void set_state_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_state( a0 );
                
            
            
        }

        return
            
                set_state
            
            
                ( a0 );
    }

    static PyObject *set_state_method_(plugin_instance_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_state_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_state");
        }

        return 0;
    }



    data_type_ get_bounds_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::get_bounds();
                
            
            
        }

        return
            
                get_bounds
            
            
                ();
    }

    static PyObject *get_bounds_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    data_type_ r = o->get_bounds_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::get_bounds");
        }

        return 0;
    }



    void set_bounds_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_bounds( a0 );
                
            
            
        }

        return
            
                set_bounds
            
            
                ( a0 );
    }

    static PyObject *set_bounds_method_(plugin_instance_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bounds_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_bounds");
        }

        return 0;
    }



    void set_idle_time_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::set_idle_time( a0 );
                
            
            
        }

        return
            
                set_idle_time
            
            
                ( a0 );
    }

    static PyObject *set_idle_time_method_(plugin_instance_object_ *self, PyObject *args)
    {
         float a0; 

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_idle_time_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::set_idle_time");
        }

        return 0;
    }



    void disable_idling_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return plugin_instance_type_::disable_idling();
                
            
            
        }

        return
            
                disable_idling
            
            
                ();
    }

    static PyObject *disable_idling_method_(plugin_instance_object_ *self, PyObject *args)
    {
        

        plugin_instance_wrapper_ *o = (plugin_instance_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null plugin_instance c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->disable_idling_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in plugin_instance::disable_idling");
        }

        return 0;
    }





    static PyObject *_method_lock(plugin_instance_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(plugin_instance_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void plugin_instance_delete_(plugin_instance_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            plugin_instance_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (plugin_instance_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        plugin_instance_wrapper_::__deleter__((plugin_instance_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void plugin_instance_dealloc_(plugin_instance_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    plugin_instance_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * plugin_instance_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    plugin_instance_object_ *self;

    if(!(self = (plugin_instance_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int plugin_instance_init_(plugin_instance_object_ *self, PyObject *args, PyObject *kwds)
{
    plugin_instance_delete_(self);

    

    {
         plugin_observer_type_ * a0;  midi_channel_delegate_type_ * a1;  clockdomain_ctl_type_ * a2;  cookie_type_ * a3;  cookie_type_ * a4;  status_type_ * a5; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&" ,fpcvt_plugin_observer,&a0,fpcvt_midi_channel_delegate,&a1,fpcvt_clockdomain_ctl,&a2,fpcvt_cookie_copy,&a3,fpcvt_cookie_copy,&a4,fpcvt_status_copy,&a5))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new plugin_instance_wrapper_((PyObject *)self,interp ,a0,a1,a2,*a3,*a4,*a5);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing plugin_instance");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            plugin_instance_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"plugin_instance invalid arguments");
    return -1;

}


static const char *plugin_instance_clock_domain_docstring_ = "";

static const char *plugin_instance_metronome_input_docstring_ = "";

static const char *plugin_instance_midi_from_belcanto_docstring_ = "";

static const char *plugin_instance_midi_aggregator_docstring_ = "";

static const char *plugin_instance_audio_input_docstring_ = "";

static const char *plugin_instance_set_midi_channel_docstring_ = "";

static const char *plugin_instance_set_min_midi_channel_docstring_ = "";

static const char *plugin_instance_set_max_midi_channel_docstring_ = "";

static const char *plugin_instance_set_program_change_docstring_ = "";

static const char *plugin_instance_change_program_docstring_ = "";

static const char *plugin_instance_set_bank_change_docstring_ = "";

static const char *plugin_instance_change_bank_docstring_ = "";

static const char *plugin_instance_set_cc_docstring_ = "";

static const char *plugin_instance_change_cc_docstring_ = "";

static const char *plugin_instance_parameter_input_docstring_ = "";

static const char *plugin_instance_set_mapping_docstring_ = "";

static const char *plugin_instance_get_mapping_docstring_ = "";

static const char *plugin_instance_parameter_name_changed_docstring_ = "";

static const char *plugin_instance_map_param_docstring_ = "";

static const char *plugin_instance_map_midi_docstring_ = "";

static const char *plugin_instance_unmap_param_docstring_ = "";

static const char *plugin_instance_unmap_midi_docstring_ = "";

static const char *plugin_instance_is_mapped_param_docstring_ = "";

static const char *plugin_instance_is_mapped_midi_docstring_ = "";

static const char *plugin_instance_get_info_param_docstring_ = "";

static const char *plugin_instance_get_info_midi_docstring_ = "";

static const char *plugin_instance_set_minimum_decimation_docstring_ = "";

static const char *plugin_instance_set_midi_notes_docstring_ = "";

static const char *plugin_instance_set_midi_pitchbend_docstring_ = "";

static const char *plugin_instance_open_docstring_ = "";

static const char *plugin_instance_close_docstring_ = "";

static const char *plugin_instance_input_channel_count_docstring_ = "";

static const char *plugin_instance_output_channel_count_docstring_ = "";

static const char *plugin_instance_set_showing_docstring_ = "";

static const char *plugin_instance_is_showing_docstring_ = "";

static const char *plugin_instance_set_title_docstring_ = "";

static const char *plugin_instance_set_bypassed_docstring_ = "";

static const char *plugin_instance_is_bypassed_docstring_ = "";

static const char *plugin_instance_get_description_docstring_ = "";

static const char *plugin_instance_get_state_docstring_ = "";

static const char *plugin_instance_set_state_docstring_ = "";

static const char *plugin_instance_get_bounds_docstring_ = "";

static const char *plugin_instance_set_bounds_docstring_ = "";

static const char *plugin_instance_set_idle_time_docstring_ = "";

static const char *plugin_instance_disable_idling_docstring_ = "";




static PyMethodDef plugin_instance_methods_[] = {
    
        { "clock_domain",(PyCFunction)plugin_instance_wrapper_::clock_domain_method_,METH_VARARGS,(char *)plugin_instance_clock_domain_docstring_ },
    
        { "metronome_input",(PyCFunction)plugin_instance_wrapper_::metronome_input_method_,METH_VARARGS,(char *)plugin_instance_metronome_input_docstring_ },
    
        { "midi_from_belcanto",(PyCFunction)plugin_instance_wrapper_::midi_from_belcanto_method_,METH_VARARGS,(char *)plugin_instance_midi_from_belcanto_docstring_ },
    
        { "midi_aggregator",(PyCFunction)plugin_instance_wrapper_::midi_aggregator_method_,METH_VARARGS,(char *)plugin_instance_midi_aggregator_docstring_ },
    
        { "audio_input",(PyCFunction)plugin_instance_wrapper_::audio_input_method_,METH_VARARGS,(char *)plugin_instance_audio_input_docstring_ },
    
        { "set_midi_channel",(PyCFunction)plugin_instance_wrapper_::set_midi_channel_method_,METH_VARARGS,(char *)plugin_instance_set_midi_channel_docstring_ },
    
        { "set_min_midi_channel",(PyCFunction)plugin_instance_wrapper_::set_min_midi_channel_method_,METH_VARARGS,(char *)plugin_instance_set_min_midi_channel_docstring_ },
    
        { "set_max_midi_channel",(PyCFunction)plugin_instance_wrapper_::set_max_midi_channel_method_,METH_VARARGS,(char *)plugin_instance_set_max_midi_channel_docstring_ },
    
        { "set_program_change",(PyCFunction)plugin_instance_wrapper_::set_program_change_method_,METH_VARARGS,(char *)plugin_instance_set_program_change_docstring_ },
    
        { "change_program",(PyCFunction)plugin_instance_wrapper_::change_program_method_,METH_VARARGS,(char *)plugin_instance_change_program_docstring_ },
    
        { "set_bank_change",(PyCFunction)plugin_instance_wrapper_::set_bank_change_method_,METH_VARARGS,(char *)plugin_instance_set_bank_change_docstring_ },
    
        { "change_bank",(PyCFunction)plugin_instance_wrapper_::change_bank_method_,METH_VARARGS,(char *)plugin_instance_change_bank_docstring_ },
    
        { "set_cc",(PyCFunction)plugin_instance_wrapper_::set_cc_method_,METH_VARARGS,(char *)plugin_instance_set_cc_docstring_ },
    
        { "change_cc",(PyCFunction)plugin_instance_wrapper_::change_cc_method_,METH_VARARGS,(char *)plugin_instance_change_cc_docstring_ },
    
        { "parameter_input",(PyCFunction)plugin_instance_wrapper_::parameter_input_method_,METH_VARARGS,(char *)plugin_instance_parameter_input_docstring_ },
    
        { "set_mapping",(PyCFunction)plugin_instance_wrapper_::set_mapping_method_,METH_VARARGS,(char *)plugin_instance_set_mapping_docstring_ },
    
        { "get_mapping",(PyCFunction)plugin_instance_wrapper_::get_mapping_method_,METH_VARARGS,(char *)plugin_instance_get_mapping_docstring_ },
    
        { "parameter_name_changed",(PyCFunction)plugin_instance_wrapper_::parameter_name_changed_method_,METH_VARARGS,(char *)plugin_instance_parameter_name_changed_docstring_ },
    
        { "map_param",(PyCFunction)plugin_instance_wrapper_::map_param_method_,METH_VARARGS,(char *)plugin_instance_map_param_docstring_ },
    
        { "map_midi",(PyCFunction)plugin_instance_wrapper_::map_midi_method_,METH_VARARGS,(char *)plugin_instance_map_midi_docstring_ },
    
        { "unmap_param",(PyCFunction)plugin_instance_wrapper_::unmap_param_method_,METH_VARARGS,(char *)plugin_instance_unmap_param_docstring_ },
    
        { "unmap_midi",(PyCFunction)plugin_instance_wrapper_::unmap_midi_method_,METH_VARARGS,(char *)plugin_instance_unmap_midi_docstring_ },
    
        { "is_mapped_param",(PyCFunction)plugin_instance_wrapper_::is_mapped_param_method_,METH_VARARGS,(char *)plugin_instance_is_mapped_param_docstring_ },
    
        { "is_mapped_midi",(PyCFunction)plugin_instance_wrapper_::is_mapped_midi_method_,METH_VARARGS,(char *)plugin_instance_is_mapped_midi_docstring_ },
    
        { "get_info_param",(PyCFunction)plugin_instance_wrapper_::get_info_param_method_,METH_VARARGS,(char *)plugin_instance_get_info_param_docstring_ },
    
        { "get_info_midi",(PyCFunction)plugin_instance_wrapper_::get_info_midi_method_,METH_VARARGS,(char *)plugin_instance_get_info_midi_docstring_ },
    
        { "set_minimum_decimation",(PyCFunction)plugin_instance_wrapper_::set_minimum_decimation_method_,METH_VARARGS,(char *)plugin_instance_set_minimum_decimation_docstring_ },
    
        { "set_midi_notes",(PyCFunction)plugin_instance_wrapper_::set_midi_notes_method_,METH_VARARGS,(char *)plugin_instance_set_midi_notes_docstring_ },
    
        { "set_midi_pitchbend",(PyCFunction)plugin_instance_wrapper_::set_midi_pitchbend_method_,METH_VARARGS,(char *)plugin_instance_set_midi_pitchbend_docstring_ },
    
        { "open",(PyCFunction)plugin_instance_wrapper_::open_method_,METH_VARARGS,(char *)plugin_instance_open_docstring_ },
    
        { "close",(PyCFunction)plugin_instance_wrapper_::close_method_,METH_VARARGS,(char *)plugin_instance_close_docstring_ },
    
        { "input_channel_count",(PyCFunction)plugin_instance_wrapper_::input_channel_count_method_,METH_VARARGS,(char *)plugin_instance_input_channel_count_docstring_ },
    
        { "output_channel_count",(PyCFunction)plugin_instance_wrapper_::output_channel_count_method_,METH_VARARGS,(char *)plugin_instance_output_channel_count_docstring_ },
    
        { "set_showing",(PyCFunction)plugin_instance_wrapper_::set_showing_method_,METH_VARARGS,(char *)plugin_instance_set_showing_docstring_ },
    
        { "is_showing",(PyCFunction)plugin_instance_wrapper_::is_showing_method_,METH_VARARGS,(char *)plugin_instance_is_showing_docstring_ },
    
        { "set_title",(PyCFunction)plugin_instance_wrapper_::set_title_method_,METH_VARARGS,(char *)plugin_instance_set_title_docstring_ },
    
        { "set_bypassed",(PyCFunction)plugin_instance_wrapper_::set_bypassed_method_,METH_VARARGS,(char *)plugin_instance_set_bypassed_docstring_ },
    
        { "is_bypassed",(PyCFunction)plugin_instance_wrapper_::is_bypassed_method_,METH_VARARGS,(char *)plugin_instance_is_bypassed_docstring_ },
    
        { "get_description",(PyCFunction)plugin_instance_wrapper_::get_description_method_,METH_VARARGS,(char *)plugin_instance_get_description_docstring_ },
    
        { "get_state",(PyCFunction)plugin_instance_wrapper_::get_state_method_,METH_VARARGS,(char *)plugin_instance_get_state_docstring_ },
    
        { "set_state",(PyCFunction)plugin_instance_wrapper_::set_state_method_,METH_VARARGS,(char *)plugin_instance_set_state_docstring_ },
    
        { "get_bounds",(PyCFunction)plugin_instance_wrapper_::get_bounds_method_,METH_VARARGS,(char *)plugin_instance_get_bounds_docstring_ },
    
        { "set_bounds",(PyCFunction)plugin_instance_wrapper_::set_bounds_method_,METH_VARARGS,(char *)plugin_instance_set_bounds_docstring_ },
    
        { "set_idle_time",(PyCFunction)plugin_instance_wrapper_::set_idle_time_method_,METH_VARARGS,(char *)plugin_instance_set_idle_time_docstring_ },
    
        { "disable_idling",(PyCFunction)plugin_instance_wrapper_::disable_idling_method_,METH_VARARGS,(char *)plugin_instance_disable_idling_docstring_ },
    
    
        { "py_lock",(PyCFunction)plugin_instance_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)plugin_instance_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *plugin_instance_docstring_ = "";

PyTypeObject plugin_instance_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "host_native.plugin_instance",/*tp_name*/
    sizeof(plugin_instance_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)plugin_instance_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)plugin_instance_docstring_,       /* tp_doc */
    (traverseproc)plugin_instance_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)plugin_instance_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    plugin_instance_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)plugin_instance_init_,  /* tp_init */
    0,                         /* tp_alloc */
    plugin_instance_new_,             /* tp_new */
};

PyObject *plugin_instance_wrapcopy_(plugin_instance_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *plugin_instance_wrap_(plugin_instance_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = plugin_instance_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    plugin_instance_object_ *self = 
        
        PyObject_GC_New(plugin_instance_object_, &plugin_instance_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        plugin_instance_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

plugin_instance_type_ *plugin_instance_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &plugin_instance_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected plugin_instance object");
        return 0;
    }

    plugin_instance_type_ *object = static_cast<plugin_instance_type_ *>(((plugin_instance_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null plugin_instance object in unwrap");
        return 0;
    }

    return object;
}

void plugin_instance_insert_(const std::pair< plugin_instance_type_ *, PyObject *> &p)
{




    plugin_instance_wrapped_.insert(p);


}

void plugin_instance_erase_(plugin_instance_type_ *p)
{




    plugin_instance_wrapped_.erase(p);


}

PyObject * plugin_instance_find_(plugin_instance_type_ *p)
{




    PyObject *ret = 0;
    std::map< plugin_instance_type_ *, PyObject *>::iterator where = plugin_instance_wrapped_.find(p);
    if(where != plugin_instance_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct plugin_instance_dispatch_ plugin_instance_realdispatcher_ =
{
    &plugin_instance_typeobject_,
    plugin_instance_wrap_,
    plugin_instance_wrapcopy_,
    plugin_instance_unwrap_,
    plugin_instance_insert_,
    plugin_instance_erase_,
    plugin_instance_find_
};

                 


static const char *module_method_create_plugin_description_docstring_ = "";

static PyObject *module_method_create_plugin_description(PyObject *self, PyObject *args)
{
     std::string a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_stdstr,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    plugin_description_type_ r = host::create_plugin_description  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_plugin_description_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in create_plugin_description");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "create_plugin_description",module_method_create_plugin_description,METH_VARARGS, (char *)module_method_create_plugin_description_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY inithost_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            plugin_description_dispatcher_ = &plugin_description_realdispatcher_;
        
    
        
            plugin_list_dispatcher_ = &plugin_list_realdispatcher_;
        
    
        
            plugin_observer_dispatcher_ = &plugin_observer_realdispatcher_;
        
    
        
            plugin_instance_dispatcher_ = &plugin_instance_realdispatcher_;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("picross")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_notify_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_notify_dispatch in module picross");
                return;
            }

            notify_dispatcher_ = (notify_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_status_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_status_dispatch in module picross");
                return;
            }

            status_dispatcher_ = (status_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_mapping_info_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_mapping_info_dispatch in module piw");
                return;
            }

            mapping_info_dispatcher_ = (mapping_info_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_midi_channel_delegate_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_midi_channel_delegate_dispatch in module piw");
                return;
            }

            midi_channel_delegate_dispatcher_ = (midi_channel_delegate_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&plugin_description_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&plugin_list_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&plugin_observer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&plugin_instance_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("host_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(plugin_description_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_plugin_description_dispatch",s);
            Py_INCREF(&plugin_description_typeobject_);
            PyModule_AddObject(m,"plugin_description",(PyObject *)&plugin_description_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(plugin_list_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_plugin_list_dispatch",s);
            Py_INCREF(&plugin_list_typeobject_);
            PyModule_AddObject(m,"plugin_list",(PyObject *)&plugin_list_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(plugin_observer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_plugin_observer_dispatch",s);
            Py_INCREF(&plugin_observer_typeobject_);
            PyModule_AddObject(m,"plugin_observer",(PyObject *)&plugin_observer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(plugin_instance_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_plugin_instance_dispatch",s);
            Py_INCREF(&plugin_instance_typeobject_);
            PyModule_AddObject(m,"plugin_instance",(PyObject *)&plugin_instance_typeobject_);
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

