#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "lng_wordrec.h"
#include "lng_controls.h"
#include "lng_osc.h"
#include "lng_bonjour.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef language::bonjour_t bonjour_type_; 


 
static std::map<bonjour_type_ *, PyObject *> bonjour_wrapped_;
 


struct bonjour_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(bonjour_type_ *, bool owned);
    PyObject *(*wrapcopy)(bonjour_type_ *);
    bonjour_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< bonjour_type_ *, PyObject * > &);
    void (*erase)(bonjour_type_ *);
    PyObject *(*find)(bonjour_type_ *);
};

struct bonjour_dispatch_ *bonjour_dispatcher_;



typedef language::oscserver_t oscserver_type_; 


 
static std::map<oscserver_type_ *, PyObject *> oscserver_wrapped_;
 


struct oscserver_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(oscserver_type_ *, bool owned);
    PyObject *(*wrapcopy)(oscserver_type_ *);
    oscserver_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< oscserver_type_ *, PyObject * > &);
    void (*erase)(oscserver_type_ *);
    PyObject *(*find)(oscserver_type_ *);
};

struct oscserver_dispatch_ *oscserver_dispatcher_;



typedef language::wordrec_t wordrec_type_; 


 
static std::map<wordrec_type_ *, PyObject *> wordrec_wrapped_;
 


struct wordrec_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wordrec_type_ *, bool owned);
    PyObject *(*wrapcopy)(wordrec_type_ *);
    wordrec_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wordrec_type_ *, PyObject * > &);
    void (*erase)(wordrec_type_ *);
    PyObject *(*find)(wordrec_type_ *);
};

struct wordrec_dispatch_ *wordrec_dispatcher_;



typedef language::toggle_t toggle_type_; 


  


struct toggle_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(toggle_type_ *, bool owned);
    PyObject *(*wrapcopy)(toggle_type_ *);
    toggle_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< toggle_type_ *, PyObject * > &);
    void (*erase)(toggle_type_ *);
    PyObject *(*find)(toggle_type_ *);
};

struct toggle_dispatch_ *toggle_dispatcher_;



typedef language::updown_t updown_type_; 


  


struct updown_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(updown_type_ *, bool owned);
    PyObject *(*wrapcopy)(updown_type_ *);
    updown_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< updown_type_ *, PyObject * > &);
    void (*erase)(updown_type_ *);
    PyObject *(*find)(updown_type_ *);
};

struct updown_dispatch_ *updown_dispatcher_;



typedef language::xselector_t xselector_type_; 


  


struct xselector_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(xselector_type_ *, bool owned);
    PyObject *(*wrapcopy)(xselector_type_ *);
    xselector_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< xselector_type_ *, PyObject * > &);
    void (*erase)(xselector_type_ *);
    PyObject *(*find)(xselector_type_ *);
};

struct xselector_dispatch_ *xselector_dispatcher_;



typedef piw::xxcontrolled_t xxcontrolled_type_; 





struct xxcontrolled_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(xxcontrolled_type_ *, bool owned);
    PyObject *(*wrapcopy)(xxcontrolled_type_ *);
    xxcontrolled_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< xxcontrolled_type_ *, PyObject * > &);
    void (*erase)(xxcontrolled_type_ *);
    PyObject *(*find)(xxcontrolled_type_ *);
};

struct xxcontrolled_dispatch_ *xxcontrolled_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::fastdata_t fastdata_type_; 





struct fastdata_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(fastdata_type_ *, bool owned);
    PyObject *(*wrapcopy)(fastdata_type_ *);
    fastdata_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< fastdata_type_ *, PyObject * > &);
    void (*erase)(fastdata_type_ *);
    PyObject *(*find)(fastdata_type_ *);
};

struct fastdata_dispatch_ *fastdata_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::data_nb_t data_nb_type_; 





struct data_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_nb_type_ *);
    data_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_nb_type_ *, PyObject * > &);
    void (*erase)(data_nb_type_ *);
    PyObject *(*find)(data_nb_type_ *);
};

struct data_nb_dispatch_ *data_nb_dispatcher_;





int fpcvt_bonjour(PyObject *o, void *a)
{
    bonjour_type_ *object;
    if(o == Py_None)
    {
        *((bonjour_type_ **)a) = 0;
        return 1;
    }
    object = bonjour_dispatcher_->unwrap(o);
    *((bonjour_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_bonjour_copy fpcvt_bonjour

PyObject *tpcvt_bonjour(void *a)
{
    bonjour_type_ *o = *((bonjour_type_ **)a);
    return bonjour_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_bonjour_copy(void *a)
{
    bonjour_type_ *o = (bonjour_type_ *)a;
    return bonjour_dispatcher_->wrapcopy(o);
}



int fpcvt_oscserver(PyObject *o, void *a)
{
    oscserver_type_ *object;
    if(o == Py_None)
    {
        *((oscserver_type_ **)a) = 0;
        return 1;
    }
    object = oscserver_dispatcher_->unwrap(o);
    *((oscserver_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_oscserver_copy fpcvt_oscserver

PyObject *tpcvt_oscserver(void *a)
{
    oscserver_type_ *o = *((oscserver_type_ **)a);
    return oscserver_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_oscserver_copy(void *a)
{
    oscserver_type_ *o = (oscserver_type_ *)a;
    return oscserver_dispatcher_->wrapcopy(o);
}



int fpcvt_wordrec(PyObject *o, void *a)
{
    wordrec_type_ *object;
    if(o == Py_None)
    {
        *((wordrec_type_ **)a) = 0;
        return 1;
    }
    object = wordrec_dispatcher_->unwrap(o);
    *((wordrec_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wordrec_copy fpcvt_wordrec

PyObject *tpcvt_wordrec(void *a)
{
    wordrec_type_ *o = *((wordrec_type_ **)a);
    return wordrec_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wordrec_copy(void *a)
{
    wordrec_type_ *o = (wordrec_type_ *)a;
    return wordrec_dispatcher_->wrapcopy(o);
}



int fpcvt_toggle(PyObject *o, void *a)
{
    toggle_type_ *object;
    if(o == Py_None)
    {
        *((toggle_type_ **)a) = 0;
        return 1;
    }
    object = toggle_dispatcher_->unwrap(o);
    *((toggle_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_toggle_copy fpcvt_toggle

PyObject *tpcvt_toggle(void *a)
{
    toggle_type_ *o = *((toggle_type_ **)a);
    return toggle_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_toggle_copy(void *a)
{
    toggle_type_ *o = (toggle_type_ *)a;
    return toggle_dispatcher_->wrapcopy(o);
}



int fpcvt_updown(PyObject *o, void *a)
{
    updown_type_ *object;
    if(o == Py_None)
    {
        *((updown_type_ **)a) = 0;
        return 1;
    }
    object = updown_dispatcher_->unwrap(o);
    *((updown_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_updown_copy fpcvt_updown

PyObject *tpcvt_updown(void *a)
{
    updown_type_ *o = *((updown_type_ **)a);
    return updown_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_updown_copy(void *a)
{
    updown_type_ *o = (updown_type_ *)a;
    return updown_dispatcher_->wrapcopy(o);
}



int fpcvt_xselector(PyObject *o, void *a)
{
    xselector_type_ *object;
    if(o == Py_None)
    {
        *((xselector_type_ **)a) = 0;
        return 1;
    }
    object = xselector_dispatcher_->unwrap(o);
    *((xselector_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_xselector_copy fpcvt_xselector

PyObject *tpcvt_xselector(void *a)
{
    xselector_type_ *o = *((xselector_type_ **)a);
    return xselector_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_xselector_copy(void *a)
{
    xselector_type_ *o = (xselector_type_ *)a;
    return xselector_dispatcher_->wrapcopy(o);
}



int fpcvt_xxcontrolled(PyObject *o, void *a)
{
    xxcontrolled_type_ *object;
    if(o == Py_None)
    {
        *((xxcontrolled_type_ **)a) = 0;
        return 1;
    }
    object = xxcontrolled_dispatcher_->unwrap(o);
    *((xxcontrolled_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_xxcontrolled_copy fpcvt_xxcontrolled

PyObject *tpcvt_xxcontrolled(void *a)
{
    xxcontrolled_type_ *o = *((xxcontrolled_type_ **)a);
    return xxcontrolled_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_xxcontrolled_copy(void *a)
{
    xxcontrolled_type_ *o = (xxcontrolled_type_ *)a;
    return xxcontrolled_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_fastdata(PyObject *o, void *a)
{
    fastdata_type_ *object;
    if(o == Py_None)
    {
        *((fastdata_type_ **)a) = 0;
        return 1;
    }
    object = fastdata_dispatcher_->unwrap(o);
    *((fastdata_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_fastdata_copy fpcvt_fastdata

PyObject *tpcvt_fastdata(void *a)
{
    fastdata_type_ *o = *((fastdata_type_ **)a);
    return fastdata_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_fastdata_copy(void *a)
{
    fastdata_type_ *o = (fastdata_type_ *)a;
    return fastdata_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_data_nb(PyObject *o, void *a)
{
    data_nb_type_ *object;
    if(o == Py_None)
    {
        *((data_nb_type_ **)a) = 0;
        return 1;
    }
    object = data_nb_dispatcher_->unwrap(o);
    *((data_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_nb_copy fpcvt_data_nb

PyObject *tpcvt_data_nb(void *a)
{
    data_nb_type_ *o = *((data_nb_type_ **)a);
    return data_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_nb_copy(void *a)
{
    data_nb_type_ *o = (data_nb_type_ *)a;
    return data_nb_dispatcher_->wrapcopy(o);
}



 

struct bonjour_object_ {
    PyObject_HEAD
    bonjour_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct bonjour_wrapper_: public bonjour_type_ 
{

    static void __deleter__(bonjour_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    bonjour_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): bonjour_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~bonjour_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void service_register_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bonjour_type_::service_register();
                
            
            
        }

        return
            
                service_register
            
            
                ();
    }

    static PyObject *service_register_method_(bonjour_object_ *self, PyObject *args)
    {
        

        bonjour_wrapper_ *o = (bonjour_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bonjour c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->service_register_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bonjour::service_register");
        }

        return 0;
    }





    static PyObject *_method_lock(bonjour_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(bonjour_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void bonjour_delete_(bonjour_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            bonjour_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (bonjour_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        bonjour_wrapper_::__deleter__((bonjour_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void bonjour_dealloc_(bonjour_object_* self)
{
    
    bonjour_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * bonjour_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    bonjour_object_ *self;

    if(!(self = (bonjour_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int bonjour_init_(bonjour_object_ *self, PyObject *args, PyObject *kwds)
{
    bonjour_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new bonjour_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing bonjour");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            bonjour_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"bonjour invalid arguments");
    return -1;

}


static const char *bonjour_service_register_docstring_ = "";




static PyMethodDef bonjour_methods_[] = {
    
        { "service_register",(PyCFunction)bonjour_wrapper_::service_register_method_,METH_VARARGS,(char *)bonjour_service_register_docstring_ },
    
    
        { "py_lock",(PyCFunction)bonjour_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)bonjour_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *bonjour_docstring_ = "";

PyTypeObject bonjour_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.bonjour",/*tp_name*/
    sizeof(bonjour_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)bonjour_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)bonjour_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    bonjour_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)bonjour_init_,  /* tp_init */
    0,                         /* tp_alloc */
    bonjour_new_,             /* tp_new */
};

PyObject *bonjour_wrapcopy_(bonjour_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *bonjour_wrap_(bonjour_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = bonjour_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    bonjour_object_ *self = 
        PyObject_NEW(bonjour_object_, &bonjour_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        bonjour_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

bonjour_type_ *bonjour_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &bonjour_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected bonjour object");
        return 0;
    }

    bonjour_type_ *object = static_cast<bonjour_type_ *>(((bonjour_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null bonjour object in unwrap");
        return 0;
    }

    return object;
}

void bonjour_insert_(const std::pair< bonjour_type_ *, PyObject *> &p)
{




    bonjour_wrapped_.insert(p);


}

void bonjour_erase_(bonjour_type_ *p)
{




    bonjour_wrapped_.erase(p);


}

PyObject * bonjour_find_(bonjour_type_ *p)
{




    PyObject *ret = 0;
    std::map< bonjour_type_ *, PyObject *>::iterator where = bonjour_wrapped_.find(p);
    if(where != bonjour_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct bonjour_dispatch_ bonjour_realdispatcher_ =
{
    &bonjour_typeobject_,
    bonjour_wrap_,
    bonjour_wrapcopy_,
    bonjour_unwrap_,
    bonjour_insert_,
    bonjour_erase_,
    bonjour_find_
};

  

struct oscserver_object_ {
    PyObject_HEAD
    oscserver_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct oscserver_wrapper_: public oscserver_type_ 
{

    static void __deleter__(oscserver_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    oscserver_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,const char * a1  ): oscserver_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~oscserver_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void startup_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::startup();
                
            
            
        }

        return
            
                startup
            
            
                ();
    }

    static PyObject *startup_method_(oscserver_object_ *self, PyObject *args)
    {
        

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->startup_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::startup");
        }

        return 0;
    }



    void shutdown_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::shutdown();
                
            
            
        }

        return
            
                shutdown
            
            
                ();
    }

    static PyObject *shutdown_method_(oscserver_object_ *self, PyObject *args)
    {
        

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->shutdown_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::shutdown");
        }

        return 0;
    }



    unsigned int add_widget_call_(bool owned ,const char * a0 ,fastdata_type_ * a1 ,fastdata_type_ * a2 )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::add_widget( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                add_widget
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *add_widget_method_(oscserver_object_ *self, PyObject *args)
    {
         const char * a0;  fastdata_type_ * a1;  fastdata_type_ * a2; 

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_fastdata,&a1  ,fpcvt_fastdata,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->add_widget_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::add_widget");
        }

        return 0;
    }



    void del_widget_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::del_widget( a0 );
                
            
            
        }

        return
            
                del_widget
            
            
                ( a0 );
    }

    static PyObject *del_widget_method_(oscserver_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->del_widget_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::del_widget");
        }

        return 0;
    }



    void send_call_(bool owned ,const char * a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::send( a0  ,a1 );
                
            
            
        }

        return
            
                send
            
            
                ( a0  ,a1 );
    }

    static PyObject *send_method_(oscserver_object_ *self, PyObject *args)
    {
         const char * a0;  float a1; 

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_str,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->send_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::send");
        }

        return 0;
    }



    void set_connected_call_(bool owned ,unsigned int a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                
                    return oscserver_type_::set_connected( a0  ,a1 );
                
            
            
        }

        return
            
                set_connected
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_connected_method_(oscserver_object_ *self, PyObject *args)
    {
         unsigned int a0;  bool a1; 

        oscserver_wrapper_ *o = (oscserver_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null oscserver c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_connected_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in oscserver::set_connected");
        }

        return 0;
    }





    static PyObject *_method_lock(oscserver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(oscserver_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void oscserver_delete_(oscserver_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            oscserver_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (oscserver_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        oscserver_wrapper_::__deleter__((oscserver_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void oscserver_dealloc_(oscserver_object_* self)
{
    
    oscserver_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * oscserver_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    oscserver_object_ *self;

    if(!(self = (oscserver_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int oscserver_init_(oscserver_object_ *self, PyObject *args, PyObject *kwds)
{
    oscserver_delete_(self);

    

    {
         const char * a0;  const char * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_str,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new oscserver_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing oscserver");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            oscserver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"oscserver invalid arguments");
    return -1;

}


static const char *oscserver_startup_docstring_ = "";

static const char *oscserver_shutdown_docstring_ = "";

static const char *oscserver_add_widget_docstring_ = "";

static const char *oscserver_del_widget_docstring_ = "";

static const char *oscserver_send_docstring_ = "";

static const char *oscserver_set_connected_docstring_ = "";




static PyMethodDef oscserver_methods_[] = {
    
        { "startup",(PyCFunction)oscserver_wrapper_::startup_method_,METH_VARARGS,(char *)oscserver_startup_docstring_ },
    
        { "shutdown",(PyCFunction)oscserver_wrapper_::shutdown_method_,METH_VARARGS,(char *)oscserver_shutdown_docstring_ },
    
        { "add_widget",(PyCFunction)oscserver_wrapper_::add_widget_method_,METH_VARARGS,(char *)oscserver_add_widget_docstring_ },
    
        { "del_widget",(PyCFunction)oscserver_wrapper_::del_widget_method_,METH_VARARGS,(char *)oscserver_del_widget_docstring_ },
    
        { "send",(PyCFunction)oscserver_wrapper_::send_method_,METH_VARARGS,(char *)oscserver_send_docstring_ },
    
        { "set_connected",(PyCFunction)oscserver_wrapper_::set_connected_method_,METH_VARARGS,(char *)oscserver_set_connected_docstring_ },
    
    
        { "py_lock",(PyCFunction)oscserver_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)oscserver_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *oscserver_docstring_ = "";

PyTypeObject oscserver_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.oscserver",/*tp_name*/
    sizeof(oscserver_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)oscserver_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)oscserver_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    oscserver_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)oscserver_init_,  /* tp_init */
    0,                         /* tp_alloc */
    oscserver_new_,             /* tp_new */
};

PyObject *oscserver_wrapcopy_(oscserver_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *oscserver_wrap_(oscserver_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = oscserver_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    oscserver_object_ *self = 
        PyObject_NEW(oscserver_object_, &oscserver_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        oscserver_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

oscserver_type_ *oscserver_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &oscserver_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected oscserver object");
        return 0;
    }

    oscserver_type_ *object = static_cast<oscserver_type_ *>(((oscserver_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null oscserver object in unwrap");
        return 0;
    }

    return object;
}

void oscserver_insert_(const std::pair< oscserver_type_ *, PyObject *> &p)
{




    oscserver_wrapped_.insert(p);


}

void oscserver_erase_(oscserver_type_ *p)
{




    oscserver_wrapped_.erase(p);


}

PyObject * oscserver_find_(oscserver_type_ *p)
{




    PyObject *ret = 0;
    std::map< oscserver_type_ *, PyObject *>::iterator where = oscserver_wrapped_.find(p);
    if(where != oscserver_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct oscserver_dispatch_ oscserver_realdispatcher_ =
{
    &oscserver_typeobject_,
    oscserver_wrap_,
    oscserver_wrapcopy_,
    oscserver_unwrap_,
    oscserver_insert_,
    oscserver_erase_,
    oscserver_find_
};

  

struct wordrec_object_ {
    PyObject_HEAD
    wordrec_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct wordrec_wrapper_: public wordrec_type_ 
{

    static void __deleter__(wordrec_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    wordrec_wrapper_( PyObject *o, PyInterpreterState *i  ,change_type_ a0  ): wordrec_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~wordrec_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(wordrec_object_ *self, visitproc visit, void *arg)
    {
        wordrec_wrapper_ *o = (wordrec_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(wordrec_object_ *self)
    {
        wordrec_wrapper_ *o = (wordrec_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return wordrec_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(wordrec_object_ *self, PyObject *args)
    {
        

        wordrec_wrapper_ *o = (wordrec_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null wordrec c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in wordrec::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(wordrec_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(wordrec_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void wordrec_delete_(wordrec_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            wordrec_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (wordrec_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        wordrec_wrapper_::__deleter__((wordrec_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void wordrec_dealloc_(wordrec_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    wordrec_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * wordrec_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    wordrec_object_ *self;

    if(!(self = (wordrec_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int wordrec_init_(wordrec_object_ *self, PyObject *args, PyObject *kwds)
{
    wordrec_delete_(self);

    

    {
         change_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_change_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new wordrec_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing wordrec");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            wordrec_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"wordrec invalid arguments");
    return -1;

}


static const char *wordrec_cookie_docstring_ = "";




static PyMethodDef wordrec_methods_[] = {
    
        { "cookie",(PyCFunction)wordrec_wrapper_::cookie_method_,METH_VARARGS,(char *)wordrec_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)wordrec_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)wordrec_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *wordrec_docstring_ = "";

PyTypeObject wordrec_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.wordrec",/*tp_name*/
    sizeof(wordrec_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)wordrec_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)wordrec_docstring_,       /* tp_doc */
    (traverseproc)wordrec_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)wordrec_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    wordrec_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)wordrec_init_,  /* tp_init */
    0,                         /* tp_alloc */
    wordrec_new_,             /* tp_new */
};

PyObject *wordrec_wrapcopy_(wordrec_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *wordrec_wrap_(wordrec_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = wordrec_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    wordrec_object_ *self = 
        
        PyObject_GC_New(wordrec_object_, &wordrec_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        wordrec_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

wordrec_type_ *wordrec_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &wordrec_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected wordrec object");
        return 0;
    }

    wordrec_type_ *object = static_cast<wordrec_type_ *>(((wordrec_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null wordrec object in unwrap");
        return 0;
    }

    return object;
}

void wordrec_insert_(const std::pair< wordrec_type_ *, PyObject *> &p)
{




    wordrec_wrapped_.insert(p);


}

void wordrec_erase_(wordrec_type_ *p)
{




    wordrec_wrapped_.erase(p);


}

PyObject * wordrec_find_(wordrec_type_ *p)
{




    PyObject *ret = 0;
    std::map< wordrec_type_ *, PyObject *>::iterator where = wordrec_wrapped_.find(p);
    if(where != wordrec_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct wordrec_dispatch_ wordrec_realdispatcher_ =
{
    &wordrec_typeobject_,
    wordrec_wrap_,
    wordrec_wrapcopy_,
    wordrec_unwrap_,
    wordrec_insert_,
    wordrec_erase_,
    wordrec_find_
};

  

struct toggle_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct toggle_wrapper_: public toggle_type_ 
{

    static void __deleter__(toggle_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    toggle_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_type_ & a0  ): toggle_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~toggle_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(toggle_object_ *self, visitproc visit, void *arg)
    {
        toggle_wrapper_ *o = (toggle_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(toggle_object_ *self)
    {
        toggle_wrapper_ *o = (toggle_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void reset_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return toggle_type_::reset( a0 );
                
            
            
        }

        return
            
                reset
            
            
                ( a0 );
    }

    static PyObject *reset_method_(toggle_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        toggle_wrapper_ *o = (toggle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null toggle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in toggle::reset");
        }

        return 0;
    }





    static PyObject *_method_lock(toggle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(toggle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void toggle_delete_(toggle_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (toggle_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        toggle_wrapper_::__deleter__((toggle_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void toggle_dealloc_(toggle_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    toggle_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * toggle_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    toggle_object_ *self;

    if(!(self = (toggle_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int toggle_init_(toggle_object_ *self, PyObject *args, PyObject *kwds)
{
    toggle_delete_(self);

    

    {
         data_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_data_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new toggle_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing toggle");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"toggle invalid arguments");
    return -1;

}


static const char *toggle_reset_docstring_ = "";




static PyMethodDef toggle_methods_[] = {
    
        { "reset",(PyCFunction)toggle_wrapper_::reset_method_,METH_VARARGS,(char *)toggle_reset_docstring_ },
    
    
        { "py_lock",(PyCFunction)toggle_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)toggle_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *toggle_docstring_ = "";

PyTypeObject toggle_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.toggle",/*tp_name*/
    sizeof(toggle_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)toggle_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)toggle_docstring_,       /* tp_doc */
    (traverseproc)toggle_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)toggle_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    toggle_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)toggle_init_,  /* tp_init */
    0,                         /* tp_alloc */
    toggle_new_,             /* tp_new */
};

PyObject *toggle_wrapcopy_(toggle_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *toggle_wrap_(toggle_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    toggle_object_ *self = 
        
        PyObject_GC_New(toggle_object_, &toggle_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

toggle_type_ *toggle_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &toggle_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected toggle object");
        return 0;
    }

    toggle_type_ *object = static_cast<toggle_type_ *>(((toggle_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null toggle object in unwrap");
        return 0;
    }

    return object;
}

void toggle_insert_(const std::pair< toggle_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void toggle_erase_(toggle_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * toggle_find_(toggle_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct toggle_dispatch_ toggle_realdispatcher_ =
{
    &toggle_typeobject_,
    toggle_wrap_,
    toggle_wrapcopy_,
    toggle_unwrap_,
    toggle_insert_,
    toggle_erase_,
    toggle_find_
};

  

struct updown_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct updown_wrapper_: public updown_type_ 
{

    static void __deleter__(updown_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    updown_wrapper_( PyObject *o, PyInterpreterState *i  ,const data_type_ & a0  ,float a1  ,float a2  ): updown_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~updown_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(updown_object_ *self, visitproc visit, void *arg)
    {
        updown_wrapper_ *o = (updown_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(updown_object_ *self)
    {
        updown_wrapper_ *o = (updown_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void reset_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return updown_type_::reset( a0 );
                
            
            
        }

        return
            
                reset
            
            
                ( a0 );
    }

    static PyObject *reset_method_(updown_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        updown_wrapper_ *o = (updown_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null updown c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in updown::reset");
        }

        return 0;
    }





    static PyObject *_method_lock(updown_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(updown_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void updown_delete_(updown_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (updown_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        updown_wrapper_::__deleter__((updown_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void updown_dealloc_(updown_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    updown_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * updown_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    updown_object_ *self;

    if(!(self = (updown_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int updown_init_(updown_object_ *self, PyObject *args, PyObject *kwds)
{
    updown_delete_(self);

    

    {
         data_type_ * a0;  float a1;  float a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_data_copy,&a0,fpcvt_f,&a1,fpcvt_f,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new updown_wrapper_((PyObject *)self,interp ,*a0,a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing updown");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"updown invalid arguments");
    return -1;

}


static const char *updown_reset_docstring_ = "";




static PyMethodDef updown_methods_[] = {
    
        { "reset",(PyCFunction)updown_wrapper_::reset_method_,METH_VARARGS,(char *)updown_reset_docstring_ },
    
    
        { "py_lock",(PyCFunction)updown_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)updown_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *updown_docstring_ = "";

PyTypeObject updown_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.updown",/*tp_name*/
    sizeof(updown_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)updown_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)updown_docstring_,       /* tp_doc */
    (traverseproc)updown_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)updown_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    updown_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)updown_init_,  /* tp_init */
    0,                         /* tp_alloc */
    updown_new_,             /* tp_new */
};

PyObject *updown_wrapcopy_(updown_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *updown_wrap_(updown_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    updown_object_ *self = 
        
        PyObject_GC_New(updown_object_, &updown_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

updown_type_ *updown_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &updown_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected updown object");
        return 0;
    }

    updown_type_ *object = static_cast<updown_type_ *>(((updown_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null updown object in unwrap");
        return 0;
    }

    return object;
}

void updown_insert_(const std::pair< updown_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void updown_erase_(updown_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * updown_find_(updown_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct updown_dispatch_ updown_realdispatcher_ =
{
    &updown_typeobject_,
    updown_wrap_,
    updown_wrapcopy_,
    updown_unwrap_,
    updown_insert_,
    updown_erase_,
    updown_find_
};

  

struct xselector_object_ {
    PyObject_HEAD
    xxcontrolled_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct xselector_wrapper_: public xselector_type_ 
{

    static void __deleter__(xselector_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    xselector_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ,const data_type_ & a1  ): xselector_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~xselector_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(xselector_object_ *self, visitproc visit, void *arg)
    {
        xselector_wrapper_ *o = (xselector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(xselector_object_ *self)
    {
        xselector_wrapper_ *o = (xselector_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void set_choice_call_(bool owned ,unsigned int a0 ,const data_type_ & a1 )
    {
        if(owned)
        {
            
                
                
                    return xselector_type_::set_choice( a0  ,a1 );
                
            
            
        }

        return
            
                set_choice
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_choice_method_(xselector_object_ *self, PyObject *args)
    {
         unsigned int a0;  data_type_ * a1; 

        xselector_wrapper_ *o = (xselector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xselector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_data_copy,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_choice_call_ (self->owned ,a0,*a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xselector::set_choice");
        }

        return 0;
    }



    void reset_call_(bool owned ,const data_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return xselector_type_::reset( a0 );
                
            
            
        }

        return
            
                reset
            
            
                ( a0 );
    }

    static PyObject *reset_method_(xselector_object_ *self, PyObject *args)
    {
         data_type_ * a0; 

        xselector_wrapper_ *o = (xselector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xselector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_data_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xselector::reset");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return xselector_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(xselector_object_ *self, PyObject *args)
    {
        

        xselector_wrapper_ *o = (xselector_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xselector c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xselector::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(xselector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(xselector_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void xselector_delete_(xselector_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xxcontrolled_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (xselector_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        xselector_wrapper_::__deleter__((xselector_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void xselector_dealloc_(xselector_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    xselector_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * xselector_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    xselector_object_ *self;

    if(!(self = (xselector_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int xselector_init_(xselector_object_ *self, PyObject *args, PyObject *kwds)
{
    xselector_delete_(self);

    

    {
         change_nb_type_ * a0;  data_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_data_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new xselector_wrapper_((PyObject *)self,interp ,*a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing xselector");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"xselector invalid arguments");
    return -1;

}


static const char *xselector_set_choice_docstring_ = "";

static const char *xselector_reset_docstring_ = "";

static const char *xselector_cookie_docstring_ = "";




static PyMethodDef xselector_methods_[] = {
    
        { "set_choice",(PyCFunction)xselector_wrapper_::set_choice_method_,METH_VARARGS,(char *)xselector_set_choice_docstring_ },
    
        { "reset",(PyCFunction)xselector_wrapper_::reset_method_,METH_VARARGS,(char *)xselector_reset_docstring_ },
    
        { "cookie",(PyCFunction)xselector_wrapper_::cookie_method_,METH_VARARGS,(char *)xselector_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)xselector_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)xselector_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *xselector_docstring_ = "";

PyTypeObject xselector_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "language_native.xselector",/*tp_name*/
    sizeof(xselector_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)xselector_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)xselector_docstring_,       /* tp_doc */
    (traverseproc)xselector_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)xselector_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    xselector_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)xselector_init_,  /* tp_init */
    0,                         /* tp_alloc */
    xselector_new_,             /* tp_new */
};

PyObject *xselector_wrapcopy_(xselector_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *xselector_wrap_(xselector_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xxcontrolled_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    xselector_object_ *self = 
        
        PyObject_GC_New(xselector_object_, &xselector_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xxcontrolled_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

xselector_type_ *xselector_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &xselector_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected xselector object");
        return 0;
    }

    xselector_type_ *object = static_cast<xselector_type_ *>(((xselector_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null xselector object in unwrap");
        return 0;
    }

    return object;
}

void xselector_insert_(const std::pair< xselector_type_ *, PyObject *> &p)
{


    xxcontrolled_dispatcher_->insert(p);




}

void xselector_erase_(xselector_type_ *p)
{


    xxcontrolled_dispatcher_->erase(p);




}

PyObject * xselector_find_(xselector_type_ *p)
{


    return xxcontrolled_dispatcher_->find(p);




}

static struct xselector_dispatch_ xselector_realdispatcher_ =
{
    &xselector_typeobject_,
    xselector_wrap_,
    xselector_wrapcopy_,
    xselector_unwrap_,
    xselector_insert_,
    xselector_erase_,
    xselector_find_
};

               





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initlanguage_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            bonjour_dispatcher_ = &bonjour_realdispatcher_;
        
    
        
            oscserver_dispatcher_ = &oscserver_realdispatcher_;
        
    
        
            wordrec_dispatcher_ = &wordrec_realdispatcher_;
        
    
        
            toggle_dispatcher_ = &toggle_realdispatcher_;
        
    
        
            updown_dispatcher_ = &updown_realdispatcher_;
        
    
        
            xselector_dispatcher_ = &xselector_realdispatcher_;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_nb_dispatch in module piw");
                return;
            }

            data_nb_dispatcher_ = (data_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_xxcontrolled_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_xxcontrolled_dispatch in module piw");
                return;
            }

            xxcontrolled_dispatcher_ = (xxcontrolled_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_fastdata_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_fastdata_dispatch in module piw");
                return;
            }

            fastdata_dispatcher_ = (fastdata_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
                toggle_typeobject_.tp_base = xxcontrolled_dispatcher_->type;
            
        
    
        
            
                updown_typeobject_.tp_base = xxcontrolled_dispatcher_->type;
            
        
    
        
            
                xselector_typeobject_.tp_base = xxcontrolled_dispatcher_->type;
            
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&bonjour_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&oscserver_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&wordrec_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&toggle_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&updown_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&xselector_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("language_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(bonjour_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_bonjour_dispatch",s);
            Py_INCREF(&bonjour_typeobject_);
            PyModule_AddObject(m,"bonjour",(PyObject *)&bonjour_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(oscserver_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_oscserver_dispatch",s);
            Py_INCREF(&oscserver_typeobject_);
            PyModule_AddObject(m,"oscserver",(PyObject *)&oscserver_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(wordrec_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_wordrec_dispatch",s);
            Py_INCREF(&wordrec_typeobject_);
            PyModule_AddObject(m,"wordrec",(PyObject *)&wordrec_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(toggle_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_toggle_dispatch",s);
            Py_INCREF(&toggle_typeobject_);
            PyModule_AddObject(m,"toggle",(PyObject *)&toggle_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(updown_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_updown_dispatch",s);
            Py_INCREF(&updown_typeobject_);
            PyModule_AddObject(m,"updown",(PyObject *)&updown_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(xselector_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_xselector_dispatch",s);
            Py_INCREF(&xselector_typeobject_);
            PyModule_AddObject(m,"xselector",(PyObject *)&xselector_typeobject_);
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

