#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "loop_bundle.h"
#include "loop_pinger.h"
#include "loop_file.h"
#include "loop_clicker.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef loop::player_t player_type_; 


 
static std::map<player_type_ *, PyObject *> player_wrapped_;
 


struct player_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(player_type_ *, bool owned);
    PyObject *(*wrapcopy)(player_type_ *);
    player_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< player_type_ *, PyObject * > &);
    void (*erase)(player_type_ *);
    PyObject *(*find)(player_type_ *);
};

struct player_dispatch_ *player_dispatcher_;



typedef loop::pinger_t pinger_type_; 


 
static std::map<pinger_type_ *, PyObject *> pinger_wrapped_;
 


struct pinger_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(pinger_type_ *, bool owned);
    PyObject *(*wrapcopy)(pinger_type_ *);
    pinger_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< pinger_type_ *, PyObject * > &);
    void (*erase)(pinger_type_ *);
    PyObject *(*find)(pinger_type_ *);
};

struct pinger_dispatch_ *pinger_dispatcher_;



typedef loop::clicker_t clicker_type_; 


 
static std::map<clicker_type_ *, PyObject *> clicker_wrapped_;
 


struct clicker_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clicker_type_ *, bool owned);
    PyObject *(*wrapcopy)(clicker_type_ *);
    clicker_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clicker_type_ *, PyObject * > &);
    void (*erase)(clicker_type_ *);
    PyObject *(*find)(clicker_type_ *);
};

struct clicker_dispatch_ *clicker_dispatcher_;



typedef loop::xplayer_t xplayer_type_; 


 
static std::map<xplayer_type_ *, PyObject *> xplayer_wrapped_;
 


struct xplayer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(xplayer_type_ *, bool owned);
    PyObject *(*wrapcopy)(xplayer_type_ *);
    xplayer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< xplayer_type_ *, PyObject * > &);
    void (*erase)(xplayer_type_ *);
    PyObject *(*find)(xplayer_type_ *);
};

struct xplayer_dispatch_ *xplayer_dispatcher_;



typedef loop::samplearray2ref_t samplearray2_type_; 


 
static std::map<samplearray2_type_ *, PyObject *> samplearray2_wrapped_;
 


struct samplearray2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(samplearray2_type_ *, bool owned);
    PyObject *(*wrapcopy)(samplearray2_type_ *);
    samplearray2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< samplearray2_type_ *, PyObject * > &);
    void (*erase)(samplearray2_type_ *);
    PyObject *(*find)(samplearray2_type_ *);
};

struct samplearray2_dispatch_ *samplearray2_dispatcher_;



typedef loop::loopref_t loopref_type_; 


 
static std::map<loopref_type_ *, PyObject *> loopref_wrapped_;
 


struct loopref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(loopref_type_ *, bool owned);
    PyObject *(*wrapcopy)(loopref_type_ *);
    loopref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< loopref_type_ *, PyObject * > &);
    void (*erase)(loopref_type_ *);
    PyObject *(*find)(loopref_type_ *);
};

struct loopref_dispatch_ *loopref_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_player(PyObject *o, void *a)
{
    player_type_ *object;
    if(o == Py_None)
    {
        *((player_type_ **)a) = 0;
        return 1;
    }
    object = player_dispatcher_->unwrap(o);
    *((player_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_player_copy fpcvt_player

PyObject *tpcvt_player(void *a)
{
    player_type_ *o = *((player_type_ **)a);
    return player_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_player_copy(void *a)
{
    player_type_ *o = (player_type_ *)a;
    return player_dispatcher_->wrapcopy(o);
}



int fpcvt_pinger(PyObject *o, void *a)
{
    pinger_type_ *object;
    if(o == Py_None)
    {
        *((pinger_type_ **)a) = 0;
        return 1;
    }
    object = pinger_dispatcher_->unwrap(o);
    *((pinger_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_pinger_copy fpcvt_pinger

PyObject *tpcvt_pinger(void *a)
{
    pinger_type_ *o = *((pinger_type_ **)a);
    return pinger_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_pinger_copy(void *a)
{
    pinger_type_ *o = (pinger_type_ *)a;
    return pinger_dispatcher_->wrapcopy(o);
}



int fpcvt_clicker(PyObject *o, void *a)
{
    clicker_type_ *object;
    if(o == Py_None)
    {
        *((clicker_type_ **)a) = 0;
        return 1;
    }
    object = clicker_dispatcher_->unwrap(o);
    *((clicker_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clicker_copy fpcvt_clicker

PyObject *tpcvt_clicker(void *a)
{
    clicker_type_ *o = *((clicker_type_ **)a);
    return clicker_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clicker_copy(void *a)
{
    clicker_type_ *o = (clicker_type_ *)a;
    return clicker_dispatcher_->wrapcopy(o);
}



int fpcvt_xplayer(PyObject *o, void *a)
{
    xplayer_type_ *object;
    if(o == Py_None)
    {
        *((xplayer_type_ **)a) = 0;
        return 1;
    }
    object = xplayer_dispatcher_->unwrap(o);
    *((xplayer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_xplayer_copy fpcvt_xplayer

PyObject *tpcvt_xplayer(void *a)
{
    xplayer_type_ *o = *((xplayer_type_ **)a);
    return xplayer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_xplayer_copy(void *a)
{
    xplayer_type_ *o = (xplayer_type_ *)a;
    return xplayer_dispatcher_->wrapcopy(o);
}



int fpcvt_samplearray2(PyObject *o, void *a)
{
    samplearray2_type_ *object;
    if(o == Py_None)
    {
        *((samplearray2_type_ **)a) = 0;
        return 1;
    }
    object = samplearray2_dispatcher_->unwrap(o);
    *((samplearray2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_samplearray2_copy fpcvt_samplearray2

PyObject *tpcvt_samplearray2(void *a)
{
    samplearray2_type_ *o = *((samplearray2_type_ **)a);
    return samplearray2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_samplearray2_copy(void *a)
{
    samplearray2_type_ *o = (samplearray2_type_ *)a;
    return samplearray2_dispatcher_->wrapcopy(o);
}



int fpcvt_loopref(PyObject *o, void *a)
{
    loopref_type_ *object;
    if(o == Py_None)
    {
        *((loopref_type_ **)a) = 0;
        return 1;
    }
    object = loopref_dispatcher_->unwrap(o);
    *((loopref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_loopref_copy fpcvt_loopref

PyObject *tpcvt_loopref(void *a)
{
    loopref_type_ *o = *((loopref_type_ **)a);
    return loopref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_loopref_copy(void *a)
{
    loopref_type_ *o = (loopref_type_ *)a;
    return loopref_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct player_object_ {
    PyObject_HEAD
    player_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct player_wrapper_: public player_type_ 
{

    static void __deleter__(player_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    player_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): player_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~player_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void load_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return player_type_::load( a0 );
                
            
            
        }

        return
            
                load
            
            
                ( a0 );
    }

    static PyObject *load_method_(player_object_ *self, PyObject *args)
    {
         const char * a0; 

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::load");
        }

        return 0;
    }



    void unload_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return player_type_::unload();
                
            
            
        }

        return
            
                unload
            
            
                ();
    }

    static PyObject *unload_method_(player_object_ *self, PyObject *args)
    {
        

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unload_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::unload");
        }

        return 0;
    }



    void set_volume_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return player_type_::set_volume( a0 );
                
            
            
        }

        return
            
                set_volume
            
            
                ( a0 );
    }

    static PyObject *set_volume_method_(player_object_ *self, PyObject *args)
    {
         float a0; 

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_volume_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::set_volume");
        }

        return 0;
    }



    float get_volume_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return player_type_::get_volume();
                
            
            
        }

        return
            
                get_volume
            
            
                ();
    }

    static PyObject *get_volume_method_(player_object_ *self, PyObject *args)
    {
        

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->get_volume_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::get_volume");
        }

        return 0;
    }



    void set_chop_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return player_type_::set_chop( a0 );
                
            
            
        }

        return
            
                set_chop
            
            
                ( a0 );
    }

    static PyObject *set_chop_method_(player_object_ *self, PyObject *args)
    {
         float a0; 

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_chop_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::set_chop");
        }

        return 0;
    }



    change_nb_type_ player_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return player_type_::player( a0 );
                
            
            
        }

        return
            
                player
            
            
                ( a0 );
    }

    static PyObject *player_method_(player_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->player_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::player");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return player_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(player_object_ *self, PyObject *args)
    {
        

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(player_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(player_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void player_delete_(player_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            player_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (player_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        player_wrapper_::__deleter__((player_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void player_dealloc_(player_object_* self)
{
    
    player_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * player_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    player_object_ *self;

    if(!(self = (player_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int player_init_(player_object_ *self, PyObject *args, PyObject *kwds)
{
    player_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new player_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing player");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            player_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"player invalid arguments");
    return -1;

}


static const char *player_load_docstring_ = "";

static const char *player_unload_docstring_ = "";

static const char *player_set_volume_docstring_ = "";

static const char *player_get_volume_docstring_ = "";

static const char *player_set_chop_docstring_ = "";

static const char *player_player_docstring_ = "";

static const char *player_cookie_docstring_ = "";




static PyMethodDef player_methods_[] = {
    
        { "load",(PyCFunction)player_wrapper_::load_method_,METH_VARARGS,(char *)player_load_docstring_ },
    
        { "unload",(PyCFunction)player_wrapper_::unload_method_,METH_VARARGS,(char *)player_unload_docstring_ },
    
        { "set_volume",(PyCFunction)player_wrapper_::set_volume_method_,METH_VARARGS,(char *)player_set_volume_docstring_ },
    
        { "get_volume",(PyCFunction)player_wrapper_::get_volume_method_,METH_VARARGS,(char *)player_get_volume_docstring_ },
    
        { "set_chop",(PyCFunction)player_wrapper_::set_chop_method_,METH_VARARGS,(char *)player_set_chop_docstring_ },
    
        { "player",(PyCFunction)player_wrapper_::player_method_,METH_VARARGS,(char *)player_player_docstring_ },
    
        { "cookie",(PyCFunction)player_wrapper_::cookie_method_,METH_VARARGS,(char *)player_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)player_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)player_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *player_docstring_ = "";

PyTypeObject player_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.player",/*tp_name*/
    sizeof(player_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)player_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)player_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    player_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)player_init_,  /* tp_init */
    0,                         /* tp_alloc */
    player_new_,             /* tp_new */
};

PyObject *player_wrapcopy_(player_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *player_wrap_(player_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = player_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    player_object_ *self = 
        PyObject_NEW(player_object_, &player_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        player_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

player_type_ *player_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &player_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected player object");
        return 0;
    }

    player_type_ *object = static_cast<player_type_ *>(((player_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null player object in unwrap");
        return 0;
    }

    return object;
}

void player_insert_(const std::pair< player_type_ *, PyObject *> &p)
{




    player_wrapped_.insert(p);


}

void player_erase_(player_type_ *p)
{




    player_wrapped_.erase(p);


}

PyObject * player_find_(player_type_ *p)
{




    PyObject *ret = 0;
    std::map< player_type_ *, PyObject *>::iterator where = player_wrapped_.find(p);
    if(where != player_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct player_dispatch_ player_realdispatcher_ =
{
    &player_typeobject_,
    player_wrap_,
    player_wrapcopy_,
    player_unwrap_,
    player_insert_,
    player_erase_,
    player_find_
};

  

struct pinger_object_ {
    PyObject_HEAD
    pinger_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct pinger_wrapper_: public pinger_type_ 
{

    static void __deleter__(pinger_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    pinger_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,const cookie_type_ & a1  ,const cookie_type_ & a2  ,clockdomain_ctl_type_ * a3  ,const change_type_ & a4  ,const change_type_ & a5  ,const change_type_ & a6  ): pinger_type_(  a0 ,a1 ,a2 ,a3 ,a4 ,a5 ,a6 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~pinger_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(pinger_object_ *self, visitproc visit, void *arg)
    {
        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(pinger_object_ *self)
    {
        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(pinger_object_ *self, PyObject *args)
    {
        

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::cookie");
        }

        return 0;
    }



    void set_tempo_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::set_tempo( a0 );
                
            
            
        }

        return
            
                set_tempo
            
            
                ( a0 );
    }

    static PyObject *set_tempo_method_(pinger_object_ *self, PyObject *args)
    {
         float a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_tempo_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::set_tempo");
        }

        return 0;
    }



    void set_beats_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::set_beats( a0 );
                
            
            
        }

        return
            
                set_beats
            
            
                ( a0 );
    }

    static PyObject *set_beats_method_(pinger_object_ *self, PyObject *args)
    {
         float a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_beats_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::set_beats");
        }

        return 0;
    }



    void set_range_call_(bool owned ,float a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::set_range( a0  ,a1 );
                
            
            
        }

        return
            
                set_range
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_range_method_(pinger_object_ *self, PyObject *args)
    {
         float a0;  float a1; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_range_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::set_range");
        }

        return 0;
    }



    void start_preroll_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::start_preroll( a0 );
                
            
            
        }

        return
            
                start_preroll
            
            
                ( a0 );
    }

    static PyObject *start_preroll_method_(pinger_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_preroll_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::start_preroll");
        }

        return 0;
    }



    void midi_clock_enable_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::midi_clock_enable( a0 );
                
            
            
        }

        return
            
                midi_clock_enable
            
            
                ( a0 );
    }

    static PyObject *midi_clock_enable_method_(pinger_object_ *self, PyObject *args)
    {
         bool a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->midi_clock_enable_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::midi_clock_enable");
        }

        return 0;
    }



    void midi_clock_set_latency_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::midi_clock_set_latency( a0 );
                
            
            
        }

        return
            
                midi_clock_set_latency
            
            
                ( a0 );
    }

    static PyObject *midi_clock_set_latency_method_(pinger_object_ *self, PyObject *args)
    {
         float a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->midi_clock_set_latency_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::midi_clock_set_latency");
        }

        return 0;
    }



    void set_beat_flash_persistence_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::set_beat_flash_persistence( a0 );
                
            
            
        }

        return
            
                set_beat_flash_persistence
            
            
                ( a0 );
    }

    static PyObject *set_beat_flash_persistence_method_(pinger_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_beat_flash_persistence_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::set_beat_flash_persistence");
        }

        return 0;
    }



    void play_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::play();
                
            
            
        }

        return
            
                play
            
            
                ();
    }

    static PyObject *play_method_(pinger_object_ *self, PyObject *args)
    {
        

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->play_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::play");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(pinger_object_ *self, PyObject *args)
    {
        

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::stop");
        }

        return 0;
    }



    void toggle_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return pinger_type_::toggle();
                
            
            
        }

        return
            
                toggle
            
            
                ();
    }

    static PyObject *toggle_method_(pinger_object_ *self, PyObject *args)
    {
        

        pinger_wrapper_ *o = (pinger_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null pinger c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->toggle_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in pinger::toggle");
        }

        return 0;
    }





    static PyObject *_method_lock(pinger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(pinger_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void pinger_delete_(pinger_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            pinger_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (pinger_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        pinger_wrapper_::__deleter__((pinger_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void pinger_dealloc_(pinger_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    pinger_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * pinger_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    pinger_object_ *self;

    if(!(self = (pinger_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int pinger_init_(pinger_object_ *self, PyObject *args, PyObject *kwds)
{
    pinger_delete_(self);

    

    {
         cookie_type_ * a0;  cookie_type_ * a1;  cookie_type_ * a2;  clockdomain_ctl_type_ * a3;  change_type_ * a4;  change_type_ * a5;  change_type_ * a6; 

        if(PyArg_ParseTuple(args,"O&O&O&O&O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_cookie_copy,&a1,fpcvt_cookie_copy,&a2,fpcvt_clockdomain_ctl,&a3,fpcvt_change_copy,&a4,fpcvt_change_copy,&a5,fpcvt_change_copy,&a6))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new pinger_wrapper_((PyObject *)self,interp ,*a0,*a1,*a2,a3,*a4,*a5,*a6);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing pinger");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            pinger_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"pinger invalid arguments");
    return -1;

}


static const char *pinger_cookie_docstring_ = "";

static const char *pinger_set_tempo_docstring_ = "";

static const char *pinger_set_beats_docstring_ = "";

static const char *pinger_set_range_docstring_ = "";

static const char *pinger_start_preroll_docstring_ = "";

static const char *pinger_midi_clock_enable_docstring_ = "";

static const char *pinger_midi_clock_set_latency_docstring_ = "";

static const char *pinger_set_beat_flash_persistence_docstring_ = "";

static const char *pinger_play_docstring_ = "";

static const char *pinger_stop_docstring_ = "";

static const char *pinger_toggle_docstring_ = "";




static PyMethodDef pinger_methods_[] = {
    
        { "cookie",(PyCFunction)pinger_wrapper_::cookie_method_,METH_VARARGS,(char *)pinger_cookie_docstring_ },
    
        { "set_tempo",(PyCFunction)pinger_wrapper_::set_tempo_method_,METH_VARARGS,(char *)pinger_set_tempo_docstring_ },
    
        { "set_beats",(PyCFunction)pinger_wrapper_::set_beats_method_,METH_VARARGS,(char *)pinger_set_beats_docstring_ },
    
        { "set_range",(PyCFunction)pinger_wrapper_::set_range_method_,METH_VARARGS,(char *)pinger_set_range_docstring_ },
    
        { "start_preroll",(PyCFunction)pinger_wrapper_::start_preroll_method_,METH_VARARGS,(char *)pinger_start_preroll_docstring_ },
    
        { "midi_clock_enable",(PyCFunction)pinger_wrapper_::midi_clock_enable_method_,METH_VARARGS,(char *)pinger_midi_clock_enable_docstring_ },
    
        { "midi_clock_set_latency",(PyCFunction)pinger_wrapper_::midi_clock_set_latency_method_,METH_VARARGS,(char *)pinger_midi_clock_set_latency_docstring_ },
    
        { "set_beat_flash_persistence",(PyCFunction)pinger_wrapper_::set_beat_flash_persistence_method_,METH_VARARGS,(char *)pinger_set_beat_flash_persistence_docstring_ },
    
        { "play",(PyCFunction)pinger_wrapper_::play_method_,METH_VARARGS,(char *)pinger_play_docstring_ },
    
        { "stop",(PyCFunction)pinger_wrapper_::stop_method_,METH_VARARGS,(char *)pinger_stop_docstring_ },
    
        { "toggle",(PyCFunction)pinger_wrapper_::toggle_method_,METH_VARARGS,(char *)pinger_toggle_docstring_ },
    
    
        { "py_lock",(PyCFunction)pinger_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)pinger_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *pinger_docstring_ = "";

PyTypeObject pinger_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.pinger",/*tp_name*/
    sizeof(pinger_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)pinger_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)pinger_docstring_,       /* tp_doc */
    (traverseproc)pinger_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)pinger_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    pinger_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)pinger_init_,  /* tp_init */
    0,                         /* tp_alloc */
    pinger_new_,             /* tp_new */
};

PyObject *pinger_wrapcopy_(pinger_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *pinger_wrap_(pinger_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = pinger_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    pinger_object_ *self = 
        
        PyObject_GC_New(pinger_object_, &pinger_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        pinger_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

pinger_type_ *pinger_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &pinger_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected pinger object");
        return 0;
    }

    pinger_type_ *object = static_cast<pinger_type_ *>(((pinger_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null pinger object in unwrap");
        return 0;
    }

    return object;
}

void pinger_insert_(const std::pair< pinger_type_ *, PyObject *> &p)
{




    pinger_wrapped_.insert(p);


}

void pinger_erase_(pinger_type_ *p)
{




    pinger_wrapped_.erase(p);


}

PyObject * pinger_find_(pinger_type_ *p)
{




    PyObject *ret = 0;
    std::map< pinger_type_ *, PyObject *>::iterator where = pinger_wrapped_.find(p);
    if(where != pinger_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct pinger_dispatch_ pinger_realdispatcher_ =
{
    &pinger_typeobject_,
    pinger_wrap_,
    pinger_wrapcopy_,
    pinger_unwrap_,
    pinger_insert_,
    pinger_erase_,
    pinger_find_
};

  

struct clicker_object_ {
    PyObject_HEAD
    clicker_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clicker_wrapper_: public clicker_type_ 
{

    static void __deleter__(clicker_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clicker_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ,const samplearray2_type_ & a2  ,const samplearray2_type_ & a3  ): clicker_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clicker_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clicker_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(clicker_object_ *self, PyObject *args)
    {
        

        clicker_wrapper_ *o = (clicker_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clicker c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clicker::cookie");
        }

        return 0;
    }



    void play_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return clicker_type_::play( a0 );
                
            
            
        }

        return
            
                play
            
            
                ( a0 );
    }

    static PyObject *play_method_(clicker_object_ *self, PyObject *args)
    {
         bool a0; 

        clicker_wrapper_ *o = (clicker_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clicker c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->play_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clicker::play");
        }

        return 0;
    }





    static PyObject *_method_lock(clicker_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clicker_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clicker_delete_(clicker_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clicker_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clicker_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clicker_wrapper_::__deleter__((clicker_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clicker_dealloc_(clicker_object_* self)
{
    
    clicker_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clicker_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clicker_object_ *self;

    if(!(self = (clicker_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clicker_init_(clicker_object_ *self, PyObject *args, PyObject *kwds)
{
    clicker_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1;  samplearray2_type_ * a2;  samplearray2_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1,fpcvt_samplearray2_copy,&a2,fpcvt_samplearray2_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clicker_wrapper_((PyObject *)self,interp ,*a0,a1,*a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clicker");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clicker_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clicker invalid arguments");
    return -1;

}


static const char *clicker_cookie_docstring_ = "";

static const char *clicker_play_docstring_ = "";




static PyMethodDef clicker_methods_[] = {
    
        { "cookie",(PyCFunction)clicker_wrapper_::cookie_method_,METH_VARARGS,(char *)clicker_cookie_docstring_ },
    
        { "play",(PyCFunction)clicker_wrapper_::play_method_,METH_VARARGS,(char *)clicker_play_docstring_ },
    
    
        { "py_lock",(PyCFunction)clicker_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clicker_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clicker_docstring_ = "";

PyTypeObject clicker_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.clicker",/*tp_name*/
    sizeof(clicker_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clicker_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)clicker_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clicker_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clicker_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clicker_new_,             /* tp_new */
};

PyObject *clicker_wrapcopy_(clicker_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clicker_wrap_(clicker_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clicker_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clicker_object_ *self = 
        PyObject_NEW(clicker_object_, &clicker_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clicker_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clicker_type_ *clicker_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clicker_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clicker object");
        return 0;
    }

    clicker_type_ *object = static_cast<clicker_type_ *>(((clicker_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clicker object in unwrap");
        return 0;
    }

    return object;
}

void clicker_insert_(const std::pair< clicker_type_ *, PyObject *> &p)
{




    clicker_wrapped_.insert(p);


}

void clicker_erase_(clicker_type_ *p)
{




    clicker_wrapped_.erase(p);


}

PyObject * clicker_find_(clicker_type_ *p)
{




    PyObject *ret = 0;
    std::map< clicker_type_ *, PyObject *>::iterator where = clicker_wrapped_.find(p);
    if(where != clicker_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clicker_dispatch_ clicker_realdispatcher_ =
{
    &clicker_typeobject_,
    clicker_wrap_,
    clicker_wrapcopy_,
    clicker_unwrap_,
    clicker_insert_,
    clicker_erase_,
    clicker_find_
};

  

struct xplayer_object_ {
    PyObject_HEAD
    xplayer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct xplayer_wrapper_: public xplayer_type_ 
{

    static void __deleter__(xplayer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    xplayer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ,const samplearray2_type_ & a2  ): xplayer_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~xplayer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void play_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return xplayer_type_::play( a0 );
                
            
            
        }

        return
            
                play
            
            
                ( a0 );
    }

    static PyObject *play_method_(xplayer_object_ *self, PyObject *args)
    {
         bool a0; 

        xplayer_wrapper_ *o = (xplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->play_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xplayer::play");
        }

        return 0;
    }



    void set_gain_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return xplayer_type_::set_gain( a0 );
                
            
            
        }

        return
            
                set_gain
            
            
                ( a0 );
    }

    static PyObject *set_gain_method_(xplayer_object_ *self, PyObject *args)
    {
         float a0; 

        xplayer_wrapper_ *o = (xplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null xplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_gain_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in xplayer::set_gain");
        }

        return 0;
    }





    static PyObject *_method_lock(xplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(xplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void xplayer_delete_(xplayer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            xplayer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (xplayer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        xplayer_wrapper_::__deleter__((xplayer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void xplayer_dealloc_(xplayer_object_* self)
{
    
    xplayer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * xplayer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    xplayer_object_ *self;

    if(!(self = (xplayer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int xplayer_init_(xplayer_object_ *self, PyObject *args, PyObject *kwds)
{
    xplayer_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1;  samplearray2_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1,fpcvt_samplearray2_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new xplayer_wrapper_((PyObject *)self,interp ,*a0,a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing xplayer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            xplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"xplayer invalid arguments");
    return -1;

}


static const char *xplayer_play_docstring_ = "";

static const char *xplayer_set_gain_docstring_ = "";




static PyMethodDef xplayer_methods_[] = {
    
        { "play",(PyCFunction)xplayer_wrapper_::play_method_,METH_VARARGS,(char *)xplayer_play_docstring_ },
    
        { "set_gain",(PyCFunction)xplayer_wrapper_::set_gain_method_,METH_VARARGS,(char *)xplayer_set_gain_docstring_ },
    
    
        { "py_lock",(PyCFunction)xplayer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)xplayer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *xplayer_docstring_ = "";

PyTypeObject xplayer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.xplayer",/*tp_name*/
    sizeof(xplayer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)xplayer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)xplayer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    xplayer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)xplayer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    xplayer_new_,             /* tp_new */
};

PyObject *xplayer_wrapcopy_(xplayer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *xplayer_wrap_(xplayer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = xplayer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    xplayer_object_ *self = 
        PyObject_NEW(xplayer_object_, &xplayer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        xplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

xplayer_type_ *xplayer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &xplayer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected xplayer object");
        return 0;
    }

    xplayer_type_ *object = static_cast<xplayer_type_ *>(((xplayer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null xplayer object in unwrap");
        return 0;
    }

    return object;
}

void xplayer_insert_(const std::pair< xplayer_type_ *, PyObject *> &p)
{




    xplayer_wrapped_.insert(p);


}

void xplayer_erase_(xplayer_type_ *p)
{




    xplayer_wrapped_.erase(p);


}

PyObject * xplayer_find_(xplayer_type_ *p)
{




    PyObject *ret = 0;
    std::map< xplayer_type_ *, PyObject *>::iterator where = xplayer_wrapped_.find(p);
    if(where != xplayer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct xplayer_dispatch_ xplayer_realdispatcher_ =
{
    &xplayer_typeobject_,
    xplayer_wrap_,
    xplayer_wrapcopy_,
    xplayer_unwrap_,
    xplayer_insert_,
    xplayer_erase_,
    xplayer_find_
};

  

struct samplearray2_object_ {
    PyObject_HEAD
    samplearray2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct samplearray2_wrapper_: public samplearray2_type_ 
{

    static void __deleter__(samplearray2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    samplearray2_wrapper_( PyObject *o, PyInterpreterState *i  ): samplearray2_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    samplearray2_wrapper_( PyObject *o, PyInterpreterState *i  ,const samplearray2_type_ & a0  ): samplearray2_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~samplearray2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }



















    static PyObject *_method_lock(samplearray2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(samplearray2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void samplearray2_delete_(samplearray2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            samplearray2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (samplearray2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        samplearray2_wrapper_::__deleter__((samplearray2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void samplearray2_dealloc_(samplearray2_object_* self)
{
    
    samplearray2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * samplearray2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    samplearray2_object_ *self;

    if(!(self = (samplearray2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int samplearray2_init_(samplearray2_object_ *self, PyObject *args, PyObject *kwds)
{
    samplearray2_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new samplearray2_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing samplearray2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            samplearray2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         samplearray2_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_samplearray2_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new samplearray2_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing samplearray2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            samplearray2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"samplearray2 invalid arguments");
    return -1;

}





static PyMethodDef samplearray2_methods_[] = {
    
    
        { "py_lock",(PyCFunction)samplearray2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)samplearray2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *samplearray2_docstring_ = "";

PyTypeObject samplearray2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.samplearray2",/*tp_name*/
    sizeof(samplearray2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)samplearray2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)samplearray2_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    samplearray2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)samplearray2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    samplearray2_new_,             /* tp_new */
};

PyObject *samplearray2_wrapcopy_(samplearray2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            samplearray2_object_ *obj = 
                PyObject_NEW(samplearray2_object_, samplearray2_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new samplearray2_type_(*o);
                }

                try
                {
                    samplearray2_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying samplearray2");
                return 0;
            }
        

    
}

PyObject *samplearray2_wrap_(samplearray2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = samplearray2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    samplearray2_object_ *self = 
        PyObject_NEW(samplearray2_object_, &samplearray2_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        samplearray2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

samplearray2_type_ *samplearray2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &samplearray2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected samplearray2 object");
        return 0;
    }

    samplearray2_type_ *object = static_cast<samplearray2_type_ *>(((samplearray2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null samplearray2 object in unwrap");
        return 0;
    }

    return object;
}

void samplearray2_insert_(const std::pair< samplearray2_type_ *, PyObject *> &p)
{




    samplearray2_wrapped_.insert(p);


}

void samplearray2_erase_(samplearray2_type_ *p)
{




    samplearray2_wrapped_.erase(p);


}

PyObject * samplearray2_find_(samplearray2_type_ *p)
{




    PyObject *ret = 0;
    std::map< samplearray2_type_ *, PyObject *>::iterator where = samplearray2_wrapped_.find(p);
    if(where != samplearray2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct samplearray2_dispatch_ samplearray2_realdispatcher_ =
{
    &samplearray2_typeobject_,
    samplearray2_wrap_,
    samplearray2_wrapcopy_,
    samplearray2_unwrap_,
    samplearray2_insert_,
    samplearray2_erase_,
    samplearray2_find_
};

  

struct loopref_object_ {
    PyObject_HEAD
    loopref_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct loopref_wrapper_: public loopref_type_ 
{

    static void __deleter__(loopref_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    loopref_wrapper_( PyObject *o, PyInterpreterState *i  ,const loopref_type_ & a0  ): loopref_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~loopref_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned long samples_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->samples();
            
        }

        return
            
            
                (operator->())->samples
            
                ();
    }

    static PyObject *samples_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->samples_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::samples");
        }

        return 0;
    }



    unsigned long sample_rate_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->sample_rate();
            
        }

        return
            
            
                (operator->())->sample_rate
            
                ();
    }

    static PyObject *sample_rate_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->sample_rate_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::sample_rate");
        }

        return 0;
    }



    unsigned long beats_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->beats();
            
        }

        return
            
            
                (operator->())->beats
            
                ();
    }

    static PyObject *beats_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long r = o->beats_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::beats");
        }

        return 0;
    }



    unsigned short note_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->note();
            
        }

        return
            
            
                (operator->())->note
            
                ();
    }

    static PyObject *note_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned short r = o->note_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::note");
        }

        return 0;
    }



    unsigned short scale_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->scale();
            
        }

        return
            
            
                (operator->())->scale
            
                ();
    }

    static PyObject *scale_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned short r = o->scale_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::scale");
        }

        return 0;
    }



    unsigned short timesig_numerator_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->timesig_numerator();
            
        }

        return
            
            
                (operator->())->timesig_numerator
            
                ();
    }

    static PyObject *timesig_numerator_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned short r = o->timesig_numerator_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::timesig_numerator");
        }

        return 0;
    }



    unsigned short timesig_denominator_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->timesig_denominator();
            
        }

        return
            
            
                (operator->())->timesig_denominator
            
                ();
    }

    static PyObject *timesig_denominator_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned short r = o->timesig_denominator_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::timesig_denominator");
        }

        return 0;
    }



    unsigned short looping_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->looping();
            
        }

        return
            
            
                (operator->())->looping
            
                ();
    }

    static PyObject *looping_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned short r = o->looping_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_us((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::looping");
        }

        return 0;
    }



    unsigned int ntags_call_(bool owned )
    {
        if(owned)
        {
            
            
                return (operator->())->ntags();
            
        }

        return
            
            
                (operator->())->ntags
            
                ();
    }

    static PyObject *ntags_method_(loopref_object_ *self, PyObject *args)
    {
        

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->ntags_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::ntags");
        }

        return 0;
    }



    std::string tag_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
            
                return (operator->())->tag( a0 );
            
        }

        return
            
            
                (operator->())->tag
            
                ( a0 );
    }

    static PyObject *tag_method_(loopref_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        loopref_wrapper_ *o = (loopref_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loopref c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    std::string r = o->tag_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loopref::tag");
        }

        return 0;
    }





    static PyObject *_method_lock(loopref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(loopref_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void loopref_delete_(loopref_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            loopref_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (loopref_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        loopref_wrapper_::__deleter__((loopref_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void loopref_dealloc_(loopref_object_* self)
{
    
    loopref_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * loopref_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    loopref_object_ *self;

    if(!(self = (loopref_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int loopref_init_(loopref_object_ *self, PyObject *args, PyObject *kwds)
{
    loopref_delete_(self);

    

    {
         loopref_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_loopref_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new loopref_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing loopref");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            loopref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"loopref invalid arguments");
    return -1;

}


static const char *loopref_samples_docstring_ = "";

static const char *loopref_sample_rate_docstring_ = "";

static const char *loopref_beats_docstring_ = "";

static const char *loopref_note_docstring_ = "";

static const char *loopref_scale_docstring_ = "";

static const char *loopref_timesig_numerator_docstring_ = "";

static const char *loopref_timesig_denominator_docstring_ = "";

static const char *loopref_looping_docstring_ = "";

static const char *loopref_ntags_docstring_ = "";

static const char *loopref_tag_docstring_ = "";




static PyMethodDef loopref_methods_[] = {
    
        { "samples",(PyCFunction)loopref_wrapper_::samples_method_,METH_VARARGS,(char *)loopref_samples_docstring_ },
    
        { "sample_rate",(PyCFunction)loopref_wrapper_::sample_rate_method_,METH_VARARGS,(char *)loopref_sample_rate_docstring_ },
    
        { "beats",(PyCFunction)loopref_wrapper_::beats_method_,METH_VARARGS,(char *)loopref_beats_docstring_ },
    
        { "note",(PyCFunction)loopref_wrapper_::note_method_,METH_VARARGS,(char *)loopref_note_docstring_ },
    
        { "scale",(PyCFunction)loopref_wrapper_::scale_method_,METH_VARARGS,(char *)loopref_scale_docstring_ },
    
        { "timesig_numerator",(PyCFunction)loopref_wrapper_::timesig_numerator_method_,METH_VARARGS,(char *)loopref_timesig_numerator_docstring_ },
    
        { "timesig_denominator",(PyCFunction)loopref_wrapper_::timesig_denominator_method_,METH_VARARGS,(char *)loopref_timesig_denominator_docstring_ },
    
        { "looping",(PyCFunction)loopref_wrapper_::looping_method_,METH_VARARGS,(char *)loopref_looping_docstring_ },
    
        { "ntags",(PyCFunction)loopref_wrapper_::ntags_method_,METH_VARARGS,(char *)loopref_ntags_docstring_ },
    
        { "tag",(PyCFunction)loopref_wrapper_::tag_method_,METH_VARARGS,(char *)loopref_tag_docstring_ },
    
    
        { "py_lock",(PyCFunction)loopref_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)loopref_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *loopref_docstring_ = "";

PyTypeObject loopref_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "loop_native.loopref",/*tp_name*/
    sizeof(loopref_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)loopref_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)loopref_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    loopref_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)loopref_init_,  /* tp_init */
    0,                         /* tp_alloc */
    loopref_new_,             /* tp_new */
};

PyObject *loopref_wrapcopy_(loopref_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            loopref_object_ *obj = 
                PyObject_NEW(loopref_object_, loopref_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new loopref_type_(*o);
                }

                try
                {
                    loopref_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying loopref");
                return 0;
            }
        

    
}

PyObject *loopref_wrap_(loopref_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = loopref_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    loopref_object_ *self = 
        PyObject_NEW(loopref_object_, &loopref_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        loopref_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

loopref_type_ *loopref_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &loopref_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected loopref object");
        return 0;
    }

    loopref_type_ *object = static_cast<loopref_type_ *>(((loopref_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null loopref object in unwrap");
        return 0;
    }

    return object;
}

void loopref_insert_(const std::pair< loopref_type_ *, PyObject *> &p)
{




    loopref_wrapped_.insert(p);


}

void loopref_erase_(loopref_type_ *p)
{




    loopref_wrapped_.erase(p);


}

PyObject * loopref_find_(loopref_type_ *p)
{




    PyObject *ret = 0;
    std::map< loopref_type_ *, PyObject *>::iterator where = loopref_wrapped_.find(p);
    if(where != loopref_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct loopref_dispatch_ loopref_realdispatcher_ =
{
    &loopref_typeobject_,
    loopref_wrap_,
    loopref_wrapcopy_,
    loopref_unwrap_,
    loopref_insert_,
    loopref_erase_,
    loopref_find_
};

         


static const char *module_method_read_aiff_docstring_ = "";

static PyObject *module_method_read_aiff(PyObject *self, PyObject *args)
{
     const char * a0;  bool a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_b,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    loopref_type_ r = loop::read_aiff  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_loopref_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in read_aiff");
    }

    return 0;
}


static const char *module_method_canonicalise_samples_docstring_ = "";

static PyObject *module_method_canonicalise_samples(PyObject *self, PyObject *args)
{
     std::string a0;  float a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_stdstr,&a0,fpcvt_f,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    samplearray2_type_ r = loop::canonicalise_samples  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_samplearray2_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in canonicalise_samples");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "read_aiff",module_method_read_aiff,METH_VARARGS, (char *)module_method_read_aiff_docstring_ }, 
{ "canonicalise_samples",module_method_canonicalise_samples,METH_VARARGS, (char *)module_method_canonicalise_samples_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initloop_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            player_dispatcher_ = &player_realdispatcher_;
        
    
        
            pinger_dispatcher_ = &pinger_realdispatcher_;
        
    
        
            clicker_dispatcher_ = &clicker_realdispatcher_;
        
    
        
            xplayer_dispatcher_ = &xplayer_realdispatcher_;
        
    
        
            samplearray2_dispatcher_ = &samplearray2_realdispatcher_;
        
    
        
            loopref_dispatcher_ = &loopref_realdispatcher_;
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&player_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&pinger_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clicker_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&xplayer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&samplearray2_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&loopref_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("loop_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(player_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_player_dispatch",s);
            Py_INCREF(&player_typeobject_);
            PyModule_AddObject(m,"player",(PyObject *)&player_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(pinger_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_pinger_dispatch",s);
            Py_INCREF(&pinger_typeobject_);
            PyModule_AddObject(m,"pinger",(PyObject *)&pinger_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clicker_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clicker_dispatch",s);
            Py_INCREF(&clicker_typeobject_);
            PyModule_AddObject(m,"clicker",(PyObject *)&clicker_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(xplayer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_xplayer_dispatch",s);
            Py_INCREF(&xplayer_typeobject_);
            PyModule_AddObject(m,"xplayer",(PyObject *)&xplayer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(samplearray2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_samplearray2_dispatch",s);
            Py_INCREF(&samplearray2_typeobject_);
            PyModule_AddObject(m,"samplearray2",(PyObject *)&samplearray2_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(loopref_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_loopref_dispatch",s);
            Py_INCREF(&loopref_typeobject_);
            PyModule_AddObject(m,"loopref",(PyObject *)&loopref_typeobject_);
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

