#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "audio.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pi_audio::audioctl_t audioctl_type_; 


 
static std::map<audioctl_type_ *, PyObject *> audioctl_wrapped_;
 


struct audioctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(audioctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(audioctl_type_ *);
    audioctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< audioctl_type_ *, PyObject * > &);
    void (*erase)(audioctl_type_ *);
    PyObject *(*find)(audioctl_type_ *);
};

struct audioctl_dispatch_ *audioctl_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_audioctl(PyObject *o, void *a)
{
    audioctl_type_ *object;
    if(o == Py_None)
    {
        *((audioctl_type_ **)a) = 0;
        return 1;
    }
    object = audioctl_dispatcher_->unwrap(o);
    *((audioctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_audioctl_copy fpcvt_audioctl

PyObject *tpcvt_audioctl(void *a)
{
    audioctl_type_ *o = *((audioctl_type_ **)a);
    return audioctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_audioctl_copy(void *a)
{
    audioctl_type_ *o = (audioctl_type_ *)a;
    return audioctl_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct audioctl_object_ {
    PyObject_HEAD
    audioctl_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct audioctl_wrapper_: public audioctl_type_ 
{

    static void __deleter__(audioctl_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    audioctl_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,const std::string & a1  ): audioctl_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~audioctl_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void device_changed(  const char * a0  ,unsigned long a1  ,unsigned int a2  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"device_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&O&)" ,tpcvt_str,&a0,tpcvt_ul,&a1,tpcvt_ui,&a2);
        //printf("device_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("device_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void device_list_changed(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"device_list_changed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("device_list_changed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("device_list_changed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::cookie");
        }

        return 0;
    }



    bool open_device_call_(bool owned ,const std::string & a0 ,unsigned long a1 ,unsigned int a2 ,bool a3 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::open_device( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                open_device
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *open_device_method_(audioctl_object_ *self, PyObject *args)
    {
         std::string a0;  unsigned long a1;  unsigned int a2;  bool a3; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_ul,&a1  ,fpcvt_ui,&a2  ,fpcvt_b,&a3 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_device_call_ (self->owned ,a0 ,a1 ,a2 ,a3  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::open_device");
        }

        return 0;
    }



    void close_device_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::close_device();
                
            
            
        }

        return
            
                close_device
            
            
                ();
    }

    static PyObject *close_device_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_device_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::close_device");
        }

        return 0;
    }



    void mute_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::mute();
                
            
            
        }

        return
            
                mute
            
            
                ();
    }

    static PyObject *mute_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->mute_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::mute");
        }

        return 0;
    }



    void unmute_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::unmute();
                
            
            
        }

        return
            
                unmute
            
            
                ();
    }

    static PyObject *unmute_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unmute_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::unmute");
        }

        return 0;
    }



    unsigned int num_devices_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::num_devices();
                
            
            
        }

        return
            
                num_devices
            
            
                ();
    }

    static PyObject *num_devices_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->num_devices_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::num_devices");
        }

        return 0;
    }



    std::string device_name_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::device_name( a0 );
                
            
            
        }

        return
            
                device_name
            
            
                ( a0 );
    }

    static PyObject *device_name_method_(audioctl_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->device_name_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::device_name");
        }

        return 0;
    }



    std::string device_uid_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::device_uid( a0 );
                
            
            
        }

        return
            
                device_uid
            
            
                ( a0 );
    }

    static PyObject *device_uid_method_(audioctl_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->device_uid_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::device_uid");
        }

        return 0;
    }



    std::string get_uid_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::get_uid();
                
            
            
        }

        return
            
                get_uid
            
            
                ();
    }

    static PyObject *get_uid_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->get_uid_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::get_uid");
        }

        return 0;
    }



    void reset_dropout_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::reset_dropout_count();
                
            
            
        }

        return
            
                reset_dropout_count
            
            
                ();
    }

    static PyObject *reset_dropout_count_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->reset_dropout_count_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::reset_dropout_count");
        }

        return 0;
    }



    unsigned long get_dropout_count_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::get_dropout_count();
                
            
            
        }

        return
            
                get_dropout_count
            
            
                ();
    }

    static PyObject *get_dropout_count_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->get_dropout_count_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::get_dropout_count");
        }

        return 0;
    }



    void device_changed_call_(bool owned ,const char * a0 ,unsigned long a1 ,unsigned int a2 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::device_changed( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                device_changed
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *device_changed_method_(audioctl_object_ *self, PyObject *args)
    {
         const char * a0;  unsigned long a1;  unsigned int a2; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_str,&a0  ,fpcvt_ul,&a1  ,fpcvt_ui,&a2 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->device_changed_call_ (self->owned ,a0,a1,a2 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::device_changed");
        }

        return 0;
    }



    void device_list_changed_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::device_list_changed();
                
            
            
        }

        return
            
                device_list_changed
            
            
                ();
    }

    static PyObject *device_list_changed_method_(audioctl_object_ *self, PyObject *args)
    {
        

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->device_list_changed_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::device_list_changed");
        }

        return 0;
    }



    void enable_callbacks_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::enable_callbacks( a0 );
                
            
            
        }

        return
            
                enable_callbacks
            
            
                ( a0 );
    }

    static PyObject *enable_callbacks_method_(audioctl_object_ *self, PyObject *args)
    {
         bool a0; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_callbacks_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::enable_callbacks");
        }

        return 0;
    }



    void show_gui_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return audioctl_type_::show_gui( a0 );
                
            
            
        }

        return
            
                show_gui
            
            
                ( a0 );
    }

    static PyObject *show_gui_method_(audioctl_object_ *self, PyObject *args)
    {
         bool a0; 

        audioctl_wrapper_ *o = (audioctl_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null audioctl c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->show_gui_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in audioctl::show_gui");
        }

        return 0;
    }





    static PyObject *_method_lock(audioctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(audioctl_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void audioctl_delete_(audioctl_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            audioctl_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (audioctl_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        audioctl_wrapper_::__deleter__((audioctl_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void audioctl_dealloc_(audioctl_object_* self)
{
    
    audioctl_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * audioctl_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    audioctl_object_ *self;

    if(!(self = (audioctl_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int audioctl_init_(audioctl_object_ *self, PyObject *args, PyObject *kwds)
{
    audioctl_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  std::string a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_stdstr,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new audioctl_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing audioctl");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            audioctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"audioctl invalid arguments");
    return -1;

}


static const char *audioctl_cookie_docstring_ = "";

static const char *audioctl_open_device_docstring_ = "";

static const char *audioctl_close_device_docstring_ = "";

static const char *audioctl_mute_docstring_ = "";

static const char *audioctl_unmute_docstring_ = "";

static const char *audioctl_num_devices_docstring_ = "";

static const char *audioctl_device_name_docstring_ = "";

static const char *audioctl_device_uid_docstring_ = "";

static const char *audioctl_get_uid_docstring_ = "";

static const char *audioctl_reset_dropout_count_docstring_ = "";

static const char *audioctl_get_dropout_count_docstring_ = "";

static const char *audioctl_device_changed_docstring_ = "";

static const char *audioctl_device_list_changed_docstring_ = "";

static const char *audioctl_enable_callbacks_docstring_ = "";

static const char *audioctl_show_gui_docstring_ = "";




static PyMethodDef audioctl_methods_[] = {
    
        { "cookie",(PyCFunction)audioctl_wrapper_::cookie_method_,METH_VARARGS,(char *)audioctl_cookie_docstring_ },
    
        { "open_device",(PyCFunction)audioctl_wrapper_::open_device_method_,METH_VARARGS,(char *)audioctl_open_device_docstring_ },
    
        { "close_device",(PyCFunction)audioctl_wrapper_::close_device_method_,METH_VARARGS,(char *)audioctl_close_device_docstring_ },
    
        { "mute",(PyCFunction)audioctl_wrapper_::mute_method_,METH_VARARGS,(char *)audioctl_mute_docstring_ },
    
        { "unmute",(PyCFunction)audioctl_wrapper_::unmute_method_,METH_VARARGS,(char *)audioctl_unmute_docstring_ },
    
        { "num_devices",(PyCFunction)audioctl_wrapper_::num_devices_method_,METH_VARARGS,(char *)audioctl_num_devices_docstring_ },
    
        { "device_name",(PyCFunction)audioctl_wrapper_::device_name_method_,METH_VARARGS,(char *)audioctl_device_name_docstring_ },
    
        { "device_uid",(PyCFunction)audioctl_wrapper_::device_uid_method_,METH_VARARGS,(char *)audioctl_device_uid_docstring_ },
    
        { "get_uid",(PyCFunction)audioctl_wrapper_::get_uid_method_,METH_VARARGS,(char *)audioctl_get_uid_docstring_ },
    
        { "reset_dropout_count",(PyCFunction)audioctl_wrapper_::reset_dropout_count_method_,METH_VARARGS,(char *)audioctl_reset_dropout_count_docstring_ },
    
        { "get_dropout_count",(PyCFunction)audioctl_wrapper_::get_dropout_count_method_,METH_VARARGS,(char *)audioctl_get_dropout_count_docstring_ },
    
        { "device_changed",(PyCFunction)audioctl_wrapper_::device_changed_method_,METH_VARARGS,(char *)audioctl_device_changed_docstring_ },
    
        { "device_list_changed",(PyCFunction)audioctl_wrapper_::device_list_changed_method_,METH_VARARGS,(char *)audioctl_device_list_changed_docstring_ },
    
        { "enable_callbacks",(PyCFunction)audioctl_wrapper_::enable_callbacks_method_,METH_VARARGS,(char *)audioctl_enable_callbacks_docstring_ },
    
        { "show_gui",(PyCFunction)audioctl_wrapper_::show_gui_method_,METH_VARARGS,(char *)audioctl_show_gui_docstring_ },
    
    
        { "py_lock",(PyCFunction)audioctl_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)audioctl_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *audioctl_docstring_ = "";

PyTypeObject audioctl_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "audio_native.audioctl",/*tp_name*/
    sizeof(audioctl_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)audioctl_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)audioctl_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    audioctl_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)audioctl_init_,  /* tp_init */
    0,                         /* tp_alloc */
    audioctl_new_,             /* tp_new */
};

PyObject *audioctl_wrapcopy_(audioctl_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *audioctl_wrap_(audioctl_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = audioctl_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    audioctl_object_ *self = 
        PyObject_NEW(audioctl_object_, &audioctl_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        audioctl_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

audioctl_type_ *audioctl_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &audioctl_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected audioctl object");
        return 0;
    }

    audioctl_type_ *object = static_cast<audioctl_type_ *>(((audioctl_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null audioctl object in unwrap");
        return 0;
    }

    return object;
}

void audioctl_insert_(const std::pair< audioctl_type_ *, PyObject *> &p)
{




    audioctl_wrapped_.insert(p);


}

void audioctl_erase_(audioctl_type_ *p)
{




    audioctl_wrapped_.erase(p);


}

PyObject * audioctl_find_(audioctl_type_ *p)
{




    PyObject *ret = 0;
    std::map< audioctl_type_ *, PyObject *>::iterator where = audioctl_wrapped_.find(p);
    if(where != audioctl_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct audioctl_dispatch_ audioctl_realdispatcher_ =
{
    &audioctl_typeobject_,
    audioctl_wrap_,
    audioctl_wrapcopy_,
    audioctl_unwrap_,
    audioctl_insert_,
    audioctl_erase_,
    audioctl_find_
};

         





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initaudio_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            audioctl_dispatcher_ = &audioctl_realdispatcher_;
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw_native");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw_native");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw_native");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw_native");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&audioctl_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("audio_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(audioctl_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_audioctl_dispatch",s);
            Py_INCREF(&audioctl_typeobject_);
            PyModule_AddObject(m,"audioctl",(PyObject *)&audioctl_typeobject_);
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

