#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <plg_midi/midi_merge_output.h>
#include <plg_midi/midi_output.h>
#include <plg_midi/midi_clock.h>
#include <plg_midi/midi_input.h>


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pi_midi::midi_merge_output_t midi_merge_output_type_; 


 
static std::map<midi_merge_output_type_ *, PyObject *> midi_merge_output_wrapped_;
 


struct midi_merge_output_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_merge_output_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_merge_output_type_ *);
    midi_merge_output_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_merge_output_type_ *, PyObject * > &);
    void (*erase)(midi_merge_output_type_ *);
    PyObject *(*find)(midi_merge_output_type_ *);
};

struct midi_merge_output_dispatch_ *midi_merge_output_dispatcher_;



typedef pi_midi::midi_output_t midi_output_type_; 


 
static std::map<midi_output_type_ *, PyObject *> midi_output_wrapped_;
 


struct midi_output_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_output_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_output_type_ *);
    midi_output_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_output_type_ *, PyObject * > &);
    void (*erase)(midi_output_type_ *);
    PyObject *(*find)(midi_output_type_ *);
};

struct midi_output_dispatch_ *midi_output_dispatcher_;



typedef pi_midi::midi_clock_t midi_clock_type_; 


 
static std::map<midi_clock_type_ *, PyObject *> midi_clock_wrapped_;
 


struct midi_clock_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_clock_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_clock_type_ *);
    midi_clock_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_clock_type_ *, PyObject * > &);
    void (*erase)(midi_clock_type_ *);
    PyObject *(*find)(midi_clock_type_ *);
};

struct midi_clock_dispatch_ *midi_clock_dispatcher_;



typedef pi_midi::midi_input_t midi_input_type_; 


 
static std::map<midi_input_type_ *, PyObject *> midi_input_wrapped_;
 


struct midi_input_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(midi_input_type_ *, bool owned);
    PyObject *(*wrapcopy)(midi_input_type_ *);
    midi_input_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< midi_input_type_ *, PyObject * > &);
    void (*erase)(midi_input_type_ *);
    PyObject *(*find)(midi_input_type_ *);
};

struct midi_input_dispatch_ *midi_input_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_midi_merge_output(PyObject *o, void *a)
{
    midi_merge_output_type_ *object;
    if(o == Py_None)
    {
        *((midi_merge_output_type_ **)a) = 0;
        return 1;
    }
    object = midi_merge_output_dispatcher_->unwrap(o);
    *((midi_merge_output_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_merge_output_copy fpcvt_midi_merge_output

PyObject *tpcvt_midi_merge_output(void *a)
{
    midi_merge_output_type_ *o = *((midi_merge_output_type_ **)a);
    return midi_merge_output_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_merge_output_copy(void *a)
{
    midi_merge_output_type_ *o = (midi_merge_output_type_ *)a;
    return midi_merge_output_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_output(PyObject *o, void *a)
{
    midi_output_type_ *object;
    if(o == Py_None)
    {
        *((midi_output_type_ **)a) = 0;
        return 1;
    }
    object = midi_output_dispatcher_->unwrap(o);
    *((midi_output_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_output_copy fpcvt_midi_output

PyObject *tpcvt_midi_output(void *a)
{
    midi_output_type_ *o = *((midi_output_type_ **)a);
    return midi_output_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_output_copy(void *a)
{
    midi_output_type_ *o = (midi_output_type_ *)a;
    return midi_output_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_clock(PyObject *o, void *a)
{
    midi_clock_type_ *object;
    if(o == Py_None)
    {
        *((midi_clock_type_ **)a) = 0;
        return 1;
    }
    object = midi_clock_dispatcher_->unwrap(o);
    *((midi_clock_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_clock_copy fpcvt_midi_clock

PyObject *tpcvt_midi_clock(void *a)
{
    midi_clock_type_ *o = *((midi_clock_type_ **)a);
    return midi_clock_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_clock_copy(void *a)
{
    midi_clock_type_ *o = (midi_clock_type_ *)a;
    return midi_clock_dispatcher_->wrapcopy(o);
}



int fpcvt_midi_input(PyObject *o, void *a)
{
    midi_input_type_ *object;
    if(o == Py_None)
    {
        *((midi_input_type_ **)a) = 0;
        return 1;
    }
    object = midi_input_dispatcher_->unwrap(o);
    *((midi_input_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_midi_input_copy fpcvt_midi_input

PyObject *tpcvt_midi_input(void *a)
{
    midi_input_type_ *o = *((midi_input_type_ **)a);
    return midi_input_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_midi_input_copy(void *a)
{
    midi_input_type_ *o = (midi_input_type_ *)a;
    return midi_input_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct midi_merge_output_object_ {
    PyObject_HEAD
    midi_merge_output_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_merge_output_wrapper_: public midi_merge_output_type_ 
{

    static void __deleter__(midi_merge_output_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    midi_merge_output_wrapper_( PyObject *o, PyInterpreterState *i  ,const change_nb_type_ & a0  ,clockdomain_ctl_type_ * a1  ): midi_merge_output_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_merge_output_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_merge_output_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(midi_merge_output_object_ *self, PyObject *args)
    {
        

        midi_merge_output_wrapper_ *o = (midi_merge_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_merge_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_merge_output::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_merge_output_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_merge_output_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_merge_output_delete_(midi_merge_output_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_merge_output_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_merge_output_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_merge_output_wrapper_::__deleter__((midi_merge_output_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_merge_output_dealloc_(midi_merge_output_object_* self)
{
    
    midi_merge_output_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_merge_output_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_merge_output_object_ *self;

    if(!(self = (midi_merge_output_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_merge_output_init_(midi_merge_output_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_merge_output_delete_(self);

    

    {
         change_nb_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_change_nb_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_merge_output_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_merge_output");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_merge_output_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_merge_output invalid arguments");
    return -1;

}


static const char *midi_merge_output_cookie_docstring_ = "";




static PyMethodDef midi_merge_output_methods_[] = {
    
        { "cookie",(PyCFunction)midi_merge_output_wrapper_::cookie_method_,METH_VARARGS,(char *)midi_merge_output_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_merge_output_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_merge_output_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_merge_output_docstring_ = "";

PyTypeObject midi_merge_output_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "midi_native.midi_merge_output",/*tp_name*/
    sizeof(midi_merge_output_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_merge_output_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_merge_output_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_merge_output_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_merge_output_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_merge_output_new_,             /* tp_new */
};

PyObject *midi_merge_output_wrapcopy_(midi_merge_output_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_merge_output_wrap_(midi_merge_output_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_merge_output_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_merge_output_object_ *self = 
        PyObject_NEW(midi_merge_output_object_, &midi_merge_output_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_merge_output_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_merge_output_type_ *midi_merge_output_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_merge_output_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_merge_output object");
        return 0;
    }

    midi_merge_output_type_ *object = static_cast<midi_merge_output_type_ *>(((midi_merge_output_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_merge_output object in unwrap");
        return 0;
    }

    return object;
}

void midi_merge_output_insert_(const std::pair< midi_merge_output_type_ *, PyObject *> &p)
{




    midi_merge_output_wrapped_.insert(p);


}

void midi_merge_output_erase_(midi_merge_output_type_ *p)
{




    midi_merge_output_wrapped_.erase(p);


}

PyObject * midi_merge_output_find_(midi_merge_output_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_merge_output_type_ *, PyObject *>::iterator where = midi_merge_output_wrapped_.find(p);
    if(where != midi_merge_output_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_merge_output_dispatch_ midi_merge_output_realdispatcher_ =
{
    &midi_merge_output_typeobject_,
    midi_merge_output_wrap_,
    midi_merge_output_wrapcopy_,
    midi_merge_output_unwrap_,
    midi_merge_output_insert_,
    midi_merge_output_erase_,
    midi_merge_output_find_
};

  

struct midi_output_object_ {
    PyObject_HEAD
    midi_output_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_output_wrapper_: public midi_output_type_ 
{

    static void __deleter__(midi_output_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    midi_output_wrapper_( PyObject *o, PyInterpreterState *i  ): midi_output_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_output_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void sink_added(  long a0  ,const std::string & a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"sink_added");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_l,&a0,tpcvt_stdstr,&a1);
        //printf("sink_added before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("sink_added after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void sink_removed(  long a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"sink_removed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_l,&a0);
        //printf("sink_removed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("sink_removed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    bool set_port_call_(bool owned ,long a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::set_port( a0 );
                
            
            
        }

        return
            
                set_port
            
            
                ( a0 );
    }

    static PyObject *set_port_method_(midi_output_object_ *self, PyObject *args)
    {
         long a0; 

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_l,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->set_port_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::set_port");
        }

        return 0;
    }



    long get_port_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::get_port();
                
            
            
        }

        return
            
                get_port
            
            
                ();
    }

    static PyObject *get_port_method_(midi_output_object_ *self, PyObject *args)
    {
        

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    long r = o->get_port_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_l((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::get_port");
        }

        return 0;
    }



    change_nb_type_ get_midi_output_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::get_midi_output_functor();
                
            
            
        }

        return
            
                get_midi_output_functor
            
            
                ();
    }

    static PyObject *get_midi_output_functor_method_(midi_output_object_ *self, PyObject *args)
    {
        

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->get_midi_output_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::get_midi_output_functor");
        }

        return 0;
    }



    void set_source_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::set_source( a0 );
                
            
            
        }

        return
            
                set_source
            
            
                ( a0 );
    }

    static PyObject *set_source_method_(midi_output_object_ *self, PyObject *args)
    {
         std::string a0; 

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_source_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::set_source");
        }

        return 0;
    }



    void sink_added_call_(bool owned ,long a0 ,const std::string & a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::sink_added( a0  ,a1 );
                
            
            
        }

        return
            
                sink_added
            
            
                ( a0  ,a1 );
    }

    static PyObject *sink_added_method_(midi_output_object_ *self, PyObject *args)
    {
         long a0;  std::string a1; 

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_l,&a0  ,fpcvt_stdstr,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sink_added_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::sink_added");
        }

        return 0;
    }



    void sink_removed_call_(bool owned ,long a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::sink_removed( a0 );
                
            
            
        }

        return
            
                sink_removed
            
            
                ( a0 );
    }

    static PyObject *sink_removed_method_(midi_output_object_ *self, PyObject *args)
    {
         long a0; 

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_l,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->sink_removed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::sink_removed");
        }

        return 0;
    }



    void run_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::run();
                
            
            
        }

        return
            
                run
            
            
                ();
    }

    static PyObject *run_method_(midi_output_object_ *self, PyObject *args)
    {
        

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->run_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::run");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_output_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(midi_output_object_ *self, PyObject *args)
    {
        

        midi_output_wrapper_ *o = (midi_output_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_output c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_output::stop");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_output_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_output_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_output_delete_(midi_output_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_output_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_output_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_output_wrapper_::__deleter__((midi_output_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_output_dealloc_(midi_output_object_* self)
{
    
    midi_output_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_output_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_output_object_ *self;

    if(!(self = (midi_output_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_output_init_(midi_output_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_output_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_output_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_output");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_output_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_output invalid arguments");
    return -1;

}


static const char *midi_output_set_port_docstring_ = "";

static const char *midi_output_get_port_docstring_ = "";

static const char *midi_output_get_midi_output_functor_docstring_ = "";

static const char *midi_output_set_source_docstring_ = "";

static const char *midi_output_sink_added_docstring_ = "";

static const char *midi_output_sink_removed_docstring_ = "";

static const char *midi_output_run_docstring_ = "";

static const char *midi_output_stop_docstring_ = "";




static PyMethodDef midi_output_methods_[] = {
    
        { "set_port",(PyCFunction)midi_output_wrapper_::set_port_method_,METH_VARARGS,(char *)midi_output_set_port_docstring_ },
    
        { "get_port",(PyCFunction)midi_output_wrapper_::get_port_method_,METH_VARARGS,(char *)midi_output_get_port_docstring_ },
    
        { "get_midi_output_functor",(PyCFunction)midi_output_wrapper_::get_midi_output_functor_method_,METH_VARARGS,(char *)midi_output_get_midi_output_functor_docstring_ },
    
        { "set_source",(PyCFunction)midi_output_wrapper_::set_source_method_,METH_VARARGS,(char *)midi_output_set_source_docstring_ },
    
        { "sink_added",(PyCFunction)midi_output_wrapper_::sink_added_method_,METH_VARARGS,(char *)midi_output_sink_added_docstring_ },
    
        { "sink_removed",(PyCFunction)midi_output_wrapper_::sink_removed_method_,METH_VARARGS,(char *)midi_output_sink_removed_docstring_ },
    
        { "run",(PyCFunction)midi_output_wrapper_::run_method_,METH_VARARGS,(char *)midi_output_run_docstring_ },
    
        { "stop",(PyCFunction)midi_output_wrapper_::stop_method_,METH_VARARGS,(char *)midi_output_stop_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_output_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_output_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_output_docstring_ = "";

PyTypeObject midi_output_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "midi_native.midi_output",/*tp_name*/
    sizeof(midi_output_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_output_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_output_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_output_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_output_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_output_new_,             /* tp_new */
};

PyObject *midi_output_wrapcopy_(midi_output_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_output_wrap_(midi_output_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_output_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_output_object_ *self = 
        PyObject_NEW(midi_output_object_, &midi_output_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_output_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_output_type_ *midi_output_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_output_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_output object");
        return 0;
    }

    midi_output_type_ *object = static_cast<midi_output_type_ *>(((midi_output_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_output object in unwrap");
        return 0;
    }

    return object;
}

void midi_output_insert_(const std::pair< midi_output_type_ *, PyObject *> &p)
{




    midi_output_wrapped_.insert(p);


}

void midi_output_erase_(midi_output_type_ *p)
{




    midi_output_wrapped_.erase(p);


}

PyObject * midi_output_find_(midi_output_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_output_type_ *, PyObject *>::iterator where = midi_output_wrapped_.find(p);
    if(where != midi_output_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_output_dispatch_ midi_output_realdispatcher_ =
{
    &midi_output_typeobject_,
    midi_output_wrap_,
    midi_output_wrapcopy_,
    midi_output_unwrap_,
    midi_output_insert_,
    midi_output_erase_,
    midi_output_find_
};

  

struct midi_clock_object_ {
    PyObject_HEAD
    midi_clock_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_clock_wrapper_: public midi_clock_type_ 
{

    static void __deleter__(midi_clock_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    midi_clock_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): midi_clock_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_clock_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_clock_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(midi_clock_object_ *self, PyObject *args)
    {
        

        midi_clock_wrapper_ *o = (midi_clock_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_clock c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_clock::cookie");
        }

        return 0;
    }



    void set_delay_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_clock_type_::set_delay( a0 );
                
            
            
        }

        return
            
                set_delay
            
            
                ( a0 );
    }

    static PyObject *set_delay_method_(midi_clock_object_ *self, PyObject *args)
    {
         float a0; 

        midi_clock_wrapper_ *o = (midi_clock_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_clock c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_delay_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_clock::set_delay");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_clock_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_clock_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_clock_delete_(midi_clock_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_clock_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_clock_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_clock_wrapper_::__deleter__((midi_clock_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_clock_dealloc_(midi_clock_object_* self)
{
    
    midi_clock_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_clock_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_clock_object_ *self;

    if(!(self = (midi_clock_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_clock_init_(midi_clock_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_clock_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_clock_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_clock");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_clock_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_clock invalid arguments");
    return -1;

}


static const char *midi_clock_cookie_docstring_ = "";

static const char *midi_clock_set_delay_docstring_ = "";




static PyMethodDef midi_clock_methods_[] = {
    
        { "cookie",(PyCFunction)midi_clock_wrapper_::cookie_method_,METH_VARARGS,(char *)midi_clock_cookie_docstring_ },
    
        { "set_delay",(PyCFunction)midi_clock_wrapper_::set_delay_method_,METH_VARARGS,(char *)midi_clock_set_delay_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_clock_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_clock_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_clock_docstring_ = "";

PyTypeObject midi_clock_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "midi_native.midi_clock",/*tp_name*/
    sizeof(midi_clock_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_clock_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)midi_clock_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_clock_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_clock_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_clock_new_,             /* tp_new */
};

PyObject *midi_clock_wrapcopy_(midi_clock_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_clock_wrap_(midi_clock_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_clock_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_clock_object_ *self = 
        PyObject_NEW(midi_clock_object_, &midi_clock_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_clock_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_clock_type_ *midi_clock_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_clock_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_clock object");
        return 0;
    }

    midi_clock_type_ *object = static_cast<midi_clock_type_ *>(((midi_clock_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_clock object in unwrap");
        return 0;
    }

    return object;
}

void midi_clock_insert_(const std::pair< midi_clock_type_ *, PyObject *> &p)
{




    midi_clock_wrapped_.insert(p);


}

void midi_clock_erase_(midi_clock_type_ *p)
{




    midi_clock_wrapped_.erase(p);


}

PyObject * midi_clock_find_(midi_clock_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_clock_type_ *, PyObject *>::iterator where = midi_clock_wrapped_.find(p);
    if(where != midi_clock_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_clock_dispatch_ midi_clock_realdispatcher_ =
{
    &midi_clock_typeobject_,
    midi_clock_wrap_,
    midi_clock_wrapcopy_,
    midi_clock_unwrap_,
    midi_clock_insert_,
    midi_clock_erase_,
    midi_clock_find_
};

  

struct midi_input_object_ {
    PyObject_HEAD
    midi_input_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct midi_input_wrapper_: public midi_input_type_ 
{

    static void __deleter__(midi_input_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
    }
*/



    midi_input_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,const cookie_type_ & a1  ): midi_input_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~midi_input_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void source_added(  long a0  ,const std::string & a1  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"source_added");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&O&)" ,tpcvt_l,&a0,tpcvt_stdstr,&a1);
        //printf("source_added before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("source_added after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void source_removed(  long a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"source_removed");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_l,&a0);
        //printf("source_removed before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("source_removed after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }













    static int gc_traverse_method_(midi_input_object_ *self, visitproc visit, void *arg)
    {
        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(midi_input_object_ *self)
    {
        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    bool setport_call_(bool owned ,long a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::setport( a0 );
                
            
            
        }

        return
            
                setport
            
            
                ( a0 );
    }

    static PyObject *setport_method_(midi_input_object_ *self, PyObject *args)
    {
         long a0; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_l,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->setport_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::setport");
        }

        return 0;
    }



    long getport_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::getport();
                
            
            
        }

        return
            
                getport
            
            
                ();
    }

    static PyObject *getport_method_(midi_input_object_ *self, PyObject *args)
    {
        

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    long r = o->getport_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_l((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::getport");
        }

        return 0;
    }



    unsigned int current_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::current( a0 );
                
            
            
        }

        return
            
                current
            
            
                ( a0 );
    }

    static PyObject *current_method_(midi_input_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned int r = o->current_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::current");
        }

        return 0;
    }



    void set_trim_call_(bool owned ,unsigned int a0 ,float a1 ,float a2 ,bool a3 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::set_trim( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                set_trim
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *set_trim_method_(midi_input_object_ *self, PyObject *args)
    {
         unsigned int a0;  float a1;  float a2;  bool a3; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_f,&a1  ,fpcvt_f,&a2  ,fpcvt_b,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_trim_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::set_trim");
        }

        return 0;
    }



    void clear_trim_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::clear_trim();
                
            
            
        }

        return
            
                clear_trim
            
            
                ();
    }

    static PyObject *clear_trim_method_(midi_input_object_ *self, PyObject *args)
    {
        

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->clear_trim_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::clear_trim");
        }

        return 0;
    }



    void set_destination_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::set_destination( a0 );
                
            
            
        }

        return
            
                set_destination
            
            
                ( a0 );
    }

    static PyObject *set_destination_method_(midi_input_object_ *self, PyObject *args)
    {
         std::string a0; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_destination_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::set_destination");
        }

        return 0;
    }



    void source_added_call_(bool owned ,long a0 ,const std::string & a1 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::source_added( a0  ,a1 );
                
            
            
        }

        return
            
                source_added
            
            
                ( a0  ,a1 );
    }

    static PyObject *source_added_method_(midi_input_object_ *self, PyObject *args)
    {
         long a0;  std::string a1; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_l,&a0  ,fpcvt_stdstr,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->source_added_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::source_added");
        }

        return 0;
    }



    void source_removed_call_(bool owned ,long a0 )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::source_removed( a0 );
                
            
            
        }

        return
            
                source_removed
            
            
                ( a0 );
    }

    static PyObject *source_removed_method_(midi_input_object_ *self, PyObject *args)
    {
         long a0; 

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_l,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->source_removed_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::source_removed");
        }

        return 0;
    }



    void run_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::run();
                
            
            
        }

        return
            
                run
            
            
                ();
    }

    static PyObject *run_method_(midi_input_object_ *self, PyObject *args)
    {
        

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->run_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::run");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return midi_input_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(midi_input_object_ *self, PyObject *args)
    {
        

        midi_input_wrapper_ *o = (midi_input_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null midi_input c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in midi_input::stop");
        }

        return 0;
    }





    static PyObject *_method_lock(midi_input_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(midi_input_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void midi_input_delete_(midi_input_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            midi_input_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (midi_input_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        midi_input_wrapper_::__deleter__((midi_input_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void midi_input_dealloc_(midi_input_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    midi_input_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * midi_input_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    midi_input_object_ *self;

    if(!(self = (midi_input_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int midi_input_init_(midi_input_object_ *self, PyObject *args, PyObject *kwds)
{
    midi_input_delete_(self);

    

    {
         cookie_type_ * a0;  cookie_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_cookie_copy,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new midi_input_wrapper_((PyObject *)self,interp ,*a0,*a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing midi_input");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            midi_input_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"midi_input invalid arguments");
    return -1;

}


static const char *midi_input_setport_docstring_ = "";

static const char *midi_input_getport_docstring_ = "";

static const char *midi_input_current_docstring_ = "";

static const char *midi_input_set_trim_docstring_ = "";

static const char *midi_input_clear_trim_docstring_ = "";

static const char *midi_input_set_destination_docstring_ = "";

static const char *midi_input_source_added_docstring_ = "";

static const char *midi_input_source_removed_docstring_ = "";

static const char *midi_input_run_docstring_ = "";

static const char *midi_input_stop_docstring_ = "";




static PyMethodDef midi_input_methods_[] = {
    
        { "setport",(PyCFunction)midi_input_wrapper_::setport_method_,METH_VARARGS,(char *)midi_input_setport_docstring_ },
    
        { "getport",(PyCFunction)midi_input_wrapper_::getport_method_,METH_VARARGS,(char *)midi_input_getport_docstring_ },
    
        { "current",(PyCFunction)midi_input_wrapper_::current_method_,METH_VARARGS,(char *)midi_input_current_docstring_ },
    
        { "set_trim",(PyCFunction)midi_input_wrapper_::set_trim_method_,METH_VARARGS,(char *)midi_input_set_trim_docstring_ },
    
        { "clear_trim",(PyCFunction)midi_input_wrapper_::clear_trim_method_,METH_VARARGS,(char *)midi_input_clear_trim_docstring_ },
    
        { "set_destination",(PyCFunction)midi_input_wrapper_::set_destination_method_,METH_VARARGS,(char *)midi_input_set_destination_docstring_ },
    
        { "source_added",(PyCFunction)midi_input_wrapper_::source_added_method_,METH_VARARGS,(char *)midi_input_source_added_docstring_ },
    
        { "source_removed",(PyCFunction)midi_input_wrapper_::source_removed_method_,METH_VARARGS,(char *)midi_input_source_removed_docstring_ },
    
        { "run",(PyCFunction)midi_input_wrapper_::run_method_,METH_VARARGS,(char *)midi_input_run_docstring_ },
    
        { "stop",(PyCFunction)midi_input_wrapper_::stop_method_,METH_VARARGS,(char *)midi_input_stop_docstring_ },
    
    
        { "py_lock",(PyCFunction)midi_input_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)midi_input_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *midi_input_docstring_ = "";

PyTypeObject midi_input_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "midi_native.midi_input",/*tp_name*/
    sizeof(midi_input_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)midi_input_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)midi_input_docstring_,       /* tp_doc */
    (traverseproc)midi_input_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)midi_input_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    midi_input_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)midi_input_init_,  /* tp_init */
    0,                         /* tp_alloc */
    midi_input_new_,             /* tp_new */
};

PyObject *midi_input_wrapcopy_(midi_input_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *midi_input_wrap_(midi_input_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = midi_input_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    midi_input_object_ *self = 
        
        PyObject_GC_New(midi_input_object_, &midi_input_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        midi_input_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

midi_input_type_ *midi_input_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &midi_input_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected midi_input object");
        return 0;
    }

    midi_input_type_ *object = static_cast<midi_input_type_ *>(((midi_input_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null midi_input object in unwrap");
        return 0;
    }

    return object;
}

void midi_input_insert_(const std::pair< midi_input_type_ *, PyObject *> &p)
{




    midi_input_wrapped_.insert(p);


}

void midi_input_erase_(midi_input_type_ *p)
{




    midi_input_wrapped_.erase(p);


}

PyObject * midi_input_find_(midi_input_type_ *p)
{




    PyObject *ret = 0;
    std::map< midi_input_type_ *, PyObject *>::iterator where = midi_input_wrapped_.find(p);
    if(where != midi_input_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct midi_input_dispatch_ midi_input_realdispatcher_ =
{
    &midi_input_typeobject_,
    midi_input_wrap_,
    midi_input_wrapcopy_,
    midi_input_unwrap_,
    midi_input_insert_,
    midi_input_erase_,
    midi_input_find_
};

         





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initmidi_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            midi_merge_output_dispatcher_ = &midi_merge_output_realdispatcher_;
        
    
        
            midi_output_dispatcher_ = &midi_output_realdispatcher_;
        
    
        
            midi_clock_dispatcher_ = &midi_clock_realdispatcher_;
        
    
        
            midi_input_dispatcher_ = &midi_input_realdispatcher_;
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&midi_merge_output_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_output_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_clock_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&midi_input_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("midi_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(midi_merge_output_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_merge_output_dispatch",s);
            Py_INCREF(&midi_merge_output_typeobject_);
            PyModule_AddObject(m,"midi_merge_output",(PyObject *)&midi_merge_output_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_output_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_output_dispatch",s);
            Py_INCREF(&midi_output_typeobject_);
            PyModule_AddObject(m,"midi_output",(PyObject *)&midi_output_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_clock_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_clock_dispatch",s);
            Py_INCREF(&midi_clock_typeobject_);
            PyModule_AddObject(m,"midi_clock",(PyObject *)&midi_clock_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(midi_input_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_midi_input_dispatch",s);
            Py_INCREF(&midi_input_typeobject_);
            PyModule_AddObject(m,"midi_input",(PyObject *)&midi_input_typeobject_);
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

