#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "pkbd_bundle.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pkbd::bundle_t bundle_type_; 


 
static std::map<bundle_type_ *, PyObject *> bundle_wrapped_;
 


struct bundle_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(bundle_type_ *, bool owned);
    PyObject *(*wrapcopy)(bundle_type_ *);
    bundle_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< bundle_type_ *, PyObject * > &);
    void (*erase)(bundle_type_ *);
    PyObject *(*find)(bundle_type_ *);
};

struct bundle_dispatch_ *bundle_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef pic::notify_t notify_type_; 





struct notify_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(notify_type_ *, bool owned);
    PyObject *(*wrapcopy)(notify_type_ *);
    notify_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< notify_type_ *, PyObject * > &);
    void (*erase)(notify_type_ *);
    PyObject *(*find)(notify_type_ *);
};

struct notify_dispatch_ *notify_dispatcher_;





int fpcvt_bundle(PyObject *o, void *a)
{
    bundle_type_ *object;
    if(o == Py_None)
    {
        *((bundle_type_ **)a) = 0;
        return 1;
    }
    object = bundle_dispatcher_->unwrap(o);
    *((bundle_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_bundle_copy fpcvt_bundle

PyObject *tpcvt_bundle(void *a)
{
    bundle_type_ *o = *((bundle_type_ **)a);
    return bundle_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_bundle_copy(void *a)
{
    bundle_type_ *o = (bundle_type_ *)a;
    return bundle_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_notify(PyObject *o, void *a)
{
    notify_type_ *object;
    if(o == Py_None)
    {
        *((notify_type_ **)a) = 0;
        return 1;
    }
    object = notify_dispatcher_->unwrap(o);
    *((notify_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_notify_copy fpcvt_notify

PyObject *tpcvt_notify(void *a)
{
    notify_type_ *o = *((notify_type_ **)a);
    return notify_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_notify_copy(void *a)
{
    notify_type_ *o = (notify_type_ *)a;
    return notify_dispatcher_->wrapcopy(o);
}



 

struct bundle_object_ {
    PyObject_HEAD
    bundle_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct bundle_wrapper_: public bundle_type_ 
{

    static void __deleter__(bundle_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    bundle_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,const cookie_type_ & a1  ,const notify_type_ & a2  ,const change_type_ & a3  ): bundle_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~bundle_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void close_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::close();
                
            
            
        }

        return
            
                close
            
            
                ();
    }

    static PyObject *close_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->close_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::close");
        }

        return 0;
    }



    std::string name_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::name();
                
            
            
        }

        return
            
                name
            
            
                ();
    }

    static PyObject *name_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->name_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::name");
        }

        return 0;
    }



    change_nb_type_ led_functor_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::led_functor();
                
            
            
        }

        return
            
                led_functor
            
            
                ();
    }

    static PyObject *led_functor_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_nb_type_ r = o->led_functor_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::led_functor");
        }

        return 0;
    }



    void set_threshold1_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::set_threshold1( a0 );
                
            
            
        }

        return
            
                set_threshold1
            
            
                ( a0 );
    }

    static PyObject *set_threshold1_method_(bundle_object_ *self, PyObject *args)
    {
         float a0; 

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_threshold1_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::set_threshold1");
        }

        return 0;
    }



    float get_threshold1_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::get_threshold1();
                
            
            
        }

        return
            
                get_threshold1
            
            
                ();
    }

    static PyObject *get_threshold1_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->get_threshold1_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::get_threshold1");
        }

        return 0;
    }



    void set_threshold2_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::set_threshold2( a0 );
                
            
            
        }

        return
            
                set_threshold2
            
            
                ( a0 );
    }

    static PyObject *set_threshold2_method_(bundle_object_ *self, PyObject *args)
    {
         float a0; 

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_threshold2_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::set_threshold2");
        }

        return 0;
    }



    float get_threshold2_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::get_threshold2();
                
            
            
        }

        return
            
                get_threshold2
            
            
                ();
    }

    static PyObject *get_threshold2_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->get_threshold2_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::get_threshold2");
        }

        return 0;
    }



    void set_roll_axis_window_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::set_roll_axis_window( a0 );
                
            
            
        }

        return
            
                set_roll_axis_window
            
            
                ( a0 );
    }

    static PyObject *set_roll_axis_window_method_(bundle_object_ *self, PyObject *args)
    {
         float a0; 

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_roll_axis_window_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::set_roll_axis_window");
        }

        return 0;
    }



    float get_roll_axis_window_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::get_roll_axis_window();
                
            
            
        }

        return
            
                get_roll_axis_window
            
            
                ();
    }

    static PyObject *get_roll_axis_window_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->get_roll_axis_window_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::get_roll_axis_window");
        }

        return 0;
    }



    void set_yaw_axis_window_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::set_yaw_axis_window( a0 );
                
            
            
        }

        return
            
                set_yaw_axis_window
            
            
                ( a0 );
    }

    static PyObject *set_yaw_axis_window_method_(bundle_object_ *self, PyObject *args)
    {
         float a0; 

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_yaw_axis_window_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::set_yaw_axis_window");
        }

        return 0;
    }



    float get_yaw_axis_window_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::get_yaw_axis_window();
                
            
            
        }

        return
            
                get_yaw_axis_window
            
            
                ();
    }

    static PyObject *get_yaw_axis_window_method_(bundle_object_ *self, PyObject *args)
    {
        

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->get_yaw_axis_window_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::get_yaw_axis_window");
        }

        return 0;
    }



    void enable_key_logging_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return bundle_type_::enable_key_logging( a0 );
                
            
            
        }

        return
            
                enable_key_logging
            
            
                ( a0 );
    }

    static PyObject *enable_key_logging_method_(bundle_object_ *self, PyObject *args)
    {
         bool a0; 

        bundle_wrapper_ *o = (bundle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bundle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->enable_key_logging_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bundle::enable_key_logging");
        }

        return 0;
    }





    static PyObject *_method_lock(bundle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(bundle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void bundle_delete_(bundle_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            bundle_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (bundle_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        bundle_wrapper_::__deleter__((bundle_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void bundle_dealloc_(bundle_object_* self)
{
    
    bundle_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * bundle_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    bundle_object_ *self;

    if(!(self = (bundle_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int bundle_init_(bundle_object_ *self, PyObject *args, PyObject *kwds)
{
    bundle_delete_(self);

    

    {
         const char * a0;  cookie_type_ * a1;  notify_type_ * a2;  change_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_str,&a0,fpcvt_cookie_copy,&a1,fpcvt_notify_copy,&a2,fpcvt_change_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new bundle_wrapper_((PyObject *)self,interp ,a0,*a1,*a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing bundle");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            bundle_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"bundle invalid arguments");
    return -1;

}


static const char *bundle_close_docstring_ = "";

static const char *bundle_name_docstring_ = "";

static const char *bundle_led_functor_docstring_ = "";

static const char *bundle_set_threshold1_docstring_ = "";

static const char *bundle_get_threshold1_docstring_ = "";

static const char *bundle_set_threshold2_docstring_ = "";

static const char *bundle_get_threshold2_docstring_ = "";

static const char *bundle_set_roll_axis_window_docstring_ = "";

static const char *bundle_get_roll_axis_window_docstring_ = "";

static const char *bundle_set_yaw_axis_window_docstring_ = "";

static const char *bundle_get_yaw_axis_window_docstring_ = "";

static const char *bundle_enable_key_logging_docstring_ = "";




static PyMethodDef bundle_methods_[] = {
    
        { "close",(PyCFunction)bundle_wrapper_::close_method_,METH_VARARGS,(char *)bundle_close_docstring_ },
    
        { "name",(PyCFunction)bundle_wrapper_::name_method_,METH_VARARGS,(char *)bundle_name_docstring_ },
    
        { "led_functor",(PyCFunction)bundle_wrapper_::led_functor_method_,METH_VARARGS,(char *)bundle_led_functor_docstring_ },
    
        { "set_threshold1",(PyCFunction)bundle_wrapper_::set_threshold1_method_,METH_VARARGS,(char *)bundle_set_threshold1_docstring_ },
    
        { "get_threshold1",(PyCFunction)bundle_wrapper_::get_threshold1_method_,METH_VARARGS,(char *)bundle_get_threshold1_docstring_ },
    
        { "set_threshold2",(PyCFunction)bundle_wrapper_::set_threshold2_method_,METH_VARARGS,(char *)bundle_set_threshold2_docstring_ },
    
        { "get_threshold2",(PyCFunction)bundle_wrapper_::get_threshold2_method_,METH_VARARGS,(char *)bundle_get_threshold2_docstring_ },
    
        { "set_roll_axis_window",(PyCFunction)bundle_wrapper_::set_roll_axis_window_method_,METH_VARARGS,(char *)bundle_set_roll_axis_window_docstring_ },
    
        { "get_roll_axis_window",(PyCFunction)bundle_wrapper_::get_roll_axis_window_method_,METH_VARARGS,(char *)bundle_get_roll_axis_window_docstring_ },
    
        { "set_yaw_axis_window",(PyCFunction)bundle_wrapper_::set_yaw_axis_window_method_,METH_VARARGS,(char *)bundle_set_yaw_axis_window_docstring_ },
    
        { "get_yaw_axis_window",(PyCFunction)bundle_wrapper_::get_yaw_axis_window_method_,METH_VARARGS,(char *)bundle_get_yaw_axis_window_docstring_ },
    
        { "enable_key_logging",(PyCFunction)bundle_wrapper_::enable_key_logging_method_,METH_VARARGS,(char *)bundle_enable_key_logging_docstring_ },
    
    
        { "py_lock",(PyCFunction)bundle_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)bundle_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *bundle_docstring_ = "";

PyTypeObject bundle_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "pkbd_native.bundle",/*tp_name*/
    sizeof(bundle_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)bundle_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)bundle_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    bundle_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)bundle_init_,  /* tp_init */
    0,                         /* tp_alloc */
    bundle_new_,             /* tp_new */
};

PyObject *bundle_wrapcopy_(bundle_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *bundle_wrap_(bundle_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = bundle_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    bundle_object_ *self = 
        PyObject_NEW(bundle_object_, &bundle_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        bundle_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

bundle_type_ *bundle_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &bundle_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected bundle object");
        return 0;
    }

    bundle_type_ *object = static_cast<bundle_type_ *>(((bundle_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null bundle object in unwrap");
        return 0;
    }

    return object;
}

void bundle_insert_(const std::pair< bundle_type_ *, PyObject *> &p)
{




    bundle_wrapped_.insert(p);


}

void bundle_erase_(bundle_type_ *p)
{




    bundle_wrapped_.erase(p);


}

PyObject * bundle_find_(bundle_type_ *p)
{




    PyObject *ret = 0;
    std::map< bundle_type_ *, PyObject *>::iterator where = bundle_wrapped_.find(p);
    if(where != bundle_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct bundle_dispatch_ bundle_realdispatcher_ =
{
    &bundle_typeobject_,
    bundle_wrap_,
    bundle_wrapcopy_,
    bundle_unwrap_,
    bundle_insert_,
    bundle_erase_,
    bundle_find_
};

         





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpkbd_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            bundle_dispatcher_ = &bundle_realdispatcher_;
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw_native");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw_native");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw_native");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    
        if(!(m=PyImport_ImportModule("picross_native")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_notify_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_notify_dispatch in module picross_native");
                return;
            }

            notify_dispatcher_ = (notify_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&bundle_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("pkbd_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(bundle_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_bundle_dispatch",s);
            Py_INCREF(&bundle_typeobject_);
            PyModule_AddObject(m,"bundle",(PyObject *)&bundle_typeobject_);
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

