#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "rec_recorder.h"
#include "rec_recording.h"
#include "rec_player.h"
#include "rec_nplayer.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef recorder::readevent_t readevent_type_; 


 
static std::map<readevent_type_ *, PyObject *> readevent_wrapped_;
 


struct readevent_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(readevent_type_ *, bool owned);
    PyObject *(*wrapcopy)(readevent_type_ *);
    readevent_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< readevent_type_ *, PyObject * > &);
    void (*erase)(readevent_type_ *);
    PyObject *(*find)(readevent_type_ *);
};

struct readevent_dispatch_ *readevent_dispatcher_;



typedef recorder::recording_t recording_type_; 


 
static std::map<recording_type_ *, PyObject *> recording_wrapped_;
 


struct recording_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(recording_type_ *, bool owned);
    PyObject *(*wrapcopy)(recording_type_ *);
    recording_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< recording_type_ *, PyObject * > &);
    void (*erase)(recording_type_ *);
    PyObject *(*find)(recording_type_ *);
};

struct recording_dispatch_ *recording_dispatcher_;



typedef recorder::recorder_t recorder_type_; 


 
static std::map<recorder_type_ *, PyObject *> recorder_wrapped_;
 


struct recorder_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(recorder_type_ *, bool owned);
    PyObject *(*wrapcopy)(recorder_type_ *);
    recorder_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< recorder_type_ *, PyObject * > &);
    void (*erase)(recorder_type_ *);
    PyObject *(*find)(recorder_type_ *);
};

struct recorder_dispatch_ *recorder_dispatcher_;



typedef recorder::player_t kplayer_type_; 


 
static std::map<kplayer_type_ *, PyObject *> kplayer_wrapped_;
 


struct kplayer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(kplayer_type_ *, bool owned);
    PyObject *(*wrapcopy)(kplayer_type_ *);
    kplayer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< kplayer_type_ *, PyObject * > &);
    void (*erase)(kplayer_type_ *);
    PyObject *(*find)(kplayer_type_ *);
};

struct kplayer_dispatch_ *kplayer_dispatcher_;



typedef recorder::nplayer_t nplayer_type_; 


 
static std::map<nplayer_type_ *, PyObject *> nplayer_wrapped_;
 


struct nplayer_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(nplayer_type_ *, bool owned);
    PyObject *(*wrapcopy)(nplayer_type_ *);
    nplayer_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< nplayer_type_ *, PyObject * > &);
    void (*erase)(nplayer_type_ *);
    PyObject *(*find)(nplayer_type_ *);
};

struct nplayer_dispatch_ *nplayer_dispatcher_;



typedef bct_clocksink_t clockref_type_; 





struct clockref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockref_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockref_type_ *);
    clockref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockref_type_ *, PyObject * > &);
    void (*erase)(clockref_type_ *);
    PyObject *(*find)(clockref_type_ *);
};

struct clockref_dispatch_ *clockref_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::change_t change_type_; 





struct change_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_type_ *);
    change_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_type_ *, PyObject * > &);
    void (*erase)(change_type_ *);
    PyObject *(*find)(change_type_ *);
};

struct change_dispatch_ *change_dispatcher_;



typedef piw::change_nb_t change_nb_type_; 





struct change_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(change_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(change_nb_type_ *);
    change_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< change_nb_type_ *, PyObject * > &);
    void (*erase)(change_nb_type_ *);
    PyObject *(*find)(change_nb_type_ *);
};

struct change_nb_dispatch_ *change_nb_dispatcher_;



typedef piw::data_t data_type_; 





struct data_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_type_ *);
    data_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_type_ *, PyObject * > &);
    void (*erase)(data_type_ *);
    PyObject *(*find)(data_type_ *);
};

struct data_dispatch_ *data_dispatcher_;



typedef piw::data_nb_t data_nb_type_; 





struct data_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(data_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(data_nb_type_ *);
    data_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< data_nb_type_ *, PyObject * > &);
    void (*erase)(data_nb_type_ *);
    PyObject *(*find)(data_nb_type_ *);
};

struct data_nb_dispatch_ *data_nb_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_readevent(PyObject *o, void *a)
{
    readevent_type_ *object;
    if(o == Py_None)
    {
        *((readevent_type_ **)a) = 0;
        return 1;
    }
    object = readevent_dispatcher_->unwrap(o);
    *((readevent_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_readevent_copy fpcvt_readevent

PyObject *tpcvt_readevent(void *a)
{
    readevent_type_ *o = *((readevent_type_ **)a);
    return readevent_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_readevent_copy(void *a)
{
    readevent_type_ *o = (readevent_type_ *)a;
    return readevent_dispatcher_->wrapcopy(o);
}



int fpcvt_recording(PyObject *o, void *a)
{
    recording_type_ *object;
    if(o == Py_None)
    {
        *((recording_type_ **)a) = 0;
        return 1;
    }
    object = recording_dispatcher_->unwrap(o);
    *((recording_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_recording_copy fpcvt_recording

PyObject *tpcvt_recording(void *a)
{
    recording_type_ *o = *((recording_type_ **)a);
    return recording_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_recording_copy(void *a)
{
    recording_type_ *o = (recording_type_ *)a;
    return recording_dispatcher_->wrapcopy(o);
}



int fpcvt_recorder(PyObject *o, void *a)
{
    recorder_type_ *object;
    if(o == Py_None)
    {
        *((recorder_type_ **)a) = 0;
        return 1;
    }
    object = recorder_dispatcher_->unwrap(o);
    *((recorder_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_recorder_copy fpcvt_recorder

PyObject *tpcvt_recorder(void *a)
{
    recorder_type_ *o = *((recorder_type_ **)a);
    return recorder_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_recorder_copy(void *a)
{
    recorder_type_ *o = (recorder_type_ *)a;
    return recorder_dispatcher_->wrapcopy(o);
}



int fpcvt_kplayer(PyObject *o, void *a)
{
    kplayer_type_ *object;
    if(o == Py_None)
    {
        *((kplayer_type_ **)a) = 0;
        return 1;
    }
    object = kplayer_dispatcher_->unwrap(o);
    *((kplayer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_kplayer_copy fpcvt_kplayer

PyObject *tpcvt_kplayer(void *a)
{
    kplayer_type_ *o = *((kplayer_type_ **)a);
    return kplayer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_kplayer_copy(void *a)
{
    kplayer_type_ *o = (kplayer_type_ *)a;
    return kplayer_dispatcher_->wrapcopy(o);
}



int fpcvt_nplayer(PyObject *o, void *a)
{
    nplayer_type_ *object;
    if(o == Py_None)
    {
        *((nplayer_type_ **)a) = 0;
        return 1;
    }
    object = nplayer_dispatcher_->unwrap(o);
    *((nplayer_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_nplayer_copy fpcvt_nplayer

PyObject *tpcvt_nplayer(void *a)
{
    nplayer_type_ *o = *((nplayer_type_ **)a);
    return nplayer_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_nplayer_copy(void *a)
{
    nplayer_type_ *o = (nplayer_type_ *)a;
    return nplayer_dispatcher_->wrapcopy(o);
}



int fpcvt_clockref(PyObject *o, void *a)
{
    clockref_type_ *object;
    if(o == Py_None)
    {
        *((clockref_type_ **)a) = 0;
        return 1;
    }
    object = clockref_dispatcher_->unwrap(o);
    *((clockref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockref_copy fpcvt_clockref

PyObject *tpcvt_clockref(void *a)
{
    clockref_type_ *o = *((clockref_type_ **)a);
    return clockref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockref_copy(void *a)
{
    clockref_type_ *o = (clockref_type_ *)a;
    return clockref_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_change(PyObject *o, void *a)
{
    change_type_ *object;
    if(o == Py_None)
    {
        *((change_type_ **)a) = 0;
        return 1;
    }
    object = change_dispatcher_->unwrap(o);
    *((change_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_copy fpcvt_change

PyObject *tpcvt_change(void *a)
{
    change_type_ *o = *((change_type_ **)a);
    return change_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_copy(void *a)
{
    change_type_ *o = (change_type_ *)a;
    return change_dispatcher_->wrapcopy(o);
}



int fpcvt_change_nb(PyObject *o, void *a)
{
    change_nb_type_ *object;
    if(o == Py_None)
    {
        *((change_nb_type_ **)a) = 0;
        return 1;
    }
    object = change_nb_dispatcher_->unwrap(o);
    *((change_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_change_nb_copy fpcvt_change_nb

PyObject *tpcvt_change_nb(void *a)
{
    change_nb_type_ *o = *((change_nb_type_ **)a);
    return change_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_change_nb_copy(void *a)
{
    change_nb_type_ *o = (change_nb_type_ *)a;
    return change_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_data(PyObject *o, void *a)
{
    data_type_ *object;
    if(o == Py_None)
    {
        *((data_type_ **)a) = 0;
        return 1;
    }
    object = data_dispatcher_->unwrap(o);
    *((data_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_copy fpcvt_data

PyObject *tpcvt_data(void *a)
{
    data_type_ *o = *((data_type_ **)a);
    return data_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_copy(void *a)
{
    data_type_ *o = (data_type_ *)a;
    return data_dispatcher_->wrapcopy(o);
}



int fpcvt_data_nb(PyObject *o, void *a)
{
    data_nb_type_ *object;
    if(o == Py_None)
    {
        *((data_nb_type_ **)a) = 0;
        return 1;
    }
    object = data_nb_dispatcher_->unwrap(o);
    *((data_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_data_nb_copy fpcvt_data_nb

PyObject *tpcvt_data_nb(void *a)
{
    data_nb_type_ *o = *((data_nb_type_ **)a);
    return data_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_data_nb_copy(void *a)
{
    data_nb_type_ *o = (data_nb_type_ *)a;
    return data_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct readevent_object_ {
    PyObject_HEAD
    readevent_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct readevent_wrapper_: public readevent_type_ 
{

    static void __deleter__(readevent_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    readevent_wrapper_( PyObject *o, PyInterpreterState *i  ,const readevent_type_ & a0  ): readevent_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~readevent_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    data_nb_type_ evt_id_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::evt_id();
                
            
            
        }

        return
            
                evt_id
            
            
                ();
    }

    static PyObject *evt_id_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    data_nb_type_ r = o->evt_id_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::evt_id");
        }

        return 0;
    }



    unsigned long long evt_time_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::evt_time();
                
            
            
        }

        return
            
                evt_time
            
            
                ();
    }

    static PyObject *evt_time_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long long r = o->evt_time_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ull((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::evt_time");
        }

        return 0;
    }



    float evt_beat_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::evt_beat();
                
            
            
        }

        return
            
                evt_beat
            
            
                ();
    }

    static PyObject *evt_beat_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    float r = o->evt_beat_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::evt_beat");
        }

        return 0;
    }



    data_nb_type_ cur_value_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::cur_value();
                
            
            
        }

        return
            
                cur_value
            
            
                ();
    }

    static PyObject *cur_value_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    data_nb_type_ r = o->cur_value_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_nb_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::cur_value");
        }

        return 0;
    }



    unsigned long long cur_time_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::cur_time();
                
            
            
        }

        return
            
                cur_time
            
            
                ();
    }

    static PyObject *cur_time_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned long long r = o->cur_time_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ull((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::cur_time");
        }

        return 0;
    }



    float cur_beat_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::cur_beat();
                
            
            
        }

        return
            
                cur_beat
            
            
                ();
    }

    static PyObject *cur_beat_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    float r = o->cur_beat_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::cur_beat");
        }

        return 0;
    }



    unsigned int cur_signal_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::cur_signal();
                
            
            
        }

        return
            
                cur_signal
            
            
                ();
    }

    static PyObject *cur_signal_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->cur_signal_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::cur_signal");
        }

        return 0;
    }



    bool isvalid_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::isvalid();
                
            
            
        }

        return
            
                isvalid
            
            
                ();
    }

    static PyObject *isvalid_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    bool r = o->isvalid_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::isvalid");
        }

        return 0;
    }



    void reset_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::reset();
                
            
            
        }

        return
            
                reset
            
            
                ();
    }

    static PyObject *reset_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->reset_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::reset");
        }

        return 0;
    }



    void next_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::next();
                
            
            
        }

        return
            
                next
            
            
                ();
    }

    static PyObject *next_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->next_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::next");
        }

        return 0;
    }



    void clear_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return readevent_type_::clear();
                
            
            
        }

        return
            
                clear
            
            
                ();
    }

    static PyObject *clear_method_(readevent_object_ *self, PyObject *args)
    {
        

        readevent_wrapper_ *o = (readevent_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null readevent c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->clear_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in readevent::clear");
        }

        return 0;
    }





    static PyObject *_method_lock(readevent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(readevent_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void readevent_delete_(readevent_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            readevent_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (readevent_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        readevent_wrapper_::__deleter__((readevent_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void readevent_dealloc_(readevent_object_* self)
{
    
    readevent_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * readevent_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    readevent_object_ *self;

    if(!(self = (readevent_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int readevent_init_(readevent_object_ *self, PyObject *args, PyObject *kwds)
{
    readevent_delete_(self);

    

    {
         readevent_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_readevent_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new readevent_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing readevent");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            readevent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"readevent invalid arguments");
    return -1;

}


static const char *readevent_evt_id_docstring_ = "";

static const char *readevent_evt_time_docstring_ = "";

static const char *readevent_evt_beat_docstring_ = "";

static const char *readevent_cur_value_docstring_ = "";

static const char *readevent_cur_time_docstring_ = "";

static const char *readevent_cur_beat_docstring_ = "";

static const char *readevent_cur_signal_docstring_ = "";

static const char *readevent_isvalid_docstring_ = "";

static const char *readevent_reset_docstring_ = "";

static const char *readevent_next_docstring_ = "";

static const char *readevent_clear_docstring_ = "";




static PyMethodDef readevent_methods_[] = {
    
        { "evt_id",(PyCFunction)readevent_wrapper_::evt_id_method_,METH_VARARGS,(char *)readevent_evt_id_docstring_ },
    
        { "evt_time",(PyCFunction)readevent_wrapper_::evt_time_method_,METH_VARARGS,(char *)readevent_evt_time_docstring_ },
    
        { "evt_beat",(PyCFunction)readevent_wrapper_::evt_beat_method_,METH_VARARGS,(char *)readevent_evt_beat_docstring_ },
    
        { "cur_value",(PyCFunction)readevent_wrapper_::cur_value_method_,METH_VARARGS,(char *)readevent_cur_value_docstring_ },
    
        { "cur_time",(PyCFunction)readevent_wrapper_::cur_time_method_,METH_VARARGS,(char *)readevent_cur_time_docstring_ },
    
        { "cur_beat",(PyCFunction)readevent_wrapper_::cur_beat_method_,METH_VARARGS,(char *)readevent_cur_beat_docstring_ },
    
        { "cur_signal",(PyCFunction)readevent_wrapper_::cur_signal_method_,METH_VARARGS,(char *)readevent_cur_signal_docstring_ },
    
        { "isvalid",(PyCFunction)readevent_wrapper_::isvalid_method_,METH_VARARGS,(char *)readevent_isvalid_docstring_ },
    
        { "reset",(PyCFunction)readevent_wrapper_::reset_method_,METH_VARARGS,(char *)readevent_reset_docstring_ },
    
        { "next",(PyCFunction)readevent_wrapper_::next_method_,METH_VARARGS,(char *)readevent_next_docstring_ },
    
        { "clear",(PyCFunction)readevent_wrapper_::clear_method_,METH_VARARGS,(char *)readevent_clear_docstring_ },
    
    
        { "py_lock",(PyCFunction)readevent_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)readevent_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *readevent_docstring_ = "";

PyTypeObject readevent_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "recorder_native.readevent",/*tp_name*/
    sizeof(readevent_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)readevent_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)readevent_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    readevent_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)readevent_init_,  /* tp_init */
    0,                         /* tp_alloc */
    readevent_new_,             /* tp_new */
};

PyObject *readevent_wrapcopy_(readevent_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            readevent_object_ *obj = 
                PyObject_NEW(readevent_object_, readevent_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new readevent_type_(*o);
                }

                try
                {
                    readevent_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying readevent");
                return 0;
            }
        

    
}

PyObject *readevent_wrap_(readevent_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = readevent_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    readevent_object_ *self = 
        PyObject_NEW(readevent_object_, &readevent_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        readevent_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

readevent_type_ *readevent_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &readevent_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected readevent object");
        return 0;
    }

    readevent_type_ *object = static_cast<readevent_type_ *>(((readevent_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null readevent object in unwrap");
        return 0;
    }

    return object;
}

void readevent_insert_(const std::pair< readevent_type_ *, PyObject *> &p)
{




    readevent_wrapped_.insert(p);


}

void readevent_erase_(readevent_type_ *p)
{




    readevent_wrapped_.erase(p);


}

PyObject * readevent_find_(readevent_type_ *p)
{




    PyObject *ret = 0;
    std::map< readevent_type_ *, PyObject *>::iterator where = readevent_wrapped_.find(p);
    if(where != readevent_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct readevent_dispatch_ readevent_realdispatcher_ =
{
    &readevent_typeobject_,
    readevent_wrap_,
    readevent_wrapcopy_,
    readevent_unwrap_,
    readevent_insert_,
    readevent_erase_,
    readevent_find_
};

  

struct recording_object_ {
    PyObject_HEAD
    recording_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct recording_wrapper_: public recording_type_ 
{

    static void __deleter__(recording_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    recording_wrapper_( PyObject *o, PyInterpreterState *i  ): recording_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    recording_wrapper_( PyObject *o, PyInterpreterState *i  ,const recording_type_ & a0  ): recording_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~recording_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    unsigned int signals_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::signals();
                
            
            
        }

        return
            
                signals
            
            
                ();
    }

    static PyObject *signals_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->signals_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::signals");
        }

        return 0;
    }



    unsigned int wires_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::wires();
                
            
            
        }

        return
            
                wires
            
            
                ();
    }

    static PyObject *wires_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->wires_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::wires");
        }

        return 0;
    }



    unsigned int events_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::events();
                
            
            
        }

        return
            
                events
            
            
                ();
    }

    static PyObject *events_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    unsigned int r = o->events_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::events");
        }

        return 0;
    }



    bool isvalid_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::isvalid();
                
            
            
        }

        return
            
                isvalid
            
            
                ();
    }

    static PyObject *isvalid_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    bool r = o->isvalid_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::isvalid");
        }

        return 0;
    }



    void reset_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::reset();
                
            
            
        }

        return
            
                reset
            
            
                ();
    }

    static PyObject *reset_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->reset_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::reset");
        }

        return 0;
    }



    void next_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::next();
                
            
            
        }

        return
            
                next
            
            
                ();
    }

    static PyObject *next_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->next_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::next");
        }

        return 0;
    }



    data_type_ get_tag_call_(bool owned ,unsigned char a0 )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::get_tag( a0 );
                
            
            
        }

        return
            
                get_tag
            
            
                ( a0 );
    }

    static PyObject *get_tag_method_(recording_object_ *self, PyObject *args)
    {
         unsigned char a0; 

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_uc,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    data_type_ r = o->get_tag_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_data_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::get_tag");
        }

        return 0;
    }



    readevent_type_ cur_event_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::cur_event();
                
            
            
        }

        return
            
                cur_event
            
            
                ();
    }

    static PyObject *cur_event_method_(recording_object_ *self, PyObject *args)
    {
        

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  (true);
                {
                    readevent_type_ r = o->cur_event_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_readevent_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::cur_event");
        }

        return 0;
    }



    void write_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return recording_type_::write( a0 );
                
            
            
        }

        return
            
                write
            
            
                ( a0 );
    }

    static PyObject *write_method_(recording_object_ *self, PyObject *args)
    {
         const char * a0; 

        recording_wrapper_ *o = (recording_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recording c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock (true);
                o->write_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recording::write");
        }

        return 0;
    }





    static PyObject *_method_lock(recording_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(recording_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void recording_delete_(recording_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            recording_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (recording_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        recording_wrapper_::__deleter__((recording_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void recording_dealloc_(recording_object_* self)
{
    
    recording_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * recording_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    recording_object_ *self;

    if(!(self = (recording_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int recording_init_(recording_object_ *self, PyObject *args, PyObject *kwds)
{
    recording_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new recording_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing recording");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            recording_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         recording_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_recording_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new recording_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing recording");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            recording_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"recording invalid arguments");
    return -1;

}


static const char *recording_signals_docstring_ = "";

static const char *recording_wires_docstring_ = "";

static const char *recording_events_docstring_ = "";

static const char *recording_isvalid_docstring_ = "";

static const char *recording_reset_docstring_ = "";

static const char *recording_next_docstring_ = "";

static const char *recording_get_tag_docstring_ = "";

static const char *recording_cur_event_docstring_ = "";

static const char *recording_write_docstring_ = "";




static PyMethodDef recording_methods_[] = {
    
        { "signals",(PyCFunction)recording_wrapper_::signals_method_,METH_VARARGS,(char *)recording_signals_docstring_ },
    
        { "wires",(PyCFunction)recording_wrapper_::wires_method_,METH_VARARGS,(char *)recording_wires_docstring_ },
    
        { "events",(PyCFunction)recording_wrapper_::events_method_,METH_VARARGS,(char *)recording_events_docstring_ },
    
        { "isvalid",(PyCFunction)recording_wrapper_::isvalid_method_,METH_VARARGS,(char *)recording_isvalid_docstring_ },
    
        { "reset",(PyCFunction)recording_wrapper_::reset_method_,METH_VARARGS,(char *)recording_reset_docstring_ },
    
        { "next",(PyCFunction)recording_wrapper_::next_method_,METH_VARARGS,(char *)recording_next_docstring_ },
    
        { "get_tag",(PyCFunction)recording_wrapper_::get_tag_method_,METH_VARARGS,(char *)recording_get_tag_docstring_ },
    
        { "cur_event",(PyCFunction)recording_wrapper_::cur_event_method_,METH_VARARGS,(char *)recording_cur_event_docstring_ },
    
        { "write",(PyCFunction)recording_wrapper_::write_method_,METH_VARARGS,(char *)recording_write_docstring_ },
    
    
        { "py_lock",(PyCFunction)recording_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)recording_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *recording_docstring_ = "";

PyTypeObject recording_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "recorder_native.recording",/*tp_name*/
    sizeof(recording_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)recording_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)recording_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    recording_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)recording_init_,  /* tp_init */
    0,                         /* tp_alloc */
    recording_new_,             /* tp_new */
};

PyObject *recording_wrapcopy_(recording_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            recording_object_ *obj = 
                PyObject_NEW(recording_object_, recording_dispatcher_->type); 
                

            if(!obj)
            {
                return 0;
            }

            

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new recording_type_(*o);
                }

                try
                {
                    recording_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying recording");
                return 0;
            }
        

    
}

PyObject *recording_wrap_(recording_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = recording_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    recording_object_ *self = 
        PyObject_NEW(recording_object_, &recording_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        recording_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

recording_type_ *recording_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &recording_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected recording object");
        return 0;
    }

    recording_type_ *object = static_cast<recording_type_ *>(((recording_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null recording object in unwrap");
        return 0;
    }

    return object;
}

void recording_insert_(const std::pair< recording_type_ *, PyObject *> &p)
{




    recording_wrapped_.insert(p);


}

void recording_erase_(recording_type_ *p)
{




    recording_wrapped_.erase(p);


}

PyObject * recording_find_(recording_type_ *p)
{




    PyObject *ret = 0;
    std::map< recording_type_ *, PyObject *>::iterator where = recording_wrapped_.find(p);
    if(where != recording_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct recording_dispatch_ recording_realdispatcher_ =
{
    &recording_typeobject_,
    recording_wrap_,
    recording_wrapcopy_,
    recording_unwrap_,
    recording_insert_,
    recording_erase_,
    recording_find_
};

  

struct recorder_object_ {
    PyObject_HEAD
    recorder_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct recorder_wrapper_: public recorder_type_ 
{

    static void __deleter__(recorder_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
    }
*/



    recorder_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,unsigned int a1  ): recorder_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~recorder_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void record_started(  const recording_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"record_started");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_recording_copy,&a0);
        //printf("record_started before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("record_started after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void record_done(  const recording_type_ & a0  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"record_done");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"(O&)" ,tpcvt_recording_copy,&a0);
        //printf("record_done before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("record_done after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void record_aborted(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"record_aborted");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("record_aborted before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("record_aborted after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    int record_call_(bool owned ,unsigned int a0 )
    {
        if(owned)
        {
            
                
                
                    return recorder_type_::record( a0 );
                
            
            
        }

        return
            
                record
            
            
                ( a0 );
    }

    static PyObject *record_method_(recorder_object_ *self, PyObject *args)
    {
         unsigned int a0; 

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ui,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    int r = o->record_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_i((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::record");
        }

        return 0;
    }



    void record_started_call_(bool owned ,const recording_type_ & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method recorder::record_started");
                
                
            
            
        }

        return
            
                record_started
            
            
                ( a0 );
    }

    static PyObject *record_started_method_(recorder_object_ *self, PyObject *args)
    {
         recording_type_ * a0; 

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_recording_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->record_started_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::record_started");
        }

        return 0;
    }



    void record_done_call_(bool owned ,const recording_type_ & a0 )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method recorder::record_done");
                
                
            
            
        }

        return
            
                record_done
            
            
                ( a0 );
    }

    static PyObject *record_done_method_(recorder_object_ *self, PyObject *args)
    {
         recording_type_ * a0; 

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_recording_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->record_done_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::record_done");
        }

        return 0;
    }



    void record_aborted_call_(bool owned )
    {
        if(owned)
        {
            
                
                    throw pip_err_t("calling pure virtual method recorder::record_aborted");
                
                
            
            
        }

        return
            
                record_aborted
            
            
                ();
    }

    static PyObject *record_aborted_method_(recorder_object_ *self, PyObject *args)
    {
        

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->record_aborted_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::record_aborted");
        }

        return 0;
    }



    void abort_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recorder_type_::abort();
                
            
            
        }

        return
            
                abort
            
            
                ();
    }

    static PyObject *abort_method_(recorder_object_ *self, PyObject *args)
    {
        

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->abort_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::abort");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return recorder_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(recorder_object_ *self, PyObject *args)
    {
        

        recorder_wrapper_ *o = (recorder_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null recorder c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in recorder::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(recorder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(recorder_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void recorder_delete_(recorder_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            recorder_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (recorder_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        recorder_wrapper_::__deleter__((recorder_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void recorder_dealloc_(recorder_object_* self)
{
    
    recorder_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * recorder_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    recorder_object_ *self;

    if(!(self = (recorder_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int recorder_init_(recorder_object_ *self, PyObject *args, PyObject *kwds)
{
    recorder_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new recorder_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing recorder");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            recorder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"recorder invalid arguments");
    return -1;

}


static const char *recorder_record_docstring_ = "";

static const char *recorder_record_started_docstring_ = "";

static const char *recorder_record_done_docstring_ = "";

static const char *recorder_record_aborted_docstring_ = "";

static const char *recorder_abort_docstring_ = "";

static const char *recorder_cookie_docstring_ = "";




static PyMethodDef recorder_methods_[] = {
    
        { "record",(PyCFunction)recorder_wrapper_::record_method_,METH_VARARGS,(char *)recorder_record_docstring_ },
    
        { "record_started",(PyCFunction)recorder_wrapper_::record_started_method_,METH_VARARGS,(char *)recorder_record_started_docstring_ },
    
        { "record_done",(PyCFunction)recorder_wrapper_::record_done_method_,METH_VARARGS,(char *)recorder_record_done_docstring_ },
    
        { "record_aborted",(PyCFunction)recorder_wrapper_::record_aborted_method_,METH_VARARGS,(char *)recorder_record_aborted_docstring_ },
    
        { "abort",(PyCFunction)recorder_wrapper_::abort_method_,METH_VARARGS,(char *)recorder_abort_docstring_ },
    
        { "cookie",(PyCFunction)recorder_wrapper_::cookie_method_,METH_VARARGS,(char *)recorder_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)recorder_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)recorder_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *recorder_docstring_ = "";

PyTypeObject recorder_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "recorder_native.recorder",/*tp_name*/
    sizeof(recorder_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)recorder_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)recorder_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    recorder_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)recorder_init_,  /* tp_init */
    0,                         /* tp_alloc */
    recorder_new_,             /* tp_new */
};

PyObject *recorder_wrapcopy_(recorder_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *recorder_wrap_(recorder_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = recorder_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    recorder_object_ *self = 
        PyObject_NEW(recorder_object_, &recorder_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        recorder_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

recorder_type_ *recorder_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &recorder_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected recorder object");
        return 0;
    }

    recorder_type_ *object = static_cast<recorder_type_ *>(((recorder_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null recorder object in unwrap");
        return 0;
    }

    return object;
}

void recorder_insert_(const std::pair< recorder_type_ *, PyObject *> &p)
{




    recorder_wrapped_.insert(p);


}

void recorder_erase_(recorder_type_ *p)
{




    recorder_wrapped_.erase(p);


}

PyObject * recorder_find_(recorder_type_ *p)
{




    PyObject *ret = 0;
    std::map< recorder_type_ *, PyObject *>::iterator where = recorder_wrapped_.find(p);
    if(where != recorder_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct recorder_dispatch_ recorder_realdispatcher_ =
{
    &recorder_typeobject_,
    recorder_wrap_,
    recorder_wrapcopy_,
    recorder_unwrap_,
    recorder_insert_,
    recorder_erase_,
    recorder_find_
};

  

struct kplayer_object_ {
    PyObject_HEAD
    kplayer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct kplayer_wrapper_: public kplayer_type_ 
{

    static void __deleter__(kplayer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    kplayer_wrapper_( PyObject *o, PyInterpreterState *i  ,clockdomain_ctl_type_ * a0  ,unsigned int a1  ,unsigned int a2  ,const cookie_type_ & a3  ): kplayer_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~kplayer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(kplayer_object_ *self, PyObject *args)
    {
        

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::cookie");
        }

        return 0;
    }



    unsigned long load_call_(bool owned ,const std::string & a0 ,const recording_type_ & a1 ,unsigned int a2 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::load( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                load
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *load_method_(kplayer_object_ *self, PyObject *args)
    {
         std::string a0;  recording_type_ * a1;  unsigned int a2; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_recording_copy,&a1  ,fpcvt_ui,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->load_call_ (self->owned ,a0 ,*a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::load");
        }

        return 0;
    }



    void unload_call_(bool owned ,unsigned long a0 ,bool a1 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::unload( a0  ,a1 );
                
            
            
        }

        return
            
                unload
            
            
                ( a0  ,a1 );
    }

    static PyObject *unload_method_(kplayer_object_ *self, PyObject *args)
    {
         unsigned long a0;  bool a1; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ul,&a0  ,fpcvt_b,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unload_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::unload");
        }

        return 0;
    }



    void abort_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::abort( a0 );
                
            
            
        }

        return
            
                abort
            
            
                ( a0 );
    }

    static PyObject *abort_method_(kplayer_object_ *self, PyObject *args)
    {
         std::string a0; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->abort_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::abort");
        }

        return 0;
    }



    void abortcookie_call_(bool owned ,unsigned long a0 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::abortcookie( a0 );
                
            
            
        }

        return
            
                abortcookie
            
            
                ( a0 );
    }

    static PyObject *abortcookie_method_(kplayer_object_ *self, PyObject *args)
    {
         unsigned long a0; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ul,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->abortcookie_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::abortcookie");
        }

        return 0;
    }



    void abortall_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::abortall();
                
            
            
        }

        return
            
                abortall
            
            
                ();
    }

    static PyObject *abortall_method_(kplayer_object_ *self, PyObject *args)
    {
        

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->abortall_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::abortall");
        }

        return 0;
    }



    change_type_ player_call_(bool owned ,unsigned long a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::player( a0  ,a1 );
                
            
            
        }

        return
            
                player
            
            
                ( a0  ,a1 );
    }

    static PyObject *player_method_(kplayer_object_ *self, PyObject *args)
    {
         unsigned long a0;  unsigned int a1; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ul,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->player_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::player");
        }

        return 0;
    }



    unsigned long getcookie_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::getcookie( a0 );
                
            
            
        }

        return
            
                getcookie
            
            
                ( a0 );
    }

    static PyObject *getcookie_method_(kplayer_object_ *self, PyObject *args)
    {
         std::string a0; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->getcookie_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::getcookie");
        }

        return 0;
    }



    unsigned long clonecookie_call_(bool owned ,const std::string & a0 ,unsigned int a1 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::clonecookie( a0  ,a1 );
                
            
            
        }

        return
            
                clonecookie
            
            
                ( a0  ,a1 );
    }

    static PyObject *clonecookie_method_(kplayer_object_ *self, PyObject *args)
    {
         std::string a0;  unsigned int a1; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_stdstr,&a0  ,fpcvt_ui,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    unsigned long r = o->clonecookie_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ul((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::clonecookie");
        }

        return 0;
    }



    std::string cookiename_call_(bool owned ,unsigned long a0 )
    {
        if(owned)
        {
            
                
                
                    return kplayer_type_::cookiename( a0 );
                
            
            
        }

        return
            
                cookiename
            
            
                ( a0 );
    }

    static PyObject *cookiename_method_(kplayer_object_ *self, PyObject *args)
    {
         unsigned long a0; 

        kplayer_wrapper_ *o = (kplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null kplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ul,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->cookiename_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in kplayer::cookiename");
        }

        return 0;
    }





    static PyObject *_method_lock(kplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(kplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void kplayer_delete_(kplayer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            kplayer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (kplayer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        kplayer_wrapper_::__deleter__((kplayer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void kplayer_dealloc_(kplayer_object_* self)
{
    
    kplayer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * kplayer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    kplayer_object_ *self;

    if(!(self = (kplayer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int kplayer_init_(kplayer_object_ *self, PyObject *args, PyObject *kwds)
{
    kplayer_delete_(self);

    

    {
         clockdomain_ctl_type_ * a0;  unsigned int a1;  unsigned int a2;  cookie_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_clockdomain_ctl,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_cookie_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new kplayer_wrapper_((PyObject *)self,interp ,a0,a1,a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing kplayer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            kplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"kplayer invalid arguments");
    return -1;

}


static const char *kplayer_cookie_docstring_ = "";

static const char *kplayer_load_docstring_ = "";

static const char *kplayer_unload_docstring_ = "";

static const char *kplayer_abort_docstring_ = "";

static const char *kplayer_abortcookie_docstring_ = "";

static const char *kplayer_abortall_docstring_ = "";

static const char *kplayer_player_docstring_ = "";

static const char *kplayer_getcookie_docstring_ = "";

static const char *kplayer_clonecookie_docstring_ = "";

static const char *kplayer_cookiename_docstring_ = "";




static PyMethodDef kplayer_methods_[] = {
    
        { "cookie",(PyCFunction)kplayer_wrapper_::cookie_method_,METH_VARARGS,(char *)kplayer_cookie_docstring_ },
    
        { "load",(PyCFunction)kplayer_wrapper_::load_method_,METH_VARARGS,(char *)kplayer_load_docstring_ },
    
        { "unload",(PyCFunction)kplayer_wrapper_::unload_method_,METH_VARARGS,(char *)kplayer_unload_docstring_ },
    
        { "abort",(PyCFunction)kplayer_wrapper_::abort_method_,METH_VARARGS,(char *)kplayer_abort_docstring_ },
    
        { "abortcookie",(PyCFunction)kplayer_wrapper_::abortcookie_method_,METH_VARARGS,(char *)kplayer_abortcookie_docstring_ },
    
        { "abortall",(PyCFunction)kplayer_wrapper_::abortall_method_,METH_VARARGS,(char *)kplayer_abortall_docstring_ },
    
        { "player",(PyCFunction)kplayer_wrapper_::player_method_,METH_VARARGS,(char *)kplayer_player_docstring_ },
    
        { "getcookie",(PyCFunction)kplayer_wrapper_::getcookie_method_,METH_VARARGS,(char *)kplayer_getcookie_docstring_ },
    
        { "clonecookie",(PyCFunction)kplayer_wrapper_::clonecookie_method_,METH_VARARGS,(char *)kplayer_clonecookie_docstring_ },
    
        { "cookiename",(PyCFunction)kplayer_wrapper_::cookiename_method_,METH_VARARGS,(char *)kplayer_cookiename_docstring_ },
    
    
        { "py_lock",(PyCFunction)kplayer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)kplayer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *kplayer_docstring_ = "";

PyTypeObject kplayer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "recorder_native.kplayer",/*tp_name*/
    sizeof(kplayer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)kplayer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)kplayer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    kplayer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)kplayer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    kplayer_new_,             /* tp_new */
};

PyObject *kplayer_wrapcopy_(kplayer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *kplayer_wrap_(kplayer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = kplayer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    kplayer_object_ *self = 
        PyObject_NEW(kplayer_object_, &kplayer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        kplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

kplayer_type_ *kplayer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &kplayer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected kplayer object");
        return 0;
    }

    kplayer_type_ *object = static_cast<kplayer_type_ *>(((kplayer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null kplayer object in unwrap");
        return 0;
    }

    return object;
}

void kplayer_insert_(const std::pair< kplayer_type_ *, PyObject *> &p)
{




    kplayer_wrapped_.insert(p);


}

void kplayer_erase_(kplayer_type_ *p)
{




    kplayer_wrapped_.erase(p);


}

PyObject * kplayer_find_(kplayer_type_ *p)
{




    PyObject *ret = 0;
    std::map< kplayer_type_ *, PyObject *>::iterator where = kplayer_wrapped_.find(p);
    if(where != kplayer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct kplayer_dispatch_ kplayer_realdispatcher_ =
{
    &kplayer_typeobject_,
    kplayer_wrap_,
    kplayer_wrapcopy_,
    kplayer_unwrap_,
    kplayer_insert_,
    kplayer_erase_,
    kplayer_find_
};

  

struct nplayer_object_ {
    PyObject_HEAD
    nplayer_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct nplayer_wrapper_: public nplayer_type_ 
{

    static void __deleter__(nplayer_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    nplayer_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,unsigned int a1  ,unsigned int a2  ,clockdomain_ctl_type_ * a3  ): nplayer_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~nplayer_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    change_type_ play_call_(bool owned ,unsigned int a0 ,unsigned int a1 ,unsigned long long a2 )
    {
        if(owned)
        {
            
                
                
                    return nplayer_type_::play( a0  ,a1  ,a2 );
                
            
            
        }

        return
            
                play
            
            
                ( a0  ,a1  ,a2 );
    }

    static PyObject *play_method_(nplayer_object_ *self, PyObject *args)
    {
         unsigned int a0;  unsigned int a1;  unsigned long long a2; 

        nplayer_wrapper_ *o = (nplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null nplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&"  ,fpcvt_ui,&a0  ,fpcvt_ui,&a1  ,fpcvt_ull,&a2 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    change_type_ r = o->play_call_ (self->owned ,a0 ,a1 ,a2  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_change_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in nplayer::play");
        }

        return 0;
    }



    clockref_type_ * get_clock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return nplayer_type_::get_clock();
                
            
            
        }

        return
            
                get_clock
            
            
                ();
    }

    static PyObject *get_clock_method_(nplayer_object_ *self, PyObject *args)
    {
        

        nplayer_wrapper_ *o = (nplayer_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null nplayer c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    clockref_type_ * r = o->get_clock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_clockref((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in nplayer::get_clock");
        }

        return 0;
    }





    static PyObject *_method_lock(nplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(nplayer_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void nplayer_delete_(nplayer_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            nplayer_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (nplayer_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        nplayer_wrapper_::__deleter__((nplayer_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void nplayer_dealloc_(nplayer_object_* self)
{
    
    nplayer_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * nplayer_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    nplayer_object_ *self;

    if(!(self = (nplayer_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int nplayer_init_(nplayer_object_ *self, PyObject *args, PyObject *kwds)
{
    nplayer_delete_(self);

    

    {
         cookie_type_ * a0;  unsigned int a1;  unsigned int a2;  clockdomain_ctl_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_ui,&a1,fpcvt_ui,&a2,fpcvt_clockdomain_ctl,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new nplayer_wrapper_((PyObject *)self,interp ,*a0,a1,a2,a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing nplayer");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            nplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"nplayer invalid arguments");
    return -1;

}


static const char *nplayer_play_docstring_ = "";

static const char *nplayer_get_clock_docstring_ = "";




static PyMethodDef nplayer_methods_[] = {
    
        { "play",(PyCFunction)nplayer_wrapper_::play_method_,METH_VARARGS,(char *)nplayer_play_docstring_ },
    
        { "get_clock",(PyCFunction)nplayer_wrapper_::get_clock_method_,METH_VARARGS,(char *)nplayer_get_clock_docstring_ },
    
    
        { "py_lock",(PyCFunction)nplayer_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)nplayer_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *nplayer_docstring_ = "";

PyTypeObject nplayer_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "recorder_native.nplayer",/*tp_name*/
    sizeof(nplayer_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)nplayer_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)nplayer_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    nplayer_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)nplayer_init_,  /* tp_init */
    0,                         /* tp_alloc */
    nplayer_new_,             /* tp_new */
};

PyObject *nplayer_wrapcopy_(nplayer_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *nplayer_wrap_(nplayer_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = nplayer_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    nplayer_object_ *self = 
        PyObject_NEW(nplayer_object_, &nplayer_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        nplayer_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

nplayer_type_ *nplayer_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &nplayer_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected nplayer object");
        return 0;
    }

    nplayer_type_ *object = static_cast<nplayer_type_ *>(((nplayer_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null nplayer object in unwrap");
        return 0;
    }

    return object;
}

void nplayer_insert_(const std::pair< nplayer_type_ *, PyObject *> &p)
{




    nplayer_wrapped_.insert(p);


}

void nplayer_erase_(nplayer_type_ *p)
{




    nplayer_wrapped_.erase(p);


}

PyObject * nplayer_find_(nplayer_type_ *p)
{




    PyObject *ret = 0;
    std::map< nplayer_type_ *, PyObject *>::iterator where = nplayer_wrapped_.find(p);
    if(where != nplayer_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct nplayer_dispatch_ nplayer_realdispatcher_ =
{
    &nplayer_typeobject_,
    nplayer_wrap_,
    nplayer_wrapcopy_,
    nplayer_unwrap_,
    nplayer_insert_,
    nplayer_erase_,
    nplayer_find_
};

               


static const char *module_method_read_docstring_ = "";

static PyObject *module_method_read(PyObject *self, PyObject *args)
{
     const char * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_str,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    recording_type_ r = recorder::read  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_recording_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in read");
    }

    return 0;
}


static const char *module_method_read_meta_docstring_ = "";

static PyObject *module_method_read_meta(PyObject *self, PyObject *args)
{
     const char * a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_str,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock(true);
                {
                    recording_type_ r = recorder::read_meta  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_recording_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in read_meta");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "read",module_method_read,METH_VARARGS, (char *)module_method_read_docstring_ }, 
{ "read_meta",module_method_read_meta,METH_VARARGS, (char *)module_method_read_meta_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initrecorder_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            readevent_dispatcher_ = &readevent_realdispatcher_;
        
    
        
            recording_dispatcher_ = &recording_realdispatcher_;
        
    
        
            recorder_dispatcher_ = &recorder_realdispatcher_;
        
    
        
            kplayer_dispatcher_ = &kplayer_realdispatcher_;
        
    
        
            nplayer_dispatcher_ = &nplayer_realdispatcher_;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_dispatch in module piw");
                return;
            }

            change_dispatcher_ = (change_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_change_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_change_nb_dispatch in module piw");
                return;
            }

            change_nb_dispatcher_ = (change_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_dispatch in module piw");
                return;
            }

            data_dispatcher_ = (data_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_data_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_data_nb_dispatch in module piw");
                return;
            }

            data_nb_dispatcher_ = (data_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockref_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockref_dispatch in module piw");
                return;
            }

            clockref_dispatcher_ = (clockref_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&readevent_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&recording_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&recorder_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&kplayer_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&nplayer_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("recorder_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(readevent_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_readevent_dispatch",s);
            Py_INCREF(&readevent_typeobject_);
            PyModule_AddObject(m,"readevent",(PyObject *)&readevent_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(recording_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_recording_dispatch",s);
            Py_INCREF(&recording_typeobject_);
            PyModule_AddObject(m,"recording",(PyObject *)&recording_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(recorder_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_recorder_dispatch",s);
            Py_INCREF(&recorder_typeobject_);
            PyModule_AddObject(m,"recorder",(PyObject *)&recorder_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(kplayer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_kplayer_dispatch",s);
            Py_INCREF(&kplayer_typeobject_);
            PyModule_AddObject(m,"kplayer",(PyObject *)&kplayer_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(nplayer_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_nplayer_dispatch",s);
            Py_INCREF(&nplayer_typeobject_);
            PyModule_AddObject(m,"nplayer",(PyObject *)&nplayer_typeobject_);
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

