#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <plg_sampler2/smp_loader.h>
#include <plg_sampler2/smp_player.h>


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef sampler2::player_t player_type_; 


 
static std::map<player_type_ *, PyObject *> player_wrapped_;
 


struct player_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(player_type_ *, bool owned);
    PyObject *(*wrapcopy)(player_type_ *);
    player_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< player_type_ *, PyObject * > &);
    void (*erase)(player_type_ *);
    PyObject *(*find)(player_type_ *);
};

struct player_dispatch_ *player_dispatcher_;



typedef sampler2::loader_t loader_type_; 


 
static std::map<loader_type_ *, PyObject *> loader_wrapped_;
 


struct loader_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(loader_type_ *, bool owned);
    PyObject *(*wrapcopy)(loader_type_ *);
    loader_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< loader_type_ *, PyObject * > &);
    void (*erase)(loader_type_ *);
    PyObject *(*find)(loader_type_ *);
};

struct loader_dispatch_ *loader_dispatcher_;



typedef piw::presetref_t preset_type_; 





struct preset_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(preset_type_ *, bool owned);
    PyObject *(*wrapcopy)(preset_type_ *);
    preset_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< preset_type_ *, PyObject * > &);
    void (*erase)(preset_type_ *);
    PyObject *(*find)(preset_type_ *);
};

struct preset_dispatch_ *preset_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_player(PyObject *o, void *a)
{
    player_type_ *object;
    if(o == Py_None)
    {
        *((player_type_ **)a) = 0;
        return 1;
    }
    object = player_dispatcher_->unwrap(o);
    *((player_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_player_copy fpcvt_player

PyObject *tpcvt_player(void *a)
{
    player_type_ *o = *((player_type_ **)a);
    return player_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_player_copy(void *a)
{
    player_type_ *o = (player_type_ *)a;
    return player_dispatcher_->wrapcopy(o);
}



int fpcvt_loader(PyObject *o, void *a)
{
    loader_type_ *object;
    if(o == Py_None)
    {
        *((loader_type_ **)a) = 0;
        return 1;
    }
    object = loader_dispatcher_->unwrap(o);
    *((loader_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_loader_copy fpcvt_loader

PyObject *tpcvt_loader(void *a)
{
    loader_type_ *o = *((loader_type_ **)a);
    return loader_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_loader_copy(void *a)
{
    loader_type_ *o = (loader_type_ *)a;
    return loader_dispatcher_->wrapcopy(o);
}



int fpcvt_preset(PyObject *o, void *a)
{
    preset_type_ *object;
    if(o == Py_None)
    {
        *((preset_type_ **)a) = 0;
        return 1;
    }
    object = preset_dispatcher_->unwrap(o);
    *((preset_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_preset_copy fpcvt_preset

PyObject *tpcvt_preset(void *a)
{
    preset_type_ *o = *((preset_type_ **)a);
    return preset_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_preset_copy(void *a)
{
    preset_type_ *o = (preset_type_ *)a;
    return preset_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct player_object_ {
    PyObject_HEAD
    player_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct player_wrapper_: public player_type_ 
{

    static void __deleter__(player_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    player_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): player_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~player_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void set_fade_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return player_type_::set_fade( a0 );
                
            
            
        }

        return
            
                set_fade
            
            
                ( a0 );
    }

    static PyObject *set_fade_method_(player_object_ *self, PyObject *args)
    {
         bool a0; 

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_fade_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::set_fade");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return player_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(player_object_ *self, PyObject *args)
    {
        

        player_wrapper_ *o = (player_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null player c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in player::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(player_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(player_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void player_delete_(player_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            player_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (player_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        player_wrapper_::__deleter__((player_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void player_dealloc_(player_object_* self)
{
    
    player_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * player_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    player_object_ *self;

    if(!(self = (player_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int player_init_(player_object_ *self, PyObject *args, PyObject *kwds)
{
    player_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new player_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing player");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            player_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"player invalid arguments");
    return -1;

}


static const char *player_set_fade_docstring_ = "";

static const char *player_cookie_docstring_ = "";




static PyMethodDef player_methods_[] = {
    
        { "set_fade",(PyCFunction)player_wrapper_::set_fade_method_,METH_VARARGS,(char *)player_set_fade_docstring_ },
    
        { "cookie",(PyCFunction)player_wrapper_::cookie_method_,METH_VARARGS,(char *)player_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)player_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)player_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *player_docstring_ = "";

PyTypeObject player_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "sampler2_native.player",/*tp_name*/
    sizeof(player_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)player_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)player_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    player_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)player_init_,  /* tp_init */
    0,                         /* tp_alloc */
    player_new_,             /* tp_new */
};

PyObject *player_wrapcopy_(player_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *player_wrap_(player_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = player_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    player_object_ *self = 
        PyObject_NEW(player_object_, &player_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        player_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

player_type_ *player_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &player_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected player object");
        return 0;
    }

    player_type_ *object = static_cast<player_type_ *>(((player_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null player object in unwrap");
        return 0;
    }

    return object;
}

void player_insert_(const std::pair< player_type_ *, PyObject *> &p)
{




    player_wrapped_.insert(p);


}

void player_erase_(player_type_ *p)
{




    player_wrapped_.erase(p);


}

PyObject * player_find_(player_type_ *p)
{




    PyObject *ret = 0;
    std::map< player_type_ *, PyObject *>::iterator where = player_wrapped_.find(p);
    if(where != player_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct player_dispatch_ player_realdispatcher_ =
{
    &player_typeobject_,
    player_wrap_,
    player_wrapcopy_,
    player_unwrap_,
    player_insert_,
    player_erase_,
    player_find_
};

  

struct loader_object_ {
    PyObject_HEAD
    loader_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct loader_wrapper_: public loader_type_ 
{

    static void __deleter__(loader_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    loader_wrapper_( PyObject *o, PyInterpreterState *i  ,player_type_ * a0  ,const cookie_type_ & a1  ,clockdomain_ctl_type_ * a2  ): loader_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~loader_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void load_call_(bool owned ,const preset_type_ & a0 )
    {
        if(owned)
        {
            
                
                
                    return loader_type_::load( a0 );
                
            
            
        }

        return
            
                load
            
            
                ( a0 );
    }

    static PyObject *load_method_(loader_object_ *self, PyObject *args)
    {
         preset_type_ * a0; 

        loader_wrapper_ *o = (loader_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loader c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_preset_copy,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->load_call_ (self->owned ,*a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loader::load");
        }

        return 0;
    }



    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return loader_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(loader_object_ *self, PyObject *args)
    {
        

        loader_wrapper_ *o = (loader_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null loader c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in loader::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(loader_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(loader_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void loader_delete_(loader_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            loader_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (loader_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        loader_wrapper_::__deleter__((loader_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void loader_dealloc_(loader_object_* self)
{
    
    loader_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * loader_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    loader_object_ *self;

    if(!(self = (loader_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int loader_init_(loader_object_ *self, PyObject *args, PyObject *kwds)
{
    loader_delete_(self);

    

    {
         player_type_ * a0;  cookie_type_ * a1;  clockdomain_ctl_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_player,&a0,fpcvt_cookie_copy,&a1,fpcvt_clockdomain_ctl,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new loader_wrapper_((PyObject *)self,interp ,a0,*a1,a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing loader");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            loader_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"loader invalid arguments");
    return -1;

}


static const char *loader_load_docstring_ = "";

static const char *loader_cookie_docstring_ = "";




static PyMethodDef loader_methods_[] = {
    
        { "load",(PyCFunction)loader_wrapper_::load_method_,METH_VARARGS,(char *)loader_load_docstring_ },
    
        { "cookie",(PyCFunction)loader_wrapper_::cookie_method_,METH_VARARGS,(char *)loader_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)loader_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)loader_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *loader_docstring_ = "";

PyTypeObject loader_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "sampler2_native.loader",/*tp_name*/
    sizeof(loader_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)loader_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)loader_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    loader_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)loader_init_,  /* tp_init */
    0,                         /* tp_alloc */
    loader_new_,             /* tp_new */
};

PyObject *loader_wrapcopy_(loader_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *loader_wrap_(loader_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = loader_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    loader_object_ *self = 
        PyObject_NEW(loader_object_, &loader_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        loader_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

loader_type_ *loader_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &loader_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected loader object");
        return 0;
    }

    loader_type_ *object = static_cast<loader_type_ *>(((loader_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null loader object in unwrap");
        return 0;
    }

    return object;
}

void loader_insert_(const std::pair< loader_type_ *, PyObject *> &p)
{




    loader_wrapped_.insert(p);


}

void loader_erase_(loader_type_ *p)
{




    loader_wrapped_.erase(p);


}

PyObject * loader_find_(loader_type_ *p)
{




    PyObject *ret = 0;
    std::map< loader_type_ *, PyObject *>::iterator where = loader_wrapped_.find(p);
    if(where != loader_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct loader_dispatch_ loader_realdispatcher_ =
{
    &loader_typeobject_,
    loader_wrap_,
    loader_wrapcopy_,
    loader_unwrap_,
    loader_insert_,
    loader_erase_,
    loader_find_
};

       





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initsampler2_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            player_dispatcher_ = &player_realdispatcher_;
        
    
        
            loader_dispatcher_ = &loader_realdispatcher_;
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_preset_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_preset_dispatch in module piw");
                return;
            }

            preset_dispatcher_ = (preset_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&player_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&loader_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("sampler2_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(player_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_player_dispatch",s);
            Py_INCREF(&player_typeobject_);
            PyModule_AddObject(m,"player",(PyObject *)&player_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(loader_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_loader_dispatch",s);
            Py_INCREF(&loader_typeobject_);
            PyModule_AddObject(m,"loader",(PyObject *)&loader_typeobject_);
        
    
        
    
        
    
        
    

    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

