#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "pistk.h"
#include "Cello.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef stk::blownstring_t blownstring_type_; 


 
static std::map<blownstring_type_ *, PyObject *> blownstring_wrapped_;
 


struct blownstring_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(blownstring_type_ *, bool owned);
    PyObject *(*wrapcopy)(blownstring_type_ *);
    blownstring_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< blownstring_type_ *, PyObject * > &);
    void (*erase)(blownstring_type_ *);
    PyObject *(*find)(blownstring_type_ *);
};

struct blownstring_dispatch_ *blownstring_dispatcher_;



typedef stk::panpipe_t panpipe_type_; 


 
static std::map<panpipe_type_ *, PyObject *> panpipe_wrapped_;
 


struct panpipe_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(panpipe_type_ *, bool owned);
    PyObject *(*wrapcopy)(panpipe_type_ *);
    panpipe_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< panpipe_type_ *, PyObject * > &);
    void (*erase)(panpipe_type_ *);
    PyObject *(*find)(panpipe_type_ *);
};

struct panpipe_dispatch_ *panpipe_dispatcher_;



typedef stk::clarinet_t clarinet_type_; 


 
static std::map<clarinet_type_ *, PyObject *> clarinet_wrapped_;
 


struct clarinet_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clarinet_type_ *, bool owned);
    PyObject *(*wrapcopy)(clarinet_type_ *);
    clarinet_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clarinet_type_ *, PyObject * > &);
    void (*erase)(clarinet_type_ *);
    PyObject *(*find)(clarinet_type_ *);
};

struct clarinet_dispatch_ *clarinet_dispatcher_;



typedef stk::cello_t cello_type_; 


 
static std::map<cello_type_ *, PyObject *> cello_wrapped_;
 


struct cello_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cello_type_ *, bool owned);
    PyObject *(*wrapcopy)(cello_type_ *);
    cello_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cello_type_ *, PyObject * > &);
    void (*erase)(cello_type_ *);
    PyObject *(*find)(cello_type_ *);
};

struct cello_dispatch_ *cello_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_blownstring(PyObject *o, void *a)
{
    blownstring_type_ *object;
    if(o == Py_None)
    {
        *((blownstring_type_ **)a) = 0;
        return 1;
    }
    object = blownstring_dispatcher_->unwrap(o);
    *((blownstring_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_blownstring_copy fpcvt_blownstring

PyObject *tpcvt_blownstring(void *a)
{
    blownstring_type_ *o = *((blownstring_type_ **)a);
    return blownstring_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_blownstring_copy(void *a)
{
    blownstring_type_ *o = (blownstring_type_ *)a;
    return blownstring_dispatcher_->wrapcopy(o);
}



int fpcvt_panpipe(PyObject *o, void *a)
{
    panpipe_type_ *object;
    if(o == Py_None)
    {
        *((panpipe_type_ **)a) = 0;
        return 1;
    }
    object = panpipe_dispatcher_->unwrap(o);
    *((panpipe_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_panpipe_copy fpcvt_panpipe

PyObject *tpcvt_panpipe(void *a)
{
    panpipe_type_ *o = *((panpipe_type_ **)a);
    return panpipe_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_panpipe_copy(void *a)
{
    panpipe_type_ *o = (panpipe_type_ *)a;
    return panpipe_dispatcher_->wrapcopy(o);
}



int fpcvt_clarinet(PyObject *o, void *a)
{
    clarinet_type_ *object;
    if(o == Py_None)
    {
        *((clarinet_type_ **)a) = 0;
        return 1;
    }
    object = clarinet_dispatcher_->unwrap(o);
    *((clarinet_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clarinet_copy fpcvt_clarinet

PyObject *tpcvt_clarinet(void *a)
{
    clarinet_type_ *o = *((clarinet_type_ **)a);
    return clarinet_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clarinet_copy(void *a)
{
    clarinet_type_ *o = (clarinet_type_ *)a;
    return clarinet_dispatcher_->wrapcopy(o);
}



int fpcvt_cello(PyObject *o, void *a)
{
    cello_type_ *object;
    if(o == Py_None)
    {
        *((cello_type_ **)a) = 0;
        return 1;
    }
    object = cello_dispatcher_->unwrap(o);
    *((cello_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cello_copy fpcvt_cello

PyObject *tpcvt_cello(void *a)
{
    cello_type_ *o = *((cello_type_ **)a);
    return cello_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cello_copy(void *a)
{
    cello_type_ *o = (cello_type_ *)a;
    return cello_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct blownstring_object_ {
    PyObject_HEAD
    blownstring_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct blownstring_wrapper_: public blownstring_type_ 
{

    static void __deleter__(blownstring_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    blownstring_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): blownstring_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~blownstring_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return blownstring_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(blownstring_object_ *self, PyObject *args)
    {
        

        blownstring_wrapper_ *o = (blownstring_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null blownstring c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in blownstring::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(blownstring_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(blownstring_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void blownstring_delete_(blownstring_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            blownstring_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (blownstring_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        blownstring_wrapper_::__deleter__((blownstring_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void blownstring_dealloc_(blownstring_object_* self)
{
    
    blownstring_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * blownstring_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    blownstring_object_ *self;

    if(!(self = (blownstring_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int blownstring_init_(blownstring_object_ *self, PyObject *args, PyObject *kwds)
{
    blownstring_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new blownstring_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing blownstring");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            blownstring_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"blownstring invalid arguments");
    return -1;

}


static const char *blownstring_cookie_docstring_ = "";




static PyMethodDef blownstring_methods_[] = {
    
        { "cookie",(PyCFunction)blownstring_wrapper_::cookie_method_,METH_VARARGS,(char *)blownstring_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)blownstring_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)blownstring_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *blownstring_docstring_ = "";

PyTypeObject blownstring_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "stk_native.blownstring",/*tp_name*/
    sizeof(blownstring_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)blownstring_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)blownstring_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    blownstring_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)blownstring_init_,  /* tp_init */
    0,                         /* tp_alloc */
    blownstring_new_,             /* tp_new */
};

PyObject *blownstring_wrapcopy_(blownstring_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *blownstring_wrap_(blownstring_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = blownstring_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    blownstring_object_ *self = 
        PyObject_NEW(blownstring_object_, &blownstring_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        blownstring_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

blownstring_type_ *blownstring_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &blownstring_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected blownstring object");
        return 0;
    }

    blownstring_type_ *object = static_cast<blownstring_type_ *>(((blownstring_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null blownstring object in unwrap");
        return 0;
    }

    return object;
}

void blownstring_insert_(const std::pair< blownstring_type_ *, PyObject *> &p)
{




    blownstring_wrapped_.insert(p);


}

void blownstring_erase_(blownstring_type_ *p)
{




    blownstring_wrapped_.erase(p);


}

PyObject * blownstring_find_(blownstring_type_ *p)
{




    PyObject *ret = 0;
    std::map< blownstring_type_ *, PyObject *>::iterator where = blownstring_wrapped_.find(p);
    if(where != blownstring_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct blownstring_dispatch_ blownstring_realdispatcher_ =
{
    &blownstring_typeobject_,
    blownstring_wrap_,
    blownstring_wrapcopy_,
    blownstring_unwrap_,
    blownstring_insert_,
    blownstring_erase_,
    blownstring_find_
};

  

struct panpipe_object_ {
    PyObject_HEAD
    panpipe_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct panpipe_wrapper_: public panpipe_type_ 
{

    static void __deleter__(panpipe_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    panpipe_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): panpipe_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~panpipe_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return panpipe_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(panpipe_object_ *self, PyObject *args)
    {
        

        panpipe_wrapper_ *o = (panpipe_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null panpipe c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in panpipe::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(panpipe_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(panpipe_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void panpipe_delete_(panpipe_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            panpipe_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (panpipe_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        panpipe_wrapper_::__deleter__((panpipe_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void panpipe_dealloc_(panpipe_object_* self)
{
    
    panpipe_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * panpipe_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    panpipe_object_ *self;

    if(!(self = (panpipe_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int panpipe_init_(panpipe_object_ *self, PyObject *args, PyObject *kwds)
{
    panpipe_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new panpipe_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing panpipe");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            panpipe_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"panpipe invalid arguments");
    return -1;

}


static const char *panpipe_cookie_docstring_ = "";




static PyMethodDef panpipe_methods_[] = {
    
        { "cookie",(PyCFunction)panpipe_wrapper_::cookie_method_,METH_VARARGS,(char *)panpipe_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)panpipe_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)panpipe_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *panpipe_docstring_ = "";

PyTypeObject panpipe_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "stk_native.panpipe",/*tp_name*/
    sizeof(panpipe_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)panpipe_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)panpipe_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    panpipe_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)panpipe_init_,  /* tp_init */
    0,                         /* tp_alloc */
    panpipe_new_,             /* tp_new */
};

PyObject *panpipe_wrapcopy_(panpipe_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *panpipe_wrap_(panpipe_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = panpipe_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    panpipe_object_ *self = 
        PyObject_NEW(panpipe_object_, &panpipe_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        panpipe_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

panpipe_type_ *panpipe_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &panpipe_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected panpipe object");
        return 0;
    }

    panpipe_type_ *object = static_cast<panpipe_type_ *>(((panpipe_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null panpipe object in unwrap");
        return 0;
    }

    return object;
}

void panpipe_insert_(const std::pair< panpipe_type_ *, PyObject *> &p)
{




    panpipe_wrapped_.insert(p);


}

void panpipe_erase_(panpipe_type_ *p)
{




    panpipe_wrapped_.erase(p);


}

PyObject * panpipe_find_(panpipe_type_ *p)
{




    PyObject *ret = 0;
    std::map< panpipe_type_ *, PyObject *>::iterator where = panpipe_wrapped_.find(p);
    if(where != panpipe_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct panpipe_dispatch_ panpipe_realdispatcher_ =
{
    &panpipe_typeobject_,
    panpipe_wrap_,
    panpipe_wrapcopy_,
    panpipe_unwrap_,
    panpipe_insert_,
    panpipe_erase_,
    panpipe_find_
};

  

struct clarinet_object_ {
    PyObject_HEAD
    clarinet_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct clarinet_wrapper_: public clarinet_type_ 
{

    static void __deleter__(clarinet_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    clarinet_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): clarinet_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~clarinet_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return clarinet_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(clarinet_object_ *self, PyObject *args)
    {
        

        clarinet_wrapper_ *o = (clarinet_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null clarinet c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in clarinet::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(clarinet_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(clarinet_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void clarinet_delete_(clarinet_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            clarinet_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (clarinet_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        clarinet_wrapper_::__deleter__((clarinet_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void clarinet_dealloc_(clarinet_object_* self)
{
    
    clarinet_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * clarinet_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    clarinet_object_ *self;

    if(!(self = (clarinet_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int clarinet_init_(clarinet_object_ *self, PyObject *args, PyObject *kwds)
{
    clarinet_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new clarinet_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing clarinet");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            clarinet_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"clarinet invalid arguments");
    return -1;

}


static const char *clarinet_cookie_docstring_ = "";




static PyMethodDef clarinet_methods_[] = {
    
        { "cookie",(PyCFunction)clarinet_wrapper_::cookie_method_,METH_VARARGS,(char *)clarinet_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)clarinet_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)clarinet_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *clarinet_docstring_ = "";

PyTypeObject clarinet_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "stk_native.clarinet",/*tp_name*/
    sizeof(clarinet_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)clarinet_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)clarinet_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    clarinet_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)clarinet_init_,  /* tp_init */
    0,                         /* tp_alloc */
    clarinet_new_,             /* tp_new */
};

PyObject *clarinet_wrapcopy_(clarinet_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *clarinet_wrap_(clarinet_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = clarinet_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    clarinet_object_ *self = 
        PyObject_NEW(clarinet_object_, &clarinet_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        clarinet_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

clarinet_type_ *clarinet_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &clarinet_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected clarinet object");
        return 0;
    }

    clarinet_type_ *object = static_cast<clarinet_type_ *>(((clarinet_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null clarinet object in unwrap");
        return 0;
    }

    return object;
}

void clarinet_insert_(const std::pair< clarinet_type_ *, PyObject *> &p)
{




    clarinet_wrapped_.insert(p);


}

void clarinet_erase_(clarinet_type_ *p)
{




    clarinet_wrapped_.erase(p);


}

PyObject * clarinet_find_(clarinet_type_ *p)
{




    PyObject *ret = 0;
    std::map< clarinet_type_ *, PyObject *>::iterator where = clarinet_wrapped_.find(p);
    if(where != clarinet_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct clarinet_dispatch_ clarinet_realdispatcher_ =
{
    &clarinet_typeobject_,
    clarinet_wrap_,
    clarinet_wrapcopy_,
    clarinet_unwrap_,
    clarinet_insert_,
    clarinet_erase_,
    clarinet_find_
};

  

struct cello_object_ {
    PyObject_HEAD
    cello_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct cello_wrapper_: public cello_type_ 
{

    static void __deleter__(cello_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    cello_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): cello_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~cello_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return cello_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(cello_object_ *self, PyObject *args)
    {
        

        cello_wrapper_ *o = (cello_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cello c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cello::cookie");
        }

        return 0;
    }



    cookie_type_ bow_cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return cello_type_::bow_cookie();
                
            
            
        }

        return
            
                bow_cookie
            
            
                ();
    }

    static PyObject *bow_cookie_method_(cello_object_ *self, PyObject *args)
    {
        

        cello_wrapper_ *o = (cello_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null cello c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->bow_cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in cello::bow_cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(cello_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(cello_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void cello_delete_(cello_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            cello_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (cello_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        cello_wrapper_::__deleter__((cello_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void cello_dealloc_(cello_object_* self)
{
    
    cello_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * cello_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    cello_object_ *self;

    if(!(self = (cello_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int cello_init_(cello_object_ *self, PyObject *args, PyObject *kwds)
{
    cello_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new cello_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing cello");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            cello_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"cello invalid arguments");
    return -1;

}


static const char *cello_cookie_docstring_ = "";

static const char *cello_bow_cookie_docstring_ = "";




static PyMethodDef cello_methods_[] = {
    
        { "cookie",(PyCFunction)cello_wrapper_::cookie_method_,METH_VARARGS,(char *)cello_cookie_docstring_ },
    
        { "bow_cookie",(PyCFunction)cello_wrapper_::bow_cookie_method_,METH_VARARGS,(char *)cello_bow_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)cello_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)cello_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *cello_docstring_ = "";

PyTypeObject cello_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "stk_native.cello",/*tp_name*/
    sizeof(cello_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)cello_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)cello_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    cello_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)cello_init_,  /* tp_init */
    0,                         /* tp_alloc */
    cello_new_,             /* tp_new */
};

PyObject *cello_wrapcopy_(cello_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *cello_wrap_(cello_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = cello_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    cello_object_ *self = 
        PyObject_NEW(cello_object_, &cello_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        cello_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

cello_type_ *cello_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &cello_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected cello object");
        return 0;
    }

    cello_type_ *object = static_cast<cello_type_ *>(((cello_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null cello object in unwrap");
        return 0;
    }

    return object;
}

void cello_insert_(const std::pair< cello_type_ *, PyObject *> &p)
{




    cello_wrapped_.insert(p);


}

void cello_erase_(cello_type_ *p)
{




    cello_wrapped_.erase(p);


}

PyObject * cello_find_(cello_type_ *p)
{




    PyObject *ret = 0;
    std::map< cello_type_ *, PyObject *>::iterator where = cello_wrapped_.find(p);
    if(where != cello_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct cello_dispatch_ cello_realdispatcher_ =
{
    &cello_typeobject_,
    cello_wrap_,
    cello_wrapcopy_,
    cello_unwrap_,
    cello_insert_,
    cello_erase_,
    cello_find_
};

     





static PyMethodDef module_method_table[] = {
    
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initstk_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            blownstring_dispatcher_ = &blownstring_realdispatcher_;
        
    
        
            panpipe_dispatcher_ = &panpipe_realdispatcher_;
        
    
        
            clarinet_dispatcher_ = &clarinet_realdispatcher_;
        
    
        
            cello_dispatcher_ = &cello_realdispatcher_;
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    

    
        
            if(PyType_Ready(&blownstring_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&panpipe_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&clarinet_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&cello_typeobject_)<0)
                return;
        
    
        
    
        
    

    if(!(m=Py_InitModule("stk_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(blownstring_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_blownstring_dispatch",s);
            Py_INCREF(&blownstring_typeobject_);
            PyModule_AddObject(m,"blownstring",(PyObject *)&blownstring_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(panpipe_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_panpipe_dispatch",s);
            Py_INCREF(&panpipe_typeobject_);
            PyModule_AddObject(m,"panpipe",(PyObject *)&panpipe_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(clarinet_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_clarinet_dispatch",s);
            Py_INCREF(&clarinet_typeobject_);
            PyModule_AddObject(m,"clarinet",(PyObject *)&clarinet_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(cello_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_cello_dispatch",s);
            Py_INCREF(&cello_typeobject_);
            PyModule_AddObject(m,"cello",(PyObject *)&cello_typeobject_);
        
    
        
    
        
    

    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

