#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include "synth.h"


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = "";


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef synth::sine_t sine_type_; 


 
static std::map<sine_type_ *, PyObject *> sine_wrapped_;
 


struct sine_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(sine_type_ *, bool owned);
    PyObject *(*wrapcopy)(sine_type_ *);
    sine_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< sine_type_ *, PyObject * > &);
    void (*erase)(sine_type_ *);
    PyObject *(*find)(sine_type_ *);
};

struct sine_dispatch_ *sine_dispatcher_;



typedef synth::adsr_t adsr_type_; 


 
static std::map<adsr_type_ *, PyObject *> adsr_wrapped_;
 


struct adsr_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(adsr_type_ *, bool owned);
    PyObject *(*wrapcopy)(adsr_type_ *);
    adsr_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< adsr_type_ *, PyObject * > &);
    void (*erase)(adsr_type_ *);
    PyObject *(*find)(adsr_type_ *);
};

struct adsr_dispatch_ *adsr_dispatcher_;



typedef synth::adsr2_t adsr2_type_; 


 
static std::map<adsr2_type_ *, PyObject *> adsr2_wrapped_;
 


struct adsr2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(adsr2_type_ *, bool owned);
    PyObject *(*wrapcopy)(adsr2_type_ *);
    adsr2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< adsr2_type_ *, PyObject * > &);
    void (*erase)(adsr2_type_ *);
    PyObject *(*find)(adsr2_type_ *);
};

struct adsr2_dispatch_ *adsr2_dispatcher_;



typedef synth::sawtooth_t sawtooth_type_; 


 
static std::map<sawtooth_type_ *, PyObject *> sawtooth_wrapped_;
 


struct sawtooth_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(sawtooth_type_ *, bool owned);
    PyObject *(*wrapcopy)(sawtooth_type_ *);
    sawtooth_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< sawtooth_type_ *, PyObject * > &);
    void (*erase)(sawtooth_type_ *);
    PyObject *(*find)(sawtooth_type_ *);
};

struct sawtooth_dispatch_ *sawtooth_dispatcher_;



typedef synth::rect_t rect_type_; 


 
static std::map<rect_type_ *, PyObject *> rect_wrapped_;
 


struct rect_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(rect_type_ *, bool owned);
    PyObject *(*wrapcopy)(rect_type_ *);
    rect_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< rect_type_ *, PyObject * > &);
    void (*erase)(rect_type_ *);
    PyObject *(*find)(rect_type_ *);
};

struct rect_dispatch_ *rect_dispatcher_;



typedef synth::triangle_t triangle_type_; 


 
static std::map<triangle_type_ *, PyObject *> triangle_wrapped_;
 


struct triangle_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(triangle_type_ *, bool owned);
    PyObject *(*wrapcopy)(triangle_type_ *);
    triangle_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< triangle_type_ *, PyObject * > &);
    void (*erase)(triangle_type_ *);
    PyObject *(*find)(triangle_type_ *);
};

struct triangle_dispatch_ *triangle_dispatcher_;



typedef synth::synthfilter_t synthfilter_type_; 


 
static std::map<synthfilter_type_ *, PyObject *> synthfilter_wrapped_;
 


struct synthfilter_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(synthfilter_type_ *, bool owned);
    PyObject *(*wrapcopy)(synthfilter_type_ *);
    synthfilter_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< synthfilter_type_ *, PyObject * > &);
    void (*erase)(synthfilter_type_ *);
    PyObject *(*find)(synthfilter_type_ *);
};

struct synthfilter_dispatch_ *synthfilter_dispatcher_;



typedef synth::synthfilter2_t synthfilter2_type_; 


 
static std::map<synthfilter2_type_ *, PyObject *> synthfilter2_wrapped_;
 


struct synthfilter2_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(synthfilter2_type_ *, bool owned);
    PyObject *(*wrapcopy)(synthfilter2_type_ *);
    synthfilter2_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< synthfilter2_type_ *, PyObject * > &);
    void (*erase)(synthfilter2_type_ *);
    PyObject *(*find)(synthfilter2_type_ *);
};

struct synthfilter2_dispatch_ *synthfilter2_dispatcher_;



typedef bct_clocksink_t clockref_type_; 





struct clockref_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockref_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockref_type_ *);
    clockref_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockref_type_ *, PyObject * > &);
    void (*erase)(clockref_type_ *);
    PyObject *(*find)(clockref_type_ *);
};

struct clockref_dispatch_ *clockref_dispatcher_;



typedef piw::cookie_t cookie_type_; 





struct cookie_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(cookie_type_ *, bool owned);
    PyObject *(*wrapcopy)(cookie_type_ *);
    cookie_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< cookie_type_ *, PyObject * > &);
    void (*erase)(cookie_type_ *);
    PyObject *(*find)(cookie_type_ *);
};

struct cookie_dispatch_ *cookie_dispatcher_;



typedef piw::decode_ctl_t decode_ctl_type_; 





struct decode_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(decode_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(decode_ctl_type_ *);
    decode_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< decode_ctl_type_ *, PyObject * > &);
    void (*erase)(decode_ctl_type_ *);
    PyObject *(*find)(decode_ctl_type_ *);
};

struct decode_ctl_dispatch_ *decode_ctl_dispatcher_;



typedef piw::wire_ctl_t wire_ctl_type_; 





struct wire_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wire_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(wire_ctl_type_ *);
    wire_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wire_ctl_type_ *, PyObject * > &);
    void (*erase)(wire_ctl_type_ *);
    PyObject *(*find)(wire_ctl_type_ *);
};

struct wire_ctl_dispatch_ *wire_ctl_dispatcher_;



typedef piw::wire_t wire_type_; 





struct wire_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(wire_type_ *, bool owned);
    PyObject *(*wrapcopy)(wire_type_ *);
    wire_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< wire_type_ *, PyObject * > &);
    void (*erase)(wire_type_ *);
    PyObject *(*find)(wire_type_ *);
};

struct wire_dispatch_ *wire_dispatcher_;



typedef piw::d2d_nb_t d2d_nb_type_; 





struct d2d_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(d2d_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(d2d_nb_type_ *);
    d2d_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< d2d_nb_type_ *, PyObject * > &);
    void (*erase)(d2d_nb_type_ *);
    PyObject *(*find)(d2d_nb_type_ *);
};

struct d2d_nb_dispatch_ *d2d_nb_dispatcher_;



typedef piw::dd2d_nb_t dd2d_nb_type_; 





struct dd2d_nb_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(dd2d_nb_type_ *, bool owned);
    PyObject *(*wrapcopy)(dd2d_nb_type_ *);
    dd2d_nb_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< dd2d_nb_type_ *, PyObject * > &);
    void (*erase)(dd2d_nb_type_ *);
    PyObject *(*find)(dd2d_nb_type_ *);
};

struct dd2d_nb_dispatch_ *dd2d_nb_dispatcher_;



typedef piw::clockdomain_ctl_t clockdomain_ctl_type_; 





struct clockdomain_ctl_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(clockdomain_ctl_type_ *, bool owned);
    PyObject *(*wrapcopy)(clockdomain_ctl_type_ *);
    clockdomain_ctl_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< clockdomain_ctl_type_ *, PyObject * > &);
    void (*erase)(clockdomain_ctl_type_ *);
    PyObject *(*find)(clockdomain_ctl_type_ *);
};

struct clockdomain_ctl_dispatch_ *clockdomain_ctl_dispatcher_;





int fpcvt_sine(PyObject *o, void *a)
{
    sine_type_ *object;
    if(o == Py_None)
    {
        *((sine_type_ **)a) = 0;
        return 1;
    }
    object = sine_dispatcher_->unwrap(o);
    *((sine_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_sine_copy fpcvt_sine

PyObject *tpcvt_sine(void *a)
{
    sine_type_ *o = *((sine_type_ **)a);
    return sine_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_sine_copy(void *a)
{
    sine_type_ *o = (sine_type_ *)a;
    return sine_dispatcher_->wrapcopy(o);
}



int fpcvt_adsr(PyObject *o, void *a)
{
    adsr_type_ *object;
    if(o == Py_None)
    {
        *((adsr_type_ **)a) = 0;
        return 1;
    }
    object = adsr_dispatcher_->unwrap(o);
    *((adsr_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_adsr_copy fpcvt_adsr

PyObject *tpcvt_adsr(void *a)
{
    adsr_type_ *o = *((adsr_type_ **)a);
    return adsr_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_adsr_copy(void *a)
{
    adsr_type_ *o = (adsr_type_ *)a;
    return adsr_dispatcher_->wrapcopy(o);
}



int fpcvt_adsr2(PyObject *o, void *a)
{
    adsr2_type_ *object;
    if(o == Py_None)
    {
        *((adsr2_type_ **)a) = 0;
        return 1;
    }
    object = adsr2_dispatcher_->unwrap(o);
    *((adsr2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_adsr2_copy fpcvt_adsr2

PyObject *tpcvt_adsr2(void *a)
{
    adsr2_type_ *o = *((adsr2_type_ **)a);
    return adsr2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_adsr2_copy(void *a)
{
    adsr2_type_ *o = (adsr2_type_ *)a;
    return adsr2_dispatcher_->wrapcopy(o);
}



int fpcvt_sawtooth(PyObject *o, void *a)
{
    sawtooth_type_ *object;
    if(o == Py_None)
    {
        *((sawtooth_type_ **)a) = 0;
        return 1;
    }
    object = sawtooth_dispatcher_->unwrap(o);
    *((sawtooth_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_sawtooth_copy fpcvt_sawtooth

PyObject *tpcvt_sawtooth(void *a)
{
    sawtooth_type_ *o = *((sawtooth_type_ **)a);
    return sawtooth_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_sawtooth_copy(void *a)
{
    sawtooth_type_ *o = (sawtooth_type_ *)a;
    return sawtooth_dispatcher_->wrapcopy(o);
}



int fpcvt_rect(PyObject *o, void *a)
{
    rect_type_ *object;
    if(o == Py_None)
    {
        *((rect_type_ **)a) = 0;
        return 1;
    }
    object = rect_dispatcher_->unwrap(o);
    *((rect_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_rect_copy fpcvt_rect

PyObject *tpcvt_rect(void *a)
{
    rect_type_ *o = *((rect_type_ **)a);
    return rect_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_rect_copy(void *a)
{
    rect_type_ *o = (rect_type_ *)a;
    return rect_dispatcher_->wrapcopy(o);
}



int fpcvt_triangle(PyObject *o, void *a)
{
    triangle_type_ *object;
    if(o == Py_None)
    {
        *((triangle_type_ **)a) = 0;
        return 1;
    }
    object = triangle_dispatcher_->unwrap(o);
    *((triangle_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_triangle_copy fpcvt_triangle

PyObject *tpcvt_triangle(void *a)
{
    triangle_type_ *o = *((triangle_type_ **)a);
    return triangle_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_triangle_copy(void *a)
{
    triangle_type_ *o = (triangle_type_ *)a;
    return triangle_dispatcher_->wrapcopy(o);
}



int fpcvt_synthfilter(PyObject *o, void *a)
{
    synthfilter_type_ *object;
    if(o == Py_None)
    {
        *((synthfilter_type_ **)a) = 0;
        return 1;
    }
    object = synthfilter_dispatcher_->unwrap(o);
    *((synthfilter_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_synthfilter_copy fpcvt_synthfilter

PyObject *tpcvt_synthfilter(void *a)
{
    synthfilter_type_ *o = *((synthfilter_type_ **)a);
    return synthfilter_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_synthfilter_copy(void *a)
{
    synthfilter_type_ *o = (synthfilter_type_ *)a;
    return synthfilter_dispatcher_->wrapcopy(o);
}



int fpcvt_synthfilter2(PyObject *o, void *a)
{
    synthfilter2_type_ *object;
    if(o == Py_None)
    {
        *((synthfilter2_type_ **)a) = 0;
        return 1;
    }
    object = synthfilter2_dispatcher_->unwrap(o);
    *((synthfilter2_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_synthfilter2_copy fpcvt_synthfilter2

PyObject *tpcvt_synthfilter2(void *a)
{
    synthfilter2_type_ *o = *((synthfilter2_type_ **)a);
    return synthfilter2_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_synthfilter2_copy(void *a)
{
    synthfilter2_type_ *o = (synthfilter2_type_ *)a;
    return synthfilter2_dispatcher_->wrapcopy(o);
}



int fpcvt_clockref(PyObject *o, void *a)
{
    clockref_type_ *object;
    if(o == Py_None)
    {
        *((clockref_type_ **)a) = 0;
        return 1;
    }
    object = clockref_dispatcher_->unwrap(o);
    *((clockref_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockref_copy fpcvt_clockref

PyObject *tpcvt_clockref(void *a)
{
    clockref_type_ *o = *((clockref_type_ **)a);
    return clockref_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockref_copy(void *a)
{
    clockref_type_ *o = (clockref_type_ *)a;
    return clockref_dispatcher_->wrapcopy(o);
}



int fpcvt_cookie(PyObject *o, void *a)
{
    cookie_type_ *object;
    if(o == Py_None)
    {
        *((cookie_type_ **)a) = 0;
        return 1;
    }
    object = cookie_dispatcher_->unwrap(o);
    *((cookie_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_cookie_copy fpcvt_cookie

PyObject *tpcvt_cookie(void *a)
{
    cookie_type_ *o = *((cookie_type_ **)a);
    return cookie_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_cookie_copy(void *a)
{
    cookie_type_ *o = (cookie_type_ *)a;
    return cookie_dispatcher_->wrapcopy(o);
}



int fpcvt_decode_ctl(PyObject *o, void *a)
{
    decode_ctl_type_ *object;
    if(o == Py_None)
    {
        *((decode_ctl_type_ **)a) = 0;
        return 1;
    }
    object = decode_ctl_dispatcher_->unwrap(o);
    *((decode_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_decode_ctl_copy fpcvt_decode_ctl

PyObject *tpcvt_decode_ctl(void *a)
{
    decode_ctl_type_ *o = *((decode_ctl_type_ **)a);
    return decode_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_decode_ctl_copy(void *a)
{
    decode_ctl_type_ *o = (decode_ctl_type_ *)a;
    return decode_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_wire_ctl(PyObject *o, void *a)
{
    wire_ctl_type_ *object;
    if(o == Py_None)
    {
        *((wire_ctl_type_ **)a) = 0;
        return 1;
    }
    object = wire_ctl_dispatcher_->unwrap(o);
    *((wire_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wire_ctl_copy fpcvt_wire_ctl

PyObject *tpcvt_wire_ctl(void *a)
{
    wire_ctl_type_ *o = *((wire_ctl_type_ **)a);
    return wire_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wire_ctl_copy(void *a)
{
    wire_ctl_type_ *o = (wire_ctl_type_ *)a;
    return wire_ctl_dispatcher_->wrapcopy(o);
}



int fpcvt_wire(PyObject *o, void *a)
{
    wire_type_ *object;
    if(o == Py_None)
    {
        *((wire_type_ **)a) = 0;
        return 1;
    }
    object = wire_dispatcher_->unwrap(o);
    *((wire_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_wire_copy fpcvt_wire

PyObject *tpcvt_wire(void *a)
{
    wire_type_ *o = *((wire_type_ **)a);
    return wire_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_wire_copy(void *a)
{
    wire_type_ *o = (wire_type_ *)a;
    return wire_dispatcher_->wrapcopy(o);
}



int fpcvt_d2d_nb(PyObject *o, void *a)
{
    d2d_nb_type_ *object;
    if(o == Py_None)
    {
        *((d2d_nb_type_ **)a) = 0;
        return 1;
    }
    object = d2d_nb_dispatcher_->unwrap(o);
    *((d2d_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_d2d_nb_copy fpcvt_d2d_nb

PyObject *tpcvt_d2d_nb(void *a)
{
    d2d_nb_type_ *o = *((d2d_nb_type_ **)a);
    return d2d_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_d2d_nb_copy(void *a)
{
    d2d_nb_type_ *o = (d2d_nb_type_ *)a;
    return d2d_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_dd2d_nb(PyObject *o, void *a)
{
    dd2d_nb_type_ *object;
    if(o == Py_None)
    {
        *((dd2d_nb_type_ **)a) = 0;
        return 1;
    }
    object = dd2d_nb_dispatcher_->unwrap(o);
    *((dd2d_nb_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_dd2d_nb_copy fpcvt_dd2d_nb

PyObject *tpcvt_dd2d_nb(void *a)
{
    dd2d_nb_type_ *o = *((dd2d_nb_type_ **)a);
    return dd2d_nb_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_dd2d_nb_copy(void *a)
{
    dd2d_nb_type_ *o = (dd2d_nb_type_ *)a;
    return dd2d_nb_dispatcher_->wrapcopy(o);
}



int fpcvt_clockdomain_ctl(PyObject *o, void *a)
{
    clockdomain_ctl_type_ *object;
    if(o == Py_None)
    {
        *((clockdomain_ctl_type_ **)a) = 0;
        return 1;
    }
    object = clockdomain_ctl_dispatcher_->unwrap(o);
    *((clockdomain_ctl_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_clockdomain_ctl_copy fpcvt_clockdomain_ctl

PyObject *tpcvt_clockdomain_ctl(void *a)
{
    clockdomain_ctl_type_ *o = *((clockdomain_ctl_type_ **)a);
    return clockdomain_ctl_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_clockdomain_ctl_copy(void *a)
{
    clockdomain_ctl_type_ *o = (clockdomain_ctl_type_ *)a;
    return clockdomain_ctl_dispatcher_->wrapcopy(o);
}



 

struct sine_object_ {
    PyObject_HEAD
    sine_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct sine_wrapper_: public sine_type_ 
{

    static void __deleter__(sine_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    sine_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): sine_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~sine_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return sine_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(sine_object_ *self, PyObject *args)
    {
        

        sine_wrapper_ *o = (sine_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sine c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sine::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(sine_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(sine_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void sine_delete_(sine_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            sine_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (sine_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        sine_wrapper_::__deleter__((sine_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void sine_dealloc_(sine_object_* self)
{
    
    sine_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * sine_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    sine_object_ *self;

    if(!(self = (sine_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int sine_init_(sine_object_ *self, PyObject *args, PyObject *kwds)
{
    sine_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new sine_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing sine");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            sine_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"sine invalid arguments");
    return -1;

}


static const char *sine_cookie_docstring_ = "";




static PyMethodDef sine_methods_[] = {
    
        { "cookie",(PyCFunction)sine_wrapper_::cookie_method_,METH_VARARGS,(char *)sine_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)sine_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)sine_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *sine_docstring_ = ""
"    Sine wave oscillator."
"    "
""
;

PyTypeObject sine_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.sine",/*tp_name*/
    sizeof(sine_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)sine_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)sine_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    sine_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)sine_init_,  /* tp_init */
    0,                         /* tp_alloc */
    sine_new_,             /* tp_new */
};

PyObject *sine_wrapcopy_(sine_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *sine_wrap_(sine_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = sine_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    sine_object_ *self = 
        PyObject_NEW(sine_object_, &sine_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        sine_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

sine_type_ *sine_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &sine_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected sine object");
        return 0;
    }

    sine_type_ *object = static_cast<sine_type_ *>(((sine_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null sine object in unwrap");
        return 0;
    }

    return object;
}

void sine_insert_(const std::pair< sine_type_ *, PyObject *> &p)
{




    sine_wrapped_.insert(p);


}

void sine_erase_(sine_type_ *p)
{




    sine_wrapped_.erase(p);


}

PyObject * sine_find_(sine_type_ *p)
{




    PyObject *ret = 0;
    std::map< sine_type_ *, PyObject *>::iterator where = sine_wrapped_.find(p);
    if(where != sine_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct sine_dispatch_ sine_realdispatcher_ =
{
    &sine_typeobject_,
    sine_wrap_,
    sine_wrapcopy_,
    sine_unwrap_,
    sine_insert_,
    sine_erase_,
    sine_find_
};

  

struct adsr_object_ {
    PyObject_HEAD
    adsr_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct adsr_wrapper_: public adsr_type_ 
{

    static void __deleter__(adsr_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    adsr_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): adsr_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~adsr_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return adsr_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(adsr_object_ *self, PyObject *args)
    {
        

        adsr_wrapper_ *o = (adsr_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null adsr c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in adsr::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(adsr_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(adsr_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void adsr_delete_(adsr_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            adsr_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (adsr_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        adsr_wrapper_::__deleter__((adsr_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void adsr_dealloc_(adsr_object_* self)
{
    
    adsr_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * adsr_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    adsr_object_ *self;

    if(!(self = (adsr_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int adsr_init_(adsr_object_ *self, PyObject *args, PyObject *kwds)
{
    adsr_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new adsr_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing adsr");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            adsr_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"adsr invalid arguments");
    return -1;

}


static const char *adsr_cookie_docstring_ = "";




static PyMethodDef adsr_methods_[] = {
    
        { "cookie",(PyCFunction)adsr_wrapper_::cookie_method_,METH_VARARGS,(char *)adsr_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)adsr_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)adsr_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *adsr_docstring_ = ""
"    Envelope generator."
"    "
""
;

PyTypeObject adsr_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.adsr",/*tp_name*/
    sizeof(adsr_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)adsr_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)adsr_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    adsr_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)adsr_init_,  /* tp_init */
    0,                         /* tp_alloc */
    adsr_new_,             /* tp_new */
};

PyObject *adsr_wrapcopy_(adsr_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *adsr_wrap_(adsr_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = adsr_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    adsr_object_ *self = 
        PyObject_NEW(adsr_object_, &adsr_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        adsr_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

adsr_type_ *adsr_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &adsr_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected adsr object");
        return 0;
    }

    adsr_type_ *object = static_cast<adsr_type_ *>(((adsr_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null adsr object in unwrap");
        return 0;
    }

    return object;
}

void adsr_insert_(const std::pair< adsr_type_ *, PyObject *> &p)
{




    adsr_wrapped_.insert(p);


}

void adsr_erase_(adsr_type_ *p)
{




    adsr_wrapped_.erase(p);


}

PyObject * adsr_find_(adsr_type_ *p)
{




    PyObject *ret = 0;
    std::map< adsr_type_ *, PyObject *>::iterator where = adsr_wrapped_.find(p);
    if(where != adsr_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct adsr_dispatch_ adsr_realdispatcher_ =
{
    &adsr_typeobject_,
    adsr_wrap_,
    adsr_wrapcopy_,
    adsr_unwrap_,
    adsr_insert_,
    adsr_erase_,
    adsr_find_
};

  

struct adsr2_object_ {
    PyObject_HEAD
    adsr2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct adsr2_wrapper_: public adsr2_type_ 
{

    static void __deleter__(adsr2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    adsr2_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): adsr2_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~adsr2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return adsr2_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(adsr2_object_ *self, PyObject *args)
    {
        

        adsr2_wrapper_ *o = (adsr2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null adsr2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in adsr2::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(adsr2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(adsr2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void adsr2_delete_(adsr2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            adsr2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (adsr2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        adsr2_wrapper_::__deleter__((adsr2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void adsr2_dealloc_(adsr2_object_* self)
{
    
    adsr2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * adsr2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    adsr2_object_ *self;

    if(!(self = (adsr2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int adsr2_init_(adsr2_object_ *self, PyObject *args, PyObject *kwds)
{
    adsr2_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new adsr2_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing adsr2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            adsr2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"adsr2 invalid arguments");
    return -1;

}


static const char *adsr2_cookie_docstring_ = "";




static PyMethodDef adsr2_methods_[] = {
    
        { "cookie",(PyCFunction)adsr2_wrapper_::cookie_method_,METH_VARARGS,(char *)adsr2_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)adsr2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)adsr2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *adsr2_docstring_ = "";

PyTypeObject adsr2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.adsr2",/*tp_name*/
    sizeof(adsr2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)adsr2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)adsr2_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    adsr2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)adsr2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    adsr2_new_,             /* tp_new */
};

PyObject *adsr2_wrapcopy_(adsr2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *adsr2_wrap_(adsr2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = adsr2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    adsr2_object_ *self = 
        PyObject_NEW(adsr2_object_, &adsr2_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        adsr2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

adsr2_type_ *adsr2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &adsr2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected adsr2 object");
        return 0;
    }

    adsr2_type_ *object = static_cast<adsr2_type_ *>(((adsr2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null adsr2 object in unwrap");
        return 0;
    }

    return object;
}

void adsr2_insert_(const std::pair< adsr2_type_ *, PyObject *> &p)
{




    adsr2_wrapped_.insert(p);


}

void adsr2_erase_(adsr2_type_ *p)
{




    adsr2_wrapped_.erase(p);


}

PyObject * adsr2_find_(adsr2_type_ *p)
{




    PyObject *ret = 0;
    std::map< adsr2_type_ *, PyObject *>::iterator where = adsr2_wrapped_.find(p);
    if(where != adsr2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct adsr2_dispatch_ adsr2_realdispatcher_ =
{
    &adsr2_typeobject_,
    adsr2_wrap_,
    adsr2_wrapcopy_,
    adsr2_unwrap_,
    adsr2_insert_,
    adsr2_erase_,
    adsr2_find_
};

  

struct sawtooth_object_ {
    PyObject_HEAD
    sawtooth_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct sawtooth_wrapper_: public sawtooth_type_ 
{

    static void __deleter__(sawtooth_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    sawtooth_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): sawtooth_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~sawtooth_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return sawtooth_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(sawtooth_object_ *self, PyObject *args)
    {
        

        sawtooth_wrapper_ *o = (sawtooth_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null sawtooth c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in sawtooth::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(sawtooth_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(sawtooth_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void sawtooth_delete_(sawtooth_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            sawtooth_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (sawtooth_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        sawtooth_wrapper_::__deleter__((sawtooth_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void sawtooth_dealloc_(sawtooth_object_* self)
{
    
    sawtooth_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * sawtooth_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    sawtooth_object_ *self;

    if(!(self = (sawtooth_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int sawtooth_init_(sawtooth_object_ *self, PyObject *args, PyObject *kwds)
{
    sawtooth_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new sawtooth_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing sawtooth");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            sawtooth_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"sawtooth invalid arguments");
    return -1;

}


static const char *sawtooth_cookie_docstring_ = "";




static PyMethodDef sawtooth_methods_[] = {
    
        { "cookie",(PyCFunction)sawtooth_wrapper_::cookie_method_,METH_VARARGS,(char *)sawtooth_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)sawtooth_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)sawtooth_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *sawtooth_docstring_ = ""
"    sawtooth wave oscillator."
"    "
""
;

PyTypeObject sawtooth_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.sawtooth",/*tp_name*/
    sizeof(sawtooth_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)sawtooth_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)sawtooth_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    sawtooth_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)sawtooth_init_,  /* tp_init */
    0,                         /* tp_alloc */
    sawtooth_new_,             /* tp_new */
};

PyObject *sawtooth_wrapcopy_(sawtooth_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *sawtooth_wrap_(sawtooth_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = sawtooth_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    sawtooth_object_ *self = 
        PyObject_NEW(sawtooth_object_, &sawtooth_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        sawtooth_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

sawtooth_type_ *sawtooth_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &sawtooth_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected sawtooth object");
        return 0;
    }

    sawtooth_type_ *object = static_cast<sawtooth_type_ *>(((sawtooth_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null sawtooth object in unwrap");
        return 0;
    }

    return object;
}

void sawtooth_insert_(const std::pair< sawtooth_type_ *, PyObject *> &p)
{




    sawtooth_wrapped_.insert(p);


}

void sawtooth_erase_(sawtooth_type_ *p)
{




    sawtooth_wrapped_.erase(p);


}

PyObject * sawtooth_find_(sawtooth_type_ *p)
{




    PyObject *ret = 0;
    std::map< sawtooth_type_ *, PyObject *>::iterator where = sawtooth_wrapped_.find(p);
    if(where != sawtooth_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct sawtooth_dispatch_ sawtooth_realdispatcher_ =
{
    &sawtooth_typeobject_,
    sawtooth_wrap_,
    sawtooth_wrapcopy_,
    sawtooth_unwrap_,
    sawtooth_insert_,
    sawtooth_erase_,
    sawtooth_find_
};

  

struct rect_object_ {
    PyObject_HEAD
    rect_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct rect_wrapper_: public rect_type_ 
{

    static void __deleter__(rect_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    rect_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): rect_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~rect_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return rect_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(rect_object_ *self, PyObject *args)
    {
        

        rect_wrapper_ *o = (rect_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null rect c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in rect::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(rect_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(rect_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void rect_delete_(rect_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            rect_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (rect_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        rect_wrapper_::__deleter__((rect_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void rect_dealloc_(rect_object_* self)
{
    
    rect_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * rect_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    rect_object_ *self;

    if(!(self = (rect_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int rect_init_(rect_object_ *self, PyObject *args, PyObject *kwds)
{
    rect_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new rect_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing rect");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            rect_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"rect invalid arguments");
    return -1;

}


static const char *rect_cookie_docstring_ = "";




static PyMethodDef rect_methods_[] = {
    
        { "cookie",(PyCFunction)rect_wrapper_::cookie_method_,METH_VARARGS,(char *)rect_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)rect_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)rect_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *rect_docstring_ = ""
"    rectangle (pulse-width) wave oscillator."
"    "
""
;

PyTypeObject rect_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.rect",/*tp_name*/
    sizeof(rect_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)rect_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)rect_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    rect_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)rect_init_,  /* tp_init */
    0,                         /* tp_alloc */
    rect_new_,             /* tp_new */
};

PyObject *rect_wrapcopy_(rect_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *rect_wrap_(rect_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = rect_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    rect_object_ *self = 
        PyObject_NEW(rect_object_, &rect_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        rect_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

rect_type_ *rect_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &rect_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected rect object");
        return 0;
    }

    rect_type_ *object = static_cast<rect_type_ *>(((rect_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null rect object in unwrap");
        return 0;
    }

    return object;
}

void rect_insert_(const std::pair< rect_type_ *, PyObject *> &p)
{




    rect_wrapped_.insert(p);


}

void rect_erase_(rect_type_ *p)
{




    rect_wrapped_.erase(p);


}

PyObject * rect_find_(rect_type_ *p)
{




    PyObject *ret = 0;
    std::map< rect_type_ *, PyObject *>::iterator where = rect_wrapped_.find(p);
    if(where != rect_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct rect_dispatch_ rect_realdispatcher_ =
{
    &rect_typeobject_,
    rect_wrap_,
    rect_wrapcopy_,
    rect_unwrap_,
    rect_insert_,
    rect_erase_,
    rect_find_
};

  

struct triangle_object_ {
    PyObject_HEAD
    triangle_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct triangle_wrapper_: public triangle_type_ 
{

    static void __deleter__(triangle_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    triangle_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): triangle_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~triangle_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return triangle_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(triangle_object_ *self, PyObject *args)
    {
        

        triangle_wrapper_ *o = (triangle_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null triangle c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in triangle::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(triangle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(triangle_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void triangle_delete_(triangle_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            triangle_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (triangle_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        triangle_wrapper_::__deleter__((triangle_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void triangle_dealloc_(triangle_object_* self)
{
    
    triangle_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * triangle_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    triangle_object_ *self;

    if(!(self = (triangle_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int triangle_init_(triangle_object_ *self, PyObject *args, PyObject *kwds)
{
    triangle_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new triangle_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing triangle");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            triangle_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"triangle invalid arguments");
    return -1;

}


static const char *triangle_cookie_docstring_ = "";




static PyMethodDef triangle_methods_[] = {
    
        { "cookie",(PyCFunction)triangle_wrapper_::cookie_method_,METH_VARARGS,(char *)triangle_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)triangle_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)triangle_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *triangle_docstring_ = ""
"    triangle oscillator."
"    "
""
;

PyTypeObject triangle_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.triangle",/*tp_name*/
    sizeof(triangle_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)triangle_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)triangle_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    triangle_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)triangle_init_,  /* tp_init */
    0,                         /* tp_alloc */
    triangle_new_,             /* tp_new */
};

PyObject *triangle_wrapcopy_(triangle_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *triangle_wrap_(triangle_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = triangle_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    triangle_object_ *self = 
        PyObject_NEW(triangle_object_, &triangle_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        triangle_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

triangle_type_ *triangle_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &triangle_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected triangle object");
        return 0;
    }

    triangle_type_ *object = static_cast<triangle_type_ *>(((triangle_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null triangle object in unwrap");
        return 0;
    }

    return object;
}

void triangle_insert_(const std::pair< triangle_type_ *, PyObject *> &p)
{




    triangle_wrapped_.insert(p);


}

void triangle_erase_(triangle_type_ *p)
{




    triangle_wrapped_.erase(p);


}

PyObject * triangle_find_(triangle_type_ *p)
{




    PyObject *ret = 0;
    std::map< triangle_type_ *, PyObject *>::iterator where = triangle_wrapped_.find(p);
    if(where != triangle_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct triangle_dispatch_ triangle_realdispatcher_ =
{
    &triangle_typeobject_,
    triangle_wrap_,
    triangle_wrapcopy_,
    triangle_unwrap_,
    triangle_insert_,
    triangle_erase_,
    triangle_find_
};

  

struct synthfilter_object_ {
    PyObject_HEAD
    synthfilter_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct synthfilter_wrapper_: public synthfilter_type_ 
{

    static void __deleter__(synthfilter_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    synthfilter_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): synthfilter_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~synthfilter_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return synthfilter_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(synthfilter_object_ *self, PyObject *args)
    {
        

        synthfilter_wrapper_ *o = (synthfilter_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null synthfilter c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in synthfilter::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(synthfilter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(synthfilter_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void synthfilter_delete_(synthfilter_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            synthfilter_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (synthfilter_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        synthfilter_wrapper_::__deleter__((synthfilter_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void synthfilter_dealloc_(synthfilter_object_* self)
{
    
    synthfilter_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * synthfilter_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    synthfilter_object_ *self;

    if(!(self = (synthfilter_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int synthfilter_init_(synthfilter_object_ *self, PyObject *args, PyObject *kwds)
{
    synthfilter_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new synthfilter_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing synthfilter");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            synthfilter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"synthfilter invalid arguments");
    return -1;

}


static const char *synthfilter_cookie_docstring_ = "";




static PyMethodDef synthfilter_methods_[] = {
    
        { "cookie",(PyCFunction)synthfilter_wrapper_::cookie_method_,METH_VARARGS,(char *)synthfilter_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)synthfilter_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)synthfilter_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *synthfilter_docstring_ = ""
"    Emulation of the Moog 4 pole filter."
"    "
""
;

PyTypeObject synthfilter_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.synthfilter",/*tp_name*/
    sizeof(synthfilter_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)synthfilter_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)synthfilter_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    synthfilter_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)synthfilter_init_,  /* tp_init */
    0,                         /* tp_alloc */
    synthfilter_new_,             /* tp_new */
};

PyObject *synthfilter_wrapcopy_(synthfilter_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *synthfilter_wrap_(synthfilter_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = synthfilter_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    synthfilter_object_ *self = 
        PyObject_NEW(synthfilter_object_, &synthfilter_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        synthfilter_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

synthfilter_type_ *synthfilter_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &synthfilter_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected synthfilter object");
        return 0;
    }

    synthfilter_type_ *object = static_cast<synthfilter_type_ *>(((synthfilter_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null synthfilter object in unwrap");
        return 0;
    }

    return object;
}

void synthfilter_insert_(const std::pair< synthfilter_type_ *, PyObject *> &p)
{




    synthfilter_wrapped_.insert(p);


}

void synthfilter_erase_(synthfilter_type_ *p)
{




    synthfilter_wrapped_.erase(p);


}

PyObject * synthfilter_find_(synthfilter_type_ *p)
{




    PyObject *ret = 0;
    std::map< synthfilter_type_ *, PyObject *>::iterator where = synthfilter_wrapped_.find(p);
    if(where != synthfilter_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct synthfilter_dispatch_ synthfilter_realdispatcher_ =
{
    &synthfilter_typeobject_,
    synthfilter_wrap_,
    synthfilter_wrapcopy_,
    synthfilter_unwrap_,
    synthfilter_insert_,
    synthfilter_erase_,
    synthfilter_find_
};

  

struct synthfilter2_object_ {
    PyObject_HEAD
    synthfilter2_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct synthfilter2_wrapper_: public synthfilter2_type_ 
{

    static void __deleter__(synthfilter2_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    synthfilter2_wrapper_( PyObject *o, PyInterpreterState *i  ,const cookie_type_ & a0  ,clockdomain_ctl_type_ * a1  ): synthfilter2_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~synthfilter2_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    cookie_type_ cookie_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return synthfilter2_type_::cookie();
                
            
            
        }

        return
            
                cookie
            
            
                ();
    }

    static PyObject *cookie_method_(synthfilter2_object_ *self, PyObject *args)
    {
        

        synthfilter2_wrapper_ *o = (synthfilter2_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null synthfilter2 c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    cookie_type_ r = o->cookie_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_cookie_copy((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in synthfilter2::cookie");
        }

        return 0;
    }





    static PyObject *_method_lock(synthfilter2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(synthfilter2_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void synthfilter2_delete_(synthfilter2_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            synthfilter2_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (synthfilter2_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        synthfilter2_wrapper_::__deleter__((synthfilter2_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void synthfilter2_dealloc_(synthfilter2_object_* self)
{
    
    synthfilter2_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * synthfilter2_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    synthfilter2_object_ *self;

    if(!(self = (synthfilter2_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int synthfilter2_init_(synthfilter2_object_ *self, PyObject *args, PyObject *kwds)
{
    synthfilter2_delete_(self);

    

    {
         cookie_type_ * a0;  clockdomain_ctl_type_ * a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_cookie_copy,&a0,fpcvt_clockdomain_ctl,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new synthfilter2_wrapper_((PyObject *)self,interp ,*a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing synthfilter2");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            synthfilter2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"synthfilter2 invalid arguments");
    return -1;

}


static const char *synthfilter2_cookie_docstring_ = "";




static PyMethodDef synthfilter2_methods_[] = {
    
        { "cookie",(PyCFunction)synthfilter2_wrapper_::cookie_method_,METH_VARARGS,(char *)synthfilter2_cookie_docstring_ },
    
    
        { "py_lock",(PyCFunction)synthfilter2_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)synthfilter2_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *synthfilter2_docstring_ = ""
"    Emulation of the Moog 4 pole filter which models"
"    transistor nonlinearities."
"    "
""
;

PyTypeObject synthfilter2_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "synth_native.synthfilter2",/*tp_name*/
    sizeof(synthfilter2_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)synthfilter2_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)synthfilter2_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    synthfilter2_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)synthfilter2_init_,  /* tp_init */
    0,                         /* tp_alloc */
    synthfilter2_new_,             /* tp_new */
};

PyObject *synthfilter2_wrapcopy_(synthfilter2_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *synthfilter2_wrap_(synthfilter2_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = synthfilter2_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    synthfilter2_object_ *self = 
        PyObject_NEW(synthfilter2_object_, &synthfilter2_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        synthfilter2_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

synthfilter2_type_ *synthfilter2_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &synthfilter2_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected synthfilter2 object");
        return 0;
    }

    synthfilter2_type_ *object = static_cast<synthfilter2_type_ *>(((synthfilter2_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null synthfilter2 object in unwrap");
        return 0;
    }

    return object;
}

void synthfilter2_insert_(const std::pair< synthfilter2_type_ *, PyObject *> &p)
{




    synthfilter2_wrapped_.insert(p);


}

void synthfilter2_erase_(synthfilter2_type_ *p)
{




    synthfilter2_wrapped_.erase(p);


}

PyObject * synthfilter2_find_(synthfilter2_type_ *p)
{




    PyObject *ret = 0;
    std::map< synthfilter2_type_ *, PyObject *>::iterator where = synthfilter2_wrapped_.find(p);
    if(where != synthfilter2_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct synthfilter2_dispatch_ synthfilter2_realdispatcher_ =
{
    &synthfilter2_typeobject_,
    synthfilter2_wrap_,
    synthfilter2_wrapcopy_,
    synthfilter2_unwrap_,
    synthfilter2_insert_,
    synthfilter2_erase_,
    synthfilter2_find_
};

                 


static const char *module_method_compressor_docstring_ = "";

static PyObject *module_method_compressor(PyObject *self, PyObject *args)
{
     float a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_f,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    d2d_nb_type_ r = synth::compressor  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_d2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in compressor");
    }

    return 0;
}


static const char *module_method_sharpener_docstring_ = "";

static PyObject *module_method_sharpener(PyObject *self, PyObject *args)
{
     float a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_f,&a0))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    dd2d_nb_type_ r = synth::sharpener  (  a0 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_dd2d_nb_copy((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in sharpener");
    }

    return 0;
}





static PyMethodDef module_method_table[] = {
    
{ "compressor",module_method_compressor,METH_VARARGS, (char *)module_method_compressor_docstring_ }, 
{ "sharpener",module_method_sharpener,METH_VARARGS, (char *)module_method_sharpener_docstring_ }, 
    
    {0}
};
};



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initsynth_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            sine_dispatcher_ = &sine_realdispatcher_;
        
    
        
            adsr_dispatcher_ = &adsr_realdispatcher_;
        
    
        
            adsr2_dispatcher_ = &adsr2_realdispatcher_;
        
    
        
            sawtooth_dispatcher_ = &sawtooth_realdispatcher_;
        
    
        
            rect_dispatcher_ = &rect_realdispatcher_;
        
    
        
            triangle_dispatcher_ = &triangle_realdispatcher_;
        
    
        
            synthfilter_dispatcher_ = &synthfilter_realdispatcher_;
        
    
        
            synthfilter2_dispatcher_ = &synthfilter2_realdispatcher_;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        if(!(m=PyImport_ImportModule("piw")))
        {
            return;
        }

        
            if(!(s=PyObject_GetAttrString(m,"pip_decode_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_decode_ctl_dispatch in module piw");
                return;
            }

            decode_ctl_dispatcher_ = (decode_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockdomain_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockdomain_ctl_dispatch in module piw");
                return;
            }

            clockdomain_ctl_dispatcher_ = (clockdomain_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_clockref_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_clockref_dispatch in module piw");
                return;
            }

            clockref_dispatcher_ = (clockref_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_wire_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_wire_dispatch in module piw");
                return;
            }

            wire_dispatcher_ = (wire_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_wire_ctl_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_wire_ctl_dispatch in module piw");
                return;
            }

            wire_ctl_dispatcher_ = (wire_ctl_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_cookie_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_cookie_dispatch in module piw");
                return;
            }

            cookie_dispatcher_ = (cookie_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_d2d_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_d2d_nb_dispatch in module piw");
                return;
            }

            d2d_nb_dispatcher_ = (d2d_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        
            if(!(s=PyObject_GetAttrString(m,"pip_dd2d_nb_dispatch")) || !PyCObject_Check(s))
            {
                PyErr_SetString(PyExc_RuntimeError,"incorrect or missing pip_dd2d_nb_dispatch in module piw");
                return;
            }

            dd2d_nb_dispatcher_ = (dd2d_nb_dispatch_ *)PyCObject_AsVoidPtr(s);
            Py_DECREF(s);
        

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
        
            if(PyType_Ready(&sine_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&adsr_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&adsr2_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&sawtooth_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&rect_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&triangle_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&synthfilter_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&synthfilter2_typeobject_)<0)
                return;
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    if(!(m=Py_InitModule("synth_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(sine_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_sine_dispatch",s);
            Py_INCREF(&sine_typeobject_);
            PyModule_AddObject(m,"sine",(PyObject *)&sine_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(adsr_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_adsr_dispatch",s);
            Py_INCREF(&adsr_typeobject_);
            PyModule_AddObject(m,"adsr",(PyObject *)&adsr_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(adsr2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_adsr2_dispatch",s);
            Py_INCREF(&adsr2_typeobject_);
            PyModule_AddObject(m,"adsr2",(PyObject *)&adsr2_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(sawtooth_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_sawtooth_dispatch",s);
            Py_INCREF(&sawtooth_typeobject_);
            PyModule_AddObject(m,"sawtooth",(PyObject *)&sawtooth_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(rect_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_rect_dispatch",s);
            Py_INCREF(&rect_typeobject_);
            PyModule_AddObject(m,"rect",(PyObject *)&rect_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(triangle_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_triangle_dispatch",s);
            Py_INCREF(&triangle_typeobject_);
            PyModule_AddObject(m,"triangle",(PyObject *)&triangle_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(synthfilter_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_synthfilter_dispatch",s);
            Py_INCREF(&synthfilter_typeobject_);
            PyModule_AddObject(m,"synthfilter",(PyObject *)&synthfilter_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(synthfilter2_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_synthfilter2_dispatch",s);
            Py_INCREF(&synthfilter2_typeobject_);
            PyModule_AddObject(m,"synthfilter2",(PyObject *)&synthfilter2_typeobject_);
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

