#include <Python.h>
#include <structmember.h>
#include <map>
#include <climits>
#include <stdexcept>
#include <string>


#include <picross/pic_usb.h>
#include <picross/pic_functor.h>
#include <picross/pic_thread.h>
#include <picross/pic_error.h>
#include <picross/pic_platform.h>
#include <picross/pic_power.h>
#include <picross/pic_mlock.h>
#include <picross/pic_table.h>
#include <picross/pic_time.h>
#include <picross/pic_resources.h>

static PyObject *pic_make_notify_functor(PyObject *args);

static PyObject *pic_make_status_functor(PyObject *args);

static PyObject *pic_make_f_string_functor(PyObject *args);

static PyObject *pic_make_ff2f_functor(PyObject *args);

static PyObject *pic_make_f2f_functor(PyObject *args);


#ifdef __GNUC__
#pragma GCC visibility push(hidden)
#endif

namespace
{

#if PY_VERSION_HEX >= 0x02050000
    typedef Py_ssize_t pip_ssize_t;
#else
    typedef int pip_ssize_t;
#endif

    static bool pip_usegil = false;

    // unlocks python and relocks it on dtor
    class unlock_p2c
    {
        public:
            unlock_p2c(bool fake=false): t_(0), f_(fake) { unlock(); }
            ~unlock_p2c() { lock(); }

            void unlock()
            {
                if(f_ || t_) return;
                t_=PyEval_SaveThread();
            }
            void lock() 
            {
                if(f_ || !t_) return;
                PyEval_RestoreThread(t_);
                t_=0;
            }
        private:
            PyThreadState *t_;
            bool f_;
    };

    // locks python and unlocks it on dtor
    class lock_c2p
    {
        public:
            lock_c2p(PyInterpreterState *interp, bool fake=false): t_(0), gs_(PyGILState_LOCKED), f_(fake)
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_AcquireLock();
                        t_=PyThreadState_New(interp);
                        PyEval_ReleaseLock();
                        PyEval_AcquireThread(t_);
                        return;
                    }

                    gs_ = PyGILState_Ensure();
                }
             }

            ~lock_c2p()
            {
                if(!f_)
                {
                    if(!pip_usegil)
                    {
                        PyEval_ReleaseThread(t_);
                        PyEval_AcquireLock();
                        PyThreadState_Delete(t_);
                        PyEval_ReleaseLock();
                        return;
                    }

                    PyGILState_Release(gs_);
                }
            }

        private:
            PyThreadState *t_;
            PyGILState_STATE gs_;
            bool f_;
    };

    // relocks python inside an unlock_p2c and unlocks afterwards
    class relock_p2c
    {
        public:
            relock_p2c(unlock_p2c &u): u_(u) { u_.lock(); }
            ~relock_p2c() { u_.unlock(); }
        private:
            unlock_p2c &u_;
    };

struct pip_err_t: std::exception
{
    pip_err_t(const char *x) : what_(x) {}
    ~pip_err_t() throw() {}
    const char *what() const throw() { return what_.c_str(); }
    std::string what_;
};

static const char *module_docstring = """"
"Python interface to libpic tools"
"================================"
""
"Functors"
"--------"
""
"These are templated classes which can represent anything callable. "
"Each type of functor is individually wrapped here.  A functor object"
"obtained from c++ is called from python with the objects's invoke() method."
"Any python callable can be turned into a functor usable by c++ with one of"
"the make...functor functions."
""
""
""
;


PyObject *tpcvt_uc(void *a)
{
    return PyInt_FromLong(*((unsigned char *)a));
}

PyObject *tpcvt_s(void *a)
{
    return PyInt_FromLong(*((short *)a));
}

PyObject *tpcvt_us(void *a)
{
    return PyInt_FromLong(*((unsigned short *)a));
}

PyObject *tpcvt_i(void *a)
{
    return PyInt_FromLong(*((int *)a));
}

PyObject *tpcvt_ui(void *a)
{
    return PyInt_FromLong(*((unsigned int *)a));
}

PyObject *tpcvt_l(void *a)
{
    return PyInt_FromLong(*((long *)a));
}

PyObject *tpcvt_ul(void *a)
{
    return PyLong_FromUnsignedLong(*((unsigned long *)a));
}

PyObject *tpcvt_ll(void *a)
{
    return PyLong_FromLongLong(*((long long *)a));
}

PyObject *tpcvt_ull(void *a)
{
    return PyLong_FromUnsignedLongLong(*((unsigned long long *)a));
}

PyObject *tpcvt_f(void *a)
{
    return PyFloat_FromDouble(*((float *)a));
}

PyObject *tpcvt_d(void *a)
{
    return PyFloat_FromDouble(*((double *)a));
}

PyObject *tpcvt_b(void *a)
{
    return PyBool_FromLong(*((bool *)a));
}

PyObject *tpcvt_c(void *a)
{
    char c = *((const char *)a);
    return PyString_FromStringAndSize(&c,1);
}

PyObject *tpcvt_stdstr(void *a)
{
    std::string *s = (std::string *)a;
    return PyString_FromStringAndSize(s->c_str(),s->length());
}

PyObject *tpcvt_str(void *a)
{
    return PyString_FromString(*((const char **)a));
}

PyObject *tpcvt_ustr(void *a)
{
    return PyString_FromString((const char *)*((const unsigned char **)a));
}

PyObject *tpcvt_py(void *a)
{
    PyObject *o = *((PyObject **)a);
    if(o) Py_INCREF(o);
    return o;
}

static long fplong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static long long fplonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        return PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        return (long long)PyFloat_AsDouble(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long fpulong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLong(o);
    }

    if(PyFloat_Check(o))
    {
		double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static unsigned long long fpulonglong(PyObject *o)
{
    if(PyInt_Check(o))
    {
        long v = PyInt_AsLong(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return v;
    }

    if(PyLong_Check(o))
    {
        return PyLong_AsUnsignedLongLong(o);
    }

    if(PyFloat_Check(o))
    {
        double v = PyFloat_AsDouble(o);

        if(v<0)
        {
            PyErr_SetString(PyExc_OverflowError,"negative value supplied for conversion to unsigned");
            return 0;
        }

        return (unsigned long long)v;
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to unsigned");
    return 0;
}

static double fpdouble(PyObject *o)
{
    if(PyFloat_Check(o))
    {
        return PyFloat_AsDouble(o);
    }

    if(PyInt_Check(o))
    {
        return (double)PyInt_AsLong(o);
    }

    if(PyLong_Check(o))
    {
        return (double)PyLong_AsLongLong(o);
    }

    PyErr_SetString(PyExc_TypeError,"bad type supplied for conversion to double");
    return 0;
}

int fpcvt_uc(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UCHAR_MAX) { *((unsigned char *)a) = (unsigned char)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected small int");
    }
    return 0;
}

int fpcvt_s(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=SHRT_MIN && val<=SHRT_MAX) { *((short *)a) = (short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_us(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=USHRT_MAX) { *((unsigned short *)a) = (unsigned short)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_i(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred())
    {
        if(val>=INT_MIN && val<=INT_MAX) { *((int *)a) = (int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_ui(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred())
    {
        if(val<=UINT_MAX) { *((unsigned int *)a) = (unsigned int)val; return 1; }
        else PyErr_SetString(PyExc_TypeError,"expected int");
    }
    return 0;
}

int fpcvt_l(PyObject *o, void *a)
{
    PyErr_Clear();
    long val = fplong(o);
    if(!PyErr_Occurred()) { *((long *)a) = (long)val; return 1; }
    return 0;
}

int fpcvt_ul(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long val = fpulong(o);
    if(!PyErr_Occurred()) { *((unsigned long *)a) = (unsigned long)val; return 1; }
    return 0;
}

int fpcvt_ll(PyObject *o, void *a)
{
    PyErr_Clear();
    long long val = fplonglong(o);
    if(!PyErr_Occurred()) { *((long long *)a) = (long long)val; return 1; }
    return 0;
}

int fpcvt_ull(PyObject *o, void *a)
{
    PyErr_Clear();
    unsigned long long val = fpulonglong(o);
    if(!PyErr_Occurred()) { *((unsigned long long *)a) = (unsigned long long)val; return 1; }
    return 0;
}

int fpcvt_f(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((float *)a) = (float)val; return 1; }
    return 0;
}

int fpcvt_d(PyObject *o, void *a)
{
    PyErr_Clear();
    double val = fpdouble(o);
    if(!PyErr_Occurred()) { *((double *)a) = (double)val; return 1; }
    return 0;
}

int fpcvt_b(PyObject *o, void *a)
{
    if(o==Py_True) { *((bool *)a)=true; return 1; }
    if(o==Py_False) { *((bool *)a)=false; return 1; }
    PyErr_SetString(PyExc_TypeError,"expected bool");
    return 0;
}

int fpcvt_c(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        if(l==1)
        {
            *((char *)a)=s[0];
            return 1;
        }

        PyErr_SetString(PyExc_TypeError,"expected character");
    }
    return 0;
}

int fpcvt_stdstr(PyObject *o, void *a)
{
    char *s; pip_ssize_t l;
    if(PyString_AsStringAndSize(o,&s,&l)>=0)
    {
        *((std::string *)a) = std::string(s,l);
        return 1;
    }
    return 0;
}

int fpcvt_str(PyObject *o, void *a)
{
    const char *s = PyString_AsString(o);
    if(s) { *((const char **)a) = s; return 1; }
    return 0;
}

int fpcvt_ustr(PyObject *o, void *a)
{
    const unsigned char *s = (unsigned char *)PyString_AsString(o);
    if(s) { *((const unsigned char **)a) = s; return 1; }
    return 0;
}

int fpcvt_py(PyObject *o, void *a)
{
    if(o) Py_INCREF(o);
    *((PyObject **)a) = o; return 1;
}



typedef pic::notify_t notify_type_; 


 
static std::map<notify_type_ *, PyObject *> notify_wrapped_;
 


struct notify_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(notify_type_ *, bool owned);
    PyObject *(*wrapcopy)(notify_type_ *);
    notify_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< notify_type_ *, PyObject * > &);
    void (*erase)(notify_type_ *);
    PyObject *(*find)(notify_type_ *);
};

struct notify_dispatch_ *notify_dispatcher_;



typedef pic::status_t status_type_; 


 
static std::map<status_type_ *, PyObject *> status_wrapped_;
 


struct status_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(status_type_ *, bool owned);
    PyObject *(*wrapcopy)(status_type_ *);
    status_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< status_type_ *, PyObject * > &);
    void (*erase)(status_type_ *);
    PyObject *(*find)(status_type_ *);
};

struct status_dispatch_ *status_dispatcher_;



typedef pic::f_string_t f_string_type_; 


 
static std::map<f_string_type_ *, PyObject *> f_string_wrapped_;
 


struct f_string_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f_string_type_ *, bool owned);
    PyObject *(*wrapcopy)(f_string_type_ *);
    f_string_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f_string_type_ *, PyObject * > &);
    void (*erase)(f_string_type_ *);
    PyObject *(*find)(f_string_type_ *);
};

struct f_string_dispatch_ *f_string_dispatcher_;



typedef pic::ff2f_t ff2f_type_; 


 
static std::map<ff2f_type_ *, PyObject *> ff2f_wrapped_;
 


struct ff2f_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(ff2f_type_ *, bool owned);
    PyObject *(*wrapcopy)(ff2f_type_ *);
    ff2f_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< ff2f_type_ *, PyObject * > &);
    void (*erase)(ff2f_type_ *);
    PyObject *(*find)(ff2f_type_ *);
};

struct ff2f_dispatch_ *ff2f_dispatcher_;



typedef pic::f2f_t f2f_type_; 


 
static std::map<f2f_type_ *, PyObject *> f2f_wrapped_;
 


struct f2f_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(f2f_type_ *, bool owned);
    PyObject *(*wrapcopy)(f2f_type_ *);
    f2f_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< f2f_type_ *, PyObject * > &);
    void (*erase)(f2f_type_ *);
    PyObject *(*find)(f2f_type_ *);
};

struct f2f_dispatch_ *f2f_dispatcher_;



typedef pic::usbdevice_t::bulk_out_pipe_t bulk_out_pipe_type_; 


 
static std::map<bulk_out_pipe_type_ *, PyObject *> bulk_out_pipe_wrapped_;
 


struct bulk_out_pipe_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(bulk_out_pipe_type_ *, bool owned);
    PyObject *(*wrapcopy)(bulk_out_pipe_type_ *);
    bulk_out_pipe_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< bulk_out_pipe_type_ *, PyObject * > &);
    void (*erase)(bulk_out_pipe_type_ *);
    PyObject *(*find)(bulk_out_pipe_type_ *);
};

struct bulk_out_pipe_dispatch_ *bulk_out_pipe_dispatcher_;



typedef pic::usbdevice_t usbdevice_type_; 


 
static std::map<usbdevice_type_ *, PyObject *> usbdevice_wrapped_;
 


struct usbdevice_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(usbdevice_type_ *, bool owned);
    PyObject *(*wrapcopy)(usbdevice_type_ *);
    usbdevice_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< usbdevice_type_ *, PyObject * > &);
    void (*erase)(usbdevice_type_ *);
    PyObject *(*find)(usbdevice_type_ *);
};

struct usbdevice_dispatch_ *usbdevice_dispatcher_;



typedef pic::usbenumerator_t enumerator_type_; 


 
static std::map<enumerator_type_ *, PyObject *> enumerator_wrapped_;
 


struct enumerator_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(enumerator_type_ *, bool owned);
    PyObject *(*wrapcopy)(enumerator_type_ *);
    enumerator_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< enumerator_type_ *, PyObject * > &);
    void (*erase)(enumerator_type_ *);
    PyObject *(*find)(enumerator_type_ *);
};

struct enumerator_dispatch_ *enumerator_dispatcher_;



typedef pic::gate_t gate_type_; 


 
static std::map<gate_type_ *, PyObject *> gate_wrapped_;
 


struct gate_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(gate_type_ *, bool owned);
    PyObject *(*wrapcopy)(gate_type_ *);
    gate_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< gate_type_ *, PyObject * > &);
    void (*erase)(gate_type_ *);
    PyObject *(*find)(gate_type_ *);
};

struct gate_dispatch_ *gate_dispatcher_;



typedef pic::mutex_t mutex_type_; 


 
static std::map<mutex_type_ *, PyObject *> mutex_wrapped_;
 


struct mutex_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(mutex_type_ *, bool owned);
    PyObject *(*wrapcopy)(mutex_type_ *);
    mutex_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< mutex_type_ *, PyObject * > &);
    void (*erase)(mutex_type_ *);
    PyObject *(*find)(mutex_type_ *);
};

struct mutex_dispatch_ *mutex_dispatcher_;



typedef pic::thread_t thread_type_; 


 
static std::map<thread_type_ *, PyObject *> thread_wrapped_;
 


struct thread_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(thread_type_ *, bool owned);
    PyObject *(*wrapcopy)(thread_type_ *);
    thread_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< thread_type_ *, PyObject * > &);
    void (*erase)(thread_type_ *);
    PyObject *(*find)(thread_type_ *);
};

struct thread_dispatch_ *thread_dispatcher_;



typedef pic::ftable_t ftable_type_; 


 
static std::map<ftable_type_ *, PyObject *> ftable_wrapped_;
 


struct ftable_dispatch_
{
    PyTypeObject *type;
    PyObject *(*wrap)(ftable_type_ *, bool owned);
    PyObject *(*wrapcopy)(ftable_type_ *);
    ftable_type_ *(*unwrap)(PyObject *);
    void (*insert)(const std::pair< ftable_type_ *, PyObject * > &);
    void (*erase)(ftable_type_ *);
    PyObject *(*find)(ftable_type_ *);
};

struct ftable_dispatch_ *ftable_dispatcher_;





int fpcvt_notify(PyObject *o, void *a)
{
    notify_type_ *object;
    if(o == Py_None)
    {
        *((notify_type_ **)a) = 0;
        return 1;
    }
    object = notify_dispatcher_->unwrap(o);
    *((notify_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_notify_copy fpcvt_notify

PyObject *tpcvt_notify(void *a)
{
    notify_type_ *o = *((notify_type_ **)a);
    return notify_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_notify_copy(void *a)
{
    notify_type_ *o = (notify_type_ *)a;
    return notify_dispatcher_->wrapcopy(o);
}



int fpcvt_status(PyObject *o, void *a)
{
    status_type_ *object;
    if(o == Py_None)
    {
        *((status_type_ **)a) = 0;
        return 1;
    }
    object = status_dispatcher_->unwrap(o);
    *((status_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_status_copy fpcvt_status

PyObject *tpcvt_status(void *a)
{
    status_type_ *o = *((status_type_ **)a);
    return status_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_status_copy(void *a)
{
    status_type_ *o = (status_type_ *)a;
    return status_dispatcher_->wrapcopy(o);
}



int fpcvt_f_string(PyObject *o, void *a)
{
    f_string_type_ *object;
    if(o == Py_None)
    {
        *((f_string_type_ **)a) = 0;
        return 1;
    }
    object = f_string_dispatcher_->unwrap(o);
    *((f_string_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f_string_copy fpcvt_f_string

PyObject *tpcvt_f_string(void *a)
{
    f_string_type_ *o = *((f_string_type_ **)a);
    return f_string_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f_string_copy(void *a)
{
    f_string_type_ *o = (f_string_type_ *)a;
    return f_string_dispatcher_->wrapcopy(o);
}



int fpcvt_ff2f(PyObject *o, void *a)
{
    ff2f_type_ *object;
    if(o == Py_None)
    {
        *((ff2f_type_ **)a) = 0;
        return 1;
    }
    object = ff2f_dispatcher_->unwrap(o);
    *((ff2f_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_ff2f_copy fpcvt_ff2f

PyObject *tpcvt_ff2f(void *a)
{
    ff2f_type_ *o = *((ff2f_type_ **)a);
    return ff2f_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_ff2f_copy(void *a)
{
    ff2f_type_ *o = (ff2f_type_ *)a;
    return ff2f_dispatcher_->wrapcopy(o);
}



int fpcvt_f2f(PyObject *o, void *a)
{
    f2f_type_ *object;
    if(o == Py_None)
    {
        *((f2f_type_ **)a) = 0;
        return 1;
    }
    object = f2f_dispatcher_->unwrap(o);
    *((f2f_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_f2f_copy fpcvt_f2f

PyObject *tpcvt_f2f(void *a)
{
    f2f_type_ *o = *((f2f_type_ **)a);
    return f2f_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_f2f_copy(void *a)
{
    f2f_type_ *o = (f2f_type_ *)a;
    return f2f_dispatcher_->wrapcopy(o);
}



int fpcvt_bulk_out_pipe(PyObject *o, void *a)
{
    bulk_out_pipe_type_ *object;
    if(o == Py_None)
    {
        *((bulk_out_pipe_type_ **)a) = 0;
        return 1;
    }
    object = bulk_out_pipe_dispatcher_->unwrap(o);
    *((bulk_out_pipe_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_bulk_out_pipe_copy fpcvt_bulk_out_pipe

PyObject *tpcvt_bulk_out_pipe(void *a)
{
    bulk_out_pipe_type_ *o = *((bulk_out_pipe_type_ **)a);
    return bulk_out_pipe_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_bulk_out_pipe_copy(void *a)
{
    bulk_out_pipe_type_ *o = (bulk_out_pipe_type_ *)a;
    return bulk_out_pipe_dispatcher_->wrapcopy(o);
}



int fpcvt_usbdevice(PyObject *o, void *a)
{
    usbdevice_type_ *object;
    if(o == Py_None)
    {
        *((usbdevice_type_ **)a) = 0;
        return 1;
    }
    object = usbdevice_dispatcher_->unwrap(o);
    *((usbdevice_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_usbdevice_copy fpcvt_usbdevice

PyObject *tpcvt_usbdevice(void *a)
{
    usbdevice_type_ *o = *((usbdevice_type_ **)a);
    return usbdevice_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_usbdevice_copy(void *a)
{
    usbdevice_type_ *o = (usbdevice_type_ *)a;
    return usbdevice_dispatcher_->wrapcopy(o);
}



int fpcvt_enumerator(PyObject *o, void *a)
{
    enumerator_type_ *object;
    if(o == Py_None)
    {
        *((enumerator_type_ **)a) = 0;
        return 1;
    }
    object = enumerator_dispatcher_->unwrap(o);
    *((enumerator_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_enumerator_copy fpcvt_enumerator

PyObject *tpcvt_enumerator(void *a)
{
    enumerator_type_ *o = *((enumerator_type_ **)a);
    return enumerator_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_enumerator_copy(void *a)
{
    enumerator_type_ *o = (enumerator_type_ *)a;
    return enumerator_dispatcher_->wrapcopy(o);
}



int fpcvt_gate(PyObject *o, void *a)
{
    gate_type_ *object;
    if(o == Py_None)
    {
        *((gate_type_ **)a) = 0;
        return 1;
    }
    object = gate_dispatcher_->unwrap(o);
    *((gate_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_gate_copy fpcvt_gate

PyObject *tpcvt_gate(void *a)
{
    gate_type_ *o = *((gate_type_ **)a);
    return gate_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_gate_copy(void *a)
{
    gate_type_ *o = (gate_type_ *)a;
    return gate_dispatcher_->wrapcopy(o);
}



int fpcvt_mutex(PyObject *o, void *a)
{
    mutex_type_ *object;
    if(o == Py_None)
    {
        *((mutex_type_ **)a) = 0;
        return 1;
    }
    object = mutex_dispatcher_->unwrap(o);
    *((mutex_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_mutex_copy fpcvt_mutex

PyObject *tpcvt_mutex(void *a)
{
    mutex_type_ *o = *((mutex_type_ **)a);
    return mutex_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_mutex_copy(void *a)
{
    mutex_type_ *o = (mutex_type_ *)a;
    return mutex_dispatcher_->wrapcopy(o);
}



int fpcvt_thread(PyObject *o, void *a)
{
    thread_type_ *object;
    if(o == Py_None)
    {
        *((thread_type_ **)a) = 0;
        return 1;
    }
    object = thread_dispatcher_->unwrap(o);
    *((thread_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_thread_copy fpcvt_thread

PyObject *tpcvt_thread(void *a)
{
    thread_type_ *o = *((thread_type_ **)a);
    return thread_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_thread_copy(void *a)
{
    thread_type_ *o = (thread_type_ *)a;
    return thread_dispatcher_->wrapcopy(o);
}



int fpcvt_ftable(PyObject *o, void *a)
{
    ftable_type_ *object;
    if(o == Py_None)
    {
        *((ftable_type_ **)a) = 0;
        return 1;
    }
    object = ftable_dispatcher_->unwrap(o);
    *((ftable_type_ **)a) = object;
    return object?1:0;
}

#define fpcvt_ftable_copy fpcvt_ftable

PyObject *tpcvt_ftable(void *a)
{
    ftable_type_ *o = *((ftable_type_ **)a);
    return ftable_dispatcher_->wrap(o,false);
}

PyObject *tpcvt_ftable_copy(void *a)
{
    ftable_type_ *o = (ftable_type_ *)a;
    return ftable_dispatcher_->wrapcopy(o);
}



 

struct notify_object_ {
    PyObject_HEAD
    notify_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct notify_wrapper_: public notify_type_ 
{

    static void __deleter__(notify_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    notify_wrapper_( PyObject *o, PyInterpreterState *i  ): notify_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    notify_wrapper_( PyObject *o, PyInterpreterState *i  ,const notify_type_ & a0  ): notify_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~notify_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(notify_object_ *self, visitproc visit, void *arg)
    {
        notify_wrapper_ *o = (notify_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(notify_object_ *self)
    {
        notify_wrapper_ *o = (notify_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning  doc ""

    void special_call_call_()
    {
        return
             invoke 
            
                ();
    }

    static PyObject *special_call_method_(notify_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"notify::__call__ doesn't support keywords");
            return 0;
        }

        

        notify_wrapper_ *o = (notify_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null notify c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock  ;
                o->special_call_call_ ( );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }

        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in notify::__call__");
        }

        return 0;
    }




    void invoke_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return notify_type_::invoke();
                
            
            
        }

        return
            
                invoke
            
            
                ();
    }

    static PyObject *invoke_method_(notify_object_ *self, PyObject *args)
    {
        

        notify_wrapper_ *o = (notify_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null notify c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->invoke_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in notify::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(notify_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(notify_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void notify_delete_(notify_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            notify_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (notify_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        notify_wrapper_::__deleter__((notify_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void notify_dealloc_(notify_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    notify_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * notify_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    notify_object_ *self;

    if(!(self = (notify_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int notify_init_(notify_object_ *self, PyObject *args, PyObject *kwds)
{
    notify_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new notify_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing notify");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            notify_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         notify_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_notify_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new notify_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing notify");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            notify_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"notify invalid arguments");
    return -1;

}


static const char *notify_invoke_docstring_ = ""
"        self.invoke() -> "
"        "
"        invoke the functor."
"        "
""
;




static PyMethodDef notify_methods_[] = {
    
        { "invoke",(PyCFunction)notify_wrapper_::invoke_method_,METH_VARARGS,(char *)notify_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)notify_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)notify_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *notify_docstring_ = ""
"    Functor with no argument and no return value."
"    "
"    Notify functors can be obtained from libpic c++ objects or"
"    created from python callables using make_notify_functor"
"    "
"    Initialisers"
"    ============"
"      "
"    notify() - creates a null functor which does nothing"
"    "
""
;

PyTypeObject notify_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.notify",/*tp_name*/
    sizeof(notify_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)notify_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)notify_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)notify_docstring_,       /* tp_doc */
    (traverseproc)notify_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)notify_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    notify_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)notify_init_,  /* tp_init */
    0,                         /* tp_alloc */
    notify_new_,             /* tp_new */
};

PyObject *notify_wrapcopy_(notify_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            notify_object_ *obj = 
                
                PyObject_GC_New(notify_object_, notify_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new notify_type_(*o);
                }

                try
                {
                    notify_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying notify");
                return 0;
            }
        

    
}

PyObject *notify_wrap_(notify_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = notify_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    notify_object_ *self = 
        
        PyObject_GC_New(notify_object_, &notify_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        notify_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

notify_type_ *notify_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &notify_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected notify object");
        return 0;
    }

    notify_type_ *object = static_cast<notify_type_ *>(((notify_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null notify object in unwrap");
        return 0;
    }

    return object;
}

void notify_insert_(const std::pair< notify_type_ *, PyObject *> &p)
{




    notify_wrapped_.insert(p);


}

void notify_erase_(notify_type_ *p)
{




    notify_wrapped_.erase(p);


}

PyObject * notify_find_(notify_type_ *p)
{




    PyObject *ret = 0;
    std::map< notify_type_ *, PyObject *>::iterator where = notify_wrapped_.find(p);
    if(where != notify_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct notify_dispatch_ notify_realdispatcher_ =
{
    &notify_typeobject_,
    notify_wrap_,
    notify_wrapcopy_,
    notify_unwrap_,
    notify_insert_,
    notify_erase_,
    notify_find_
};

  

struct status_object_ {
    PyObject_HEAD
    status_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct status_wrapper_: public status_type_ 
{

    static void __deleter__(status_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    status_wrapper_( PyObject *o, PyInterpreterState *i  ): status_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    status_wrapper_( PyObject *o, PyInterpreterState *i  ,const status_type_ & a0  ): status_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~status_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(status_object_ *self, visitproc visit, void *arg)
    {
        status_wrapper_ *o = (status_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(status_object_ *self)
    {
        status_wrapper_ *o = (status_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning  doc ""

    void special_call_call_( bool a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(status_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"status::__call__ doesn't support keywords");
            return 0;
        }

         bool a0; 

        status_wrapper_ *o = (status_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null status c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock  ;
                o->special_call_call_ (a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }

        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in status::__call__");
        }

        return 0;
    }




    void invoke_call_(bool owned ,bool a0 )
    {
        if(owned)
        {
            
                
                
                    return status_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(status_object_ *self, PyObject *args)
    {
         bool a0; 

        status_wrapper_ *o = (status_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null status c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_b,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->invoke_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in status::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(status_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(status_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void status_delete_(status_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            status_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (status_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        status_wrapper_::__deleter__((status_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void status_dealloc_(status_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    status_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * status_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    status_object_ *self;

    if(!(self = (status_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int status_init_(status_object_ *self, PyObject *args, PyObject *kwds)
{
    status_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new status_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing status");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            status_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         status_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_status_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new status_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing status");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            status_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"status invalid arguments");
    return -1;

}


static const char *status_invoke_docstring_ = ""
"        self.invoke(bool) ->"
""
"        invoke the functor."
"        "
""
;




static PyMethodDef status_methods_[] = {
    
        { "invoke",(PyCFunction)status_wrapper_::invoke_method_,METH_VARARGS,(char *)status_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)status_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)status_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *status_docstring_ = ""
"    Functor with bool argument and no return value."
"    "
""
;

PyTypeObject status_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.status",/*tp_name*/
    sizeof(status_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)status_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)status_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)status_docstring_,       /* tp_doc */
    (traverseproc)status_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)status_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    status_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)status_init_,  /* tp_init */
    0,                         /* tp_alloc */
    status_new_,             /* tp_new */
};

PyObject *status_wrapcopy_(status_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            status_object_ *obj = 
                
                PyObject_GC_New(status_object_, status_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new status_type_(*o);
                }

                try
                {
                    status_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying status");
                return 0;
            }
        

    
}

PyObject *status_wrap_(status_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = status_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    status_object_ *self = 
        
        PyObject_GC_New(status_object_, &status_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        status_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

status_type_ *status_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &status_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected status object");
        return 0;
    }

    status_type_ *object = static_cast<status_type_ *>(((status_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null status object in unwrap");
        return 0;
    }

    return object;
}

void status_insert_(const std::pair< status_type_ *, PyObject *> &p)
{




    status_wrapped_.insert(p);


}

void status_erase_(status_type_ *p)
{




    status_wrapped_.erase(p);


}

PyObject * status_find_(status_type_ *p)
{




    PyObject *ret = 0;
    std::map< status_type_ *, PyObject *>::iterator where = status_wrapped_.find(p);
    if(where != status_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct status_dispatch_ status_realdispatcher_ =
{
    &status_typeobject_,
    status_wrap_,
    status_wrapcopy_,
    status_unwrap_,
    status_insert_,
    status_erase_,
    status_find_
};

  

struct f_string_object_ {
    PyObject_HEAD
    f_string_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct f_string_wrapper_: public f_string_type_ 
{

    static void __deleter__(f_string_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    f_string_wrapper_( PyObject *o, PyInterpreterState *i  ): f_string_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    f_string_wrapper_( PyObject *o, PyInterpreterState *i  ,const f_string_type_ & a0  ): f_string_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~f_string_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(f_string_object_ *self, visitproc visit, void *arg)
    {
        f_string_wrapper_ *o = (f_string_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(f_string_object_ *self)
    {
        f_string_wrapper_ *o = (f_string_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning  doc ""

    void special_call_call_( const char * a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(f_string_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"f_string::__call__ doesn't support keywords");
            return 0;
        }

         const char * a0; 

        f_string_wrapper_ *o = (f_string_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null f_string c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock  ;
                o->special_call_call_ (a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }

        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in f_string::__call__");
        }

        return 0;
    }




    void invoke_call_(bool owned ,const char * a0 )
    {
        if(owned)
        {
            
                
                
                    return f_string_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(f_string_object_ *self, PyObject *args)
    {
         const char * a0; 

        f_string_wrapper_ *o = (f_string_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null f_string c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_str,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->invoke_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in f_string::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(f_string_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(f_string_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void f_string_delete_(f_string_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            f_string_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (f_string_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        f_string_wrapper_::__deleter__((f_string_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void f_string_dealloc_(f_string_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    f_string_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * f_string_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    f_string_object_ *self;

    if(!(self = (f_string_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int f_string_init_(f_string_object_ *self, PyObject *args, PyObject *kwds)
{
    f_string_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_string_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_string");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_string_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         f_string_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_f_string_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f_string_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f_string");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f_string_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"f_string invalid arguments");
    return -1;

}


static const char *f_string_invoke_docstring_ = ""
"        self.invoke(const char *) -> ."
""
"        invoke the functor with a string argument."
"        "
""
;




static PyMethodDef f_string_methods_[] = {
    
        { "invoke",(PyCFunction)f_string_wrapper_::invoke_method_,METH_VARARGS,(char *)f_string_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)f_string_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)f_string_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *f_string_docstring_ = ""
"    Functor with 1 string argument and no return value."
"    "
"    "
"    Initialisers"
"    ============"
"      "
"    f_string() - creates a null functor"
"    "
""
;

PyTypeObject f_string_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.f_string",/*tp_name*/
    sizeof(f_string_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)f_string_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)f_string_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)f_string_docstring_,       /* tp_doc */
    (traverseproc)f_string_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)f_string_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    f_string_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)f_string_init_,  /* tp_init */
    0,                         /* tp_alloc */
    f_string_new_,             /* tp_new */
};

PyObject *f_string_wrapcopy_(f_string_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            f_string_object_ *obj = 
                
                PyObject_GC_New(f_string_object_, f_string_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new f_string_type_(*o);
                }

                try
                {
                    f_string_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying f_string");
                return 0;
            }
        

    
}

PyObject *f_string_wrap_(f_string_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = f_string_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    f_string_object_ *self = 
        
        PyObject_GC_New(f_string_object_, &f_string_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        f_string_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

f_string_type_ *f_string_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &f_string_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected f_string object");
        return 0;
    }

    f_string_type_ *object = static_cast<f_string_type_ *>(((f_string_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null f_string object in unwrap");
        return 0;
    }

    return object;
}

void f_string_insert_(const std::pair< f_string_type_ *, PyObject *> &p)
{




    f_string_wrapped_.insert(p);


}

void f_string_erase_(f_string_type_ *p)
{




    f_string_wrapped_.erase(p);


}

PyObject * f_string_find_(f_string_type_ *p)
{




    PyObject *ret = 0;
    std::map< f_string_type_ *, PyObject *>::iterator where = f_string_wrapped_.find(p);
    if(where != f_string_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct f_string_dispatch_ f_string_realdispatcher_ =
{
    &f_string_typeobject_,
    f_string_wrap_,
    f_string_wrapcopy_,
    f_string_unwrap_,
    f_string_insert_,
    f_string_erase_,
    f_string_find_
};

  

struct ff2f_object_ {
    PyObject_HEAD
    ff2f_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct ff2f_wrapper_: public ff2f_type_ 
{

    static void __deleter__(ff2f_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    ff2f_wrapper_( PyObject *o, PyInterpreterState *i  ): ff2f_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    ff2f_wrapper_( PyObject *o, PyInterpreterState *i  ,const ff2f_type_ & a0  ): ff2f_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~ff2f_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(ff2f_object_ *self, visitproc visit, void *arg)
    {
        ff2f_wrapper_ *o = (ff2f_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(ff2f_object_ *self)
    {
        ff2f_wrapper_ *o = (ff2f_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning float doc ""

    float special_call_call_( float a0  ,float a1 )
    {
        return
             invoke 
            
                ( a0  ,a1 );
    }

    static PyObject *special_call_method_(ff2f_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"ff2f::__call__ doesn't support keywords");
            return 0;
        }

         float a0;  float a1; 

        ff2f_wrapper_ *o = (ff2f_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ff2f c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->special_call_call_ (a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ff2f::__call__");
        }

        return 0;
    }




    float invoke_call_(bool owned ,float a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return ff2f_type_::invoke( a0  ,a1 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0  ,a1 );
    }

    static PyObject *invoke_method_(ff2f_object_ *self, PyObject *args)
    {
         float a0;  float a1; 

        ff2f_wrapper_ *o = (ff2f_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ff2f c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->invoke_call_ (self->owned ,a0 ,a1  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ff2f::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(ff2f_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(ff2f_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void ff2f_delete_(ff2f_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            ff2f_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (ff2f_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        ff2f_wrapper_::__deleter__((ff2f_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void ff2f_dealloc_(ff2f_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    ff2f_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * ff2f_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    ff2f_object_ *self;

    if(!(self = (ff2f_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int ff2f_init_(ff2f_object_ *self, PyObject *args, PyObject *kwds)
{
    ff2f_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new ff2f_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing ff2f");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            ff2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         ff2f_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ff2f_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new ff2f_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing ff2f");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            ff2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"ff2f invalid arguments");
    return -1;

}


static const char *ff2f_invoke_docstring_ = ""
"        self.invoke(float,float) -> float"
"        invoke the functor"
"        "
""
;




static PyMethodDef ff2f_methods_[] = {
    
        { "invoke",(PyCFunction)ff2f_wrapper_::invoke_method_,METH_VARARGS,(char *)ff2f_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)ff2f_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)ff2f_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *ff2f_docstring_ = ""
"    Functor that takes 2 floats and returns a float"
"    "
"    Initialisers"
"    ============"
"      "
"    ff2f() - creates a null functor which returns 0.0"
"    "
""
;

PyTypeObject ff2f_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.ff2f",/*tp_name*/
    sizeof(ff2f_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)ff2f_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)ff2f_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)ff2f_docstring_,       /* tp_doc */
    (traverseproc)ff2f_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)ff2f_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    ff2f_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)ff2f_init_,  /* tp_init */
    0,                         /* tp_alloc */
    ff2f_new_,             /* tp_new */
};

PyObject *ff2f_wrapcopy_(ff2f_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            ff2f_object_ *obj = 
                
                PyObject_GC_New(ff2f_object_, ff2f_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new ff2f_type_(*o);
                }

                try
                {
                    ff2f_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying ff2f");
                return 0;
            }
        

    
}

PyObject *ff2f_wrap_(ff2f_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = ff2f_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    ff2f_object_ *self = 
        
        PyObject_GC_New(ff2f_object_, &ff2f_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        ff2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

ff2f_type_ *ff2f_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &ff2f_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected ff2f object");
        return 0;
    }

    ff2f_type_ *object = static_cast<ff2f_type_ *>(((ff2f_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null ff2f object in unwrap");
        return 0;
    }

    return object;
}

void ff2f_insert_(const std::pair< ff2f_type_ *, PyObject *> &p)
{




    ff2f_wrapped_.insert(p);


}

void ff2f_erase_(ff2f_type_ *p)
{




    ff2f_wrapped_.erase(p);


}

PyObject * ff2f_find_(ff2f_type_ *p)
{




    PyObject *ret = 0;
    std::map< ff2f_type_ *, PyObject *>::iterator where = ff2f_wrapped_.find(p);
    if(where != ff2f_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct ff2f_dispatch_ ff2f_realdispatcher_ =
{
    &ff2f_typeobject_,
    ff2f_wrap_,
    ff2f_wrapcopy_,
    ff2f_unwrap_,
    ff2f_insert_,
    ff2f_erase_,
    ff2f_find_
};

  

struct f2f_object_ {
    PyObject_HEAD
    f2f_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct f2f_wrapper_: public f2f_type_ 
{

    static void __deleter__(f2f_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    f2f_wrapper_( PyObject *o, PyInterpreterState *i  ): f2f_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    f2f_wrapper_( PyObject *o, PyInterpreterState *i  ,const f2f_type_ & a0  ): f2f_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~f2f_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(f2f_object_ *self, visitproc visit, void *arg)
    {
        f2f_wrapper_ *o = (f2f_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(f2f_object_ *self)
    {
        f2f_wrapper_ *o = (f2f_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }




    // special method __call__ returning float doc ""

    float special_call_call_( float a0 )
    {
        return
             invoke 
            
                ( a0 );
    }

    static PyObject *special_call_method_(f2f_object_ *self, PyObject *args, PyObject *kwds)
    {
        if(kwds)
        {
            PyErr_SetString(PyExc_TypeError,"f2f::__call__ doesn't support keywords");
            return 0;
        }

         float a0; 

        f2f_wrapper_ *o = (f2f_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null f2f c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        

            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->special_call_call_ (a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in f2f::__call__");
        }

        return 0;
    }




    float invoke_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return f2f_type_::invoke( a0 );
                
            
            
        }

        return
            
                invoke
            
            
                ( a0 );
    }

    static PyObject *invoke_method_(f2f_object_ *self, PyObject *args)
    {
         float a0; 

        f2f_wrapper_ *o = (f2f_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null f2f c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->invoke_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in f2f::invoke");
        }

        return 0;
    }





    static PyObject *_method_lock(f2f_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(f2f_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void f2f_delete_(f2f_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            f2f_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (f2f_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        f2f_wrapper_::__deleter__((f2f_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void f2f_dealloc_(f2f_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    f2f_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * f2f_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    f2f_object_ *self;

    if(!(self = (f2f_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int f2f_init_(f2f_object_ *self, PyObject *args, PyObject *kwds)
{
    f2f_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f2f_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f2f");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         f2f_type_ * a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_f2f_copy,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new f2f_wrapper_((PyObject *)self,interp ,*a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing f2f");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            f2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"f2f invalid arguments");
    return -1;

}


static const char *f2f_invoke_docstring_ = ""
"        self.invoke(float,float) -> float"
"        invoke the functor"
"        "
""
;




static PyMethodDef f2f_methods_[] = {
    
        { "invoke",(PyCFunction)f2f_wrapper_::invoke_method_,METH_VARARGS,(char *)f2f_invoke_docstring_ },
    
    
        { "py_lock",(PyCFunction)f2f_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)f2f_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *f2f_docstring_ = ""
"    Functor that takes a float and returns a float"
"    "
"    Initialisers"
"    ============"
"      "
"    f2f() - creates a null functor which returns 0.0"
"    "
""
;

PyTypeObject f2f_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.f2f",/*tp_name*/
    sizeof(f2f_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)f2f_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
     (ternaryfunc)f2f_wrapper_::special_call_method_  , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)f2f_docstring_,       /* tp_doc */
    (traverseproc)f2f_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)f2f_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    f2f_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)f2f_init_,  /* tp_init */
    0,                         /* tp_alloc */
    f2f_new_,             /* tp_new */
};

PyObject *f2f_wrapcopy_(f2f_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        

        

            f2f_object_ *obj = 
                
                PyObject_GC_New(f2f_object_, f2f_dispatcher_->type); 

            if(!obj)
            {
                return 0;
            }

            PyObject_GC_Track((PyObject *)obj); 

            obj->wrapped=0;
            obj->locked=0;
            obj->owned=true;

            try
            {
                {   unlock_p2c unlock;
                    obj->object = new f2f_type_(*o);
                }

                try
                {
                    f2f_dispatcher_->insert(std::make_pair(obj->object,(PyObject *)obj));
                }
                catch(...)
                {
                }

                return (PyObject *)obj;
            }
            catch(const std::exception &e)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return 0;
            }
            catch(...)
            {
                Py_DECREF(obj);
                PyErr_SetString(PyExc_RuntimeError,"c++ exception copying f2f");
                return 0;
            }
        

    
}

PyObject *f2f_wrap_(f2f_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = f2f_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    f2f_object_ *self = 
        
        PyObject_GC_New(f2f_object_, &f2f_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        f2f_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

f2f_type_ *f2f_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &f2f_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected f2f object");
        return 0;
    }

    f2f_type_ *object = static_cast<f2f_type_ *>(((f2f_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null f2f object in unwrap");
        return 0;
    }

    return object;
}

void f2f_insert_(const std::pair< f2f_type_ *, PyObject *> &p)
{




    f2f_wrapped_.insert(p);


}

void f2f_erase_(f2f_type_ *p)
{




    f2f_wrapped_.erase(p);


}

PyObject * f2f_find_(f2f_type_ *p)
{




    PyObject *ret = 0;
    std::map< f2f_type_ *, PyObject *>::iterator where = f2f_wrapped_.find(p);
    if(where != f2f_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct f2f_dispatch_ f2f_realdispatcher_ =
{
    &f2f_typeobject_,
    f2f_wrap_,
    f2f_wrapcopy_,
    f2f_unwrap_,
    f2f_insert_,
    f2f_erase_,
    f2f_find_
};

  

struct bulk_out_pipe_object_ {
    PyObject_HEAD
    bulk_out_pipe_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct bulk_out_pipe_wrapper_: public bulk_out_pipe_type_ 
{

    static void __deleter__(bulk_out_pipe_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    bulk_out_pipe_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ,unsigned int a1  ): bulk_out_pipe_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~bulk_out_pipe_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void bulk_write_call_(bool owned ,const std::string & a0 )
    {
        if(owned)
        {
            
                
                
                    return bulk_out_pipe_type_::bulk_write( a0 );
                
            
            
        }

        return
            
                bulk_write
            
            
                ( a0 );
    }

    static PyObject *bulk_write_method_(bulk_out_pipe_object_ *self, PyObject *args)
    {
         std::string a0; 

        bulk_out_pipe_wrapper_ *o = (bulk_out_pipe_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null bulk_out_pipe c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_stdstr,&a0 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->bulk_write_call_ (self->owned ,a0 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in bulk_out_pipe::bulk_write");
        }

        return 0;
    }





    static PyObject *_method_lock(bulk_out_pipe_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(bulk_out_pipe_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void bulk_out_pipe_delete_(bulk_out_pipe_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            bulk_out_pipe_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (bulk_out_pipe_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        bulk_out_pipe_wrapper_::__deleter__((bulk_out_pipe_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void bulk_out_pipe_dealloc_(bulk_out_pipe_object_* self)
{
    
    bulk_out_pipe_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * bulk_out_pipe_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    bulk_out_pipe_object_ *self;

    if(!(self = (bulk_out_pipe_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int bulk_out_pipe_init_(bulk_out_pipe_object_ *self, PyObject *args, PyObject *kwds)
{
    bulk_out_pipe_delete_(self);

    

    {
         unsigned int a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_ui,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new bulk_out_pipe_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing bulk_out_pipe");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            bulk_out_pipe_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"bulk_out_pipe invalid arguments");
    return -1;

}


static const char *bulk_out_pipe_bulk_write_docstring_ = "";




static PyMethodDef bulk_out_pipe_methods_[] = {
    
        { "bulk_write",(PyCFunction)bulk_out_pipe_wrapper_::bulk_write_method_,METH_VARARGS,(char *)bulk_out_pipe_bulk_write_docstring_ },
    
    
        { "py_lock",(PyCFunction)bulk_out_pipe_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)bulk_out_pipe_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *bulk_out_pipe_docstring_ = "";

PyTypeObject bulk_out_pipe_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.bulk_out_pipe",/*tp_name*/
    sizeof(bulk_out_pipe_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)bulk_out_pipe_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)bulk_out_pipe_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    bulk_out_pipe_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)bulk_out_pipe_init_,  /* tp_init */
    0,                         /* tp_alloc */
    bulk_out_pipe_new_,             /* tp_new */
};

PyObject *bulk_out_pipe_wrapcopy_(bulk_out_pipe_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *bulk_out_pipe_wrap_(bulk_out_pipe_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = bulk_out_pipe_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    bulk_out_pipe_object_ *self = 
        PyObject_NEW(bulk_out_pipe_object_, &bulk_out_pipe_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        bulk_out_pipe_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

bulk_out_pipe_type_ *bulk_out_pipe_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &bulk_out_pipe_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected bulk_out_pipe object");
        return 0;
    }

    bulk_out_pipe_type_ *object = static_cast<bulk_out_pipe_type_ *>(((bulk_out_pipe_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null bulk_out_pipe object in unwrap");
        return 0;
    }

    return object;
}

void bulk_out_pipe_insert_(const std::pair< bulk_out_pipe_type_ *, PyObject *> &p)
{




    bulk_out_pipe_wrapped_.insert(p);


}

void bulk_out_pipe_erase_(bulk_out_pipe_type_ *p)
{




    bulk_out_pipe_wrapped_.erase(p);


}

PyObject * bulk_out_pipe_find_(bulk_out_pipe_type_ *p)
{




    PyObject *ret = 0;
    std::map< bulk_out_pipe_type_ *, PyObject *>::iterator where = bulk_out_pipe_wrapped_.find(p);
    if(where != bulk_out_pipe_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct bulk_out_pipe_dispatch_ bulk_out_pipe_realdispatcher_ =
{
    &bulk_out_pipe_typeobject_,
    bulk_out_pipe_wrap_,
    bulk_out_pipe_wrapcopy_,
    bulk_out_pipe_unwrap_,
    bulk_out_pipe_insert_,
    bulk_out_pipe_erase_,
    bulk_out_pipe_find_
};

  

struct usbdevice_object_ {
    PyObject_HEAD
    usbdevice_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct usbdevice_wrapper_: public usbdevice_type_ 
{

    static void __deleter__(usbdevice_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    usbdevice_wrapper_( PyObject *o, PyInterpreterState *i  ,const char * a0  ,unsigned int a1  ): usbdevice_type_(  a0 ,a1 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~usbdevice_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    bool add_bulk_out_call_(bool owned ,bulk_out_pipe_type_ * a0 )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::add_bulk_out( a0 );
                
            
            
        }

        return
            
                add_bulk_out
            
            
                ( a0 );
    }

    static PyObject *add_bulk_out_method_(usbdevice_object_ *self, PyObject *args)
    {
         bulk_out_pipe_type_ * a0; 

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_bulk_out_pipe,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->add_bulk_out_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::add_bulk_out");
        }

        return 0;
    }



    void control_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::control( a0  ,a1  ,a2  ,a3 );
                
            
            
        }

        return
            
                control
            
            
                ( a0  ,a1  ,a2  ,a3 );
    }

    static PyObject *control_method_(usbdevice_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3; 

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->control_call_ (self->owned ,a0,a1,a2,a3 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::control");
        }

        return 0;
    }



    void control_out_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 ,const std::string & a4 )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::control_out( a0  ,a1  ,a2  ,a3  ,a4 );
                
            
            
        }

        return
            
                control_out
            
            
                ( a0  ,a1  ,a2  ,a3  ,a4 );
    }

    static PyObject *control_out_method_(usbdevice_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3;  std::string a4; 

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3  ,fpcvt_stdstr,&a4 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->control_out_call_ (self->owned ,a0,a1,a2,a3,a4 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::control_out");
        }

        return 0;
    }



    std::string control_in_call_(bool owned ,unsigned char a0 ,unsigned char a1 ,unsigned short a2 ,unsigned short a3 ,unsigned int a4 )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::control_in( a0  ,a1  ,a2  ,a3  ,a4 );
                
            
            
        }

        return
            
                control_in
            
            
                ( a0  ,a1  ,a2  ,a3  ,a4 );
    }

    static PyObject *control_in_method_(usbdevice_object_ *self, PyObject *args)
    {
         unsigned char a0;  unsigned char a1;  unsigned short a2;  unsigned short a3;  unsigned int a4; 

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&O&O&O&"  ,fpcvt_uc,&a0  ,fpcvt_uc,&a1  ,fpcvt_us,&a2  ,fpcvt_us,&a3  ,fpcvt_ui,&a4 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    std::string r = o->control_in_call_ (self->owned ,a0 ,a1 ,a2 ,a3 ,a4  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::control_in");
        }

        return 0;
    }



    void start_pipes_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::start_pipes();
                
            
            
        }

        return
            
                start_pipes
            
            
                ();
    }

    static PyObject *start_pipes_method_(usbdevice_object_ *self, PyObject *args)
    {
        

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_pipes_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::start_pipes");
        }

        return 0;
    }



    void stop_pipes_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return usbdevice_type_::stop_pipes();
                
            
            
        }

        return
            
                stop_pipes
            
            
                ();
    }

    static PyObject *stop_pipes_method_(usbdevice_object_ *self, PyObject *args)
    {
        

        usbdevice_wrapper_ *o = (usbdevice_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null usbdevice c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_pipes_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in usbdevice::stop_pipes");
        }

        return 0;
    }





    static PyObject *_method_lock(usbdevice_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(usbdevice_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void usbdevice_delete_(usbdevice_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            usbdevice_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (usbdevice_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        usbdevice_wrapper_::__deleter__((usbdevice_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void usbdevice_dealloc_(usbdevice_object_* self)
{
    
    usbdevice_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * usbdevice_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    usbdevice_object_ *self;

    if(!(self = (usbdevice_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int usbdevice_init_(usbdevice_object_ *self, PyObject *args, PyObject *kwds)
{
    usbdevice_delete_(self);

    

    {
         const char * a0;  unsigned int a1; 

        if(PyArg_ParseTuple(args,"O&O&" ,fpcvt_str,&a0,fpcvt_ui,&a1))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new usbdevice_wrapper_((PyObject *)self,interp ,a0,a1);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing usbdevice");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            usbdevice_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"usbdevice invalid arguments");
    return -1;

}


static const char *usbdevice_add_bulk_out_docstring_ = "";

static const char *usbdevice_control_docstring_ = "";

static const char *usbdevice_control_out_docstring_ = "";

static const char *usbdevice_control_in_docstring_ = "";

static const char *usbdevice_start_pipes_docstring_ = "";

static const char *usbdevice_stop_pipes_docstring_ = "";




static PyMethodDef usbdevice_methods_[] = {
    
        { "add_bulk_out",(PyCFunction)usbdevice_wrapper_::add_bulk_out_method_,METH_VARARGS,(char *)usbdevice_add_bulk_out_docstring_ },
    
        { "control",(PyCFunction)usbdevice_wrapper_::control_method_,METH_VARARGS,(char *)usbdevice_control_docstring_ },
    
        { "control_out",(PyCFunction)usbdevice_wrapper_::control_out_method_,METH_VARARGS,(char *)usbdevice_control_out_docstring_ },
    
        { "control_in",(PyCFunction)usbdevice_wrapper_::control_in_method_,METH_VARARGS,(char *)usbdevice_control_in_docstring_ },
    
        { "start_pipes",(PyCFunction)usbdevice_wrapper_::start_pipes_method_,METH_VARARGS,(char *)usbdevice_start_pipes_docstring_ },
    
        { "stop_pipes",(PyCFunction)usbdevice_wrapper_::stop_pipes_method_,METH_VARARGS,(char *)usbdevice_stop_pipes_docstring_ },
    
    
        { "py_lock",(PyCFunction)usbdevice_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)usbdevice_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *usbdevice_docstring_ = "";

PyTypeObject usbdevice_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.usbdevice",/*tp_name*/
    sizeof(usbdevice_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)usbdevice_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)usbdevice_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    usbdevice_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)usbdevice_init_,  /* tp_init */
    0,                         /* tp_alloc */
    usbdevice_new_,             /* tp_new */
};

PyObject *usbdevice_wrapcopy_(usbdevice_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *usbdevice_wrap_(usbdevice_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = usbdevice_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    usbdevice_object_ *self = 
        PyObject_NEW(usbdevice_object_, &usbdevice_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        usbdevice_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

usbdevice_type_ *usbdevice_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &usbdevice_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected usbdevice object");
        return 0;
    }

    usbdevice_type_ *object = static_cast<usbdevice_type_ *>(((usbdevice_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null usbdevice object in unwrap");
        return 0;
    }

    return object;
}

void usbdevice_insert_(const std::pair< usbdevice_type_ *, PyObject *> &p)
{




    usbdevice_wrapped_.insert(p);


}

void usbdevice_erase_(usbdevice_type_ *p)
{




    usbdevice_wrapped_.erase(p);


}

PyObject * usbdevice_find_(usbdevice_type_ *p)
{




    PyObject *ret = 0;
    std::map< usbdevice_type_ *, PyObject *>::iterator where = usbdevice_wrapped_.find(p);
    if(where != usbdevice_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct usbdevice_dispatch_ usbdevice_realdispatcher_ =
{
    &usbdevice_typeobject_,
    usbdevice_wrap_,
    usbdevice_wrapcopy_,
    usbdevice_unwrap_,
    usbdevice_insert_,
    usbdevice_erase_,
    usbdevice_find_
};

  

struct enumerator_object_ {
    PyObject_HEAD
    enumerator_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct enumerator_wrapper_: public enumerator_type_ 
{

    static void __deleter__(enumerator_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    enumerator_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned short a0  ,unsigned short a1  ,const f_string_type_ & a2  ): enumerator_type_(  a0 ,a1 ,a2 ), object(o), _interp(i)
    {
        //init_cache();
    }



    enumerator_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned short a0  ,unsigned short a1  ,const f_string_type_ & a2  ,const f_string_type_ & a3  ): enumerator_type_(  a0 ,a1 ,a2 ,a3 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~enumerator_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }













    static int gc_traverse_method_(enumerator_object_ *self, visitproc visit, void *arg)
    {
        enumerator_wrapper_ *o = (enumerator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        return o->gc_traverse((void *)visit,(void *)arg);
    }

    static int gc_clear_method_(enumerator_object_ *self)
    {
        enumerator_wrapper_ *o = (enumerator_wrapper_ *)(self->object);

        if(!o)
        {
            return 0;
        }

        {   unlock_p2c unlock(false);
            return o->gc_clear();
        }
    }







    void start_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return enumerator_type_::start();
                
            
            
        }

        return
            
                start
            
            
                ();
    }

    static PyObject *start_method_(enumerator_object_ *self, PyObject *args)
    {
        

        enumerator_wrapper_ *o = (enumerator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null enumerator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->start_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in enumerator::start");
        }

        return 0;
    }



    void stop_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return enumerator_type_::stop();
                
            
            
        }

        return
            
                stop
            
            
                ();
    }

    static PyObject *stop_method_(enumerator_object_ *self, PyObject *args)
    {
        

        enumerator_wrapper_ *o = (enumerator_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null enumerator c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->stop_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in enumerator::stop");
        }

        return 0;
    }





    static PyObject *_method_lock(enumerator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(enumerator_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void enumerator_delete_(enumerator_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            enumerator_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (enumerator_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        enumerator_wrapper_::__deleter__((enumerator_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void enumerator_dealloc_(enumerator_object_* self)
{
    PyObject_GC_UnTrack((PyObject *)self); 
    enumerator_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * enumerator_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    enumerator_object_ *self;

    if(!(self = (enumerator_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int enumerator_init_(enumerator_object_ *self, PyObject *args, PyObject *kwds)
{
    enumerator_delete_(self);

    

    {
         unsigned short a0;  unsigned short a1;  f_string_type_ * a2; 

        if(PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_us,&a0,fpcvt_us,&a1,fpcvt_f_string_copy,&a2))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new enumerator_wrapper_((PyObject *)self,interp ,a0,a1,*a2);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing enumerator");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            enumerator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned short a0;  unsigned short a1;  f_string_type_ * a2;  f_string_type_ * a3; 

        if(PyArg_ParseTuple(args,"O&O&O&O&" ,fpcvt_us,&a0,fpcvt_us,&a1,fpcvt_f_string_copy,&a2,fpcvt_f_string_copy,&a3))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new enumerator_wrapper_((PyObject *)self,interp ,a0,a1,*a2,*a3);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing enumerator");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            enumerator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"enumerator invalid arguments");
    return -1;

}


static const char *enumerator_start_docstring_ = ""
"        enumerator.start() -> ."
"        start the enumeration process.  All existing devices will be reported first"
"        "
""
;

static const char *enumerator_stop_docstring_ = ""
"        enumerator.stop() -> ."
"        stop the enumeration process.  After stop() returns, no new devices will be reported."
"        "
""
;




static PyMethodDef enumerator_methods_[] = {
    
        { "start",(PyCFunction)enumerator_wrapper_::start_method_,METH_VARARGS,(char *)enumerator_start_docstring_ },
    
        { "stop",(PyCFunction)enumerator_wrapper_::stop_method_,METH_VARARGS,(char *)enumerator_stop_docstring_ },
    
    
        { "py_lock",(PyCFunction)enumerator_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)enumerator_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *enumerator_docstring_ = ""
"    enumerator(vendor,product,functor) -> enumerator object."
""
"    Watch a bus for device insertions.  As each device matching vendor and product"
"    is attached, call the functor with the device name. "
""
"    When the enumerator is started, all currently attached devices will be "
"    reported.  Then new devices will be reported when they are attached.  "
"    "
""
;

PyTypeObject enumerator_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.enumerator",/*tp_name*/
    sizeof(enumerator_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)enumerator_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC , /*tp_flags*/
    (char *)enumerator_docstring_,       /* tp_doc */
    (traverseproc)enumerator_wrapper_::gc_traverse_method_  ,   /* tp_traverse */
    (inquiry)enumerator_wrapper_::gc_clear_method_  ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    enumerator_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)enumerator_init_,  /* tp_init */
    0,                         /* tp_alloc */
    enumerator_new_,             /* tp_new */
};

PyObject *enumerator_wrapcopy_(enumerator_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *enumerator_wrap_(enumerator_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = enumerator_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    enumerator_object_ *self = 
        
        PyObject_GC_New(enumerator_object_, &enumerator_typeobject_); 

    if(!self) return 0;

    PyObject_GC_Track((PyObject *)self); 

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        enumerator_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

enumerator_type_ *enumerator_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &enumerator_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected enumerator object");
        return 0;
    }

    enumerator_type_ *object = static_cast<enumerator_type_ *>(((enumerator_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null enumerator object in unwrap");
        return 0;
    }

    return object;
}

void enumerator_insert_(const std::pair< enumerator_type_ *, PyObject *> &p)
{




    enumerator_wrapped_.insert(p);


}

void enumerator_erase_(enumerator_type_ *p)
{




    enumerator_wrapped_.erase(p);


}

PyObject * enumerator_find_(enumerator_type_ *p)
{




    PyObject *ret = 0;
    std::map< enumerator_type_ *, PyObject *>::iterator where = enumerator_wrapped_.find(p);
    if(where != enumerator_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct enumerator_dispatch_ enumerator_realdispatcher_ =
{
    &enumerator_typeobject_,
    enumerator_wrap_,
    enumerator_wrapcopy_,
    enumerator_unwrap_,
    enumerator_insert_,
    enumerator_erase_,
    enumerator_find_
};

  

struct gate_object_ {
    PyObject_HEAD
    gate_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct gate_wrapper_: public gate_type_ 
{

    static void __deleter__(gate_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    gate_wrapper_( PyObject *o, PyInterpreterState *i  ): gate_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~gate_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    bool open_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::open();
                
            
            
        }

        return
            
                open
            
            
                ();
    }

    static PyObject *open_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->open_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::open");
        }

        return 0;
    }



    bool shut_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::shut();
                
            
            
        }

        return
            
                shut
            
            
                ();
    }

    static PyObject *shut_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->shut_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::shut");
        }

        return 0;
    }



    void untimedpass_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::untimedpass();
                
            
            
        }

        return
            
                untimedpass
            
            
                ();
    }

    static PyObject *untimedpass_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->untimedpass_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::untimedpass");
        }

        return 0;
    }



    bool timedpass_call_(bool owned ,unsigned long long a0 )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::timedpass( a0 );
                
            
            
        }

        return
            
                timedpass
            
            
                ( a0 );
    }

    static PyObject *timedpass_method_(gate_object_ *self, PyObject *args)
    {
         unsigned long long a0; 

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_ull,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->timedpass_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::timedpass");
        }

        return 0;
    }



    bool isopen_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return gate_type_::isopen();
                
            
            
        }

        return
            
                isopen
            
            
                ();
    }

    static PyObject *isopen_method_(gate_object_ *self, PyObject *args)
    {
        

        gate_wrapper_ *o = (gate_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null gate c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->isopen_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in gate::isopen");
        }

        return 0;
    }





    static PyObject *_method_lock(gate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(gate_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void gate_delete_(gate_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            gate_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (gate_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        gate_wrapper_::__deleter__((gate_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void gate_dealloc_(gate_object_* self)
{
    
    gate_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * gate_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    gate_object_ *self;

    if(!(self = (gate_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int gate_init_(gate_object_ *self, PyObject *args, PyObject *kwds)
{
    gate_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new gate_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing gate");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            gate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"gate invalid arguments");
    return -1;

}


static const char *gate_open_docstring_ = "";

static const char *gate_shut_docstring_ = "";

static const char *gate_untimedpass_docstring_ = "";

static const char *gate_timedpass_docstring_ = "";

static const char *gate_isopen_docstring_ = "";




static PyMethodDef gate_methods_[] = {
    
        { "open",(PyCFunction)gate_wrapper_::open_method_,METH_VARARGS,(char *)gate_open_docstring_ },
    
        { "shut",(PyCFunction)gate_wrapper_::shut_method_,METH_VARARGS,(char *)gate_shut_docstring_ },
    
        { "untimedpass",(PyCFunction)gate_wrapper_::untimedpass_method_,METH_VARARGS,(char *)gate_untimedpass_docstring_ },
    
        { "timedpass",(PyCFunction)gate_wrapper_::timedpass_method_,METH_VARARGS,(char *)gate_timedpass_docstring_ },
    
        { "isopen",(PyCFunction)gate_wrapper_::isopen_method_,METH_VARARGS,(char *)gate_isopen_docstring_ },
    
    
        { "py_lock",(PyCFunction)gate_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)gate_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *gate_docstring_ = "";

PyTypeObject gate_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.gate",/*tp_name*/
    sizeof(gate_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)gate_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)gate_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    gate_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)gate_init_,  /* tp_init */
    0,                         /* tp_alloc */
    gate_new_,             /* tp_new */
};

PyObject *gate_wrapcopy_(gate_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *gate_wrap_(gate_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = gate_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    gate_object_ *self = 
        PyObject_NEW(gate_object_, &gate_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        gate_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

gate_type_ *gate_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &gate_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected gate object");
        return 0;
    }

    gate_type_ *object = static_cast<gate_type_ *>(((gate_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null gate object in unwrap");
        return 0;
    }

    return object;
}

void gate_insert_(const std::pair< gate_type_ *, PyObject *> &p)
{




    gate_wrapped_.insert(p);


}

void gate_erase_(gate_type_ *p)
{




    gate_wrapped_.erase(p);


}

PyObject * gate_find_(gate_type_ *p)
{




    PyObject *ret = 0;
    std::map< gate_type_ *, PyObject *>::iterator where = gate_wrapped_.find(p);
    if(where != gate_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct gate_dispatch_ gate_realdispatcher_ =
{
    &gate_typeobject_,
    gate_wrap_,
    gate_wrapcopy_,
    gate_unwrap_,
    gate_insert_,
    gate_erase_,
    gate_find_
};

  

struct mutex_object_ {
    PyObject_HEAD
    mutex_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct mutex_wrapper_: public mutex_type_ 
{

    static void __deleter__(mutex_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    mutex_wrapper_( PyObject *o, PyInterpreterState *i  ): mutex_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~mutex_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    void lock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return mutex_type_::lock();
                
            
            
        }

        return
            
                lock
            
            
                ();
    }

    static PyObject *lock_method_(mutex_object_ *self, PyObject *args)
    {
        

        mutex_wrapper_ *o = (mutex_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mutex c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->lock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mutex::lock");
        }

        return 0;
    }



    void unlock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return mutex_type_::unlock();
                
            
            
        }

        return
            
                unlock
            
            
                ();
    }

    static PyObject *unlock_method_(mutex_object_ *self, PyObject *args)
    {
        

        mutex_wrapper_ *o = (mutex_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mutex c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->unlock_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mutex::unlock");
        }

        return 0;
    }



    bool trylock_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return mutex_type_::trylock();
                
            
            
        }

        return
            
                trylock
            
            
                ();
    }

    static PyObject *trylock_method_(mutex_object_ *self, PyObject *args)
    {
        

        mutex_wrapper_ *o = (mutex_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null mutex c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->trylock_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in mutex::trylock");
        }

        return 0;
    }





    static PyObject *_method_lock(mutex_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(mutex_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void mutex_delete_(mutex_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            mutex_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (mutex_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        mutex_wrapper_::__deleter__((mutex_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void mutex_dealloc_(mutex_object_* self)
{
    
    mutex_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * mutex_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    mutex_object_ *self;

    if(!(self = (mutex_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int mutex_init_(mutex_object_ *self, PyObject *args, PyObject *kwds)
{
    mutex_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new mutex_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing mutex");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            mutex_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"mutex invalid arguments");
    return -1;

}


static const char *mutex_lock_docstring_ = "";

static const char *mutex_unlock_docstring_ = "";

static const char *mutex_trylock_docstring_ = "";




static PyMethodDef mutex_methods_[] = {
    
        { "lock",(PyCFunction)mutex_wrapper_::lock_method_,METH_VARARGS,(char *)mutex_lock_docstring_ },
    
        { "unlock",(PyCFunction)mutex_wrapper_::unlock_method_,METH_VARARGS,(char *)mutex_unlock_docstring_ },
    
        { "trylock",(PyCFunction)mutex_wrapper_::trylock_method_,METH_VARARGS,(char *)mutex_trylock_docstring_ },
    
    
        { "py_lock",(PyCFunction)mutex_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)mutex_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *mutex_docstring_ = "";

PyTypeObject mutex_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.mutex",/*tp_name*/
    sizeof(mutex_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)mutex_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)mutex_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    mutex_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)mutex_init_,  /* tp_init */
    0,                         /* tp_alloc */
    mutex_new_,             /* tp_new */
};

PyObject *mutex_wrapcopy_(mutex_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *mutex_wrap_(mutex_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = mutex_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    mutex_object_ *self = 
        PyObject_NEW(mutex_object_, &mutex_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        mutex_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

mutex_type_ *mutex_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &mutex_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected mutex object");
        return 0;
    }

    mutex_type_ *object = static_cast<mutex_type_ *>(((mutex_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null mutex object in unwrap");
        return 0;
    }

    return object;
}

void mutex_insert_(const std::pair< mutex_type_ *, PyObject *> &p)
{




    mutex_wrapped_.insert(p);


}

void mutex_erase_(mutex_type_ *p)
{




    mutex_wrapped_.erase(p);


}

PyObject * mutex_find_(mutex_type_ *p)
{




    PyObject *ret = 0;
    std::map< mutex_type_ *, PyObject *>::iterator where = mutex_wrapped_.find(p);
    if(where != mutex_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct mutex_dispatch_ mutex_realdispatcher_ =
{
    &mutex_typeobject_,
    mutex_wrap_,
    mutex_wrapcopy_,
    mutex_unwrap_,
    mutex_insert_,
    mutex_erase_,
    mutex_find_
};

  

struct thread_object_ {
    PyObject_HEAD
    thread_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct thread_wrapper_: public thread_type_ 
{

    static void __deleter__(thread_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
        cache_0_ = 0;
    
        cache_1_ = 0;
    
        cache_2_ = 0;
    
    }
*/

/*
    void clear_cache()
    {
    
        if(cache_0_)
        {
            Py_DECREF(cache_0_);
            cache_0_ = 0;
        }
    
        if(cache_1_)
        {
            Py_DECREF(cache_1_);
            cache_1_ = 0;
        }
    
        if(cache_2_)
        {
            Py_DECREF(cache_2_);
            cache_2_ = 0;
        }
    
    }
*/



    thread_wrapper_( PyObject *o, PyInterpreterState *i  ): thread_type_(  ), object(o), _interp(i)
    {
        //init_cache();
    }



    thread_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): thread_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~thread_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }


    //PyObject *cache_0_;
 
    void thread_init(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_0_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thread_init");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_0_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thread_init before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thread_init after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_1_;
 
    void thread_main(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_1_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thread_main");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_1_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thread_main before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thread_main after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }


    //PyObject *cache_2_;
 
    void thread_term(  )
    {
        lock_c2p lock(_interp);

        //PyObject *method = cache_2_;
        PyObject *method = 0;

        if(!method)
        {
            method = PyObject_GetAttrString(object,"thread_term");

            if(!method)
            {
                throw pip_err_t("pure virtual called");
            }

            //cache_2_ = method;
        }
        else
        {
        }

        PyObject *result = PyObject_CallFunction(method,(char *)"()" );
        //printf("thread_term before decref %ld\n",method->ob_refcnt); fflush(stdout);
        Py_DECREF(method);
        //printf("thread_term after decref\n"); fflush(stdout);

        if(!result)
        {
            PyErr_Print();
            throw pip_err_t("call problem");
        }

        
            Py_DECREF(result);
            return;
        

        
    }

















    void wait_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::wait();
                
            
            
        }

        return
            
                wait
            
            
                ();
    }

    static PyObject *wait_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->wait_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::wait");
        }

        return 0;
    }



    void run_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::run();
                
            
            
        }

        return
            
                run
            
            
                ();
    }

    static PyObject *run_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->run_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::run");
        }

        return 0;
    }



    bool isrunning_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::isrunning();
                
            
            
        }

        return
            
                isrunning
            
            
                ();
    }

    static PyObject *isrunning_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    bool r = o->isrunning_call_ (self->owned  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::isrunning");
        }

        return 0;
    }



    void thread_init_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::thread_init();
                
            
            
        }

        return
            
                thread_init
            
            
                ();
    }

    static PyObject *thread_init_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thread_init_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::thread_init");
        }

        return 0;
    }



    void thread_main_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::thread_main();
                
            
            
        }

        return
            
                thread_main
            
            
                ();
    }

    static PyObject *thread_main_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thread_main_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::thread_main");
        }

        return 0;
    }



    void thread_term_call_(bool owned )
    {
        if(owned)
        {
            
                
                
                    return thread_type_::thread_term();
                
            
            
        }

        return
            
                thread_term
            
            
                ();
    }

    static PyObject *thread_term_method_(thread_object_ *self, PyObject *args)
    {
        

        thread_wrapper_ *o = (thread_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null thread c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"" ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->thread_term_call_ (self->owned  );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in thread::thread_term");
        }

        return 0;
    }





    static PyObject *_method_lock(thread_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(thread_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void thread_delete_(thread_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            thread_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (thread_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        thread_wrapper_::__deleter__((thread_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void thread_dealloc_(thread_object_* self)
{
    
    thread_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * thread_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    thread_object_ *self;

    if(!(self = (thread_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int thread_init_(thread_object_ *self, PyObject *args, PyObject *kwds)
{
    thread_delete_(self);

    

    {
        

        if(PyArg_ParseTuple(args,"" ))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new thread_wrapper_((PyObject *)self,interp );
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing thread");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            thread_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new thread_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing thread");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            thread_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"thread invalid arguments");
    return -1;

}


static const char *thread_wait_docstring_ = "";

static const char *thread_run_docstring_ = "";

static const char *thread_isrunning_docstring_ = "";

static const char *thread_thread_init_docstring_ = "";

static const char *thread_thread_main_docstring_ = "";

static const char *thread_thread_term_docstring_ = "";




static PyMethodDef thread_methods_[] = {
    
        { "wait",(PyCFunction)thread_wrapper_::wait_method_,METH_VARARGS,(char *)thread_wait_docstring_ },
    
        { "run",(PyCFunction)thread_wrapper_::run_method_,METH_VARARGS,(char *)thread_run_docstring_ },
    
        { "isrunning",(PyCFunction)thread_wrapper_::isrunning_method_,METH_VARARGS,(char *)thread_isrunning_docstring_ },
    
        { "thread_init",(PyCFunction)thread_wrapper_::thread_init_method_,METH_VARARGS,(char *)thread_thread_init_docstring_ },
    
        { "thread_main",(PyCFunction)thread_wrapper_::thread_main_method_,METH_VARARGS,(char *)thread_thread_main_docstring_ },
    
        { "thread_term",(PyCFunction)thread_wrapper_::thread_term_method_,METH_VARARGS,(char *)thread_thread_term_docstring_ },
    
    
        { "py_lock",(PyCFunction)thread_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)thread_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *thread_docstring_ = "";

PyTypeObject thread_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.thread",/*tp_name*/
    sizeof(thread_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)thread_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)thread_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    thread_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)thread_init_,  /* tp_init */
    0,                         /* tp_alloc */
    thread_new_,             /* tp_new */
};

PyObject *thread_wrapcopy_(thread_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *thread_wrap_(thread_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = thread_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    thread_object_ *self = 
        PyObject_NEW(thread_object_, &thread_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        thread_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

thread_type_ *thread_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &thread_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected thread object");
        return 0;
    }

    thread_type_ *object = static_cast<thread_type_ *>(((thread_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null thread object in unwrap");
        return 0;
    }

    return object;
}

void thread_insert_(const std::pair< thread_type_ *, PyObject *> &p)
{




    thread_wrapped_.insert(p);


}

void thread_erase_(thread_type_ *p)
{




    thread_wrapped_.erase(p);


}

PyObject * thread_find_(thread_type_ *p)
{




    PyObject *ret = 0;
    std::map< thread_type_ *, PyObject *>::iterator where = thread_wrapped_.find(p);
    if(where != thread_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct thread_dispatch_ thread_realdispatcher_ =
{
    &thread_typeobject_,
    thread_wrap_,
    thread_wrapcopy_,
    thread_unwrap_,
    thread_insert_,
    thread_erase_,
    thread_find_
};

  

struct ftable_object_ {
    PyObject_HEAD
    ftable_type_ *object;
    unsigned short wrapped;
    unsigned short locked;
    bool owned;
};


struct ftable_wrapper_: public ftable_type_ 
{

    static void __deleter__(ftable_type_ *o)
    {
        delete o;
    }

/*
    void init_cache()
    {
    
    }
*/

/*
    void clear_cache()
    {
    
    }
*/



    ftable_wrapper_( PyObject *o, PyInterpreterState *i  ,unsigned int a0  ): ftable_type_(  a0 ), object(o), _interp(i)
    {
        //init_cache();
    }



    void __safe_dtors()
    {
        
    }

    virtual ~ftable_wrapper_()
    {
        //clear_cache();

        //PyThreadState *thread=PyEval_SaveThread();

        try
        {
            __safe_dtors();
        }
        catch(...)
        {
        }

        //PyEval_RestoreThread(thread);
    }

















    float interp_call_(bool owned ,float a0 )
    {
        if(owned)
        {
            
                
                
                    return ftable_type_::interp( a0 );
                
            
            
        }

        return
            
                interp
            
            
                ( a0 );
    }

    static PyObject *interp_method_(ftable_object_ *self, PyObject *args)
    {
         float a0; 

        ftable_wrapper_ *o = (ftable_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ftable c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&"  ,fpcvt_f,&a0 ))
        {
            return 0;
        }

        try
        {

        

        
            PyObject *ro;

            {   unlock_p2c unlock  ;
                {
                    float r = o->interp_call_ (self->owned ,a0  );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_f((void *)&r);
                    }
                } // r dtor runs here
            }

            return ro;

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ftable::interp");
        }

        return 0;
    }



    void set_bounds_call_(bool owned ,float a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return ftable_type_::set_bounds( a0  ,a1 );
                
            
            
        }

        return
            
                set_bounds
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_bounds_method_(ftable_object_ *self, PyObject *args)
    {
         float a0;  float a1; 

        ftable_wrapper_ *o = (ftable_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ftable c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_f,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_bounds_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ftable::set_bounds");
        }

        return 0;
    }



    void set_entry_call_(bool owned ,unsigned int a0 ,float a1 )
    {
        if(owned)
        {
            
                
                
                    return ftable_type_::set_entry( a0  ,a1 );
                
            
            
        }

        return
            
                set_entry
            
            
                ( a0  ,a1 );
    }

    static PyObject *set_entry_method_(ftable_object_ *self, PyObject *args)
    {
         unsigned int a0;  float a1; 

        ftable_wrapper_ *o = (ftable_wrapper_ *)(self->object);

        if(!o)
        {
            PyErr_SetString(PyExc_RuntimeError,"null ftable c++ object, probably forgot to __init__");
            return 0;
        }

        if(!PyArg_ParseTuple(args,"O&O&"  ,fpcvt_ui,&a0  ,fpcvt_f,&a1 ))
        {
            return 0;
        }

        try
        {

        
            {   unlock_p2c unlock ;
                o->set_entry_call_ (self->owned ,a0,a1 );
                {   relock_p2c relock(unlock);
                    Py_INCREF(Py_None);
                    return Py_None;
                }
            }
        

        

        }
        catch(const std::exception &e)
        {
            PyErr_SetString(PyExc_RuntimeError,e.what());
        }
        catch(...)
        {
            PyErr_SetString(PyExc_RuntimeError,"c++ exception in ftable::set_entry");
        }

        return 0;
    }





    static PyObject *_method_lock(ftable_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(self->locked++ == 0)
        {
            Py_INCREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    static PyObject *_method_unlock(ftable_object_ *self, PyObject *args)
    {
        if(!PyArg_ParseTuple(args,""))
        {
            return 0;
        }

        if(--self->locked == 0)
        {
            Py_DECREF(self);
        }

        //Py_INCREF(self);
        //return (PyObject *)self;
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *object;
    PyInterpreterState *_interp;
};

static void ftable_delete_(ftable_object_* self)
{
    if(self->object)
    {
        if(self->owned)
        {
            ftable_dispatcher_->erase(self->object);

            try
            {
                if(self->wrapped)
                {
                    {   unlock_p2c unlock;
                        delete (ftable_wrapper_ *)self->object;
                    }
                }
                else
                {
                    {   unlock_p2c unlock;
                        ftable_wrapper_::__deleter__((ftable_type_ *)self->object);
                    }
                }
            }
            catch(...)
            {
            }
        }

        self->object = 0;
    }
}

static void ftable_dealloc_(ftable_object_* self)
{
    
    ftable_delete_(self);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject * ftable_new_(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    ftable_object_ *self;

    if(!(self = (ftable_object_ *)type->tp_alloc(type, 0)))
    {
        return 0;
    }

    self->object=0;
    self->locked=0;
    self->wrapped=0;
    self->owned=true;
    return (PyObject *)self;
}

static int ftable_init_(ftable_object_ *self, PyObject *args, PyObject *kwds)
{
    ftable_delete_(self);

    

    {
         unsigned int a0; 

        if(PyArg_ParseTuple(args,"O&" ,fpcvt_ui,&a0))
        {
            try
            {
                PyInterpreterState *interp = PyThreadState_Get()->interp;
                {   unlock_p2c unlock;
                    self->object = new ftable_wrapper_((PyObject *)self,interp ,a0);
                }
            }
            catch(const std::exception &e)
            {
                PyErr_SetString(PyExc_RuntimeError,e.what());
                return -1;
            }
            catch(...)
            {
                PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception constructing ftable");
                return -1;
            }

            self->wrapped=1;
            self->locked=0;
            self->owned=true;
            ftable_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
            return 0;
        }
    }

    

    PyErr_SetString(PyExc_RuntimeError,"ftable invalid arguments");
    return -1;

}


static const char *ftable_interp_docstring_ = "";

static const char *ftable_set_bounds_docstring_ = "";

static const char *ftable_set_entry_docstring_ = "";




static PyMethodDef ftable_methods_[] = {
    
        { "interp",(PyCFunction)ftable_wrapper_::interp_method_,METH_VARARGS,(char *)ftable_interp_docstring_ },
    
        { "set_bounds",(PyCFunction)ftable_wrapper_::set_bounds_method_,METH_VARARGS,(char *)ftable_set_bounds_docstring_ },
    
        { "set_entry",(PyCFunction)ftable_wrapper_::set_entry_method_,METH_VARARGS,(char *)ftable_set_entry_docstring_ },
    
    
        { "py_lock",(PyCFunction)ftable_wrapper_::_method_lock,METH_VARARGS,"increment python reference count" },
        { "py_unlock",(PyCFunction)ftable_wrapper_::_method_unlock,METH_VARARGS,"decrement python reference count" },
    {NULL}  /* Sentinel */
};

static const char *ftable_docstring_ = "";

PyTypeObject ftable_typeobject_ = 
{
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "picross_native.ftable",/*tp_name*/
    sizeof(ftable_object_),          /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)ftable_dealloc_, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
      0 , /* tp_cmp */
      0 , /* tp_repr */
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
      0 , /* tp_hash */
      0 , /* tp_call */
      0 , /* tp_str */
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE , /*tp_flags*/
    (char *)ftable_docstring_,       /* tp_doc */
     0 ,   /* tp_traverse */
     0 ,           /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    ftable_methods_,         /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base fixed up dynamically */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)ftable_init_,  /* tp_init */
    0,                         /* tp_alloc */
    ftable_new_,             /* tp_new */
};

PyObject *ftable_wrapcopy_(ftable_type_ *o)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    

    
        
            PyErr_SetString(PyExc_RuntimeError,"passing non copyable class by value");
            return 0;
        

        

    
}

PyObject *ftable_wrap_(ftable_type_ *o, bool owned)
{
    if(!o)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }

    PyObject *po = ftable_dispatcher_->find(o);
    if(po != 0)
    {
        Py_INCREF(po);
        return po;
    }

    ftable_object_ *self = 
        PyObject_NEW(ftable_object_, &ftable_typeobject_); 
        

    if(!self) return 0;

    

    self->object = o;
    self->wrapped = 0;
    self->locked = 0;
    self->owned = owned;

    if(self->owned)
    {
        ftable_dispatcher_->insert(std::make_pair(self->object,(PyObject *)self));
    }

    return (PyObject *)self;
}

ftable_type_ *ftable_unwrap_(PyObject *o)
{
    if(o == Py_None)
    {
        return 0;
    }

    if(!PyType_IsSubtype(o->ob_type, &ftable_typeobject_))
    {
        PyErr_SetString(PyExc_TypeError,"expected ftable object");
        return 0;
    }

    ftable_type_ *object = static_cast<ftable_type_ *>(((ftable_object_ *)o)->object);

    if(!object)
    {
        PyErr_SetString(PyExc_TypeError,"null ftable object in unwrap");
        return 0;
    }

    return object;
}

void ftable_insert_(const std::pair< ftable_type_ *, PyObject *> &p)
{




    ftable_wrapped_.insert(p);


}

void ftable_erase_(ftable_type_ *p)
{




    ftable_wrapped_.erase(p);


}

PyObject * ftable_find_(ftable_type_ *p)
{




    PyObject *ret = 0;
    std::map< ftable_type_ *, PyObject *>::iterator where = ftable_wrapped_.find(p);
    if(where != ftable_wrapped_.end())
    {
        ret = where->second;
    }
    return ret;


}

static struct ftable_dispatch_ ftable_realdispatcher_ =
{
    &ftable_typeobject_,
    ftable_wrap_,
    ftable_wrapcopy_,
    ftable_unwrap_,
    ftable_insert_,
    ftable_erase_,
    ftable_find_
};

 


static const char *module_method_enumerate_docstring_ = ""
"    enumerate(vendor,product,functor) -> number."
""
"    Enumerate USB devices on the bus.  The functor is called with a device name"
"    for each device of the specified type.  The number of such devices is returned."
"    "
""
;

static PyObject *module_method_enumerate(PyObject *self, PyObject *args)
{
     unsigned short a0;  unsigned short a1;  f_string_type_ * a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_us,&a0,fpcvt_us,&a1,fpcvt_f_string_copy,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    unsigned int r = pic::usbenumerator_t::enumerate  (  a0  ,a1  ,*a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_ui((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in enumerate");
    }

    return 0;
}


static const char *module_method_find_docstring_ = ""
"    find(vendor,product) -> name."
""
"    Find a particular device on the bus.  Throws an exception if no device is found."
"    returns the name of the first device found"
"    "
""
;

static PyObject *module_method_find(PyObject *self, PyObject *args)
{
     unsigned short a0;  unsigned short a1; 

    if(!PyArg_ParseTuple(args,"O&O&" ,fpcvt_us,&a0,fpcvt_us,&a1))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::usbenumerator_t::find  (  a0  ,a1 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in find");
    }

    return 0;
}


static const char *module_method_find2_docstring_ = "";

static PyObject *module_method_find2(PyObject *self, PyObject *args)
{
     unsigned short a0;  unsigned short a1;  bool a2; 

    if(!PyArg_ParseTuple(args,"O&O&O&" ,fpcvt_us,&a0,fpcvt_us,&a1,fpcvt_b,&a2))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::usbenumerator_t::find  (  a0  ,a1  ,a2 );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in find2");
    }

    return 0;
}


static const char *module_method_release_resource_dir_docstring_ = "";

static PyObject *module_method_release_resource_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::release_resource_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in release_resource_dir");
    }

    return 0;
}


static const char *module_method_global_resource_dir_docstring_ = "";

static PyObject *module_method_global_resource_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::global_resource_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in global_resource_dir");
    }

    return 0;
}


static const char *module_method_release_library_dir_docstring_ = "";

static PyObject *module_method_release_library_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::release_library_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in release_library_dir");
    }

    return 0;
}


static const char *module_method_release_root_dir_docstring_ = "";

static PyObject *module_method_release_root_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::release_root_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in release_root_dir");
    }

    return 0;
}


static const char *module_method_global_library_dir_docstring_ = "";

static PyObject *module_method_global_library_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::global_library_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in global_library_dir");
    }

    return 0;
}


static const char *module_method_python_prefix_dir_docstring_ = "";

static PyObject *module_method_python_prefix_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::python_prefix_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in python_prefix_dir");
    }

    return 0;
}


static const char *module_method_public_tools_dir_docstring_ = "";

static PyObject *module_method_public_tools_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::public_tools_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in public_tools_dir");
    }

    return 0;
}


static const char *module_method_private_tools_dir_docstring_ = "";

static PyObject *module_method_private_tools_dir(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::private_tools_dir  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in private_tools_dir");
    }

    return 0;
}


static const char *module_method_release_docstring_ = "";

static PyObject *module_method_release(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    std::string r = pic::release  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_stdstr((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in release");
    }

    return 0;
}


static const char *module_method_is_linux_docstring_ = "";

static PyObject *module_method_is_linux(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    bool r = pic::is_linux  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in is_linux");
    }

    return 0;
}


static const char *module_method_is_macosx_docstring_ = "";

static PyObject *module_method_is_macosx(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    bool r = pic::is_macosx  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in is_macosx");
    }

    return 0;
}


static const char *module_method_is_windows_docstring_ = "";

static PyObject *module_method_is_windows(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    

    

        PyObject *ro;
        {
            {   unlock_p2c unlock;
                {
                    bool r = pic::is_windows  ( );
                    {   relock_p2c relock(unlock);
                        ro = tpcvt_b((void *)&r);
                        return ro;
                    }
                }
            }
        }

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in is_windows");
    }

    return 0;
}


static const char *module_method_display_active_docstring_ = "";

static PyObject *module_method_display_active(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            pic::display_active  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in display_active");
    }

    return 0;
}


static const char *module_method_disk_active_docstring_ = "";

static PyObject *module_method_disk_active(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            pic::display_active  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in disk_active");
    }

    return 0;
}


static const char *module_method_to_front_docstring_ = "";

static PyObject *module_method_to_front(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            pic::to_front  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in to_front");
    }

    return 0;
}


static const char *module_method_pic_init_time_docstring_ = "";

static PyObject *module_method_pic_init_time(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            pic_init_time  (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_init_time");
    }

    return 0;
}


static const char *module_method_pic_mlock_code_docstring_ = "";

static PyObject *module_method_pic_mlock_code(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             pic_mlock_code (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_mlock_code");
    }

    return 0;
}


static const char *module_method_exit_docstring_ = "";

static PyObject *module_method_exit(PyObject *self, PyObject *args)
{
     int a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_i,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
            pic::exit  (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in exit");
    }

    return 0;
}


static const char *module_method_pic_set_foreground_docstring_ = "";

static PyObject *module_method_pic_set_foreground(PyObject *self, PyObject *args)
{
     bool a0; 

    if(!PyArg_ParseTuple(args,"O&" ,fpcvt_b,&a0))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             pic_set_foreground (  a0  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_set_foreground");
    }

    return 0;
}


static const char *module_method_pic_set_fpu_docstring_ = "";

static PyObject *module_method_pic_set_fpu(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             pic_set_fpu (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_set_fpu");
    }

    return 0;
}


static const char *module_method_pic_set_interrupt_docstring_ = "";

static PyObject *module_method_pic_set_interrupt(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             pic_set_interrupt (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_set_interrupt");
    }

    return 0;
}


static const char *module_method_pic_set_core_unlimited_docstring_ = "";

static PyObject *module_method_pic_set_core_unlimited(PyObject *self, PyObject *args)
{
    

    if(!PyArg_ParseTuple(args,"" ))
    {
        return 0;
    }

    try
    {

    
        {   unlock_p2c unlock;
             pic_set_core_unlimited (  );
        }
        Py_INCREF(Py_None);
        return Py_None;
    

    

    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in pic_set_core_unlimited");
    }

    return 0;
}




static const char *module_method_make_notify_functor_docstring_ = ""
"    make_notify_functor(callable) -> new notify functor"
"    "
"    Make notify functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_notify_functor(PyObject *self, PyObject *args)
{

    return pic_make_notify_functor(args);

}

static PyObject *module_method_make_notify_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_notify_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_notify_functor");
        return 0;
    }
}


static const char *module_method_make_status_functor_docstring_ = ""
"    make_status_functor(callable) -> new status functor"
""
"    Make status functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_status_functor(PyObject *self, PyObject *args)
{

    return pic_make_status_functor(args);

}

static PyObject *module_method_make_status_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_status_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_status_functor");
        return 0;
    }
}


static const char *module_method_make_string_functor_docstring_ = ""
"    make_string_functor(callable) -> new f_string tunctor"
"    Make f_string functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_string_functor(PyObject *self, PyObject *args)
{

       return pic_make_f_string_functor(args);
    
}

static PyObject *module_method_make_string_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_string_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_string_functor");
        return 0;
    }
}


static const char *module_method_make_ff2f_functor_docstring_ = ""
"    make_ff2f_functor(callable) -> new ff2f functor"
"    "
"    Make ff2f functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_ff2f_functor(PyObject *self, PyObject *args)
{

       return pic_make_ff2f_functor(args);
    
}

static PyObject *module_method_make_ff2f_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_ff2f_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_ff2f_functor");
        return 0;
    }
}


static const char *module_method_make_f2f_functor_docstring_ = ""
"    make_f2f_functor(callable) -> new f2f functor"
"    "
"    Make f2f functor from python callable"
"    "
""
;

static PyObject *module_method_inner_make_f2f_functor(PyObject *self, PyObject *args)
{

       return pic_make_f2f_functor(args);
    
}

static PyObject *module_method_make_f2f_functor(PyObject *self, PyObject *args)
{
    try
    {
        PyObject *rv = module_method_inner_make_f2f_functor(self,args);
        return rv;
    }
    catch(const std::exception &e)
    {
        PyErr_SetString(PyExc_RuntimeError,e.what());
        return 0;
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"c++ exception in ::make_f2f_functor");
        return 0;
    }
}



static PyMethodDef module_method_table[] = {
    
{ "enumerate",module_method_enumerate,METH_VARARGS, (char *)module_method_enumerate_docstring_ }, 
{ "find",module_method_find,METH_VARARGS, (char *)module_method_find_docstring_ }, 
{ "find2",module_method_find2,METH_VARARGS, (char *)module_method_find2_docstring_ }, 
{ "release_resource_dir",module_method_release_resource_dir,METH_VARARGS, (char *)module_method_release_resource_dir_docstring_ }, 
{ "global_resource_dir",module_method_global_resource_dir,METH_VARARGS, (char *)module_method_global_resource_dir_docstring_ }, 
{ "release_library_dir",module_method_release_library_dir,METH_VARARGS, (char *)module_method_release_library_dir_docstring_ }, 
{ "release_root_dir",module_method_release_root_dir,METH_VARARGS, (char *)module_method_release_root_dir_docstring_ }, 
{ "global_library_dir",module_method_global_library_dir,METH_VARARGS, (char *)module_method_global_library_dir_docstring_ }, 
{ "python_prefix_dir",module_method_python_prefix_dir,METH_VARARGS, (char *)module_method_python_prefix_dir_docstring_ }, 
{ "public_tools_dir",module_method_public_tools_dir,METH_VARARGS, (char *)module_method_public_tools_dir_docstring_ }, 
{ "private_tools_dir",module_method_private_tools_dir,METH_VARARGS, (char *)module_method_private_tools_dir_docstring_ }, 
{ "release",module_method_release,METH_VARARGS, (char *)module_method_release_docstring_ }, 
{ "is_linux",module_method_is_linux,METH_VARARGS, (char *)module_method_is_linux_docstring_ }, 
{ "is_macosx",module_method_is_macosx,METH_VARARGS, (char *)module_method_is_macosx_docstring_ }, 
{ "is_windows",module_method_is_windows,METH_VARARGS, (char *)module_method_is_windows_docstring_ }, 
{ "display_active",module_method_display_active,METH_VARARGS, (char *)module_method_display_active_docstring_ }, 
{ "disk_active",module_method_disk_active,METH_VARARGS, (char *)module_method_disk_active_docstring_ }, 
{ "to_front",module_method_to_front,METH_VARARGS, (char *)module_method_to_front_docstring_ }, 
{ "pic_init_time",module_method_pic_init_time,METH_VARARGS, (char *)module_method_pic_init_time_docstring_ }, 
{ "pic_mlock_code",module_method_pic_mlock_code,METH_VARARGS, (char *)module_method_pic_mlock_code_docstring_ }, 
{ "exit",module_method_exit,METH_VARARGS, (char *)module_method_exit_docstring_ }, 
{ "pic_set_foreground",module_method_pic_set_foreground,METH_VARARGS, (char *)module_method_pic_set_foreground_docstring_ }, 
{ "pic_set_fpu",module_method_pic_set_fpu,METH_VARARGS, (char *)module_method_pic_set_fpu_docstring_ }, 
{ "pic_set_interrupt",module_method_pic_set_interrupt,METH_VARARGS, (char *)module_method_pic_set_interrupt_docstring_ }, 
{ "pic_set_core_unlimited",module_method_pic_set_core_unlimited,METH_VARARGS, (char *)module_method_pic_set_core_unlimited_docstring_ }, 
    
{ "make_notify_functor",module_method_make_notify_functor,METH_VARARGS, (char *)module_method_make_notify_functor_docstring_ }, 
{ "make_status_functor",module_method_make_status_functor,METH_VARARGS, (char *)module_method_make_status_functor_docstring_ }, 
{ "make_string_functor",module_method_make_string_functor,METH_VARARGS, (char *)module_method_make_string_functor_docstring_ }, 
{ "make_ff2f_functor",module_method_make_ff2f_functor,METH_VARARGS, (char *)module_method_make_ff2f_functor_docstring_ }, 
{ "make_f2f_functor",module_method_make_f2f_functor,METH_VARARGS, (char *)module_method_make_f2f_functor_docstring_ }, 
    {0}
};
};


struct pic_notify_sink_t: public pic::notify_t::sinktype_t
{
    pic_notify_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~pic_notify_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return _callable;
    }

    void invoke() const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"");

        if(r) Py_DECREF(r);
        else PyErr_Print();

        if(!r) PIC_THROW("python exception inside functor");
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *pic_make_notify_functor(PyObject *args)
{
    PyObject *o;
    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make notify functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    pic::notify_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new pic::notify_t(pic::ref(new pic_notify_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return notify_wrap_(c,true);
}

struct pic_status_sink_t: public pic::status_t::sinktype_t
{
    pic_status_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~pic_status_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return _callable;
    }

    void invoke(bool b) const
    {
        lock_c2p lock(_interp);
        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_b,&b);

        if(r) Py_DECREF(r);
        else PyErr_Print();

        if(!r) PIC_THROW("python exception inside functor");
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *pic_make_status_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make status functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    pic::status_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new pic::status_t(pic::ref(new pic_status_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return status_wrap_(c,true);
}

struct pic_f_string_sink_t: public pic::f_string_t::sinktype_t
{
    pic_f_string_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~pic_f_string_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return _callable;
    }

    void invoke(const char *d) const
    {
        lock_c2p lock(_interp);
        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_str,&d);

        if(r) Py_DECREF(r);
        else PyErr_Print();

        if(!r) PIC_THROW("python exception inside functor");
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *pic_make_f_string_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make f_string functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    pic::f_string_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new pic::f_string_t(pic::ref(new pic_f_string_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return f_string_wrap_(c,true);
}

struct pic_ff2f_sink_t: public pic::ff2f_t::sinktype_t
{
    pic_ff2f_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~pic_ff2f_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return _callable;
    }

    float invoke(float f1, float f2) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&O&",tpcvt_f,&f1,tpcvt_f,&f2);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        float rv;

        if(!fpcvt_f(r,&rv))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *pic_make_ff2f_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make ff2f functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    pic::ff2f_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new pic::ff2f_t(pic::ref(new pic_ff2f_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return ff2f_wrap_(c,true);
}

struct pic_f2f_sink_t: public pic::f2f_t::sinktype_t
{
    pic_f2f_sink_t(PyInterpreterState *i, PyObject *c): _interp(i), _callable(c)
    {
    }

    ~pic_f2f_sink_t()
    {
        if(_callable)
        {
            lock_c2p lock(_interp);
            Py_CLEAR(_callable);
        }
    }

    int gc_visit(void *visit, void *arg) const
    {
        if(_callable)
        {
            return ((visitproc)visit)(_callable,arg);
        }

        return 0;
    }

    bool iscallable() const 
    {
        return _callable;
    }

    float invoke(float f) const
    {
        lock_c2p lock(_interp);

        PyObject *r = PyObject_CallFunction(_callable,(char *)"O&",tpcvt_f,&f);

        if(!r)
        {
            PyErr_Print();
            PIC_THROW("python exception inside functor");
        }

        float rv;

        if(!fpcvt_f(r,&rv))
        {
            Py_DECREF(r);
            PyErr_Print();
            PIC_THROW("incorrect (non data) return type from functor");
        }

        {   unlock_p2c unlock;
            {
                {   relock_p2c relock(unlock);
                    Py_DECREF(r);
                }
                return rv;
            }
        }
    }

    PyInterpreterState *_interp;
    PyObject *_callable;
};

static PyObject *pic_make_f2f_functor(PyObject *args)
{
    PyObject *o;

    PyEval_InitThreads();

    if(!PyArg_ParseTuple(args,"O",&o))
    {
        return 0;
    }

    if(!PyCallable_Check(o))
    {
        PyErr_SetString(PyExc_TypeError,"can't make f2f functor from non callable");
        return 0;
    }

    Py_INCREF(o);

    pic::f2f_t *c = 0;

    try
    {
        PyInterpreterState *interp = PyThreadState_Get()->interp;
        {   unlock_p2c unlock;
            c = new pic::f2f_t(pic::ref(new pic_f2f_sink_t(interp,o)));
        }
    }
    catch(...)
    {
        PyErr_SetString(PyExc_RuntimeError,"unknown c++ exception creating functor");
        Py_DECREF(o);
        return 0;
    }

    return f2f_wrap_(c,true);
}



#ifdef __GNUC__
  #pragma GCC visibility pop
  #pragma GCC visibility push(default)
  #define MODULE_VISIBILITY __attribute__ ((visibility("default")))
#else
  #define MODULE_VISIBILITY
#endif


PyMODINIT_FUNC MODULE_VISIBILITY initpicross_native(void)
{
    PyObject *m;
    PyObject *s;

    PyEval_InitThreads();

    pip_usegil = (getenv("PIP_USEGIL")!=0);

    
        
            notify_dispatcher_ = &notify_realdispatcher_;
        
    
        
            status_dispatcher_ = &status_realdispatcher_;
        
    
        
            f_string_dispatcher_ = &f_string_realdispatcher_;
        
    
        
            ff2f_dispatcher_ = &ff2f_realdispatcher_;
        
    
        
            f2f_dispatcher_ = &f2f_realdispatcher_;
        
    
        
            bulk_out_pipe_dispatcher_ = &bulk_out_pipe_realdispatcher_;
        
    
        
            usbdevice_dispatcher_ = &usbdevice_realdispatcher_;
        
    
        
            enumerator_dispatcher_ = &enumerator_realdispatcher_;
        
    
        
            gate_dispatcher_ = &gate_realdispatcher_;
        
    
        
            mutex_dispatcher_ = &mutex_realdispatcher_;
        
    
        
            thread_dispatcher_ = &thread_realdispatcher_;
        
    
        
            ftable_dispatcher_ = &ftable_realdispatcher_;
        
    

    

    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    
        
            
        
    

    
        
            if(PyType_Ready(&notify_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&status_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&f_string_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&ff2f_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&f2f_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&bulk_out_pipe_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&usbdevice_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&enumerator_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&gate_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&mutex_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&thread_typeobject_)<0)
                return;
        
    
        
            if(PyType_Ready(&ftable_typeobject_)<0)
                return;
        
    

    if(!(m=Py_InitModule("picross_native",module_method_table)))
    {
        return;
    }

    if(!(s=PyString_FromString(module_docstring)))
    {
        return;
    }

    PyModule_AddObject(m,"__doc__",s);

    
        
            if(!(s=PyCObject_FromVoidPtr(notify_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_notify_dispatch",s);
            Py_INCREF(&notify_typeobject_);
            PyModule_AddObject(m,"notify",(PyObject *)&notify_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(status_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_status_dispatch",s);
            Py_INCREF(&status_typeobject_);
            PyModule_AddObject(m,"status",(PyObject *)&status_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(f_string_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_f_string_dispatch",s);
            Py_INCREF(&f_string_typeobject_);
            PyModule_AddObject(m,"f_string",(PyObject *)&f_string_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(ff2f_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_ff2f_dispatch",s);
            Py_INCREF(&ff2f_typeobject_);
            PyModule_AddObject(m,"ff2f",(PyObject *)&ff2f_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(f2f_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_f2f_dispatch",s);
            Py_INCREF(&f2f_typeobject_);
            PyModule_AddObject(m,"f2f",(PyObject *)&f2f_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(bulk_out_pipe_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_bulk_out_pipe_dispatch",s);
            Py_INCREF(&bulk_out_pipe_typeobject_);
            PyModule_AddObject(m,"bulk_out_pipe",(PyObject *)&bulk_out_pipe_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(usbdevice_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_usbdevice_dispatch",s);
            Py_INCREF(&usbdevice_typeobject_);
            PyModule_AddObject(m,"usbdevice",(PyObject *)&usbdevice_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(enumerator_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_enumerator_dispatch",s);
            Py_INCREF(&enumerator_typeobject_);
            PyModule_AddObject(m,"enumerator",(PyObject *)&enumerator_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(gate_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_gate_dispatch",s);
            Py_INCREF(&gate_typeobject_);
            PyModule_AddObject(m,"gate",(PyObject *)&gate_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(mutex_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_mutex_dispatch",s);
            Py_INCREF(&mutex_typeobject_);
            PyModule_AddObject(m,"mutex",(PyObject *)&mutex_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(thread_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_thread_dispatch",s);
            Py_INCREF(&thread_typeobject_);
            PyModule_AddObject(m,"thread",(PyObject *)&thread_typeobject_);
        
    
        
            if(!(s=PyCObject_FromVoidPtr(ftable_dispatcher_,0)))
            {
                return;
            }

            PyModule_AddObject(m,"pip_ftable_dispatch",s);
            Py_INCREF(&ftable_typeobject_);
            PyModule_AddObject(m,"ftable",(PyObject *)&ftable_typeobject_);
        
    

    
    
    
    
    
    
    
    
    
    
    
    
    
}

#ifdef __GNUC__
#pragma GCC visibility pop
#endif

